// BuildInitResponse() method returns a javascript object containing the data to be sent to the client for load time initialisation.
// This should include stakes, currency code, and reel bands.
// On the server, this is converted to XML then forwarded to the client. On a client only config, this object is used directly
goog.provide('game.logic.BuildInitResponse');


(function() {
	"use strict";

	var BuildInitResponse = function( currencyCode ) {
		
		var init_element = {};
		var i;

		var currCode = currencyCode; // currency code from Vegas server
		var currRate = game.logic.GameData.williamHill.currencyCodeRate[currCode];
		if (!currRate) {
			throw( { message: "Invalid currency code: "+currCode } );
		}

		var stake_steps = [];
		for (i = 0; i < game.logic.GameData.financial.availableStakesPerLine.length; i++) {
			stake_steps.push( currRate * game.logic.GameData.financial.availableStakesPerLine[i] );
		}
		
		init_element.stake = { steps: stake_steps, starting: ( currRate * game.logic.GameData.financial.defaultStartingStakePerLine ) };
		init_element.currency = { code: currCode };
		
		init_element.reel_set = [];
		
		game.logic.GameData.createReelSets();

		for (i = 0; i < game.logic.GameData.reelSets.length; i++)
		{
			// check if reel set is used in wild exchange, replace the wild symbols with the 'exchanged' symbol
			// Create copy of existing game data arrays so modifications do not effect the copy in gameData object
			var reelSet = game.logic.GameData.reelSets[i].clone();
			
			var isFreeSpinSet = game.logic.GameData.reels.freespinsReelSets.some( function( fsSetName ) {
				return fsSetName === reelSet.getName();
			} );
			
			if ( isFreeSpinSet ) {
				// Replace wild symbols in free spins sets with the free spins specific wild symbol
				reelSet.replaceAllSymbols( 'WI', 'FW' );
			}

			init_element.reel_set.push( reelSet.toArray() );
		}

		return init_element;
	};

	
	game.logic.BuildInitResponse = BuildInitResponse;

}());

