// This file provides a method called by the autorun tool for every checkpoint so any custom logging can be performed.
// For example, one may wish to separate the 'freespins' features into 'freespins_x3', 'freespins_x4', 'freespins_x5' for logging purposes
goog.provide('game.logic.customStatsLogging');


(function() {
	"use strict";
	
	var ExportedBigNumber = BigNumber;
	

	var increment = function( obj, propertyName, value ) {
		if ( !obj[ propertyName ] ) {
			obj[ propertyName ] = new ExportedBigNumber( 0 );
		}
		obj[ propertyName ] = obj[ propertyName ].plus( value === undefined ? 1 : value );
	};


	var getObject = function( obj, objectName ) {
		if ( !obj[ objectName ] ) {
			obj[ objectName ] = {};
		}
		return obj[ objectName ];
	};


	var doSpinLog = function( ExportedBigNumberArg, serverBetResults, statsObject ) {
		
		ExportedBigNumber = ExportedBigNumberArg;
		if ( serverBetResults && serverBetResults.bonus ) {

			for ( var bonusIndex=0; bonusIndex<serverBetResults.bonus.length; ++bonusIndex ) {
				var bonusObj = serverBetResults.bonus[ bonusIndex ];
				
				if ( bonusObj['name'] === "freeSpinCollect" ) {
				
					var isLastSpin = !serverBetResults['expectCheckpoint'];
					var trigger = bonusObj['trigger'];

					var newBonusName = "FreeSpins_X" + trigger;
					var bonusRootElement = getObject( statsObject, 'bonus' );
					var bonusElement = getObject( bonusRootElement, newBonusName );
					
					if ( isLastSpin ) {
						var count = bonusObj['count'];
						var won = bonusObj['accumulativeWin'] ? bonusObj['accumulativeWin'] : new BigNumber( 0 );

						increment( bonusElement, 'won', won );
						increment( bonusElement, 'frequency', 1 );
						increment( bonusElement, 'totalSpinCount', count );

						var winbreakdown = getObject( bonusElement, 'winBreakdown' );
						increment( winbreakdown, won, 1 );
					}
					
					var customEvents = getObject( bonusElement, 'customEvents' );
					
					if ( bonusObj['retrigger'] ) {
						var retriggerAmount = Number( bonusObj['retrigger'] );
						var retriggerEvent = getObject( customEvents, 'retrigger' );
						increment( retriggerEvent, 'frequency', 1 );
						increment( retriggerEvent, 'total', retriggerAmount );
					}
					if ( bonusObj['retriggerType'] ) {
						var retriggerType = bonusObj['retriggerType'];
						var retriggerTypeEvent = getObject( customEvents, 'retriggerType_' + retriggerType );
						increment( retriggerTypeEvent, 'frequency', 1 );
					}
				} else if ( bonusObj['name'] === "hiddenWilds" ) {
					
					var mysteryObject = getObject( statsObject, 'bonus' );
					var bonusElement = getObject( mysteryObject, "MysterySymbol" );

					increment( bonusElement, 'won', bonusObj['win'] );
					increment( bonusElement, 'frequency', 1 );
					
					if ( bonusObj['isWinningSpin'] > 0 ) {
						increment( bonusElement, 'winningFrequency', 1 );
					} else {
						increment( bonusElement, 'losingFrequency', 1 );
					}
				}
			}
		}
	};

	
	game.logic.customStatsLogging = {};
	game.logic.customStatsLogging.onlyLogTheseBonuses = [ 'MysterySymbol', 'FreeSpins_X3', 'FreeSpins_X4', 'FreeSpins_X5' ];
	game.logic.customStatsLogging.doSpinLog = doSpinLog;

}());

