goog.provide( 'game.logic.GameData' );

goog.require( 'libs.server.ReelBandSet' );
goog.require( 'libs.server.AwardsTable' );
goog.require( 'libs.server.OutcomeTable' );
goog.require( 'libs.server.SymbolMap' );


(function() {
	"use strict";

	// Game specific data used by buildBetResults() function
	var GameData = {};

	var numberOfReels = 5;
	var numberOfSymbolRows = 4;

	GameData.friendlyGameName = "Goddess Of The Amazon";

	// Entry bonus multiplied by total stake
	GameData.freeSpins = {
		entryBonus: { '3': 0, '4': 0, '5': 0 },
		awarded: { '3': 8, '4': 16, '5': 30 },
		lastSpinRetriggerThreshold: { '3': 5, '4': 20, '5': 40 }, // (multiplied by the total stake)
		retriggerAmount: { '3': 8, '4': 8, '5': 8 }
	};

	// Shared financial data
	GameData.financial = {
		availableStakesPerLine: [ 1, 2, 5, 10, 20, 30, 40, 50, 100, 200, 300, 400, 500, 1000, 1500, 2000, 2500 ],
		defaultStartingStakePerLine: 10,
		maxPayoutPerSpin: 25000000 // Max Payout on a spin (server units(=pence))
	};

	// Outcome tables
	var primaryReelsetOutcomeTablePerStake = { '0':3500, '1':56000, '2':19720, '3':14910 };

	var primaryReelsetOutcomeTablePerStakeChange = {
		'1':		{ '4': 0, '5': 5870 },
		'2':		{ '4': 0, '5': 5870 },
		'5':		{ '4': 0, '5': 5870 },
		'10':		{ '4': 0, '5': 5870 },
		'20':		{ '4': 0, '5': 5870 },
		'30':		{ '4': 0, '5': 5870 },
		'40':		{ '4': 0, '5': 5870 },
		'50':		{ '4': 0, '5': 5870 },
		'100':	{ '4': 0, '5': 5870 },
		'200':	{ '4': 0, '5': 5870 },
		'300':	{ '4': 0, '5': 5870 },
		'400':	{ '4': 0, '5': 5870 },
		'500':	{ '4': 0, '5': 5870 },
		'1000':	{ '4': 0, '5': 5870 },
		'1500':	{ '4': 0, '5': 5870 },
		'2000':	{ '4': 0, '5': 5870 },
		'2500':	{ '4': 0, '5': 5870 }
	};

	// number of initial symbols (so 3 is when the player wins 3x symbols on the reels)
	var freeSpinReelsetOutcomeTable = { '6': 29000, '7': 29000, '8': 29500 };

	var freeSpinReelsetChangeOutcomeTable = {
		'1':		{ '9': 0, '10': 12500 },
		'2':		{ '9': 0, '10': 12500 },
		'5':		{ '9': 0, '10': 12500 },
		'10':		{ '9': 0, '10': 12500 },
		'20':		{ '9': 0, '10': 12500 },
		'30':		{ '9': 0, '10': 12500 },
		'40':		{ '9': 0, '10': 12500 },
		'50':		{ '9': 0, '10': 12500 },
		'100':	{ '9': 0, '10': 12500 },
		'200':	{ '9': 0, '10': 12500 },
		'300':	{ '9': 0, '10': 12500 },
		'400':	{ '9': 0, '10': 12500 },
		'500':	{ '9': 0, '10': 12500 },
		'1000':	{ '9': 0, '10': 12500 },
		'1500':	{ '9': 0, '10': 12500 },
		'2000':	{ '9': 0, '10': 12500 },
		'2500':	{ '9': 0, '10': 12500 }
	};
		
	var slots_linewin_table_1 = {"3":[10,10,10,20,20,40,60,80,100,100],"4":[20,20,20,60,80,100,120,160,200,200],"5":[100,100,100,120,160,200,300,400,500,500]};
	
	// Shared reels data
	GameData.reelSetData = [
		{
			name: 'reel_set_1',
			data: [["WI","WI","E","J","GA","J","T","T","S","SS","SS","S","CG","CG","CG","GA","SS","S","S","SS","E","E","J","WI","WI","E","R","I","R","SS","SS","SS","E","E","S","S","J","SP","J","R","GA","R","GA","GA","GA","GA","I","I","I","E","E","WI","WI","WI","WI","I","I","I","I","R","T","T","T","SS","WI","R","FS","R","J","E","CG","R","E","E","R","GA","GA","GA","E","R","S","R","WI","T","T","S","R","SS","SS","SS","SS","R","WI","WI","WI","I","R","S","T","R","R","E","E","WI","SS","J","J","J","GA","GA","I","CG","T","T","I","CG","WI","WI","SS","E","E","S","S","J","S","I","I","WI","S","S","I","CG","T","T","T","T","CG","E","E","S","SS","WI","WI","WI","I","I","CG","S","T","S","T","J","J","R","E","E","S","CG","S","GA","GA","SS","SS","S","R","SS","CG","CG"],["S","T","S","WI","R","S","E","J","J","J","J","R","I","S","S","R","T","GA","E","I","R","R","WI","WI","SS","SS","SS","SS","T","S","E","J","R","SS","E","E","J","J","WI","WI","WI","WI","CG","CG","CG","R","SS","SS","GA","R","S","S","J","J","J","R","WI","WI","E","E","GA","GA","CG","I","I","I","I","SS","J","WI","S","SS","E","T","T","T","T","S","R","WI","WI","WI","S","CG","CG","CG","CG","I","I","I","WI","R","S","CG","S","SS","GA","E","E","SS","SS","SS","SS","WI","WI","S","I","I","T","T","T","E","SS","S","R","T","T","S","SS","T","SP","E","E","GA","GA","R","R","J","E","WI","WI","WI","I","S","GA","GA","GA","R","R","CG","T","E","R","FS","E","CG","E","E","I","SS","SS","T","S","S","T","E","WI","E","I","I","R","GA","GA","GA","GA","CG","I","S"],["J","J","E","R","R","I","SS","SS","SS","E","S","E","GA","GA","GA","GA","S","T","E","S","S","SS","E","WI","WI","T","S","S","E","R","I","I","I","S","J","S","R","E","SS","SS","E","GA","GA","GA","E","E","WI","WI","WI","E","S","S","GA","I","I","I","I","WI","T","T","E","E","FS","R","T","T","T","T","R","SS","WI","WI","CG","CG","CG","SP","J","T","E","R","J","SS","WI","CG","CG","CG","CG","GA","R","GA","I","R","I","R","CG","E","WI","WI","WI","S","E","R","CG","R","S","SS","S","R","WI","SS","SS","SS","SS","T","GA","GA","I","I","I","R","SS","WI","E","WI","WI","J","J","J","R","S","CG","CG","CG","SS","SS","S","S","WI","WI","SS","CG","I","T","T","T","T","S","E","J","GA","R","E","T","E","R","WI","WI","SS","R","T","I","I","S","GA","J","J","J","J","S","S"],["S","WI","I","J","J","J","J","GA","S","E","T","R","R","S","WI","WI","WI","WI","R","R","I","I","E","GA","GA","GA","GA","S","I","I","I","R","WI","WI","S","R","T","T","T","T","E","S","S","S","I","WI","R","R","GA","E","J","J","J","SS","CG","WI","WI","WI","E","T","SS","SS","SS","SS","I","E","WI","CG","S","R","CG","GA","CG","E","E","T","T","T","R","WI","WI","E","S","I","I","I","I","R","SS","SS","WI","R","R","S","E","GA","GA","GA","SS","S","J","GA","S","E","WI","WI","WI","S","SS","SS","SS","S","S","R","E","I","WI","WI","T","T","SS","R","FS","R","T","S","J","J","E","CG","R","WI","SS","WI","E","CG","CG","CG","CG","S","E","GA","GA","I","E","S","T","T","CG","WI","I","R","SS","SS","SS","GA","E","E","J","SS","SS","WI","WI","WI","T","E","T","J","SP","E","T","J","S","I","CG","CG","CG"],["S","T","E","R","WI","WI","E","R","E","GA","S","T","T","T","I","I","I","WI","J","J","J","J","GA","E","E","SS","S","WI","WI","WI","CG","R","T","T","T","T","E","CG","WI","R","R","I","GA","GA","GA","SS","R","SS","SS","R","CG","WI","WI","SS","S","GA","GA","GA","GA","T","T","S","WI","E","SS","SS","SS","SS","E","GA","GA","J","J","S","SS","R","WI","WI","WI","T","T","E","I","I","I","I","SS","SS","WI","WI","E","CG","CG","CG","CG","J","J","GA","R","R","SS","I","E","WI","S","WI","J","CG","S","S","R","E","J","GA","I","R","R","S","S","T","T","WI","S","E","T","T","E","CG","CG","E","E","I","S","WI","WI","WI","E","E","R","R","SS","SS","SS","S","FS","E","J","J","J","I","WI","R","R","T","S","CG","CG","CG","GA","I","S","WI","WI","S","E","SS","SS","WI","S","WI","S","R","I","I","I","S"]]
		},
		{
			name: 'reel_set_2',
			data: [["GA","T","E","R","GA","CG","CG","S","FS","E","SS","S","CG","CG","CG","CG","SS","S","GA","SS","E","SS","E","T","T","J","I","R","CG","SS","SS","SS","E","E","R","J","J","J","R","SP","S","R","GA","GA","GA","GA","R","I","I","E","E","J","T","SS","SS","I","I","I","I","T","T","T","R","FS","R","I","R","GA","E","S","R","E","E","J","E","T","J","J","SS","R","CG","R","FS","R","CG","S","S","SS","SS","SS","SS","E","WI","SS","E","SS","R","R","GA","S","R","SS","E","FS","S","J","J","J","J","GA","E","CG","S","S","I","CG","E","S","SS","E","I","T","S","S","T","I","S","FS","S","I","I","S","T","T","T","T","S","E","E","R","I","S","S","R","SS","I","S","S","J","S","T","J","E","E","S","E","R","S","S","R","CG","R","T","T","WI"],["S","I","S","WI","R","S","S","J","FS","J","J","T","T","E","S","S","CG","J","E","I","R","J","I","WI","SS","SS","SS","R","T","S","FS","E","R","SS","E","E","T","J","J","I","S","FS","R","CG","CG","CG","R","I","GA","SS","S","S","J","J","J","SS","E","R","E","GA","GA","GA","T","I","I","I","I","SS","J","R","FS","E","E","T","T","T","T","S","R","SS","R","T","S","CG","CG","CG","CG","R","R","R","SS","R","S","S","CG","S","J","E","E","SS","SS","SS","SS","E","R","S","I","I","T","T","T","S","SS","SS","SS","CG","CG","S","SS","T","E","E","GA","GA","R","R","J","E","S","T","S","E","S","GA","GA","GA","E","R","I","R","I","T","I","E","GA","E","FS","R","S","E","T","SS","SS","S","E","S","I","E","E","R","CG","S","S","J"],["J","J","J","R","FS","T","E","S","J","E","E","S","GA","GA","GA","GA","SS","S","FS","S","CG","S","E","J","S","I","WI","S","SP","E","S","S","I","I","T","J","S","I","T","E","E","GA","GA","GA","GA","SS","SS","E","R","E","S","E","S","S","I","I","I","I","E","E","CG","T","T","E","SS","E","T","T","T","T","E","S","R","E","CG","CG","E","GA","E","R","I","SS","S","CG","CG","CG","CG","E","R","I","CG","T","I","J","R","E","FS","R","SS","SS","E","R","S","R","I","I","R","FS","R","SS","SS","SS","SS","R","J","J","S","S","SS","R","SS","S","T","SS","E","J","J","J","J","R","SS","CG","CG","R","SS","R","S","WI","SS","SS","CG","I","T","T","T","R","S","E","S","GA","R","T","R","S","T","J","I","S","S","FS","R","SS","R"],["S","WI","I","T","J","T","J","R","S","J","T","E","I","S","FS","R","J","T","CG","R","I","R","WI","GA","GA","GA","GA","S","I","SS","I","R","S","T","S","R","J","T","T","J","I","S","S","SS","I","WI","R","R","GA","S","J","T","J","WI","S","SS","E","FS","E","T","SS","I","SS","S","E","E","J","CG","S","WI","CG","GA","E","E","J","T","T","I","R","J","E","T","S","I","SS","T","I","SS","SS","E","FS","R","SS","I","R","GA","GA","GA","SS","S","R","SS","S","E","WI","CG","T","S","SS","CG","SS","GA","S","CG","I","E","FS","E","CG","CG","R","R","SS","T","E","E","J","J","CG","SS","R","I","S","WI","E","J","CG","SS","R","S","E","T","E","R","SP","E","S","T","E","CG","SS","I","R","SS","I","R","S","R","CG","E","E","J","SS"],["S","T","R","I","WI","S","E","FS","E","GA","S","T","S","T","I","J","I","WI","T","R","E","I","J","SS","J","I","E","E","T","S","GA","R","SS","T","T","E","J","R","FS","E","CG","R","GA","GA","E","SS","R","CG","J","R","CG","I","S","CG","T","SS","GA","GA","GA","R","I","WI","S","E","I","SS","CG","SS","E","T","J","E","SS","S","T","R","WI","GA","SS","T","CG","E","SS","I","SS","I","CG","J","R","FS","E","SS","CG","R","CG","T","S","GA","R","R","SS","I","E","T","R","SS","J","S","S","T","I","E","R","GA","SS","R","CG","J","E","T","S","CG","S","E","J","R","S","SS","CG","J","T","I","S","J","S","I","E","E","CG","R","SS","I","SS","S","FS","E","J","S","J","S","SP","S","R","R","S","SS","I","S","J","E","E"]]
		},
		{
			name: 'reel_set_3',
			data: [["GA","T","E","J","GA","SS","CG","CG","S","SS","I","S","CG","CG","CG","CG","SS","S","S","SS","E","E","SS","T","S","E","I","R","E","SS","SS","SS","E","R","FS","R","J","J","J","SS","R","GA","GA","GA","GA","R","I","I","E","S","J","T","R","R","I","I","I","I","T","T","T","E","FS","R","E","E","R","J","T","I","E","R","R","E","GA","GA","GA","S","R","SS","R","CG","CG","CG","S","E","SS","SS","SS","SS","E","S","R","S","I","R","R","E","T","S","I","S","T","S","J","J","T","T","E","S","CG","T","T","I","S","GA","S","R","E","E","S","J","S","J","I","R"],["S","S","T","S","R","S","E","J","J","J","J","R","T","S","S","R","R","GA","E","I","R","R","I","S","SS","SS","SS","R","T","S","R","T","R","R","E","E","J","J","S","I","E","FS","R","CG","CG","CG","GA","R","SS","GA","S","SP","S","J","J","J","SS","E","E","I","GA","GA","GA","T","I","I","GA","GA","SS","J","E","E","SS","E","T","T","T","T","S","I","E","R","I","S","CG","CG","CG","CG","I","I","I","S","R","S","SS","S","E","GA","T","S","SS","SS","SS","SS","E","R","CG","I","I","T","T","T","E","SS","E","FS","S","CG","CG","SS","E","GA","R","R","E","R","E"],["J","J","S","R","R","S","S","J","E","T","E","GA","GA","GA","GA","E","S","I","E","S","SS","E","I","R","S","T","S","E","E","I","I","I","T","J","S","S","E","J","T","E","GA","GA","GA","SS","SS","R","E","E","R","J","R","SS","E","E","I","I","R","E","S","T","T","S","FS","S","T","T","T","T","SS","J","R","CG","CG","CG","GA","GA","E","R","I","SS","S","CG","CG","CG","CG","T","R","I","R","CG","I","R","J","S","CG","R","S","SS","E","R","T","R","SS","S","S","SP","R","E","SS","SS","SS","SS","R","GA","CG","I","I","I","S","SS","R","T","SS","E","J","J"],["S","E","CG","T","J","I","J","T","T","R","SP","S","CG","CG","S","R","SS","I","J","R","J","R","R","E","GA","GA","GA","GA","S","I","J","I","R","SS","E","S","J","E","E","T","T","E","S","S","SS","I","J","R","R","GA","E","R","J","S","SS","CG","R","S","FS","E","T","SS","I","SS","SS","I","E","T","CG","S","R","CG","GA","E","J","E","I","T","T","R","E","E","CG","S","SS","I","T","I","J","S","S","FS","R","R","SS","E","GA","GA","GA","R","S","SS","CG","S","E","SS","I","T","R","SS","I","CG","GA","S","SS","E","I","T","T","SS","CG"],["S","T","I","T","S","S","J","R","E","GA","I","T","SS","T","E","SP","S","I","I","T","R","E","E","J","J","S","T","E","E","T","S","SS","R","T","S","T","R","E","FS","R","SS","SS","I","CG","E","E","SS","R","I","R","J","E","R","R","SS","GA","GA","GA","GA","SS","CG","CG","S","I","E","CG","SS","CG","SS","E","GA","GA","I","J","J","SS","T","I","S","T","R","S","E","J","I","S","I","SS","SS","R","GA","E","CG","CG","R","CG","S","S","GA","R","R","SS","I","E","T","CG","SS","S","J","J","S","CG","E","J","GA","R","I","R","S","S","T"]]
		},
		{
			name: 'reel_set_4',
			data: [["GA","T","E","FS","E","GA","T","CG","R","I","I","I","CG","CG","CG","CG","T","GA","E","E","E","R","T","T","T","E","I","R","GA","T","CG","T","E","GA","E","CG","T","R","R","R","R","GA","GA","GA","GA","E","I","I","I","E","CG","T","R","E","I","I","I","I","CG","T","T","T","T","R","R","R","R","E","CG","I","E","E","E","E","GA","GA","GA","I","R","R","R","E","CG","CG","CG","T","E","GA","I","CG","E","SP","E","T","I","R","R","GA","R"],["R","S","S","S","R","R","R","J","J","J","J","I","SS","GA","S","I","R","GA","S","I","R","SP","R","I","SS","SS","SS","R","I","S","SS","SS","SS","SS","S","R","J","J","S","SS","I","S","GA","GA","GA","R","R","R","SS","S","S","S","J","J","J","SS","GA","I","SS","S","GA","GA","J","I","I","I","I","SS","S","J","I","SS","S","GA","GA","GA","GA","SS","R","R","R","R","S","FS","R","I","SS","I","I","I","SS","R","S","S","S","S","GA","SS","J"],["J","J","J","J","SS","T","T","T","E","E","E","E","T","S","FS","E","CG","S","S","S","S","SS","E","J","CG","S","S","S","E","SS","SS","SS","SS","T","SS","S","SS","E","E","E","E","J","T","S","SS","SS","E","T","CG","S","S","S","SS","J","E","T","SS","S","E","E","T","CG","S","E","T","T","T","T","S","SS","S","E","CG","CG","CG","J","T","S","E","T","SS","S","CG","CG","CG","CG","J","J","J","SS","SS","SS","E","CG","E","T","S","SS"],["S","WI","I","J","J","J","J","T","S","E","CG","SS","R","S","WI","SS","J","CG","R","R","R","R","E","GA","GA","GA","GA","S","I","I","I","R","SS","E","SP","R","T","T","T","T","E","S","S","S","S","WI","R","R","GA","E","J","J","J","SS","CG","I","R","FS","E","T","SS","SS","SS","SS","S","E","T","CG","I","S","CG","E","E","E","E","T","T","T","R","J","E","CG","S","I","I","I","I","J","SS","SS","WI","E","GA","R","R","S","E","R"],["S","T","I","R","S","WI","E","E","E","GA","S","T","T","T","S","I","I","WI","E","R","E","J","J","J","J","I","E","E","R","S","CG","R","T","T","T","T","J","S","WI","SS","R","I","S","E","R","R","R","R","R","J","I","S","SS","S","GA","GA","GA","GA","S","S","S","S","T","SS","SS","SS","SS","E","FS","R","J","J","CG","SS","R","WI","SS","T","R","E","S","I","I","I","I","SS","SS","J","E","E","CG","CG","CG","CG","E","J","GA","SP"]]
		},
		{
			name: 'progressive_reels',
			data: [["GA","T","SP","E","E","E","T","CG","R","SP","I","I","I","CG","CG","SP","CG","T","GA","E","E","E","SP","T","T","T","E","SP","I","R","GA","T","CG","T","E","GA","E","SP","T","R","R","R","SP","GA","GA","GA","GA","E","I","I","I","E","E","T","SP","E","I","I","I","I","CG","T","T","T","T","R","R","R","SP","E","CG","I","E","SP","E","E","CG","CG","CG","I","R","R","R","SP","CG","CG","CG","T","E","GA","I","E","CG","SP","E","T","I","R","R","GA","R"],["R","S","J","I","R","SP","R","J","J","J","J","I","SS","SP","S","I","R","GA","R","I","R","SP","R","I","SS","SS","SS","SS","R","SP","S","SS","SS","SS","SS","S","SP","J","J","J","SS","I","SP","GA","GA","GA","R","SP","R","SS","S","S","S","J","J","J","SS","SP","I","SS","J","GA","GA","SP","I","I","I","I","SS","S","FS","S","SP","SS","I","GA","GA","GA","GA","SS","R","R","SP","R","S","FS","R","I","SS","I","I","I","SS","SP","S","S","S","S"],["J","J","J","J","SP","T","T","T","E","CG","T","SS","T","S","SP","CG","E","SS","SS","SS","SS","J","J","SP","CG","S","S","S","E","SS","SS","SS","SS","T","SP","S","SS","E","E","E","E","SP","J","T","S","SS","SS","E","SP","CG","S","S","S","SS","J","E","T","SP","S","E","E","T","CG","S","E","T","T","T","T","S","SP","S","E","CG","CG","CG","SP","T","S","E","T","SP","S","CG","CG","CG","CG","SP","J","J","J","SS","SS","SS","E","SP","E","T","S","SS","SS"],["S","WI","I","J","J","J","J","T","S","SP","CG","SS","R","S","WI","SS","J","CG","R","R","SP","R","E","GA","GA","GA","GA","SP","S","I","I","I","R","SS","E","SP","S","T","T","T","T","E","S","SP","S","S","WI","J","CG","SP","J","J","J","J","SS","CG","I","SP","R","E","T","SS","SS","SS","SS","SP","S","E","T","CG","I","S","SP","E","E","GA","GA","T","T","T","R","SP","E","CG","S","I","I","I","I","SP","SS","SS","WI","E","SP","R","R","S","E","R"],["S","T","I","R","S","SP","E","E","E","SP","S","T","T","T","S","I","I","SP","E","SS","E","J","J","J","J","I","S","S","R","SP","CG","R","T","T","T","T","J","S","SP","SS","GA","I","S","E","FS","R","SP","R","R","R","J","J","I","S","SP","S","GA","GA","GA","GA","S","SP","S","S","T","SS","SS","SS","SS","SP","E","FS","R","J","J","CG","SS","R","SP","SS","T","R","E","S","I","I","I","I","SS","SS","SP","E","E","CG","CG","CG","CG","SP","J","GA"]]
		},
		{
			name: 'losing_reels',
			data: [["GA","T","E","E","GA","T","CG","R","SP","I","I","I","CG","CG","CG","CG","T","GA","E","E","E","R","T","T","T","E","I","R","GA","T","CG","T","E","GA","E","CG","T","R","R","R","R","GA","GA","GA","GA","E","I","I","I","E","CG","T","R","E","I","I","I","I","CG","T","T","T","T","R","R","R","R","E","CG","I","E","E","E","E","GA","GA","GA","I","R","R","R","E","CG","CG","CG","T","E","GA","I","E","CG","R","E","T","I","R","R","GA"],["R","S","S","S","R","R","R","J","J","J","J","I","SS","GA","S","I","R","GA","S","I","R","FS","R","I","SS","SS","SS","R","I","S","SS","SS","SS","SS","S","R","J","J","S","SS","I","S","GA","GA","GA","R","R","R","SS","S","S","S","J","J","J","SS","GA","I","SS","S","GA","GA","J","I","I","I","I","SS","S","J","I","SS","S","GA","GA","GA","GA","SS","R","R","R","R","S","FS","R","I","SS","I","I","I","SS","R","S","S","S","S","GA","SS","J"],["J","J","J","J","SS","T","T","T","E","E","E","E","T","S","SS","CG","E","S","S","S","S","SS","E","J","CG","S","S","S","E","SS","SS","SS","SS","T","SP","S","SS","E","E","E","E","J","T","S","SS","SS","E","T","CG","S","S","S","SS","J","E","T","SS","S","E","E","T","CG","S","E","T","T","T","T","S","SS","S","E","CG","CG","CG","J","T","S","E","T","SS","S","CG","CG","CG","CG","J","J","J","SS","SS","SS","E","CG","E","T","S","SS"],["S","WI","I","J","J","J","J","T","S","E","CG","SS","R","S","WI","SS","J","CG","R","R","R","R","E","GA","GA","GA","GA","S","I","I","I","R","SS","E","R","R","T","T","T","T","E","S","S","S","S","WI","R","R","GA","E","J","J","J","SS","CG","I","SP","R","E","T","SS","SS","SS","SS","S","E","T","CG","I","S","CG","E","E","E","E","T","T","T","R","J","E","CG","S","I","I","I","I","J","SS","SS","WI","E","GA","R","R","S","E"],["S","T","I","R","S","WI","E","E","E","GA","S","T","T","T","S","I","I","WI","E","R","E","J","J","J","J","I","E","E","R","S","CG","R","T","T","T","T","J","S","WI","SS","R","I","S","E","R","FS","R","R","R","R","J","I","S","SS","S","GA","GA","GA","GA","S","S","S","S","T","SS","SS","SS","SS","E","FS","R","J","J","CG","SS","R","WI","SS","T","SP","E","S","I","I","I","I","SS","SS","J","E","E","CG","CG","CG","CG","E","J","GA","R"]]
		},
		{
			name: 'free_spins_set_1',
			data: [["WI","WI","WI","WI","T","E","E","T","CG","R","E","I","I","I","J","J","E","T","E","R","GP","T","T","T","E","WI","WI","WI","WI","S","S","I","R","T","T","T","E","R","E","CG","T","WI","WI","WI","WI","R","T","CG","R","E","I","I","I","E","R","GA","R","E","I","I","I","I","WI","WI","WI","WI","CG","R","T","T","T","R","CG","R","CG","E","R","I","E","I","CG","E","WI","WI","WI","WI","GA","GA","GA","E","R","T","R","E","GP","CG","CG","CG","R","E","GA","I","E","E","R","T","SS","SS","I","E","R","GA","R"],["R","S","I","I","S","R","J","J","J","SS","R","R","S","S","CG","I","R","S","T","T","T","T","I","I","R","R","R","SS","SS","SS","I","S","SS","SS","SS","SS","S","R","SS","S","S","E","E","E","E","R","R","R","SS","S","S","J","J","J","SS","S","I","SS","S","E","E","E","E","GA","GA","GA","CG","I","I","I","I","SS","S","R","R","SS","S","SS","R","R","I","S","I","R","S","I","I","I","SS","R","S","T","T","T","T","SS","SS","S","S"],["J","J","J","J","WI","WI","WI","WI","E","T","T","T","S","S","E","E","I","I","T","GA","GA","GA","S","E","E","S","SS","SS","S","E","GP","E","J","CG","S","J","S","E","WI","WI","WI","WI","SS","SS","SS","SS","E","S","SS","T","E","CG","E","S","T","S","SS","SS","E","GP","S","CG","S","T","WI","WI","WI","WI","S","SS","J","E","T","R","R","SS","S","E","E","T","CG","S","E","T","T","T","T","WI","WI","WI","WI","S","SS","S","E","CG","CG","CG","J","T","S","E","T","SS","S","WI","WI","WI","WI","CG","CG","CG","CG","J","J","J","SS","SS","SS","E","CG","E","T","S","SS","SS"],["S","J","J","R","S","E","CG","SS","GP","R","S","J","SS","J","CG","R","T","R","E","R","GA","GA","GA","GA","S","I","I","I","R","SS","E","R","T","T","T","T","E","CG","S","SS","S","J","R","R","GA","E","J","J","J","S","I","S","R","E","J","SS","SS","SS","SS","S","E","T","CG","I","S","CG","T","E","I","E","T","T","T","R","J","E","CG","GP","S","I","I","I","E","E","SS","SS","I","E","GA","R","R","S","E","R"],["S","E","I","R","S","WI","WI","WI","WI","GA","S","T","T","T","S","I","I","T","E","R","E","GP","J","J","J","J","I","E","E","R","S","CG","R","T","T","T","T","J","S","E","SS","R","I","S","E","R","GA","R","SS","SS","R","J","I","S","R","S","GP","GA","GA","GA","GA","R","S","S","CG","T","SS","SS","SS","SS","E","R","J","J","WI","WI","WI","WI","S","S","R","E","SS","T","R","E","S","I","I","I","I","SS","SS","J","E","E","CG","CG","CG","CG","E","J"]]
		},
		{
			name: 'free_spins_set_3',
			data: [["T","E","E","T","CG","R","E","I","I","I","J","J","WI","WI","WI","WI","E","T","E","R","T","T","T","E","S","S","I","R","T","T","T","E","R","E","CG","T","WI","WI","WI","WI","R","T","CG","R","E","I","I","I","E","R","GA","R","GP","E","I","WI","WI","WI","WI","I","I","I","CG","R","T","T","T","R","CG","R","CG","WI","WI","WI","WI","E","R","I","E","I","CG","E","GA","GA","GA","E","R","T","R","E","CG","CG","CG","R","WI","WI","WI","WI","E","GA","I","E","E","R","GP","T","SS","SS","I","E","R","GA","R"],["J","J","J","J","WI","WI","WI","WI","E","T","T","T","S","S","E","E","I","I","T","GA","GA","GA","S","E","E","GP","S","SS","SS","S","E","E","J","CG","S","J","S","E","SS","SS","SS","SS","E","S","WI","WI","WI","WI","SS","T","E","CG","E","S","T","S","SS","SS","E","WI","WI","WI","WI","S","CG","S","T","S","SS","J","E","T","R","R","SS","S","E","E","T","CG","GP","S","E","T","T","WI","WI","WI","WI","T","T","S","SS","S","E","CG","CG","CG","J","T","S","E","T","SS","S","CG","CG","CG","CG","WI","WI","WI","WI","J","J","J","SS","SS","SS","E","CG","E","T","S","SS","SS"],["R","S","I","I","S","R","J","J","J","SS","R","GP","R","S","S","CG","I","R","S","T","T","T","T","I","I","R","R","R","SS","SS","SS","I","S","SS","SS","SS","SS","S","R","SS","S","S","E","E","E","E","R","R","R","SS","S","S","J","J","J","SS","S","I","SS","S","E","E","E","E","GA","GA","GA","GP","CG","I","I","I","I","SS","S","R","R","SS","S","SS","R","R","I","S","I","R","S","I","I","I","SS","R","S","T","T","T","T","SS","SS","S","S"],["S","GA","I","J","J","J","J","T","S","E","T","I","I","S","I","T","R","R","R","R","E","GA","GA","GA","GA","S","GP","I","I","I","R","SS","E","S","R","T","T","T","T","E","S","S","S","I","T","R","R","GA","E","J","J","J","SS","CG","S","SS","CG","E","T","SS","SS","SS","SS","I","E","T","CG","S","R","CG","GA","WI","WI","WI","WI","E","E","E","T","T","T","R","J","E","CG","S","I","I","I","I","J","SS","SS","SS","R","R","R","E","GA","GA","GA","SS","S","S","S","S","E","SS","I","T","R","SS","SS","SS","T","GP","S","CG","E","I","E","T","CG","CG","SS","R","R","R","SS","S","J","J","E","SS","R","E","E","E","E","CG","CG","CG","CG","S","J","S","S","I"],["S","E","I","R","S","WI","WI","WI","WI","GA","S","T","T","T","S","I","I","T","E","R","E","J","J","J","J","I","E","E","R","S","CG","R","T","T","T","T","J","S","E","SS","R","I","S","E","R","GA","R","SS","SS","R","J","I","S","R","S","GA","GA","GA","GA","R","S","S","CG","T","SS","SS","SS","SS","E","R","J","J","S","S","R","E","SS","T","R","E","S","I","I","I","I","SS","SS","J","E","E","CG","CG","CG","CG","E","J"]]
		},
		{
			name: 'free_spins_set_4',
			data: [["R","S","I","I","S","GP","R","J","J","J","SS","R","R","S","S","CG","I","R","S","T","T","T","T","I","I","R","R","R","SS","SS","SS","I","S","SS","SS","SS","SS","S","R","SS","S","S","E","E","E","E","R","R","R","SS","S","S","J","J","J","SS","S","I","SS","S","E","E","E","E","GA","GA","GA","CG","I","I","I","I","SS","S","R","R","SS","S","SS","R","R","I","S","I","GP","R","S","I","I","I","SS","R","S","T","T","T","T","SS","SS","S","S"],["T","E","E","T","CG","R","E","I","I","I","J","J","WI","WI","WI","WI","E","T","E","R","T","T","T","E","S","S","I","R","T","T","T","E","R","E","CG","T","WI","WI","WI","WI","R","T","CG","R","E","I","I","I","E","WI","WI","WI","WI","R","GA","R","E","I","I","I","I","CG","R","T","T","T","R","WI","WI","WI","WI","CG","R","CG","E","R","I","E","I","CG","E","GA","GA","GA","WI","WI","WI","WI","E","R","T","R","E","CG","CG","CG","R","E","GA","I","E","E","R","T","SS","SS","I","E","R","GA","R"],["J","J","J","J","WI","WI","WI","WI","E","T","T","T","S","S","E","E","I","I","T","GA","GA","GA","S","E","E","GP","S","SS","SS","S","E","E","WI","WI","WI","WI","J","CG","S","J","S","E","SS","SS","SS","SS","E","S","WI","WI","WI","WI","SS","T","E","CG","E","S","T","S","SS","SS","E","GP","S","CG","S","T","S","SS","J","E","T","R","WI","WI","WI","WI","R","SS","S","E","E","T","CG","S","E","T","T","T","T","S","SS","S","E","CG","CG","CG","WI","WI","WI","WI","J","T","S","E","T","SS","S","CG","CG","CG","CG","J","J","J","SS","SS","SS","E","CG","E","T","S","SS","SS"],["WI","WI","WI","WI","S","J","J","R","S","E","CG","SS","R","S","GP","J","SS","J","CG","R","T","R","E","R","GA","GA","GA","GA","S","I","I","I","R","SS","E","R","T","T","T","T","E","CG","S","SS","S","J","R","R","GA","E","WI","WI","WI","WI","J","J","J","S","I","S","R","E","J","SS","SS","SS","SS","S","E","T","CG","I","S","CG","T","E","I","E","T","T","T","R","J","E","CG","S","I","I","I","GP","E","E","SS","SS","I","E","GA","R","R","S","E","R"],["S","E","I","R","S","GA","S","T","T","T","S","I","I","T","E","R","GP","E","J","J","J","J","I","E","E","R","S","CG","R","T","T","T","T","J","S","E","SS","R","I","S","E","R","GA","R","SS","SS","R","J","I","S","R","S","GA","GA","GA","GA","R","S","S","CG","T","SS","SS","SS","SS","E","R","J","GP","J","S","S","R","E","SS","T","R","E","S","I","I","I","I","SS","SS","J","E","E","CG","CG","CG","CG","E","J"]]
		},
		{
			name: 'free_spins_progressive_set',
			data: [["SS","CG","E","GP","J","CG","S","S","S","S","GP","E","SS","S","CG","CG","CG","GP","CG","SS","S","S","SS","E","GP","E","J","T","S","E","E","GP","T","CG","SS","SS","SS","S","S","S","GP","SS","J","S","T","T","T","GP","CG","CG","CG","CG","S","S","T","GP","E","E","J","T","T","SS","J","J","J","J","GP","S","T","T","T","GP","E","T","T","T","T","S","T","J","GP","CG","E","E","E","E","GP","CG","CG","SS","E","E","E","GP","J","CG","E","E","S","SS","SS","SS","SS","E","GP","S","SS","T","J","GP","E","S","SS","E","E","S","E","T","SS","GP","J","J","J","J","E","J","GP","CG","T","T","J","CG","E","GP","S","SS","E","E","S","S","J","GP","S","E","J","E","GP","E","S","J","CG","GP","T","T","T","T"],["S","S","S","GP","GA","GA","GA","GP","I","I","I","I","J","I","J","GP","S","S","GA","GA","GA","R","R","R","R","GP","J","S","SS","SS","SS","R","J","S","GP","SS","J","R","SS","I","GP","R","R","R","R","SS","I","S","SS","SS","GP","GA","R","R","R","GA","SS","I","GP","S","J","J","J","SS","GA","GP","I","I","S","R","GA","S","GP","I","I","I","I","GP","SS","J","J","R","SS","I","GA","GA","GA","GA","GP","S","R","I","R","GP","I","S","GA","GA","GA","GA","GP","I","I","I","SS","GP","R","S","S","S","S","GA","S","S","GP","SS","SS","SS","SS","I","R","S","I","GP","R","J","J","J","SS","SS","SS","GP","J","J","S","S","SS","J","GP","S","SS","R","GA","S","R","J","S","GP","R","J","SS"],["GA","GA","GP","E","R","R","GP","T","T","T","CG","E","E","E","GP","GA","GA","GA","GA","GP","T","R","R","R","R","GP","CG","E","I","R","GP","T","T","T","E","GP","E","I","I","I","T","GP","GA","T","I","E","E","E","E","GP","GA","GA","GA","CG","CG","E","GP","R","T","E","T","E","GA","I","I","I","I","GA","T","E","T","T","GP","GA","GA","T","T","T","T","GP","CG","CG","R","T","CG","CG","GP","R","I","T","E","R","I","GP","I","R","CG","CG","CG","CG","GA","R","R","GP","R","R","I","CG","CG","E","E","I","GP","I","I","E","R","R","R","GP","E","E","T","CG","T","I","I","GP","I","I","R","GA","CG","GP","I","I","I","R","I","R","T","I","GP","E","GA","GA","GA","GA","R","I","CG","CG","R","I","R","GP","R","CG","I","I","CG","I","T","T","T","CG","CG"],["S","GA","I","GP","J","J","J","J","GP","T","S","E","T","I","I","S","GP","I","T","R","R","R","GP","R","E","GA","GA","GA","GA","S","GP","I","I","I","R","SS","E","GP","S","R","T","T","T","T","E","GP","S","S","S","I","T","R","R","GA","GP","E","J","J","J","SS","CG","S","SS","CG","GP","E","T","SS","SS","SS","SS","I","GP","E","T","CG","S","GP","R","CG","GA","E","E","E","T","T","T","GP","R","J","E","CG","S","I","I","I","I","GP","J","SS","SS","SS","R","R","R","GP","E","GA","GA","GA","SS","GP","S","S","S","S","E","SS","I","T","R","GP","SS","SS","SS","T","S","CG","E","I","E","T","GP","CG","CG","SS","R","R","R","GP","SS","S","J","J","E","SS","R","GP","E","E","E","E","GP","CG","CG","CG","CG","S","J","S","S","I"],["S","T","I","E","E","E","GP","GA","S","T","T","T","GP","R","I","I","GP","S","T","R","E","J","J","J","GP","J","I","E","E","SS","S","CG","R","GP","T","T","T","T","GP","E","R","R","SS","SS","I","GA","E","GP","J","SS","R","R","R","R","CG","I","GP","J","SS","S","GA","GA","GA","GA","GP","CG","CG","S","S","GP","E","SS","SS","SS","SS","GP","E","GA","GA","J","J","GP","S","SS","R","I","J","T","T","CG","E","GP","I","I","I","I","SS","SS","GP","R","GA","E","CG","CG","CG","CG","J","J","SS","R","GP","CG","SS","I","E","T","R","SS","GP","S","S","S","S","I","GP","E","J","GA","R","R","R","S","S","GP","T","T","S","S","E","E","E","E","GP","CG","CG","T","T","I","S","S","S","GP","I","E","E","R","GP","R","SS","SS","SS","S","I"]]
		},
		{
			name: 'free_spins_losing_set',
			data: [["SS","CG","E","GP","J","CG","S","S","S","S","GP","E","SS","S","CG","CG","CG","CG","GP","SS","S","S","SS","E","E","GP","J","T","S","E","E","GP","T","CG","SS","SS","SS","S","S","S","SS","J","S","T","T","T","GP","CG","CG","CG","CG","GP","S","S","T","E","E","GP","J","T","T","SS","J","J","J","J","GP","S","T","T","T","GP","E","T","T","T","T","GP","S","T","J","CG","E","E","E","E","GP","CG","CG","SS","E","E","E","GP","J","CG","E","E","S","SS","SS","SS","SS","E","GP","S","SS","T","J","E","S","SS","E","GP","E","S","E","T","SS","J","J","J","J","E","GP","J","CG","T","T","J","CG","E","S","GP","SS","E","E","S","S","J","S","E","J","GP","E","E","S","J","CG","T","T","T","T"],["S","S","S","GA","GA","GA","I","I","I","I","J","GP","I","J","S","S","GA","GA","GA","GP","R","R","R","R","GP","J","S","SS","SS","SS","R","J","S","GP","SS","J","R","SS","I","R","R","R","R","GP","SS","I","S","SS","SS","GP","GA","R","R","R","GA","SS","I","GP","S","J","J","J","GP","SS","GA","I","I","S","R","GA","S","GP","I","I","I","I","SS","J","J","GP","R","SS","I","GA","GA","GA","GA","GP","S","R","I","R","GP","I","S","GA","GA","GA","GA","GP","I","I","I","SS","R","S","S","S","S","GP","GA","S","S","SS","SS","SS","SS","GP","I","R","S","I","R","GP","J","J","J","SS","SS","SS","J","J","S","GP","S","SS","J","S","SS","R","GA","S","GP","R","J","S","R","J","SS"],["GA","GA","GP","E","R","R","T","T","T","GP","CG","E","E","E","GP","GA","GA","GA","GA","T","R","R","R","R","GP","CG","E","I","R","T","T","T","E","GP","E","I","I","I","T","GA","GP","T","I","E","E","E","E","GP","GA","GA","GA","CG","CG","E","GP","R","T","E","T","E","GA","GP","I","I","I","I","GA","T","E","T","T","GA","GA","T","T","T","T","CG","CG","R","T","CG","CG","GP","R","I","T","E","R","I","GP","I","R","CG","CG","CG","CG","GA","R","R","GP","R","R","I","CG","CG","E","E","I","GP","I","I","E","R","R","R","GP","E","E","T","CG","T","I","I","GP","I","I","R","GA","CG","I","I","I","R","I","R","T","I","GP","E","GA","GA","GA","GA","R","I","CG","CG","R","I","R","GP","R","CG","I","I","CG","I","T","GP","T","T","CG","CG"],["S","GA","I","S","J","J","J","J","T","S","GA","E","T","I","I","S","I","T","R","R","R","S","R","E","GA","GA","GA","GA","S","E","I","I","I","R","SS","E","T","S","R","T","T","T","T","E","CG","S","S","S","I","T","R","R","GA","E","E","J","J","J","SS","CG","S","SS","CG","SS","E","T","SS","SS","SS","SS","I","E","T","CG","S","R","CG","GA","CG","E","E","E","T","T","T","R","R","J","E","CG","S","I","I","I","I","J","J","SS","SS","SS","R","R","R","E","GA","GA","GA","R","SS","S","S","S","S","E","SS","I","T","R","I","SS","SS","SS","T","S","T","CG","E","I","E","T","J","CG","CG","SS","R","R","R","S","SS","S","J","J","E","I","SS","R","E","E","E","E","CG","CG","CG","CG","SS","S","J","S","S","I"],["S","T","I","E","E","E","GP","GA","S","T","T","T","R","I","I","GP","S","T","R","E","J","J","J","GP","J","I","E","E","SS","S","GP","CG","R","T","T","T","T","E","GP","R","R","SS","SS","I","GA","GP","E","J","SS","R","R","R","R","CG","I","GP","J","SS","S","GA","GA","GP","GA","GA","CG","CG","S","S","GP","E","SS","SS","SS","SS","E","GA","GA","J","J","S","GP","SS","R","I","J","T","T","CG","E","GP","I","I","I","I","SS","SS","R","GA","E","CG","CG","CG","CG","GP","J","J","SS","R","CG","SS","I","E","T","R","SS","GP","S","S","S","S","I","GP","E","J","GA","R","R","R","S","S","GP","T","T","S","S","GP","E","E","E","E","CG","CG","T","GP","T","I","S","S","S","I","E","E","R","GP","R","SS","SS","SS","S","I"]]
		}
	];
	
	
	var symbols = new libs.server.SymbolMap( [
		{ shortName: 'E', 	longName: 'Emerald' },
		{ shortName: 'S', 	longName: 'Sapphire' },
		{ shortName: 'R', 	longName: 'Ruby' },
		{ shortName: 'SS', 	longName: 'SwordsAndShield' },
		{ shortName: 'T', 	longName: 'Toucan' },
		{ shortName: 'I', 	longName: 'Iguana' },
		{ shortName: 'J', 	longName: 'Jaguar' },
		{ shortName: 'CG', 	longName: 'CityOfGold' },
		{ shortName: 'GA', 	longName: 'GoddessOfTheAmazon' },
		{ shortName: 'WI', 	longName: 'Wild' },
		{ shortName: 'FW', 	longName: 'FreeSpinsWild' },
		{ shortName: 'MS', 	longName: 'Mystery' },
		{ shortName: 'FS', 	longName: 'FreeSpins' },
		{ shortName: 'GP', 	longName: 'GoldJackpot' },
		{ shortName: 'SP', 	longName: 'SilverJackpot' }
	] );
	
	
	GameData.reels = {
		symbolAdjacencyRules: [
			// the freespins symbol must only be next to the sapphire, emerald or ruby symbol.
			{ symbol:'FS', only:[ 'S', 'E', 'R' ], distance: 1 },
			// the freespins symbol cannot be within 3 symbols of another WK symbol.
			{ symbol:'FS', not:[ 'FS' ], distance: 3 },
			// the GP (gold progressive) symbol cannot be within 3 symbols of another GP or SP symbol.
			{ symbol:'GP', not:[ 'GP', 'SP' ], distance: 3 },
			// the SP (gold progressive) symbol cannot be within 3 symbols of another GP or SP symbol.
			{ symbol:'SP', not:[ 'GP', 'SP' ], distance: 3 }
		],
		nreels: numberOfReels,
		nrows: numberOfSymbolRows,
		valid_number_lines_arr: [ 20 ],
		winLineDefinitionArray: [
			[1,1,1,1,1],
			[2,2,2,2,2],
			[0,0,0,0,0],
			[3,3,3,3,3],
			[1,2,3,2,1],
			[2,1,0,1,2],
			[0,0,1,2,3],
			[3,3,2,1,0],
			[1,0,0,0,1],
			[2,3,3,3,2],
			[0,1,2,3,3],
			[3,2,1,0,0],
			[1,0,1,2,1],
			[2,3,2,1,2],
			[0,1,0,1,0],
			[3,2,3,2,3],
			[1,2,1,0,1],
			[2,1,2,3,2],
			[0,1,1,1,0],
			[3,2,2,2,3]
		],
		symbols: symbols,
		primaryReelSetOutcomeTable: new libs.server.OutcomeTable( 'primary-reel-set', primaryReelsetOutcomeTablePerStake, primaryReelsetOutcomeTablePerStakeChange ),
		freeSpinsReelSetOutcomeTable: new libs.server.OutcomeTable( 'freespins-reel-set', freeSpinReelsetOutcomeTable, freeSpinReelsetChangeOutcomeTable ),
		matchable_symbols_list: [ 'S', 'E', 'R', 'SS', 'T', 'I', 'J', 'CG', 'GA' ],
		linewinAwardTable: new libs.server.AwardsTable( 'PrimaryAwards', symbols, slots_linewin_table_1 ),
		freespinsReelSets: [
			'free_spins_set_1',
			'free_spins_set_3',
			'free_spins_set_4',
			'free_spins_progressive_set',
			'free_spins_losing_set'
		]
	};
	
	// Jackpot data
	GameData.jackpots = {
		list: [
			{ initialValue:10000000, reseedValue:10000000, maximumValue: 100000000, spreadsheetAverage: 19314038, stakeContribution:0.02, anotherPlayerWonMessage: "Another player has just won the Gold Jackpot." }, // Gold
			{ initialValue:300000, reseedValue:300000, maximumValue: 100000000, spreadsheetAverage: 686595, stakeContribution:0.01, anotherPlayerWonMessage: "Another player has just won the Silver Jackpot." } // Silver
		]
	};
	
	// William Hill required data
	GameData.williamHill = {
		currencyCodeRate: { // Currency codes & 'rates' (scaling for valid stakes, limits etc, and reelset pick)
			'GBP':1,
			'EUR':1,
			'USD':2,
			'AUD':2,
			'CAD':2,
			'SGD':2,
			'CHF':2,
			'DKK':10,
			'SEK':10,
			'HKD':10,
			'JPY':100,
			'XXX':1
		}
	};

	// Virgo required data
	GameData.virgo = {
		gameUID: "GAME-GODDESSOFTHEAMAZON"
	};
	
	
	// Turn reelSetData member into ReelSet class instances. This is an expensive operation and is not required by all components that use GameData so it is not done automatically.
	GameData.createReelSets = function() {
		if ( !GameData.reelSets ) {
			GameData.reelSets = GameData.reelSetData.map( function( reelSetData ) {
				return libs.server.ReelBandSet.createReelBandSet( reelSetData.data, GameData.reels.symbols, reelSetData.name );
			} );
		}
		return GameData.reelSets;
	};
	

	// Data verification - ensures all values are above are within the limits they should be. This should be used to
	// catch programmer / game design input error.
	GameData.validateData = function() {
	
		// Make sure reel set outcome tables all add up to the correct total
		GameData.reels.freeSpinsReelSetOutcomeTable.assertTotalAllStakes( 100000, GameData.financial.availableStakesPerLine );
		GameData.reels.primaryReelSetOutcomeTable.assertTotalAllStakes( 100000, GameData.financial.availableStakesPerLine );
		
		var reelSets = GameData.createReelSets();
	};


	GameData.numberOfReels = numberOfReels;
	GameData.numberOfSymbolRows = numberOfSymbolRows;

	game.logic.GameData = GameData;

}());