/**
 * The core components needed for the client
 *
 * @module libs_ClientCore
 */
goog.provide( 'libs.globalEventBusNames' );

(function() {
	"use strict";

	/**
	 * Enum of events that shouldn't be logged
	 * @readonly
	 */
	libs.eventsDontLog = [
		'libs.event.notify.tick',
		'libs.event.notify.symbolPassedWindow'
	];

	libs.eventNames = {};
	libs.activityNames = {};

	// events
	/**
	 * Enum defining the events fired when loading
	 * @readonly
	 */
	libs.eventNames.loading = [
		'libs.event.notify.manifestsToLoad', // arg0: array (manifestsToLoad)
		'libs.event.notify.loadingProgress', // arg0: number (percentage)
		'libs.event.notify.imagesLoaded'
	];

	/**
	 * Enum defining the events used for system functions
	 * @readonly
	 */
	libs.eventNames.system = [
		'libs.event.notify.isDebugMode', // arg0: bool (isDebugMode)
		'libs.event.notify.platformType', // arg0: string (platform type)
		'libs.event.notify.tick', // arg0: number (timeDelta)
		'libs.event.notify.defaultUpdateFunctions', // arg0: object (updateFunctions)
		'libs.event.notify.playerPersistentData', // arg0: object (persistent data held by the server)
		'libs.event.request.useOverrideUpdateFunctions', // arg0: object (updateFunctions)
		'libs.event.request.clearOverrideUpdateFunctions'
	];

	/**
	 * Enum defining the events used for rendering
	 * @readonly
	 */
	libs.eventNames.rendering = [
		'libs.event.request.canvasFullDraw', // arg0: bool (fullDrawBase), arg1: bool (fullDrawTop)
		'libs.event.request.performFullRender', // arg0: number (framesToPerformFullRender)
		'libs.event.request.enableCanvasOverlayMode',
		'libs.event.request.disableCanvasOverlayMode',
		'libs.event.request.changeTopCanvasZIndex', // arg0: number (zIndex)
		'libs.event.request.enableZIndexHack',
		'libs.event.request.disableZIndexHack',
		'libs.event.notify.interactiveSurface' // arg0: object (surface) 
	];

	/**
	 * Enum defining the events used for input
	 * @readonly
	 */
	libs.eventNames.input = [
		'libs.event.notify.keyPressed', // arg0: number (keyCode), arg1: string (keyChar)
		'libs.event.notify.canvasPointerDown', // arg0: object (point - x, y)
		'libs.event.notify.canvasPointerUp', // arg0: object (point - x, y)
		'libs.event.notify.canvasPointerMove', // arg0: object (point - x, y)
		'libs.event.notify.canvasPointerClick', // arg0: object (point - x, y)
		'libs.event.notify.inputGesture' // arg0: string (gesture)
	];

	libs.eventNames.screen = [
		'libs.event.notify.orientationChanged', // arg0: string (orientation - portrait or landscape)
		'libs.event.notify.displayWidthChanged', // arg0: number (width)
		'libs.event.notify.displayHeightChanged', // arg0: number (height)
		'libs.event.notify.snugVerticalFit', // arg0: bool (isSnugVerticalFit)
		'libs.event.notify.fontScaleChanged' // arg0: number (fontScale)
	];

	/**
	 * Enum defining the events used for haptic functions
	 * @readonly
	 */
	libs.eventNames.haptics = [
		'libs.event.request.enableHaptics',
		'libs.event.request.disableHaptics',
		'libs.event.request.playHaptic', // arg0: number/array (pattern)
		'libs.event.request.stopHaptic',
		'libs.event.notify.hapticsState' // arg0: bool (enabled)
	];

	/**
	 * Enum defining the events used for sound
	 * @readonly
	 */
	libs.eventNames.sound = [
		'libs.event.request.enableSounds',
		'libs.event.request.disableSounds',
		'libs.event.request.playSound', // arg0: array (quality - LQ, MQ, HQ), arg1: string (soundName), arg2: object (settings)
		'libs.event.request.stopSound', // arg0: array (quality - LQ, MQ, HQ), arg1: string (soundName)
		'libs.event.notify.soundsState' // arg0: bool (enabled)
	];

	/**
	 * Enum defining the events used for language settings
	 * @readonly
	 * @enum {string}
	 */
	libs.eventNames.language = [
		'libs.event.request.changeLanguage', // arg0: string (langCode - in format of - http://msdn.microsoft.com/en-us/library/ms533052(v=vs.85).aspx)
		'libs.event.notify.languageChanged' // arg0: string (langCode - in format of - http://msdn.microsoft.com/en-us/library/ms533052(v=vs.85).aspx)
	];

	/**
	 * Enum defining the events used for display options
	 * @readonly
	 */
	libs.eventNames.display = [
		'libs.event.request.showError', // arg0: string (errorText) - arg1: bool (fatal)
		'libs.event.request.showCustomError', // arg0: string (errorText) - arg1: object (negativeButton) - arg2: object (positiveButton)
		'libs.event.request.showBackdrop', // arg0: jquery element (el) - arg1: object (options)
		'libs.event.request.hideBackdrop', // arg0: jquery element (el) - arg1: string (class)
		'libs.event.request.stakeSelector',
		'libs.event.request.autoplaySelector',
		'libs.event.request.openPaytable',
		'libs.event.request.closePaytable',
		'libs.event.request.openHistory',
		'libs.event.request.closeHistory',
		'libs.event.request.showMessageBar', //arg0: string (message) - arg1: number (timeTilAutoHide)
		'libs.event.request.hideMessageBar',
		'libs.event.request.forceShowLogo', //arg0: bool (force show)
		'libs.event.request.changeLogoAnimation', //arg0: string (image name)
		'libs.event.notify.messageBarState' //arg0: bool (isShown)
	];

	/**
	 * Enum defining the events used for financial events
	 * @readonly
	 */
	libs.eventNames.financial = [
		'libs.event.request.changeStakePerLine', // arg0: number (stakePerLine)
		'libs.event.request.showMoneyWin', // arg0: number (win)
		'libs.event.request.changeCurrency', // arg0: string (currencyCode), arg1: object (formatForCurrency)
		'libs.event.notify.currencyChanged', // arg0: string (currencyCode), arg1: object (formatForCurrency)
		'libs.event.notify.stakePerLine', // arg0: number (stakePerLine)
		'libs.event.notify.stakeTotal', // arg0: number (stakeTotal)
		'libs.event.notify.balance', // arg0: number (balance)
		'libs.event.notify.totalWin', // arg0: number (totalWin)
		'libs.event.notify.winTotalToDisplay', // arg0: number (totalWinToDisplay)
		'libs.event.notify.jackpotValues', // arg0: array (jackpotValues)
		'libs.event.notify.jackpotWins', // arg0: array (jackpotWins)
		'libs.event.notify.availableStakesPerLine', // arg0: array (availableStakes)
		'libs.event.notify.jackpotValueDecreased' // arg0: number (jackpotIndex)
	];

	libs.eventNames.meters = [
		'libs.event.notify.meterBaseModel', // arg0: object (meterBaseModel)
		'libs.event.request.pauseMeterUpdating', // arg0: array (metersArray)
		'libs.event.request.resumeMeterUpdating' // arg0: array (metersArray)
	];

	libs.eventNames.gameplay = [
		'libs.event.request.startBetPostData', // arg0: object (data) - Data sent by this message will be posted to the server with the next startBet request
		'libs.event.request.startBet', // arg0: object (options)
		'libs.event.notify.quickPlayState', // arg0: bool (isQuickPlay)
		'libs.event.notify.betAllowed'
	];

	/**
	 * Enum defining the events used for reel events
	 * @readonly
	 */
	libs.eventNames.reels = [
		'libs.event.request.enableReelDragger',
		'libs.event.request.disableReelDragger',
		'libs.event.request.enableTurboMode',
		'libs.event.request.disableTurboMode',
		'libs.event.request.startBetRecovery',
		'libs.event.request.startSymbolAnimations', // arg0: array (symbolsToAnimate)
		'libs.event.request.skipWinAnimations',
		'libs.event.request.forceSymbols', // arg0: array (symbolsToChange), arg1: number (stopPosition), arg2: bool (dontUpdate)
		'libs.event.request.hideSymbolOnReel', // arg0: number (reelIndex), arg1: number (rowIndex)
		'libs.event.request.showSymbolOnReel', // arg0: number (reelIndex), arg1: number (rowIndex)
		'libs.event.request.disableWinAnimationsOnReel', // arg0: number (reelIndex), arg1: number (rowIndex)
		'libs.event.request.enableWinAnimationsOnReel', // arg0: number (reelIndex), arg1: number (rowIndex)
		'libs.event.request.enableWinlineBoxOverEntireReel', // arg0: number (reelIndex)
		'libs.event.request.disableWinlineBoxOverEntireReel', // arg0: number (reelIndex)
		'libs.event.request.showAllSymbolsOnReel', // arg0: number (reelIndex)
		'libs.event.request.startForceReelFadedBackground', // arg0: number (reelIndex), arg1: bool (forceDimmed)
		'libs.event.request.stopForceReelFadedBackground', // arg0: number (reelIndex)
		'libs.event.request.startForcedRenderReelBackground', // arg0: number (reelIndex)
		'libs.event.request.stopForcedRenderReelBackground', // arg0: number (reelIndex)
		'libs.event.request.showWinlines', // arg0: array (lineInfo), arg1: number (positionInRun)
		'libs.event.request.forceReelSet', // arg0: reel set index, arg1: (optional) new reel positions
		'libs.event.request.changeReelSpeed', // arg0: reel index, arg1: speed (symbols per second)
		'libs.event.request.updateSpinParameters', // arg0: object (spinParameters) - Merged with
		'libs.event.request.changeWinMultiplier', // arg0: float (global win multiplier)
		'libs.event.notify.reelSymbolAnimationFinished', // arg0: number (reelIndex), arg1: number (rowIndex)
		'libs.event.notify.betRecoveryInfo', // arg0: object (betRecoveryInfo)
		'libs.event.notify.betRecoveryStarted',
		'libs.event.notify.numberOfWinlines', // arg0: number (numberOfWinlines)
		'libs.event.notify.turboModeState', // arg0: bool (enabled)
		'libs.event.notify.spinParameters', // arg0: object (spinParameters)
		'libs.event.notify.betResult', // arg0: object (betResult)
		'libs.event.notify.reelsStopping',
		'libs.event.notify.reelAboutToLand', // arg0: number (reelIndex), arg1: number (distance in pixels that reel will have to spin until it reaches it's landing position), arg2: number (current speed [Note: This could be dynamically changed during it's journey])
		'libs.event.notify.reelBounced', // arg0: number (reelIndex), arg1: array (finalSymbols)
		'libs.event.notify.reelStopped', // arg0: number (reelIndex), arg1: array (finalSymbols)
		'libs.event.notify.allReelsStopped',
		'libs.event.notify.showingAllSymbolsOfWin', // arg0: number (symbolId), arg1: number (win)
		'libs.event.notify.showingAllSymbolsOfWinComplete',
		'libs.event.notify.showingScatterWin', // arg0: number (symbolId)
		'libs.event.notify.winlineShown', // arg0: number (positionInRun), arg1: number (win) 
		'libs.event.notify.winlineHidden',
		'libs.event.notify.betBonusInfo', // arg0: object (bonus)
		'libs.event.notify.symbolPassedWindow', // arg0: number (reelIndex), arg1: number (symbolId), arg2: array (reelStrip), arg3: number (reelPos), arg4: number (targetStopPos)
		'libs.event.notify.reelsAtSpeed', // arg0: array (reelsToDelayArray)
		'libs.event.notify.reelMoved', // arg0: number (reelIndex), arg1: number (distance), arg2: number (position)
		'libs.event.request.disableReelStates',
		'libs.event.request.enableReelStates'
	];

	/**
	 * Enum defining the events used for server events
	 * @readonly
	 */
	libs.eventNames.reelsServer = [
		'libs.event.request.serverStartBet', // arg0: number (stakePerLine), arg1: number (numberOfWinlines)
		'libs.event.request.serverLoadHistory',
		'libs.event.request.serverFinishBet', // arg0: bool (inBetSequence)
		'libs.event.notify.serverBetResult', // arg0: object (serverBetResult)
		'libs.event.notify.serverBetFinished',
		'libs.event.notify.serverBetAborted',
		'libs.event.notify.serverPolled',
		'libs.event.notify.serverBetHistory', // arg0: array (historyData)
		'libs.event.notify.serverBalance',
		'libs.event.notify.serverStartingStakePerLine',
		'libs.event.notify.serverAvailableStakesPerLine', // arg0: array (availableStakes)
		'libs.event.notify.serverJackpots', // arg0: array (jackpots)
		'libs.event.notify.serverReelsets', // arg0: array (reelsets)
		'libs.event.notify.externalMenuLinks'
	];

	/**
	 * Enum defining the events used for button events
	 * @readonly
	 */
	libs.eventNames.buttons = [
		'libs.event.notify.buttonBaseModel', // arg0: object (buttonBaseModel)
		'libs.event.request.disableAllButtons',
		'libs.event.request.updateAllButtons'
	];

	/**
	 * Enum defining the events used for menu events
	 * @readonly
	 */
	libs.eventNames.menu = [
		'libs.event.request.openMenu',
		'libs.event.request.closeMenu',
		'libs.event.request.enableMenu',
		'libs.event.request.disableMenu'
	];

	/**
	 * Enum defining the events used for game modes
	 * @readonly
	 */
	libs.eventNames.gameMode = [
		'libs.event.request.startAutoplay', // arg0: number (numberOfAutoplays)
		'libs.event.request.stopAutoplay',
		'libs.event.notify.autoplaysRemaining' // arg0: number (numberOfAutoplays)
	];

	// activities
	/**
	 * Enum defining the activities that are exposed in the system state
	 * @readonly
	 */
	libs.activityNames.systemState = [
		'libs.activity.systemState.systemInit',
		'libs.activity.systemState.gameLoading',
		'libs.activity.systemState.serverInit',
		'libs.activity.systemState.gameInit',
		'libs.activity.systemState.gamePlaying'
	];

	/**
	 * Enum defining the activities that are exposed in the bet state
	 * @readonly
	 */
	libs.activityNames.betState = [
		'libs.activity.betState.idle',
		'libs.activity.betState.preBet',
		'libs.activity.betState.playingBet',
		'libs.activity.betState.postBet'
	];

	/**
	 * Enum defining the activities that are exposed in the reels state
	 * @readonly
	 */
	libs.activityNames.reelsState = [
		'libs.activity.reelsState.reelsStarting',
		'libs.activity.reelsState.preReelsLanding',
		'libs.activity.reelsState.reelsLanding',
		'libs.activity.reelsState.postReelsLanding', // game specific
		'libs.activity.reelsState.preDisplayReelsWin', // used by 'bell alarm' effect
		'libs.activity.reelsState.displayReelsWin',
		'libs.activity.reelsState.postDisplayReelsWin',
		'libs.activity.reelsState.reelsBonusFeature', // game specific
		'libs.activity.reelsState.endOfReelsGameModals'
	];

	/**
	 * Enum defining the activities that are exposed in the display state
	 * @readonly
	 */
	libs.activityNames.display = [
		'libs.activity.display.bigWinner',
		'libs.activity.display.compatibility',
		'libs.activity.display.error', // arg1: bool (isFatal)
		'libs.activity.display.popupMessage',
		'libs.activity.display.history',
		'libs.activity.display.introVideo',
		'libs.activity.display.menu',
		'libs.activity.display.paytable',
		'libs.activity.display.soundSelection',
		'libs.activity.display.stakeSelector',
		'libs.activity.display.autoplaySelector',
		'libs.activity.display.xOfAKindSplash', // arg1: number (numberOfWins)
		'libs.activity.display.modalVisible',
		'libs.activity.display.freespinsUI' // arg0: bool (inBetRecovery)
	];

	/**
	 * Enum defining reel activities
	 * @readonly
	 */
	libs.activityNames.reels = [
		'libs.activity.reels.heavyDrop', // arg1: number (reelIndex)
		'libs.activity.reels.anticipation',
		'libs.activity.reels.reelStoppingTime' // arg1: number (reelIndex) - This is invoked after libs.activity.reelsState.reelsLanding has started, this can be used to fine-tune in what order each reel lands (e.g. anticipation spins)
	];

	/**
	 * Enum defining system activities
	 * @readonly
	 */
	libs.activityNames.system = [
		'libs.activity.system.tickManagerActive'
	];

	/**
	 * Enum defining game mode activities
	 * @readonly
	 */
	libs.activityNames.gameMode = [
		'libs.activity.gameMode.autoplay',
		'libs.activity.gameMode.betSequence',
		'libs.activity.gameMode.freespins',
		'libs.activity.gameMode.fortuneSpins'
	];

	/**
	 * Enum defining local server activities
	 * @readonly
	 */
	libs.activityNames.localServer = [
		'libs.activity.localServer.preBetResultCalculation'
	];

}());