goog.provide( 'libs.client.core.InputGestureController' );

goog.require( 'libs.globalEventBus' );


(function() {
	"use strict";

	var defaultSettings = {
		swipeSpeed: 500, // vertical movement pixels/sec that triggers a spin
		swipeDistance: 120 // min vertical movement pixels that's needed for a swipe
	};

	var InputGestureController = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._enabled = true;

		this._dragging = false;
		this._downPoint = null;
		this._lastPoint = null;

		this._gestureList = [];

		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerDown', function( point ) { that._onNotifyCanvasPointerDown( point ); } );
		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerUp', function( point ) { that._onNotifyCanvasPointerUp( point ); } );
		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerMove', function( point ) { that._onNotifyCanvasPointerMove( point ); } );
	};

	InputGestureController.prototype._onNotifyCanvasPointerDown = function( point ) {
		if ( this._enabled ) {
			this._dragging = true;
			this._downPoint = point;
			this._downTime = Date.now();
			this._lastPoint = point;
		}
	};

	InputGestureController.prototype._onNotifyCanvasPointerUp = function( point ) {
		if ( this._enabled && this._dragging ) {
			this._checkForGesture( point );
		}
		this._dragging = false;
		this._downPoint = null;
		this._lastPoint = null;
	};

	InputGestureController.prototype._onNotifyCanvasPointerMove = function( point ) {
		if ( this._enabled ) {
			if ( this._dragging ) {
				this._checkForGesture( point );
				this._lastPoint = point;
			}
		}
	};

	InputGestureController.prototype._checkForGesture = function( point ) {
		var absDeltaY = 0;
		var absDeltaX = 0;
		var speed = 0;
		this._gestureList.length = 0;

		var elapsed = Date.now() - this._downTime; // milli-secs
		if ( elapsed ) {
			var deltaX = point.x - this._downPoint.x;
			absDeltaX = Math.abs( deltaX );

			if ( absDeltaX >= this._settings.swipeDistance ) {
				deltaX = point.x - this._lastPoint.x;
				speed = 1000 * absDeltaX / elapsed;
				if ( speed > this._settings.swipeSpeed ) {
					if ( deltaX < 0 ) {
						this._gestureList.push( 'left' );
					} else {
						this._gestureList.push( 'right' );
					}
				}
			}

			var deltaY = point.y - this._downPoint.y;
			absDeltaY = Math.abs( deltaY );

			if ( absDeltaY >= this._settings.swipeDistance ) {
				deltaY = point.y - this._lastPoint.y;
				speed = 1000 * absDeltaY / elapsed;
				if ( speed > this._settings.swipeSpeed ) {
					if ( deltaY < 0 ) {
						this._gestureList.push( 'up' );
					} else {
						this._gestureList.push( 'down' );
					}
				}
			}
		}

		if ( this._gestureList.length > 0 ) {
			var gesture = 'swipe' + this._gestureList.join( '' );
			libs.globalEventBus.invoke( 'libs.event.notify.inputGesture', gesture );
		}
	};


	libs.client.core.InputGestureController = InputGestureController;

}());