(function() {
	"use strict";
	// Polyfill for request animation frame.
	// https://gist.github.com/paulirish/1579671
	(function( window ) {
		var lastTime = 0,
			vendors = ['webkit', 'moz'],
			requestAnimationFrame = window.requestAnimationFrame,
			cancelAnimationFrame = window.cancelAnimationFrame,
			i = vendors.length;

		// try to un-prefix existing raf
		while ( --i >= 0 && !requestAnimationFrame ) {
			requestAnimationFrame = window[vendors[i] + 'RequestAnimationFrame'];
			cancelAnimationFrame = window[vendors[i] + 'CancelAnimationFrame'];
		}

		// polyfill with setTimeout fallback
		// heavily inspired from @darius gist mod: https://gist.github.com/paulirish/1579671#comment-837945
		if ( !requestAnimationFrame || !cancelAnimationFrame ) {
			requestAnimationFrame = function( callback ) {
				var now = Date.now(), nextTime = Math.max( lastTime + 16, now );
				return setTimeout( function() {
					callback( lastTime = nextTime );
				}, nextTime - now );
			};

			cancelAnimationFrame = clearTimeout;
		}

		// export to window
		window.requestAnimationFrame = requestAnimationFrame;
		window.cancelAnimationFrame = cancelAnimationFrame;
	}( window ));


	// Sort of PollyFill for webaudio.
	// Just treat the webkit version as a standard version.
	(function() {
		if ( !window.AudioContext && window.webkitAudioContext ) {
			window.AudioContext = window.webkitAudioContext;
		}
	}());

	// Polyfill to move new vibrate API to standard version
	(function() {
		if ( !window.navigator.vibrate ) {
			if ( window.navigator.mozVibrate ) {
				window.navigator.vibrate = window.navigator.mozVibrate;
			} else if ( window.navigator.webkitVibrate ) {
				window.navigator.vibrate = window.navigator.webkitVibrate;
			}
		}
	}());

	// Allow knowledge of correct visibilitychange
	(function() {
		window.visibilityChangeEventName = null;
		window.visibilityHiddenName = null;
		if ( !document.hidden ) {
			window.visibilityChangeEventName = 'visibilitychange';
			window.visibilityHiddenName = 'hidden';
		} else if ( !document.mozHidden ) {
			window.visibilityChangeEventName = 'mozvisibilitychange';
			window.visibilityHiddenName = 'mozHidden';
		} else if ( !document.msHidden ) {
			window.visibilityChangeEventName = 'msvisibilitychange';
			window.visibilityHiddenName = 'msHidden';
		} else if ( !document.webkitHidden ) {
			window.visibilityChangeEventName = 'webkitvisibilitychange';
			window.visibilityHiddenName = 'webkitHidden';
		}
	}());

	// Polyfill to add create function to javascript objects
	(function() {
		if ( !Object.create ) {
			Object.create = (function() {
				function F() {}

				return function( o ) {
					if ( arguments.length !== 1 ) {
						throw new Error( 'Object.create implementation only accepts one parameter.' );
					}
					F.prototype = o;
					return new F();
				};
			})();
		}
	}());

	/*! GetDevicePixelRatio | Author: Tyson Matanich, 2012 | License: MIT - https://github.com/tysonmatanich/GetDevicePixelRatio/blob/master/getDevicePixelRatio.js */
	(function( window ) {
		if ( !window.devicePixelRatio ) {
			var ratio = 1;
			// To account for zoom, change to use deviceXDPI instead of systemXDPI
			if ( window.screen.systemXDPI !== undefined && window.screen.logicalXDPI !== undefined && window.screen.systemXDPI > window.screen.logicalXDPI ) {
				// Only allow for values > 1
				ratio = window.screen.systemXDPI / window.screen.logicalXDPI;
			}
			else if ( window.devicePixelRatio !== undefined ) {
				ratio = window.devicePixelRatio;
			}
			window.devicePixelRatio = ratio;
		}
	})( window );

	// Paul Irish performance.now() polyfill --- https://gist.github.com/paulirish/5438650

	// relies on Date.now() which has been supported everywhere modern for years.
	// as Safari 6 doesn't have support for NavigationTiming, we use a Date.now() timestamp for relative values

	// if you want values similar to what you'd get with real perf.now, place this towards the head of the page
	// but in reality, you're just getting the delta between now() calls, so it's not terribly important where it's placed

	(function() {
		// prepare base perf object
		if ( typeof window.performance === 'undefined' ) {
			window.performance = {};
		}

		if ( !window.performance.now ) {

			var nowOffset = Date.now();

			if ( window.performance.timing && window.performance.timing.navigationStart ) {
				nowOffset = window.performance.timing.navigationStart;
			}


			window.performance.now = function now() {
				return Date.now() - nowOffset;
			};

		}

	})();

	(function() {
		var IEXMLHttpRequest = window.XDomainRequest && function() {
				var xdr = new XDomainRequest();
				if ( !xdr ) {
					return null;
				}

				var request;

				var changeState = function( state ) {
					if ( state !== request.readyState ) {
						request.readyState = state;
						if ( request.onreadystatechange ) {
							request.onreadystatechange();
						}
					}
				};

				xdr.onerror = function() {
					request.status = 500;
					request.statusText = 'ERROR';
					changeState( 4 );
				};
				xdr.ontimeout = function() {
					request.status = 408;
					request.statusText = 'TIMEOUT';
					changeState( 4 );
				};
				xdr.onprogress = function() {
					changeState( 3 );
				};
				xdr.onload = function() {
					request.status = 200;
					request.statusText = 'OK';
					request.responseText = xdr.responseText;
					request.responseXML = new ActiveXObject( "Microsoft.XMLDOM" );
					request.responseXML.async = "false";
					request.responseXML.loadXML( xdr.responseText );
					changeState( 4 );
				};


				request = {
					open: function( method, url, async, user, password ) {
						xdr.open( method, url );
						changeState( 1 );
					},
					setRequestHeader: function() {
						// can I do this?
					},
					send: function( data ) {
						xdr.send( data );
						changeState( 2 );
					},
					abort: function() {
						xdr.abort();
					},
					status: '',
					statusText: '',
					getResponseHeader: function() {
					},
					getAllResponseHeaders: function() {
					},
					responseText: '',
					responseXML: '',
					readyState: 0,
					onreadystatechange: null
				};
				return request;
			};
	})();

}());