/**
 * The core components needed for the client
 *
 * @module libs_ClientCore
 */

goog.provide( 'libs.client.core.SystemStateController' );

goog.require( 'libs.globalEventBus' );


(function() {
	"use strict";

	var defaultSettings = {};

	/**
	 * The SystemStateController handles the moving between activities
	 * @class SystemStateController
	 * @constructor
	 */
	var SystemStateController = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.systemInit', function() { that._onSystemInitEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.gameLoading', function() { that._onLoadingEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.serverInit', function() { that._onServerInitEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.gameInit', function() { that._onGameInitEnd(); } );

		libs.globalEventBus.invokeActivity( 'libs.activity.systemState.systemInit' );
	};

	/**
	 * Fires on the systemInitEnd activity and triggers the gameLoading activity
	 * @private
	 * @method _onSystemInitEnd
	 */
	SystemStateController.prototype._onSystemInitEnd = function() {
		this._invokeActivity( 'libs.activity.systemState.gameLoading' );
	};

	/**
	 * Fires on the loadingEnd activity and triggers the serverInit activity
	 * @private
	 * @method _onLoadingEnd
	 */
	SystemStateController.prototype._onLoadingEnd = function() {
		this._invokeActivity( 'libs.activity.systemState.serverInit' );
	};

	/**
	 * Fires on the ServerInitEnd activity and triggers the serverInit gameInit
	 * @private
	 * @method _onServerInitEnd
	 */
	SystemStateController.prototype._onServerInitEnd = function() {
		this._invokeActivity( 'libs.activity.systemState.gameInit' );
	};

	/**
	 * Fires on the GameInitEnd activity and triggers the serverInit gamePlaying
	 * @private
	 * @method _onGameInitEnd
	 */
	SystemStateController.prototype._onGameInitEnd = function() {
		this._invokeActivity( 'libs.activity.systemState.gamePlaying' );
	};

	/**
	 * Invokes the given activity on the global event bus
	 * @private
	 * @method _invokeActivity
	 * @param {string} activityName
	 */
	SystemStateController.prototype._invokeActivity = function( activityName ) {
		// makes sure all calls on an activity end are processed before a new activity starts
		libs.setImmediate( function() {
			libs.globalEventBus.invokeActivity( activityName );
		} );
	};


	libs.client.core.SystemStateController = SystemStateController;

}());