goog.provide( 'libs.Timer' );


(function() {
	"use strict";

	var defaultSettings = {
		connectTickEvent: true
	};

	var Timer = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._duration = 0.0;
		this._hasExpired = true;
		this._paused = false;
		this._passed = 0.0;
		if ( this._settings.connectTickEvent ) {
			var that = this;
			libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._update( timeDelta ); } );
		}
	};

	Timer.prototype.expired = function() {
		return this._hasExpired;
	};

	Timer.prototype.update = function( timeDelta ) {
		if ( this._settings.connectTickEvent ) {
			throw new Error( "Timer has connectTickEvent setting - no need to call update() manually!" );
		}
		this._update( timeDelta );
	};

	Timer.prototype._update = function( timeDelta ) {
		if ( !this._paused && !this._hasExpired ) {
			this._passed += timeDelta;
			if ( this._passed >= this._duration ) {
				this._hasExpired = true;
			}
		}
	};

	Timer.prototype.start = function( duration_ms ) {
		if ( !isFinite( duration_ms ) ) {
			throw new Error( "Timer.start requires integer value!" );
		}
		this.reset( duration_ms );
	};

	Timer.prototype.reset = function( duration_ms ) {
		this._duration = isFinite( duration_ms ) ? duration_ms : this._duration;
		this._hasExpired = false;
		this._passed = 0.0;
	};

	Timer.prototype.getProgress = function() {
		if ( this._hasExpired ) {
			return 1.0;
		} else {
			return this._passed / this._duration;
		}
	};

	Timer.prototype.resume = function() {
		this._paused = false;
	};

	Timer.prototype.pause = function() {
		this._paused = true;
	};

	Timer.prototype.stop = function() {
		this._hasExpired = true;
	};

	libs.Timer = Timer;

}());