goog.provide( 'libs.client.hillsVegas.DesktopMenu' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.utils.misc' );
goog.require( 'libs.client.utils.DOM' );


( function() {
	"use strict";

	var defaultSettings = {
		links: undefined,
		text: {
			mainLinks: {
				fontSize: 16,
				maxWidth: 135
			},
			subLinks: {
				//fontSize: 16 commented out until Terry's scss is fixed
			},
			balance: {
				fontSize: 16
			}
		}
	};

	var selectors = {
		header: $( '#header' ),
		menuOptions: $('#menu-options'),
		menuOptionsHideClass: 'menu-options-hide',
		menuNotification: $( '#menu-notification' ),
		menuNotificationCTAContainer: $( '#menu-notification .cta-container' ),
		buttonHowToPlay: $( '#menu-button-how-to-play' ),
		buttonBettingHistory: $( '#menu-button-betting-history' ),
		buttonSound: $( '#menu-button-sound' ),
		buttonSoundText: $( '#menu-button-sound' ).children( '.toggle' ),
		buttonBackgroundSound: $( '#menu-button-background-sound' ),
		buttonTurbo: $( '#menu-button-turbo' ),
		buttonTurboText: $( '#menu-button-turbo' ).children( '.toggle' ),
		buttonHelp: $( '#menu-button-help' ),
		buttonMute: $( '#menu-button-mute' ),
		buttonMuteIcon: $( '#menu-button-mute' ).children( '.icon' ),
		balancePtsText: $( '#menu-balance-points-text' ),
		buttonPlayForReal: $( '#menu-button-play-for-real' ),
		buttonCashier: $( '#menu-button-cashier' ),
		buttonExit: $( '#menu-button-exit' ),
		soundButtons: $( '#menu-button-sound, #menu-button-mute' ),
		menuButtons: $( '#menu-button-help, #menu-button-how-to-play' ),
		menuOptionsButtonHeader: $( '#menu-options-button-options' ),
		menuButtonHowToPlay: $( '#menu-button-how-to-play' ),
		menuButtonBettingHistory: $( '#menu-button-betting-history' ),
		menuButtonSound: $( '#menu-button-sound-text' ),
		menuButtonSoundToggle: $( '#menu-button-sound .toggle' ),
		menuButtonTurbo: $( '#menu-button-turbo-text' ),
		menuButtonTurboToggle: $( '#menu-button-turbo .toggle' ),
		menuButtonHelpText: $( '#menu-button-help-text' ),
		menuButtonMuteText: $( '#menu-button-mute-text' ),
		menuButtonPlayForRealText: $( '#menu-button-play-for-real-text' ),
		menuButtonCashierText: $( '#menu-button-cashier-text' ),
		menuButtonExitText: $( '#menu-button-exit-text' ),
		menuBalancePointsText: $( '#menu-balance-points-text' )
	};

	selectors.header.hide();

	var DesktopMenu = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._isLocalServerMode = null;
		this._soundsEnabled = true;
		this._turboModeEnabled = false;

		this._handlePopupMessage = null;

		this._onNotifySoundsState( this._soundsEnabled );
		this._onNotifyTurboModeState( this._turboModeEnabled );

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.popupMessage', function( activity ) { that._onPopupMessageStart( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.platformType', function( platformType ) { that._onNotifyPlatformType( platformType ); } );
		libs.globalEventBus.connect( 'libs.event.notify.soundsState', function( enabled ) { that._onNotifySoundsState( enabled ); } );
		libs.globalEventBus.connect( 'libs.event.notify.turboModeState', function( enabled ) { that._onNotifyTurboModeState( enabled ); } );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connect( 'libs.event.request.enableMenu', function() { that._onRequestEnableMenu(); } );
		libs.globalEventBus.connect( 'libs.event.request.disableMenu', function() { that._onRequestDisableMenu(); } );

	};

	DesktopMenu.prototype._onGamePlayingStart = function( activity ) {
		selectors.header.show();
	};

	DesktopMenu.prototype._onPopupMessageStart = function( activity ) {
		// might have multiple popups, so don't create a new activity if one already exists
		if ( !this._handlePopupMessage ) {
			this._handlePopupMessage = activity.join();

		}
	};

	DesktopMenu.prototype._onNotifyPlatformType = function( platformType ) {
		var that = this;

		if ( this._isLocalServerMode === null ) {
			this._isLocalServerMode = platformType !== 'hillsLive';

			if ( this._isLocalServerMode ) {
				selectors.buttonCashier.hide();
				selectors.buttonPlayForReal.show();
			} else {
				selectors.buttonCashier.show();
				selectors.buttonPlayForReal.hide();
				selectors.balancePtsText.hide();
			}

			selectors.menuButtons.click( function() {
				if ( that._isLocalServerMode ) {
					that._showNotification( true, libs.getTranslation( 'This_functionality_is_only_available' ) );
				} else {
					libs.client.utils.misc.openUrl( that._settings.links.help, true );
				}
			} );

			selectors.soundButtons.click( function() {
				if ( that._soundsEnabled ) {
					libs.globalEventBus.invoke( 'libs.event.request.disableSounds' );
				} else {
					libs.globalEventBus.invoke( 'libs.event.request.enableSounds' );
				}
			} );

			selectors.buttonTurbo.click( function() {
				if ( that._turboModeEnabled ) {
					libs.globalEventBus.invoke( 'libs.event.request.disableTurboMode' );
				} else {
					libs.globalEventBus.invoke( 'libs.event.request.enableTurboMode' );
				}
			} );

			selectors.buttonBettingHistory.click( function() {
				if ( that._isLocalServerMode ) {
					that._showNotification( true, libs.getTranslation( 'This_functionality_is_only_available' ) );
				} else {
					libs.globalEventBus.invoke( 'libs.event.request.openHistory' );
				}
			} );

			selectors.buttonPlayForReal.click( function() {
				libs.client.utils.misc.openUrl( that._settings.links.playForReal, false );
			} );

			selectors.buttonCashier.click( function() {
				libs.client.utils.misc.openUrl( that._settings.links.deposit, true );
			} );

			selectors.buttonExit.click( function() {
				that._showNotification( false, libs.getTranslation( 'Are_you_sure_you_want_to_leave_the_game' ) );
			} );
		}
	};

	DesktopMenu.prototype._onNotifySoundsState = function( enabled ) {
		this._soundsEnabled = enabled;

		if ( this._soundsEnabled ) {
			selectors.buttonMuteIcon.removeClass( 'unmute' );
			selectors.buttonMuteIcon.addClass( 'mute' );
			libs.client.utils.DOM.setText( selectors.buttonSoundText, libs.getTranslation( 'ON' ), this._settings.text.subLinks );
		} else {
			selectors.buttonMuteIcon.removeClass( 'mute' );
			selectors.buttonMuteIcon.addClass( 'unmute' );
			libs.client.utils.DOM.setText( selectors.buttonSoundText, libs.getTranslation( 'OFF' ), this._settings.text.subLinks );
		}
	};

	DesktopMenu.prototype._onNotifyTurboModeState = function( enabled ) {
		this._turboModeEnabled = enabled;

		if ( this._turboModeEnabled ) {
			libs.client.utils.DOM.setText( selectors.buttonTurboText, libs.getTranslation( 'ON' ), this._settings.text.subLinks );
		} else {
			libs.client.utils.DOM.setText( selectors.buttonTurboText, libs.getTranslation( 'OFF' ), this._settings.text.subLinks );
		}
	};

	DesktopMenu.prototype._onNotifyLanguageChanged = function( langCode ) {
		libs.client.utils.DOM.setText( selectors.menuOptionsButtonHeader, libs.getTranslation( 'OPTIONS' ), this._settings.text.mainLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonHelpText, libs.getTranslation( 'HELP' ), this._settings.text.mainLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonMuteText, libs.getTranslation( 'MUTE' ), this._settings.text.mainLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonPlayForRealText, libs.getTranslation( 'PLAY_FOR_REAL' ), this._settings.text.mainLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonCashierText, libs.getTranslation( 'CASHIER' ), this._settings.text.mainLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonExitText, libs.getTranslation( 'EXIT' ), this._settings.text.mainLinks );

		libs.client.utils.DOM.setText( selectors.menuButtonHowToPlay, libs.getTranslation( 'HOW_TO_PLAY_THIS_GAME' ), this._settings.text.subLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonBettingHistory, libs.getTranslation( 'BETTING_HISTORY' ), this._settings.text.subLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonSound, libs.getTranslation( 'SOUND' ), this._settings.text.subLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonSoundToggle, libs.getTranslation( 'OFF' ), this._settings.text.subLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonTurbo, libs.getTranslation( 'TURBO_MODE' ), this._settings.text.subLinks );
		libs.client.utils.DOM.setText( selectors.menuButtonTurboToggle, libs.getTranslation( 'OFF' ), this._settings.text.subLinks );

		libs.client.utils.DOM.setText( selectors.menuBalancePointsText, libs.getTranslation( 'PTS' ), this._settings.text.balance );
	};

	DesktopMenu.prototype._onRequestEnableMenu = function() {
		selectors.menuOptions.removeClass( selectors.menuOptionsHideClass );
	};

	DesktopMenu.prototype._onRequestDisableMenu = function() {
		selectors.menuOptions.addClass( selectors.menuOptionsHideClass );
	};

	DesktopMenu.prototype._showNotification = function( justExitButton, text ) {
		var that = this;

		if ( !this._handlePopupMessage ) {
			libs.globalEventBus.invokeActivity( 'libs.activity.display.popupMessage' );
		}

		libs.globalEventBus.invoke( 'libs.event.request.closePaytable' );
		libs.globalEventBus.invoke( 'libs.event.request.closeHistory' );

		libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
			backdropClass: 'backdrop-menu',
			zIndex: 590,
			animationTime: 200,
			closeButton: '.close',
			confirmButton: '.confirm',

			confirmActions: function( ev, obj ) {
				//does not work with windows that have not been opened by JavaScript in Firefox
				// this should not be a problem in live as WH opens windows with JS
				window.open( window.location, '_parent' ).close();
				that._hideNotification();
			},
			cancelActions: function( ev, obj ) {
				that._hideNotification();
			},
			preInitFunction: function( ev, obj ) {
				libs.client.utils.DOM.setText( selectors.menuNotification.children( 'p' ), text );
				libs.client.utils.DOM.fadeIn( selectors.menuNotification, 10 );
				that._showNotificationCTA( justExitButton );
				libs.setTimeout( function() {
					$( '#viewport' ).redrawGentle();
				}, 500 );
			}
		} );
	};

	DesktopMenu.prototype._hideNotification = function() {
		libs.client.utils.DOM.fadeOut( selectors.menuNotification, 10 );
		libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-menu' );
		this._handlePopupMessage.leave();
		this._handlePopupMessage = null;
	};

	DesktopMenu.prototype._showNotificationCTA = function( justExitButton ) {
		var ok = '<a class="menuButton confirm" href="#">' + libs.getTranslation( 'OK' ) + '</a>',
			cancel = '<a class="menuButton close" href="#">' + libs.getTranslation( 'CANCEL' ) + '</a>',
			close = '<a class="menuButton close" href="#">' + libs.getTranslation( 'OK' ) + '</a>';

		if ( justExitButton ) {
			selectors.menuNotificationCTAContainer.html( close );
		} else {
			selectors.menuNotificationCTAContainer.html( ok + cancel );
		}
	};


	libs.client.hillsVegas.DesktopMenu = DesktopMenu;

}());