goog.provide( 'libs.client.hillsVegas.MobileMenu' );

goog.requireThirdParty( 'addToHome' );
goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.misc' );

goog.require( 'game.logic' );


( function() {
	"use strict";

	var defaultSettings = {
		title: game.logic.GameData.friendlyGameName,
		menuXML: 'bower_components/Libs_ClientHillsVegas/data/mobileMenu.xml',
		links: undefined
	};

	var selectors = {
		buttonBack: $( '#menu-button-back' ),
		buttonClose: $( '#menu-button-close' ),
		buttonSound: $( '#menu-button-sound' ),
		container: $( '#menu-container' ),
		containerClass: '#menu-container',
		buttonMenuClass: '.menu',
		title: $( '.top-menu span.title' ),
		submenuClass: '.sub-menu',
		menuClass: '.main-menu',
		menuitemClass: '.menu-item',
		menulinkClass: '.menu-link',
		menucontentClass: '.menu-content'
	};

	var clientInfo = libs.client.core.clientInformation;

	var MobileMenu = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._menuOpen = false;

		this._menuParent = null;
		this._menuContent = null;

		this._expanded = false;
		this._expandedObject = null;
		this._originalHeight = 0;
		this._expandToHeight = 0;
		this._menuStartHeight = 0;
		this._inSubMenu = false;
		this._justClicked = false;
		this._childMenuOpen = false;

		this._itemArray = [];

		this._soundsEnabled = true;

		this._handleDisplayMenu = null;

		libs.globalEventBus.connect( 'libs.event.notify.soundsState', function( enabled ) { that._onNotifySoundsState( enabled ); } );
		libs.globalEventBus.connect( 'libs.event.request.openMenu', function() { that._onRequestOpenMenu(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.menu', function( activity ) { that._onDisplayMenuStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.error', function( activity, isFatal ) { that._onDisplayErrorStart( activity, isFatal ); } );
		libs.globalEventBus.connect( 'libs.event.notify.orientationChanged', function( orientation ) { that._onNotifyOrientationChanged( orientation ); } );

		$( window ).resize( function() { that._onWindowResize(); } );

		libs.client.utils.misc.onMenuTappedAddCallback( 'Help', function() { that._selectHelp(); } );
		libs.client.utils.misc.onMenuTappedAddCallback( 'Paytable', function() { that._selectPaytable(); } );
		libs.client.utils.misc.onMenuTappedAddCallback( 'AddHomeScreen', function() { that._selectAddHomeScreen(); } );
		libs.client.utils.misc.onMenuTappedAddCallback( 'Deposit', function() { libs.client.utils.misc.openUrl( that._settings.links.deposit, true ); } );
		libs.client.utils.misc.onMenuTappedAddCallback( 'Withdraw', function() { libs.client.utils.misc.openUrl( that._settings.links.withdraw, true ); } );
		libs.client.utils.misc.onMenuTappedAddCallback( 'Lobby', function() { libs.client.utils.misc.openUrl( that._settings.links.lobby, false ); } );

		this._loadMenu();
	};

	MobileMenu.prototype._onNotifySoundsState = function( enabled ) {
		this._soundsEnabled = enabled;
		if ( this._soundsEnabled ) {
			selectors.buttonSound.removeClass( 'off' );
			selectors.buttonSound.addClass( 'on' );
		} else {
			selectors.buttonSound.removeClass( 'on' );
			selectors.buttonSound.addClass( 'off' );
		}
	};

	MobileMenu.prototype._onRequestOpenMenu = function() {
		var that = this;

		if ( !this._menuOpen ) {
			libs.globalEventBus.invokeActivity( 'libs.activity.display.menu' );

			libs.globalEventBus.invoke( 'libs.event.request.disableReelDragger' );
			libs.globalEventBus.invoke( 'libs.event.request.disableZIndexHack' );

			libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
				backdropClass: 'backdrop-menu',
				zIndex: 300,
				animationTime: 300,

				preInitFunction: function() {},

				clickEvent: function() {
					if ( !that._justClicked && !that._childMenuOpen ) {
						if ( !clientInfo.isWindowsPhone() ) {
							that._closeMenu();
						}
					}
				}
			} );

			$( '.col' ).redrawHarsh();
			$( '.top-menu' ).redrawHarsh();

			this._menuStartHeight = $( '.col' ).height();
			$( '.top-menu' ).each( function() {
				that._menuStartHeight += $( this ).height();
			} );
			selectors.container.stop().animate( {
				'top': 0,
				'height': that._menuStartHeight
			}, 400 );
		}
	};

	MobileMenu.prototype._onDisplayMenuStart = function( activity ) {
		this._handleDisplayMenu = activity.join();
		this._menuOpen = true;
	};

	MobileMenu.prototype._onDisplayErrorStart = function( activity ) {
		if ( this._menuOpen ) {
			this._closeMenu();
		}
	};

	MobileMenu.prototype._onNotifyOrientationChanged = function( orientation ) {
		if ( this._menuOpen ) {
			if ( orientation === 'portrait' ) {
				this._closeMenu();
			}
		}
	};

	MobileMenu.prototype._onWindowResize = function() {
		if ( this._menuOpen ) {
			this._closeMenu( 1 );
		}
	};

	MobileMenu.prototype._loadMenu = function() {
		var that = this;

		// request the menu info
		$.ajax( { url: ( libs.client.assetUrlPrefix + that._settings.menuXML ), type: 'GET' } )
			.done( function( data ) {
				that._menuLoaded( data );
				that._menuReady();
				selectors.title.html( that._settings.title );
			} )
			.fail( function( xmlHttpRequest, statusText, errorThrown ) {
				// No action - menu will remain not ready
			} );
	};

	MobileMenu.prototype._menuLoaded = function( data ) {
		var that = this;
		var $xml = $( data );

		this._itemArray.length = 0;
		var items = $xml.find( 'item' );
		items.each( function( index, item ) {
			var itemobj = {
				obj: $( this ),
				order: $( item ).attr( 'order' ),
				title: $( item ).attr( 'title' ),
				description: $( item ).attr( 'description' ),
				url: ($( item ).attr( 'url' )) ? $( item ).attr( 'url' ) : '#',
				childOF: '',
				hasChildren: ($( item ).children().length) ? true : false,
				htmlClass: '',
				html: ''
			};
			itemobj.htmlClass = itemobj.title.toLowerCase();
			itemobj.htmlClass = itemobj.htmlClass.replace( / /gi, '-' );
			itemobj.obj.attr( 'class', itemobj.htmlClass );

			if ( $( item ).parents().length > 1 ) {
				var i = $( item ).index();
				if ( i === 0 ) {
					selectors.container.append( '<div class="sub-menu ' + $( item ).parents().attr( 'class' ) + ' collapsed"></div>' );
				}
				$( selectors.containerClass + ' .' + $( item ).parents().attr( 'class' ) ).append(
					'<div class="menu-item">' +
					'<a href="' + itemobj.url + '" class="menu-link" ' +
					'data-order="' + itemobj.order + '" ' +
					'title="' + itemobj.description + '">' +
					itemobj.title + ' <span class="chevron">&raquo</span></a>' +
					'</div>' );
			} else {
				var htmlItem = '';
				htmlItem += '<div class="menu-item" data-order="' + itemobj.order + '">' +
				'<a href="' + itemobj.url + '" class="menu-link" ';
				htmlItem += (itemobj.hasChildren) ? 'data-child-item="' + itemobj.htmlClass + '" ' : '';
				htmlItem += 'title="' + itemobj.description + '">' +
				itemobj.title + '</a>' +
				'</div>';
				that._itemArray.splice( itemobj.order, 0, htmlItem );
			}

		} );
		//$('.main-menu').append(this._itemArray);
		this._itemArray.splice( 0, 0, '<div class="col">' );
		this._itemArray.splice( (this._itemArray.length * 0.5) + 1, 0, '</div><div class="col">' );
		this._itemArray.push( '</div>' );
		$( '.main-menu' ).append( this._itemArray.join( '' ) );
	};

	MobileMenu.prototype._menuReady = function() {
		var that = this;
		$( selectors.menuitemClass + ' a' ).click( function( e ) {
			e.preventDefault();
		} );

		// dim out and stop add to homescreen menu option if not in native ios web browser
		if ( !clientInfo.isIOS() || clientInfo.isIOSNativeApp() || clientInfo.isIOSWebApp() || clientInfo.isChrome() ) {
			$( selectors.menuitemClass + ' a[title="Add to HomeScreen"]' )
				.css( 'color', '#444' )
				.attr( 'href', '#' );
		}

		selectors.buttonClose.downEvent( function() {
			if ( that._justClicked ) {
				return;
			}

			that._closeMenu();
			that._tempDisableClick();
		} );

		selectors.buttonBack.downEvent( function() {
			if ( that._justClicked ) {
				return;
			}
			that._tempDisableClick();

			if ( that._expanded ) {
				$( selectors.menuitemClass ).hasClass( 'expended' ).animate( {
					'height': that._originalHeight
				}, 300, function() {
					$( this ).removeClass( 'expanded' );
					that._expanded = false;
					$( selectors.submenuClass ).animate( {
						'height': 0,
						'padding': 0
					}, 500, function() {
						$( this ).addClass( 'collapsed' );

						$( selectors.menuClass ).animate( {
							'height': that._menuStartHeight
						} ).removeClass( 'collapsed' );
					} );
				} );
			} else if ( that._childMenuOpen ) {
				that._contractChildMenu();
			} else {
				that._contractSubMenu();
				selectors.buttonClose.show();
				selectors.buttonBack.hide();
			}
		} );

		selectors.buttonSound.downEvent( function() {
			if ( that._soundsEnabled ) {
				libs.globalEventBus.invoke( 'libs.event.request.disableSounds' );
			} else {
				libs.globalEventBus.invoke( 'libs.event.request.enableSounds' );
			}
		} );

		$( selectors.menuClass + ' ' + selectors.menuitemClass + ' ' + selectors.menulinkClass ).downEvent( function() {
			if ( that._justClicked ) {
				return;
			}
			that._tempDisableClick();

			if ( !$( this ).data( 'childItem' ) ) {
				window.location = $( this ).attr( 'href' );
				that._closeMenu();
				return;
			}
			var target = $( this ).data( 'childItem' );
			$( selectors.menuClass ).animate( {
				'height': 0
			}, 300, function() {
				$( this ).addClass( 'collapsed' );
				var totalHeight = 0;
				$( selectors.submenuClass + '.' + target ).children( selectors.menuitemClass ).each( function() {
					totalHeight += $( this ).outerHeight();
				} );
				$( selectors.submenuClass + '.' + target ).animate( {
					'height': totalHeight
				} ).removeClass( 'collapsed' );

				selectors.buttonClose.hide();
				selectors.buttonBack.show();
				that._inSubMenu = true;
			} );
		} );

		$( selectors.submenuClass + ' ' + selectors.menulinkClass ).downEvent( function( e ) {
			if ( that._justClicked ) {
				return;
			}
			that._tempDisableClick();
			var obj = $( this );
			var url = obj.attr( 'href' );
			url = url.replace( 'javascript:', '' );
			eval( url );
		} );
	};

	MobileMenu.prototype._tempDisableClick = function() {
		// disable clicks in quick succession so menu has time to finish animating
		var that = this;
		this._justClicked = true;
		libs.setTimeout( function() {
			that._justClicked = false;
		}, 500 );
	};

	MobileMenu.prototype._contractSubMenu = function() {
		var that = this;

		if ( that._inSubMenu ) {
			$( selectors.submenuClass ).animate( {
				'height': 0,
				'padding': 0
			}, 500, function() {
				$( this ).addClass( 'collapsed' );
				$( selectors.menuClass ).animate( {
					'height': that._menuStartHeight
				} ).removeClass( 'collapsed' );
			} );
			that._inSubMenu = false;
		}
	};

	MobileMenu.prototype._expandChildMenu = function( obj ) {
		this._menuParent = obj.parent( selectors.menuitemClass );
		this._menuContent = obj.siblings( selectors.menucontentClass );
		var that = this;
		var objectsToExpand = obj.parents( selectors.menuitemClass + ', ' + selectors.submenuClass );

		if ( !this._menuParent.hasClass( 'expanded' ) ) {
			var menuItemHeight = 0;
			this._menuParent.parent().children( selectors.menuitemClass ).each( function() {
				menuItemHeight += $( this ).outerHeight();
			} );

			var windowHeight = $( window ).height();
			if ( libs.client.core.clientInformation.isIOS() ) {
				windowHeight = window.innerHeight ? window.innerHeight : $( window ).height();
			}

			this._expandToHeight = (windowHeight + menuItemHeight) - selectors.container.height();
			this._originalHeight = this._menuParent.height();

			this._menuParent.addClass( 'expanded' );
			this._menuContent.css( 'height', that._expandToHeight );
			$( selectors.submenuClass + '.sub-menu.game-info' ).stop().animate( {
				'top': -that._menuParent.outerHeight()
			}, 300, function() {
				objectsToExpand.animate( {
					height: '+=' + that._expandToHeight
				} );
				selectors.container.animate( {
					height: windowHeight
				} );
			} );

			that._childMenuOpen = true;
		} else {
			that._contractChildMenu( objectsToExpand );
		}
	};

	MobileMenu.prototype._contractChildMenu = function( objectsToContract ) {
		var that = this;
		if ( this._childMenuOpen ) {
			if ( !objectsToContract ) {
				objectsToContract = this._expandedObject.parents( selectors.menuitemClass + ', ' + selectors.submenuClass );
			}

			that._menuParent.css( 'height', '' );

			$( selectors.submenuClass + '.sub-menu.game-info' ).animate( {
				'top': 0
			} );

			selectors.container.stop().animate( {
				'height': that._menuStartHeight
			}, 400 );

			objectsToContract.animate( {
				height: '-=' + that._expandToHeight
			}, 400, function() {
				that._menuParent.css( 'height', '' );
			} ).removeClass( 'expanded' );

			$( '#helpfile' ).remove();

			that._childMenuOpen = false;
		}
	};

	MobileMenu.prototype._closeMenu = function( timeout ) {
		if ( this._menuOpen ) {
			this._contractSubMenu();
			this._contractChildMenu();

			if ( selectors.buttonBack.is( ':visible' ) ) {
				selectors.buttonClose.show();
				selectors.buttonBack.hide();
			}
			selectors.container.stop().animate( {
				'top': "-=" + this._menuStartHeight,
				'height': 0
			}, timeout || 400 );

			this._handleDisplayMenu.leave();
			this._handleDisplayMenu = null;

			this._menuOpen = false;

			libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-menu' );
			libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
			libs.globalEventBus.invoke( 'libs.event.request.enableReelDragger' );
		}
	};

	MobileMenu.prototype._selectHelp = function() {
		if ( this._menuOpen ) {
			var that = this;

			var $this = $( ".menu-link[title=Help]" );
			var thisParent = $this.parent( selectors.menuitemClass );
			if ( !$this.siblings( selectors.menucontentClass ).length ) {
				$.ajax( { url: that._settings.links.help, type: 'GET' } ).done( function( data ) {
					data = libs.client.utils.misc.replaceAssetPathsInText( data );
					thisParent.append( data );
					$( ".menu-link[title=Help]" ).siblings( selectors.menucontentClass ).height( that._expandToHeight );
				} ).fail( function( data ) {
					libs.globalEventBus.invoke( 'libs.event.request.showError', libs.getTranslation( 'Connection_failed_please_restart_the_game' ), false );
				} );
			}

			this._expandChildMenu( $this );
			this._expandedObject = $this;
		}
	};

	MobileMenu.prototype._selectPaytable = function() {
		if ( this._menuOpen ) {
			this._closeMenu();
			libs.globalEventBus.invoke( 'libs.event.request.openPaytable' );
		}
	};

	MobileMenu.prototype._selectAddHomeScreen = function() {
		addToHome.show( true );
	};


	libs.client.hillsVegas.MobileMenu = MobileMenu;

}());