goog.provide( 'libs.client.hillsVegas.createLocalServer' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getDefaultFormatForCurrencyCode' );
goog.require( 'libs.client.hillsVegas.LocalServerPlatformInterface' );
goog.require( 'libs.server.platform.debug.logRandomNumberStringLocal' );

goog.require( 'game.logic' );


(function() {
	"use strict";

	var availableLanguages = ['bg_BG', 'cs_CZ', 'da_DK', 'de_DE', 'el_GR', 'en_GB', 'es_ES', 'hu_HU', 'it_IT', 'nb_NO', 'pl_PL', 'pt_PT', 'ro_RO', 'ru_RU', 'sk_SK', 'sv_SE', 'zh_CN', 'zh_TW'];
	var availableCurrencies = ['ARS', 'AUD', 'BGN', 'BRL', 'CAD', 'CHF', 'CZK', 'DKK', 'EUR', 'GBP', 'HKD', 'HUF', 'INR', 'ISK', 'JPY', 'MXN', 'MYR', 'NOK', 'NZD', 'PLN', 'CNY', 'RON', 'SEK', 'SGD', 'THB', 'TWD', 'USD', 'ZAR', 'XXX'];

	var g_localStorageBetResultItemKeyPrefix = "ingg.localserver.betrecovery.";
	var g_localStoragePersistentDataItemKeyPrefix = "ingg.localserver.persistentdata.";

	var LocalServer = function( localClientSettings, fakePollTime ) {
		var that = this;

		this._localStorageBetResultKey = g_localStorageBetResultItemKeyPrefix + game.logic.GameData.friendlyGameName;
		this._localStoragePersistentDataKey = g_localStoragePersistentDataItemKeyPrefix + game.logic.GameData.friendlyGameName;

		this._delayConnect = 100;
		this._delayStartBet = 100;
		this._delayCloseBet = 100;
		this._delayBetHistory = 100;
		this._delayForcePoll = 100;
		this._delayPoll = fakePollTime || 5000;

		this.sent_stake = null;
		this.sent_lines = null;
		this._postBackData = null;
		this.inBetSequence = false;
		this._currentSequence = 0;
		this._startingReelSet = 0;

		this._preBetDelayComplete = false;
		this._preBetActivityComplete = false;

		libs.globalEventBus.connectActivityEnd( 'libs.activity.localServer.preBetResultCalculation', function() {
			that._preBetActivityComplete = true;
			that._canFireBet();
		} );
		libs.globalEventBus.connect( 'libs.event.notify.keyPressed', function( keyCode, keyChar ) { that._onKeyPressed( keyCode, keyChar ); } );
		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerDown', function( point ) { that._onNotifyCanvasPointerDown( point ); } );

		this._platformInterface = new libs.client.hillsVegas.LocalServerPlatformInterface( localClientSettings );

		this._debugLanguageIndex = 0;
		this._languageCode = this._platformInterface.getLanguageCode();

		this._debugCurrencyIndex = 0;
		this._currencyCode = this._platformInterface.getCurrencyCode();
		this._currencyFormatting = libs.getDefaultFormatForCurrencyCode( this._currencyCode );

		this._updatePlatformValues();

		var stateString = this._loadLocal( this._localStoragePersistentDataKey );
		this._currentState = libs.xmlUtils.jsonParse( stateString );
		if ( !this._currentState ) {
			this._currentState = {};
		} else {
			if ( this._currentState['game'] && isFinite( this._currentState['game']['nextCheckpointId'] ) ) {
				this._currentSequence = this._currentState['game']['nextCheckpointId'];
			}
		}

		var initObject = game.logic.BuildInitResponse( this._platformInterface.getCurrencyCode(), { 'serverState': this._currentState } );

		this.availableStakesPerLine = initObject.stake.steps.map( function( stake ) { return new BigNumber( stake ); } );
		this.startingStake = new BigNumber( initObject.stake.starting );
		this._reelSets = initObject.reel_set;
		this._startingReelSet = initObject.startingReelSet || 0;

		this._initCallback = null;

		libs.setTimeout( function() {that._normalPoll();}, this._delayPoll );
	};

	LocalServer.prototype._updatePlatformValues = function() {
		//No need to divide anymore -> using decimals
		this.balance = this._platformInterface.getBalance();
		this.jackpots = [];
		for ( var jackpotIndex = 0; jackpotIndex < this._platformInterface.getJackpotCount(); ++jackpotIndex ) {
			this.jackpots.push( this._platformInterface.getJackpotValue( jackpotIndex ) );
		}
	};

	LocalServer.prototype.onInit = function( callback ) {
		this._initCallback = callback;
	};

	LocalServer.prototype.getReels = function() {
		return this._reelSets;
	};

	LocalServer.prototype.getStartingReelSet = function() {
		return this._startingReelSet;
	};

	LocalServer.prototype.connect = function() {
		var that = this;

		libs.setTimeout( function() {that._fireConnect();}, this._delayConnect );
	};

	LocalServer.prototype.startBet = function( stake, lines, postBackData ) {
		var that = this;

		this.sent_stake = stake;
		this.sent_lines = lines;
		this._postBackData = postBackData;
		this._preBetDelayComplete = false;
		this._preBetActivityComplete = false;

		// Start pre bet calculation activity. _fireBet() won't start until this activity has completed.
		// This gives the debug server interface object time to contact the debug server to see if a forced game has been selected for the next spin.
		libs.globalEventBus.invokeActivity( 'libs.activity.localServer.preBetResultCalculation', this._currentSequence );
		libs.setTimeout( function() {
				that._preBetDelayComplete = true;
				that._canFireBet();
			},
			this._delayStartBet );
	};

	LocalServer.prototype._canFireBet = function() {

		if ( this._preBetDelayComplete && this._preBetActivityComplete ) {
			this._fireBet();
		}
	};

	LocalServer.prototype.betFinished = function( inFreeSpin ) {
		var that = this;

		libs.setTimeout( function() {that._fireClose();}, this._delayCloseBet );
	};

	LocalServer.prototype.forcePoll = function() {
		var that = this;

		libs.setTimeout( function() {that._firePoll();}, this._delayForcePoll );
	};

	LocalServer.prototype.loadHistory = function() {
		libs.setTimeout( function() { libs.globalEventBus.invoke( 'libs.event.notify.serverBetHistory', [] );}, this._delayBetHistory );
	};


	LocalServer.prototype._firePoll = function() {
		//this._updatePlatformValues();
		libs.globalEventBus.invoke( 'libs.event.notify.serverPolled' );
	};

	LocalServer.prototype._fireConnect = function() {
		if ( this._initCallback ) {
			var unfinishedBet = null;
			var persistentState = null;

			// load bet recovery result from local storage if available
			var itemString = this._loadLocal( this._localStorageBetResultKey );
			if ( itemString ) {
				unfinishedBet = libs.xmlUtils.jsonParse( itemString );
			}

			if ( this._currentState && this._currentState['player'] ) {
				persistentState = this._currentState['player'];
			}

			libs.globalEventBus.invoke( 'libs.event.request.changeLanguage', this._languageCode );
			libs.globalEventBus.invoke( 'libs.event.request.changeCurrency', this._currencyCode, this._currencyFormatting );
			this._initCallback( unfinishedBet, persistentState );
		}
	};

	LocalServer.prototype._fireBet = function() {

		var isFirstOperation = this._currentSequence === 0;
		if ( isFirstOperation ) {
			var forcedStaticDataString = libs.client.hillsVegas.LocalServerPlatformInterface.getForcedStaticData();
			if ( forcedStaticDataString && forcedStaticDataString.length > 0 ) {
				try {
					this._currentState['player'] = JSON.parse( forcedStaticDataString );
				}
				catch ( err ) {
					throw new Error( "Could not parse forced static data. Are you sure it's valid json?" );
				}
				libs.client.hillsVegas.LocalServerPlatformInterface.forceStaticData( '' );
			}
			this._platformInterface.startFirstBetOperation();
		}

		var stakePerLine = this.sent_stake;
		var totalStake = stakePerLine.times( this.sent_lines );
		var betSelection = {
			'stake': stakePerLine,
			'total_stake': totalStake,
			'lines': this.sent_lines,
			'checkpointId': this._currentSequence/*, 'jackpot': this.jackpots*/
		};
		var betResult = game.logic.BuildBetResults( this._platformInterface, {
			'betSelection': betSelection,
			'serverState': this._currentState,
			'postData': this._postBackData
		} );
		this._postBackData = null;
		betResult['isCheckpoint'] = !isFirstOperation;

		// Tell server what happened. ??Do we do this if have no win or if won a jackpot?
		this._platformInterface.takeStake( this._currentSequence, totalStake, betResult['win'], true );

		// update sequence number and server state
		if ( betResult['expectCheckpoint'] ) {
			this._currentSequence++;
		} else {
			this._currentSequence = 0;
			this._currentState['game'] = {};
		}

		betResult['reelDelays'] = [];

		this.inBetSequence = betResult['inBetSequence'];

		this._updatePlatformValues();
		libs.server.platform.debug.logRandomNumberStringLocal( this._platformInterface.getPreviousRngString() );

		// store game result for bet recovery
		this._saveLocal( this._localStorageBetResultKey, libs.xmlUtils.jsonStringify( betResult ) );
		this._saveLocal( this._localStoragePersistentDataKey, libs.xmlUtils.jsonStringify( this._currentState ) );

		libs.globalEventBus.invoke( 'libs.event.notify.serverBetResult', betResult );
	};

	LocalServer.prototype._fireClose = function() {
		this._platformInterface.closeBet();
		this._updatePlatformValues();

		// clear stored game result on successful close bet
		this._clearLocal( this._localStorageBetResultKey );
		libs.globalEventBus.invoke( 'libs.event.notify.serverBetFinished' );
	};

	LocalServer.prototype._normalPoll = function() {
		var that = this;
		this._firePoll();
		libs.setTimeout( function() {that._normalPoll();}, this._delayPoll );
	};


	LocalServer.prototype._saveLocal = function( key, item ) {
		if ( BUILD_TYPE === 'dev' && BUILD_TARGET === 'desktop' ) {
			try {
				if ( localStorage ) {
					localStorage.setItem( key, item );
				}
			}
			catch ( err ) {}
		}
	};


	LocalServer.prototype._loadLocal = function( key ) {
		if ( BUILD_TYPE === 'dev' ) {
			try {
				if ( localStorage ) {
					return localStorage.getItem( key );
				}
			}
			catch ( err ) {}
		}
		return null;
	};


	LocalServer.prototype._clearLocal = function( key ) {
		if ( BUILD_TYPE === 'dev' ) {
			try {
				if ( localStorage ) {
					localStorage.removeItem( key );
				}
			}
			catch ( err ) {}
		}
	};


	LocalServer.prototype._onKeyPressed = function( keyCode, keyChar ) {
		if ( BUILD_TYPE === 'dev' ) {
			if ( keyChar === 'c' || keyChar === 'C' ) {
				this._clearLocalStorage();
			}

			if ( keyChar === 'm' || keyChar === 'M' ) {
				this._cycleThroughCurrencies();
			}

			if ( keyChar === 'l' || keyChar === 'L' ) {
				this._cycleThroughLanguages();
			}
		}
	};


	LocalServer.prototype._onNotifyCanvasPointerDown = function( point ) {
		if ( BUILD_TYPE === 'dev' ) {
			if ( point.x < 50 ) {
				if ( point.y < 100 ) {
					this._clearLocalStorage();
				} else if ( point.y > 100 && point.y < 325 ) {
					this._cycleThroughCurrencies();
				} else if ( point.y > 325 && point.y < 550 ) {
					this._cycleThroughLanguages();
				}
			}
		}
	};


	LocalServer.prototype._clearLocalStorage = function() {
		this._clearLocal( this._localStorageBetResultKey );
		this._clearLocal( this._localStoragePersistentDataKey );
		libs.log( 'LocalServer: Local storage cleared' );
	};

	LocalServer.prototype._cycleThroughCurrencies = function() {
		this._currencyCode = availableCurrencies[this._debugCurrencyIndex];
		this._currencyFormatting = libs.getDefaultFormatForCurrencyCode( this._currencyCode );
		libs.globalEventBus.invoke( 'libs.event.request.changeCurrency', this._currencyCode, this._currencyFormatting );
		libs.log( 'LocalServer: Currency changed to ' + this._currencyCode + '(' + this._currencyFormatting.description + ')' );
		++this._debugCurrencyIndex;
		this._debugCurrencyIndex %= availableCurrencies.length;
	};

	LocalServer.prototype._cycleThroughLanguages = function() {
		++this._debugLanguageIndex;
		this._debugLanguageIndex %= availableLanguages.length;

		this._languageCode = availableLanguages[this._debugLanguageIndex];
		libs.globalEventBus.invoke( 'libs.event.request.changeLanguage', this._languageCode );
		libs.log( 'LocalServer: Language changed to ' + this._languageCode );

	};

	libs.client.hillsVegas.createLocalServer = function( settings ) {
		return new LocalServer( settings || {} );
	};

}());
