goog.provide( 'libs.client.hillsVegas.HistoryDisplay' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.presentation.HtmlButton' );
goog.require( 'libs.client.rendering.Sprite' );
goog.require( 'libs.client.rendering.LegacySprite' );
goog.require( 'libs.client.rendering.SpriteGroup' );
goog.require( 'libs.client.rendering.CanvasText' );
goog.require( 'libs.client.utils.DOM' );
goog.require( 'libs.client.utils.misc' );

goog.require( 'game.logic' );


(function() {
	"use strict";

	var defaultSettings = {
		historyPage: "../Engine/gethistory.asp",
		symbolsData: undefined,
		detailsBackground: {
			imageName: 'DetailsPanel',
			drawSettings: {
				x: 160,
				y: 80
			}
		},
		detailsClose: {
			x: 423, //Relative to the background.
			y: -10, //Relative to the background.
			height: 50,
			width: 50
		},
		symbolClip: {
			x: 180,
			y: 140,
			height: 249,
			width: 456
		},
		symbolBackground: {
			colour: "#f9f9f3",
			x: 180,
			y: 140,
			height: 249,
			width: 456
		},
		gameTypeLabel: {
			colours: ["#000000"], font: "12px Arial",
			boundaries: { x: 337, y: 110, width: 140, height: 32 }
		},
		labels: [
			{
				text: "STAKE_PER_LINE", colours: ["#000000"], font: "12px Arial",
				boundaries: { x: 165, y: 396, width: 140, height: 32 }
			},
			{
				text: "TOTAL_STAKE", colours: ["#000000"], font: "12px Arial",
				boundaries: { x: 280, y: 396, width: 140, height: 32 }
			},
			{
				text: "NUMBER_OF_WINS", colours: ["#000000"], font: "12px Arial",
				boundaries: { x: 395, y: 396, width: 140, height: 32 }
			},
			{
				text: "TOTAL_WINNINGS", colours: ["#000000"], font: "12px Arial",
				boundaries: { x: 510, y: 396, width: 140, height: 32 }
			}
		],
		values: [
			{
				text: "stakePerLine", colours: ["#000000"], font: "12px Arial",
				boundaries: { x: 165, y: 427, width: 140, height: 32 }
			},
			{
				text: "amount", colours: ["#000000"], font: "12px Arial",
				boundaries: { x: 280, y: 427, width: 140, height: 32 }
			},
			{
				text: "result", colours: ["#000000"], font: "12px Arial",
				boundaries: { x: 395, y: 427, width: 140, height: 32 }
			},
			{
				text: "won", colours: ["#000000"], font: "12px Arial",
				boundaries: { x: 510, y: 427, width: 140, height: 32 }
			}
		],
		symbolPositions: {
			'15': [ // number of symbols
				{ x: 160, y: 92, scaley: 0.68, scalex: 0.66 },
				{ x: 251, y: 92, scaley: 0.68, scalex: 0.66 },
				{ x: 343, y: 92, scaley: 0.68, scalex: 0.66 },
				{ x: 435, y: 92, scaley: 0.68, scalex: 0.66 },
				{ x: 524, y: 92, scaley: 0.68, scalex: 0.66 },

				{ x: 160, y: 176, scaley: 0.68, scalex: 0.66 },
				{ x: 251, y: 176, scaley: 0.68, scalex: 0.66 },
				{ x: 343, y: 176, scaley: 0.68, scalex: 0.66 },
				{ x: 435, y: 176, scaley: 0.68, scalex: 0.66 },
				{ x: 524, y: 176, scaley: 0.68, scalex: 0.66 },

				{ x: 160, y: 260, scaley: 0.68, scalex: 0.66 },
				{ x: 251, y: 260, scaley: 0.68, scalex: 0.66 },
				{ x: 343, y: 260, scaley: 0.68, scalex: 0.66 },
				{ x: 435, y: 260, scaley: 0.68, scalex: 0.66 },
				{ x: 524, y: 260, scaley: 0.68, scalex: 0.66 }
			],
			'20': [ // number of symbols
				{ x: 180, y: 102, scaley: 0.5, scalex: 0.5 },
				{ x: 271, y: 102, scaley: 0.5, scalex: 0.5 },
				{ x: 363, y: 102, scaley: 0.5, scalex: 0.5 },
				{ x: 455, y: 102, scaley: 0.5, scalex: 0.5 },
				{ x: 544, y: 102, scaley: 0.5, scalex: 0.5 },

				{ x: 180, y: 166, scaley: 0.5, scalex: 0.5 },
				{ x: 271, y: 166, scaley: 0.5, scalex: 0.5 },
				{ x: 363, y: 166, scaley: 0.5, scalex: 0.5 },
				{ x: 455, y: 166, scaley: 0.5, scalex: 0.5 },
				{ x: 544, y: 166, scaley: 0.5, scalex: 0.5 },

				{ x: 180, y: 230, scaley: 0.5, scalex: 0.5 },
				{ x: 271, y: 230, scaley: 0.5, scalex: 0.5 },
				{ x: 363, y: 230, scaley: 0.5, scalex: 0.5 },
				{ x: 455, y: 230, scaley: 0.5, scalex: 0.5 },
				{ x: 544, y: 230, scaley: 0.5, scalex: 0.5 },

				{ x: 180, y: 294, scaley: 0.5, scalex: 0.5 },
				{ x: 271, y: 294, scaley: 0.5, scalex: 0.5 },
				{ x: 363, y: 294, scaley: 0.5, scalex: 0.5 },
				{ x: 455, y: 294, scaley: 0.5, scalex: 0.5 },
				{ x: 544, y: 294, scaley: 0.5, scalex: 0.5 }
			]
		},
		text: {
			title: {
				fontSize: 16
			},
			columnHeadings: {
				fontSize: 16
			}
		}
	};

	var selectors = {
		container: $( '#history-container' ),
		caption: $( '#history-caption' ),
		table: $( '#history-list-table' )[0],
		rowClass: 'bet-history-row',
		buttons: $( ".historyButton" ),
		listMoreClass: 'history-list-more',
		headerTime: $( '#history-list-table-header-time' ),
		headerAction: $( '#history-list-table-header-action' ),
		headerRef: $( '#history-list-table-header-ref' ),
		headerAmount: $( '#history-list-table-header-amount' ),
		headerWon: $( '#history-list-table-header-won' ),
		headerStakePerLine: $( '#history-list-table-header-stake-per-line' ),
		headerSelected: $( '#history-list-table-header-selected' ),
		headerResults: $( '#history-list-table-header-results' ),
		print: $( '#history-print-text' )
	};

	selectors.container.hide();

	var HistoryDisplay = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._featureActive = false;

		this._symbolImages = null;
		this._historyDetails = null;

		this._handleDisplayHistory = null;

		this._buttonClose = new libs.client.presentation.HtmlButton(
			$( '.historyClose' ), true, function() {
				that._closeClicked();
			}
		);

		this._buttonPrint = new libs.client.presentation.HtmlButton(
			$( '.historyPrint' ), true, function() {
				that._printClicked();
			}
		);

		libs.globalEventBus.connect( 'libs.event.request.openHistory', function() { that._onRequestOpenHistory(); } );
		libs.globalEventBus.connect( 'libs.event.request.closeHistory', function() { that._onRequestCloseHistory(); } );
		libs.globalEventBus.connect( 'libs.event.notify.serverBetHistory', function( historyData ) { that._onNotifyServerBetHistory( historyData ); } );
		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerClick', function( point ) { that._onNotifyCanvasPointerClick( point ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.history', function( activity ) { that._onDisplayHistoryStart( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
	};

	HistoryDisplay.prototype._onRequestOpenHistory = function() {
		libs.globalEventBus.invokeActivity( 'libs.activity.display.history' );

		if ( !this._symbolImages ) {
			this._symbolImages = {};
			for ( var symbolName in this._settings.symbolsData ) {
				if ( this._settings.symbolsData.hasOwnProperty( symbolName ) ) {
					var symbolData = this._settings.symbolsData[symbolName];
					this._symbolImages[symbolName] = libs.loadedImages[symbolData.name];
				}
			}
		}

		libs.globalEventBus.invoke( 'libs.event.request.enableCanvasOverlayMode' );
		libs.globalEventBus.invoke( 'libs.event.request.disableReelDragger' );
		libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
			backdropClass: 'backdrop-history',
			zIndex: libs.surfaceSettings.top.zIndexMovedToFront - 1,
			animationTime: 200
		} );

		this._showHistoryList();
	};

	HistoryDisplay.prototype._onNotifyLanguageChanged = function( langCode ) {
		libs.client.utils.DOM.setText( selectors.caption, libs.getTranslation( 'BET_HISTORY' ), this._settings.text.title );
		libs.client.utils.DOM.setText( selectors.headerTime, libs.getTranslation( 'TIME' ), this._settings.text.columnHeadings );
		libs.client.utils.DOM.setText( selectors.headerAction, libs.getTranslation( 'ACTION' ), this._settings.text.columnHeadings );
		libs.client.utils.DOM.setText( selectors.headerRef, libs.getTranslation( 'REF' ), this._settings.text.columnHeadings );
		libs.client.utils.DOM.setText( selectors.headerAmount, libs.getTranslation( 'AMOUNT' ), this._settings.text.columnHeadings );
		libs.client.utils.DOM.setText( selectors.headerWon, libs.getTranslation( 'WON' ), this._settings.text.columnHeadings );
		libs.client.utils.DOM.setText( selectors.headerStakePerLine, libs.getTranslation( 'STAKE_PER_LINE' ), this._settings.text.columnHeadings );
		libs.client.utils.DOM.setText( selectors.headerSelected, libs.getTranslation( 'SELECTED' ), this._settings.text.columnHeadings );
		libs.client.utils.DOM.setText( selectors.headerResults, libs.getTranslation( 'RESULTS' ), this._settings.text.columnHeadings );
		libs.client.utils.DOM.setText( selectors.print, libs.getTranslation( 'PRINT' ), this._settings.text.columnHeadings );
	};

	HistoryDisplay.prototype._onRequestCloseHistory = function() {
		this._closeHistory();
	};

	HistoryDisplay.prototype._onNotifyServerBetHistory = function( historyData ) {
		var that = this;

		this._clearRows();

		historyData.forEach( function( bet ) {
			that._addRow( selectors.table, bet );
		} );
	};

	HistoryDisplay.prototype._onNotifyCanvasPointerClick = function( point ) {
		if ( this._historyDetails ) {
			this._historyDetails.mouseClick( point );
		}
	};

	HistoryDisplay.prototype._onDisplayHistoryStart = function( activity ) {
		if ( !this._handleDisplayHistory ) {
			this._handleDisplayHistory = activity.join();
		}
	};

	HistoryDisplay.prototype._showHistoryList = function() {
		selectors.container.show();
	};

	HistoryDisplay.prototype._hideHistoryList = function() {
		selectors.container.hide();
	};

	HistoryDisplay.prototype._clearRows = function() {
		$( '.' + selectors.rowClass ).remove();
	};

	HistoryDisplay.prototype._closeHistory = function() {
		if ( this._handleDisplayHistory && this._handleDisplayHistory.isJoined() ) {
			this._hideHistoryList();
			this._historyDetails = null;

			libs.globalEventBus.invoke( 'libs.event.request.disableCanvasOverlayMode' );
			libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-history' );
			libs.globalEventBus.invoke( 'libs.event.request.enableReelDragger' );

			this._handleDisplayHistory.leave();
			this._handleDisplayHistory = null;
		}
	};

	HistoryDisplay.prototype._closeClicked = function() {
		this._closeHistory();
	};

	HistoryDisplay.prototype._printClicked = function() {
		libs.client.utils.misc.openUrl( this._settings.historyPage, true );
	};

	HistoryDisplay.prototype._addRow = function( table, bet ) {
		var that = this;

		bet['result'] = winResult( bet );

		var row = '<tr class="' + selectors.rowClass + '">';
		var time = '<td>';
		time += bet['time'];
		time += '</td>';
		var action = '<td>';
		action += bet['action'];
		action += '</td>';
		var ref = '<td>';
		ref += bet['ref'];
		ref += '</td>';
		var amount = '<td>';
		amount += bet['amount'];
		amount += '</td>';
		var won = '<td>';
		won += bet['won'];
		won += '</td>';
		var stakePerLine = '<td>';
		stakePerLine += bet['detail']['extras']['stakePerLine'];
		stakePerLine += '</td>';
		var selected = '<td>';
		selected += getBetSelectionText( bet );
		selected += '</td>';
		var results = '<td>';
		results += bet['result'];
		results += '</td>';

		var more = '<td>';
		more += '<a href="#" id="' + selectors.listMoreClass + '">' + libs.getTranslation( 'MORE' ) + '</a>';
		more += '</td>';

		row += time + action + ref + amount + won + stakePerLine + selected + results + more + '</tr>';

		$( table ).append( row );

		var lastRow = table.rows[table.rows.length - 1];
		var lastCell = lastRow.cells[lastRow.cells.length - 1];
		lastCell.onclick = function() {
			that._showDetails( bet );
		};
	};

	HistoryDisplay.prototype._showDetails = function( bet ) {
		var that = this;

		this._historyDetails = new HistoryDetails( this._settings, bet, this._symbolImages );
		this._historyDetails.onDetailsClosed( function() {that._hideDetails();} );

		this._featureActive = true;

		this._hideHistoryList();
	};

	HistoryDisplay.prototype._hideDetails = function() {
		this._showHistoryList();
		this._historyDetails = null;
		this._featureActive = false;
	};

	HistoryDisplay.prototype.draw = function( ctx ) {
		if ( this._historyDetails ) {
			this._historyDetails.draw( ctx );
		}
	};

	HistoryDisplay.prototype.drawIfDirty = function( ctx ) {
		if ( this._historyDetails ) {
			this._historyDetails.drawIfDirty( ctx );
		}
	};

	HistoryDisplay.prototype.isFeatureActive = function() {
		return this._featureActive;
	};


	var HistoryDetails = function( settings, bet, symbolImages ) {
		this._settings = settings;

		var that = this;

		this._isDirty = true;
		this._detailsClosedCallback = null;

		this._detailsBackground = new libs.client.rendering.Sprite( this._settings.detailsBackground );

		this._detailsClose = {
			x: this._settings.detailsBackground.drawSettings.x + this._settings.detailsClose.x,
			y: this._settings.detailsBackground.drawSettings.y + this._settings.detailsClose.y,
			height: this._settings.detailsClose.height,
			width: this._settings.detailsClose.width
		};

		var labelTexts = this._settings.labels.map( function( settings ) {
			return new libs.client.rendering.CanvasText( libs.getTranslation( settings.text ), settings );
		} );

		labelTexts.push( new libs.client.rendering.CanvasText( getBetSelectionText( bet ), this._settings.gameTypeLabel ) );

		this._labels = new libs.client.rendering.SpriteGroup( labelTexts );

		var valueTexts = this._settings.values.map( function( settings ) {
			var betText; // need to convert string setting to variable (who's var names could get mangled in closure)
			if ( settings.text === "stakePerLine" ) {
				betText = bet['detail']['extras']['stakePerLine'];
			} else if ( settings.text === "amount" ) {
				betText = bet['amount'];
			} else if ( settings.text === "result" ) {
				betText = bet['detail']['extras']['winCount'];
			} else if ( settings.text === "won" ) {
				betText = bet['won'];
			}
			return new libs.client.rendering.CanvasText( betText, settings );
		} );

		this._values = new libs.client.rendering.SpriteGroup( valueTexts );

		var symbolArray = [];
		bet['detail']['visibleSymbols'].forEach( function( row ) {
			symbolArray = symbolArray.concat( row );
		} );
		var symbolCount = symbolArray.length;

		if ( !this._settings.symbolPositions[symbolCount] ) {
			throw new Error( 'HistoryDisplay: symbolPositions in settings object does not match the number of symbols to render' );
		}

		var symbolSprites = symbolArray.map( function( symbolId, index ) {
			return new libs.client.rendering.LegacySprite( symbolImages[game.logic.GameData.reels.symbols.longNameToShortName( symbolId )], that._settings.symbolPositions[symbolCount][index] );
		} );

		this._symbolImages = new libs.client.rendering.SpriteGroup( symbolSprites );
	};

	HistoryDetails.prototype.onDetailsClosed = function( callback ) {
		this._detailsClosedCallback = callback;
	};

	HistoryDetails.prototype.draw = function( ctx ) {
		var clipRegion = this._settings.symbolClip;
		var background = this._settings.symbolBackground;
		this._detailsBackground.draw( ctx );
		this._labels.draw( ctx );
		this._values.draw( ctx );

		ctx.save();
		ctx.beginPath();
		ctx.rect( background.x, background.y, background.width, background.height );
		ctx.fillStyle = background.colour;
		ctx.fill();

		ctx.beginPath();
		ctx.rect( clipRegion.x, clipRegion.y, clipRegion.width, clipRegion.height );
		ctx.clip();

		this._symbolImages.draw( ctx );

		ctx.restore();

		//this._isDirty = false;
	};

	HistoryDetails.prototype.drawIfDirty = function( ctx ) {
		if ( this._isDirty ) {
			this.draw( ctx );
		}
	};

	HistoryDetails.prototype.mouseClick = function( point ) {
		if ( hitTest( point, this._detailsClose ) && this._detailsClosedCallback ) {
			this._detailsClosedCallback();
		}

	};

	var getBetSelectionText = function( bet ) {
		var selected;
		if ( bet['selection'] === "NormalSpin" ) {
			selected = libs.getTranslation( 'Normal_Spin' );
		} else if ( bet['selection'] === "FreeSpin" ) {
			selected = libs.getTranslation( 'Free_Spin' );
		} else {
			selected = bet.selection;
		}
		return selected;
	};

	var winResult = function( bet ) {
		var result;
		var winCount = bet['detail']['extras']['winCount'];

		if ( winCount > 1 ) {
			result = winCount + " " + libs.getTranslation( 'Wins' );
		} else if ( winCount === 1 ) {
			result = "1 " + libs.getTranslation( 'Win' );
		} else if ( winCount === 0 && bet.won > 0 ) {
			result = libs.getTranslation( 'Win' );
		} else {
			result = libs.getTranslation( 'No_Win' );
		}

		return result;
	};

	var hitTest = function( point, rect ) {
		return point.x >= rect.x && point.x <= rect.x + rect.width &&
			point.y >= rect.y && point.y <= rect.y + rect.height;
	};


	libs.client.hillsVegas.HistoryDisplay = HistoryDisplay;

}());