goog.provide( 'libs.client.presentation.BigWinner' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.moneyFormat' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.presentation.BigWinnerFountain' );
goog.require( 'libs.client.utils.DOM' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var defaultSettings = {
		awardLevels: [
			{ multiplier: 20, translation: 'BIG_WIN', cssClass: 'level0', animName: undefined },
			{ multiplier: 40, translation: 'HUGE_WIN', cssClass: 'level1', animName: undefined },
			{ multiplier: 60, translation: 'MEGA_BIG_WIN', cssClass: 'level2', animName: undefined }
		],
		displayTime: 5000,
		multilineMessage: false,
		gradientText: true,
		overrideStyles: [],
		maxLines: 1,
		fountain: {},
		text: {
			title: {
				fontSize: 100,
				maxWidth: 580
			},
			value: {
				fontSize: 50,
				maxWidth: 460
			}
		}
	};

	var selectors = {
		container: $( '#big-winner-container' ),
		multilineContainer: $( '#big-winner-multiline' ),
		title: $( '#big-winner-title' ),
		value: $( '#big-winner-value' )
	};

	selectors.container.downEvent( function() {} );
	selectors.container.hide();

	var BigWinner = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._stakeTotal = new BigNumber( 0 );
		this._jackpotWins = [];
		this._winTotalToDisplay = new BigNumber( 0 );

		this._bigWinnerValue = new BigNumber( 0 );
		this._bigWinnerLevel = null;

		this._playerJustWonAJackpot = false;
		this._isInBetSequence = false;

		this._preBigWinTimeoutId = null;
		this._bigWinTimeoutId = null;

		this._bigWinnerFountain = null;

		var animationFrames = [];
		for ( var i = 0; i < this._settings.awardLevels.length; ++i ) {
			var animName = this._settings.awardLevels[i].animName;
			if ( animName !== '' ) {
				animationFrames.push( libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, animName ) );
			}
		}
		this._bigWinnerFountain = new libs.client.presentation.BigWinnerFountain( this._settings.fountain, animationFrames );

		this._handleBigWinner = null;

		libs.client.utils.DOM.setText( selectors.title, libs.getTranslation( this._settings.awardLevels[0].translation ), this._settings.text.title, this._settings.gradientText );
		libs.client.utils.DOM.setText( selectors.value, libs.moneyFormat( new BigNumber( 0 ), true ), this._settings.text.value );

		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connect( 'libs.event.notify.currencyChanged', function( currencyCode, formatForCurrency ) { that._onNotifyCurrencyChanged( currencyCode, formatForCurrency ); } );

		libs.globalEventBus.connect( 'libs.event.notify.stakeTotal', function( stakeTotal ) { that._onNotifyStakeTotal( stakeTotal ); } );
		libs.globalEventBus.connect( 'libs.event.notify.jackpotWins', function( jackpotWins ) { that._onNotifyJackpotWins( jackpotWins ); } );
		libs.globalEventBus.connect( 'libs.event.notify.winTotalToDisplay', function( winTotalToDisplay ) { that._onNotifyWinTotalToDisplay( winTotalToDisplay ); } );
		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) { that._onNotifyBetResult( betResult ); } );
		libs.globalEventBus.connect( 'libs.event.notify.keyPressed', function( keyCode, keyChar ) { that._onNotifyKeyPressed( keyCode, keyChar ); } );
		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerDown', function( point ) { that._onNotifyCanvasPointerDown( point ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.bigWinner', function( activity ) { that._onBigWinnerStart( activity ); } );
	};

	BigWinner.prototype._onNotifyLanguageChanged = function( langCode ) {
		if ( this._playerJustWonAJackpot ) {
			libs.client.utils.DOM.setText( selectors.title, libs.getTranslation( 'JACKPOT_WIN' ), this._settings.text.title, this._settings.gradientText );
		} else {
			var translation = libs.getTranslation( this._settings.awardLevels[this._bigWinnerLevel || 0].translation );
			libs.client.utils.DOM.setText( selectors.title, translation, this._settings.text.title, this._settings.gradientText );
		}
	};

	BigWinner.prototype._onNotifyCurrencyChanged = function( currencyCode, formatForCurrency ) {
		this._countUpWinnings( this._bigWinnerValue );
	};

	BigWinner.prototype._onNotifyStakeTotal = function( stakeTotal ) {
		this._stakeTotal = stakeTotal;
	};

	BigWinner.prototype._onNotifyJackpotWins = function( jackpotWins ) {
		this._jackpotWins = jackpotWins;
		this._playerJustWonAJackpot = this._jackpotWins.length > 0;
	};

	BigWinner.prototype._onNotifyWinTotalToDisplay = function( winTotalToDisplay ) {
		this._winTotalToDisplay = winTotalToDisplay;
	};

	BigWinner.prototype._onBigWinnerStart = function( activity ) {
		var that = this;

		// TODO: What if multiple progressive wins? Probably never happen, but....
		this._bigWinnerValue = this._playerJustWonAJackpot ? this._jackpotWins[0]['win'] : this._winTotalToDisplay;
		this._bigWinnerLevel = that._getBigWinLevel( this._bigWinnerValue );

		if ( this._bigWinnerValue.greaterThan( 0 ) ) {
			if ( this._playerJustWonAJackpot || (!this._isInBetSequence && this._bigWinnerLevel !== null) ) {
				this._handleBigWinner = activity.join();
				this._showBigWinner( this._bigWinnerValue, this._playerJustWonAJackpot, this._bigWinnerLevel );
			}
		}
	};

	BigWinner.prototype._onNotifyBetResult = function( betResult ) {
		this._bigWinnerValue = new BigNumber( 0 );
		this._bigWinnerLevel = null;
		this._isInBetSequence = betResult['expectCheckpoint'];
	};

	BigWinner.prototype._onNotifyKeyPressed = function( keyCode, keyChar ) {
		if ( keyChar === ' ' ) {
			this._hideBigWinner();
		}
	};

	BigWinner.prototype._onNotifyCanvasPointerDown = function( point ) {
		this._hideBigWinner();
	};

	BigWinner.prototype._showBigWinner = function( winningsValue, isJackpotWin, bigWinLevel ) {
		var that = this;

		if ( !libs.globalEventBus.isActivityRunning( 'libs.activity.display.modalVisible' ) ) {
			libs.globalEventBus.invokeActivity( 'libs.activity.display.modalVisible' );
		}

		libs.globalEventBus.invoke( 'libs.event.request.performFullRender', 3 );

		// delay a tad before showing, to make sure last win was shown to the player. Also helps native android
		// rendering issue, where symbols would stay stuck on whilst the big win animation is active
		this._preBigWinTimeoutId = libs.setTimeoutTick( function() {
			that._preBigWinTimeoutId = null;

			libs.globalEventBus.invoke( 'libs.event.request.enableCanvasOverlayMode' );

			libs.globalEventBus.invoke( 'libs.event.request.playSound', ['MQ', 'HQ'], 'SndBigWinner' );

			libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
				backdropClass: 'backdrop-bigWinner',
				zIndex: libs.surfaceSettings.top.zIndexMovedToFront - 2,
				animationTime: 200
			} );

			var translation = isJackpotWin ?  libs.getTranslation( 'JACKPOT_WIN' ) : libs.getTranslation( that._settings.awardLevels[bigWinLevel].translation );

			libs.client.utils.DOM.setText( selectors.title, translation, that._settings.text.title, that._settings.gradientText );

			for ( var i = 0, len = that._settings.awardLevels.length; i < len; i++ ) {
				selectors.container.removeClass( that._settings.awardLevels[i].cssClass );
			}

			var messageLines = libs.client.utils.misc.wordSplitter( translation, that._settings.maxLines );
			var length = messageLines.length;
			var separation = 100 / (length + 1);

			for ( var i = 0; i < length; i++ ) {
				var messageLine = messageLines[i];
				var overrideStyles = that._settings.overrideStyles[i] || {};
				var verticalPosition = overrideStyles.verticalPosition ? '' + overrideStyles.verticalPosition : (separation * i) + '%';

				if ( !verticalPosition.match( /%/g ) ) {
					verticalPosition += '%';
				}

				var lineContainer = $( '<div>' ).addClass( 'multititle' ).css( { top: verticalPosition } );

				var lineText = $( '<span>' ).text( messageLine );
				var titleSettings = libs.client.utils.misc.mergeSettings( that._settings.text.title, overrideStyles.font || {} );
				libs.client.utils.DOM.setText( lineText, messageLine, titleSettings, that._settings.gradientText );
				selectors.multilineContainer.append( lineContainer.append( lineText ) );
			}

			selectors.container.addClass( that._settings.awardLevels[bigWinLevel].cssClass );


			selectors.container.css( 'z-index', libs.surfaceSettings.top.zIndexMovedToFront + 1 );
			libs.client.utils.DOM.fadeIn( selectors.container, 750 );

			that._countUpWinnings( winningsValue );

			that._bigWinnerFountain.start( bigWinLevel );

			// set the function to touch big winner graphic to close it in a 1ms timeout otherwise previous touch event
			// to potentially skip free spins will come through and cause the big winner to skip immediately
			libs.setTimeout( function() {
				selectors.container.downEvent( libs.callback( function() { that._hideBigWinner( isJackpotWin ); }, (that) ) );
			}, 1 );

			that._bigWinTimeoutId = libs.setTimeoutTick( function() {
				that._hideBigWinner();
			}, that._settings.displayTime );
		}, 200 );
	};

	BigWinner.prototype._countUpWinnings = function( winningsValue ) {
		var that = this;

		var displayValue = new BigNumber( 0 );
		var totalTime = this._settings.displayTime / 2;
		var loopCount = 0;
		var loops = totalTime / 100;
		var refreshInterval = totalTime / loops;
		var interval = setInterval( updateTimer, refreshInterval );
		var winningsString = libs.moneyFormat( winningsValue, true );
		var incrementAmount = Math.round( winningsValue.div( loops ) );

		function updateTimer() {
			loopCount++;
			displayValue = displayValue.plus( incrementAmount );

			libs.client.utils.DOM.setText( selectors.value, libs.moneyFormat( displayValue, true ), that._settings.text.value );

			if ( loopCount >= loops ) {
				clearInterval( interval );
				libs.client.utils.DOM.setText( selectors.value, winningsString, that._settings.text.value );
			}
		}
	};

	BigWinner.prototype._hideBigWinner = function() {
		if ( this._handleBigWinner && !this._preBigWinTimeoutId ) {
			this._handleBigWinner.leave();
			this._handleBigWinner = null;

			libs.clearTimeoutTick( this._bigWinTimeoutId );
			this._bigWinTimeoutId = null;

			this._bigWinnerFountain.end();

			selectors.container.downEvent( function() {} );
			selectors.multilineContainer.html( '' );

			libs.client.utils.DOM.fadeOut( selectors.container, 0, function() {
				libs.client.utils.DOM.forceDOMRedraw( libs.surfaceTop.jCanvas );
			} );

			libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-bigWinner' );
			libs.globalEventBus.invoke( 'libs.event.request.disableCanvasOverlayMode' );
		}
	};

	BigWinner.prototype._getBigWinLevel = function( winAmount ) {
		var bigWinLevel = null;
		if ( this._jackpotWins.length > 0 ) {
			bigWinLevel = this._settings.awardLevels.length - 1;
		} else {
			if ( winAmount.equals( 0 ) ) {
				return null;
			}
			for ( var i = 0; i < this._settings.awardLevels.length; ++i ) {
				var bigWin = this._settings.awardLevels[i];
				var bigWinThreshold = this._stakeTotal.times( bigWin.multiplier );
				if ( winAmount.greaterThanOrEqualTo( bigWinThreshold ) ) {
					bigWinLevel = i;
				} else {
					break;
				}
			}
		}

		return bigWinLevel;
	};

	BigWinner.prototype.isDirty = function() {
		return !!this._handleBigWinner;
	};

	BigWinner.prototype.draw = function( ctx ) {
		if ( this._handleBigWinner ) {
			this._bigWinnerFountain.draw( ctx );
		}
	};

	BigWinner.prototype.isFeatureActive = function() {
		return !!this._handleBigWinner;
	};


	libs.client.presentation.BigWinner = BigWinner;

}());