goog.provide( 'libs.client.presentation.CanvasButton' );

goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var CanvasButton = function( up, down, disabled ) {
		this.x = 0;
		this.y = 0;

		this.up = up;
		this.down = down;
		this.disabled = disabled;
		this.current = this.up;
		this.x = up.x;
		this.y = up.y;
		this.width = up.width;
		this.height = up.height;

		this._clickCallback = null;

		this.enabled = true;
		this.dirtyFlag = true;
	};

	CanvasButton.prototype.onClicked = function( callback ) {
		this._clickCallback = callback;
	};

	CanvasButton.prototype.isDirty = function() {
		return this.dirtyFlag;
	};

	CanvasButton.prototype.move = function( x, y ) {
		this.x = x;
		this.y = y;
		this.up.x = x;
		this.up.y = y;
		this.down.x = x;
		this.down.y = y;
		this.disabled.x = x;
		this.disabled.y = y;
	};

	CanvasButton.prototype.mousedown = function( point ) {
		if ( this._isMouseEvent( point ) ) {
			this.current = this.down;
			this.dirtyFlag = true;
		}
	};

	CanvasButton.prototype.mouseup = function( point ) {
		if ( this._isMouseEvent( point ) && this._clickCallback && this.current === this.down ) {
			this._clickCallback();
		}

		if ( this.enabled && this.current !== this.up ) {
			this.current = this.up;
			this.dirtyFlag = true;
		}
	};

	CanvasButton.prototype.enable = function() {
		this.enabled = true;
		this.current = this.up;
		this.dirtyFlag = true;
	};

	CanvasButton.prototype.disable = function() {
		this.enabled = false;
		this.current = this.disabled;
		this.dirtyFlag = true;
	};

	CanvasButton.prototype.draw = function( ctx ) {
		this.dirtyFlag = false;
		this.current.draw( ctx );
	};

	CanvasButton.prototype.isDirty = function() {
		return this.dirtyFlag;
	};

	CanvasButton.prototype.drawIfDirty = function( ctx ) {
		if ( this.dirtyFlag ) {
			this.draw( ctx );
		}
	};

	CanvasButton.prototype._isMouseEvent = function( point ) {
		return libs.client.utils.misc.collision( this, point ) && this.enabled;
	};


	libs.client.presentation.CanvasButton = CanvasButton;

}());