goog.provide( 'libs.client.presentation.DeviceCompatibilityDisplay' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.DOM' );
goog.require( 'libs.client.utils.misc' );


(function() {
	"use strict";

	var defaultSettings = {
		links: undefined,
		text: {
			large: {
				fontSize: 30
			},
			small: {
				fontSize: 25
			}
		}
	};

	var selectors = {
		viewport: $( '#viewport' ),
		browserIssueContainer: $( '#browser-issue-container' ),
		browserIssueOldBrowser: $( '#browser-issue-old-browser' ),
		browserIssueInvalidSettings: $( '#browser-issue-invalid-settings' ),
		textGameRequiresModernBrowser: $( '#browser-issue-text-game-requires-modern-browser' ),
		textPleaseUpdateBrowser: $( '#browser-issue-text-please-update-your-browser' ),
		textIncompatibleSettings: $( '#browser-issue-text-incompatible-settings' ),
		textCustomerServices: $( '#browser-issue-text-please-contact-customer-service' )
	};

	var clientInfo = libs.client.core.clientInformation;

	var DeviceCompatibilityDisplay = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._handleDisplayCompatibility = null;
		this._passedDeviceCompatibility = false;

		libs.globalEventBus.connectActivityStart( 'libs.activity.display.compatibility', function( activity ) { that._onDisplayCompatibilityStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.compatibility', function( activity ) { that._onDisplayCompatibilityEnd( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
	};

	DeviceCompatibilityDisplay.prototype._onDisplayCompatibilityStart = function( activity ) {
		var that = this;

		this._handleDisplayCompatibility = activity.join();

		// We R changes - disabled lack of compatiblity message.
		if ( clientInfo.getDeviceType() === 'mobile' ) {
			if ( clientInfo.isDeviceOnBlacklist() ) {
				// libs.globalEventBus.invoke( 'libs.event.request.showError', libs.getTranslation( 'This_game_does_not_currently_support' ), true );
			} else if ( !clientInfo.isDeviceOnWhitelist() ) {
				// var negativeButton = {
				// 	func: function() { libs.client.utils.misc.openUrl( that._settings.links.error ); },
				// 	text: libs.getTranslation( 'EXIT' )
				// };
				// var positiveButton = {
				// 	func: function() {
				// 		that._handleDisplayCompatibility.leave();
				// 		that._handleDisplayCompatibility = null;
				// 	},
				// 	text: libs.getTranslation( 'OK' )
				// };

				// libs.globalEventBus.invoke( 'libs.event.request.showCustomError', libs.getTranslation( 'This_game_may_not_be_fully_optimised' ), negativeButton, positiveButton );
			} else {
				that._handleDisplayCompatibility.leave();
				that._handleDisplayCompatibility = null;
			}
		} else if ( clientInfo.getDeviceType() === 'desktop' ) {
			if ( clientInfo.isIE() && !clientInfo.isIEDocumentModeCompatible() && clientInfo.isIEInCompatabiltyMode() ) {
				// detection for compatability modes where doctype is lower than it should be
				selectors.viewport.hide();
				selectors.browserIssueContainer.show();
				selectors.browserIssueInvalidSettings.show();
				selectors.browserIssueOldBrowser.remove();
			} else if ( clientInfo.isDeviceOnBlacklist() || (clientInfo.isIE() && !clientInfo.isIEDocumentModeCompatible()) ) {
				selectors.viewport.hide();
				selectors.browserIssueContainer.show();
				selectors.browserIssueOldBrowser.show();
				selectors.browserIssueInvalidSettings.remove();
			} else {
				selectors.browserIssueContainer.remove();
				this._handleDisplayCompatibility.leave();
				this._handleDisplayCompatibility = null;
			}
		}
	};

	DeviceCompatibilityDisplay.prototype._onDisplayCompatibilityEnd = function( activity ) {
		this._passedDeviceCompatibility = true;
	};

	DeviceCompatibilityDisplay.prototype._onNotifyLanguageChanged = function( langCode ) {
		if ( !this._passedDeviceCompatibility ) {
			libs.client.utils.DOM.setText( selectors.textGameRequiresModernBrowser, libs.getTranslation( 'This_game_requires_that_you_use_a_modern_browser' ), this._settings.text.large );
			libs.client.utils.DOM.setText( selectors.textPleaseUpdateBrowser, libs.getTranslation( 'Please_update_your_browser_to_either' ), this._settings.text.small );
			libs.client.utils.DOM.setText( selectors.textIncompatibleSettings, libs.getTranslation( 'The_settings_on_your_browser' ), this._settings.text.large );
			libs.client.utils.DOM.setText( selectors.textCustomerServices, libs.getTranslation( 'Please_contact_customer_service_for_assistance' ), this._settings.text.small );
		}
	};


	libs.client.presentation.DeviceCompatibilityDisplay = DeviceCompatibilityDisplay;

}());