goog.provide( 'libs.client.presentation.DigitalClock' );

goog.require( 'libs.globalEventBus' );
// WeR: Required to check if WeR amendments active
goog.require( 'game.settings' );


(function() {
	'use strict';

	var defaultSettings = {
		text: {
			fontSize: 15
		}
	};

	var selectors = {
		main: $( '#main' ),
		container: $( '#digital-clock-container' ),
		clock: $( '#digital-clock' ),
		hour: $( '#digital-clock-hours' ),
		minute: $( '#digital-clock-minutes' )
	};

	selectors.container.hide();

	//instantiate globals
	var time = null;
	var timeOut = null;

	/**
	 * DigitalClock constructor
	 * @construtor DigitalClock
	 */
	var DigitalClock = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		//update time on create, if DOM elements exist
		if ( selectors.hour && selectors.minute ) {
			this._updateTime();

			libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );
			libs.globalEventBus.connect( 'libs.event.notify.fontScaleChanged', function( fontScale ) { that._onNotifyFontScaleChanged( fontScale );} );
		}
	};

	DigitalClock.prototype._onGamePlayingStart = function( activity ) {
        //WeR: Not required for Marketing build
        if (! game.settings.WeR) {
            selectors.container.show();
        }
	};

	DigitalClock.prototype._onNotifyFontScaleChanged = function( height ) {
		libs.setIntervalCount( function(){
			selectors.container.css( 'margin-top', selectors.main.css( 'top' ) );
		}, 50, 10);
	};

	/**
	 * Updates time at set timeouts
	 * @method _updateTime
	 */
	DigitalClock.prototype._updateTime = function() {
		var that = this;

		//get current system time
		time = this._getTimeObj();

		//update DOM
		libs.client.utils.DOM.setText( selectors.hour, time.hour, this._settings.text );
		libs.client.utils.DOM.setText( selectors.minute, time.minute, this._settings.text );

		//update time at the next minute timeout
		timeOut = setTimeout( function() {
			that._updateTime();
		}, time.diffToMin );
	};

	/**
	 * Returns current time object
	 * @method _getTimeObj
	 * @return {Object} object with all the relevant time data
	 */
	DigitalClock.prototype._getTimeObj = function() {
		//get now time
		var cTime = new Date();
		//get hours
		var hour = cTime.getHours();
		//get minutes
		var minute = cTime.getMinutes();
		//get milliseconds till the next minute
		var diffToMin = (60000 - (cTime.getTime() % 60000));

		//return formatted data
		return {
			hour: hour.toString(),
			minute: (minute < 10 ? '0' + minute : minute).toString(),
			diffToMin: diffToMin
		};
	};

	/**
	 * Returns current time object, Public
	 * @method getTime
	 * @return {Object} object with all the relevant time data
	 */
	DigitalClock.prototype.getTime = function() {
		return time;
	};


	libs.client.presentation.DigitalClock = DigitalClock;

})();