goog.provide( 'libs.client.presentation.FpsDisplay' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		text: {
			fontSize: 16
		}
	};

	var selectors = {
		mainContainer: $( '#main' ),
		fpsDisplay: $( '#fps-display' )
	};

	var FpsDisplay = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._time = 33;
		this._lastTime = Date.now();
		this._FPS = 0;
		this._lastFPS = 0;

		this._showFPS = false;

		this._buildHTML();

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
		libs.globalEventBus.connect( 'libs.event.notify.keyPressed', function( keyCode, keyChar ) { that._onKeyPressed( keyCode, keyChar ); } );
	};

	FpsDisplay.prototype._buildHTML = function() {
		if ( BUILD_TYPE === 'dev' ) {
			var that = this;

			var html = '<div id="fps-display">&nbsp;</div>';

			selectors.mainContainer.append( html );
			selectors.fpsDisplay = $( '#fps-display' );
			libs.client.utils.DOM.setText( selectors.fpsDisplay, '0', this._settings.text );
			selectors.fpsDisplay.downEvent( function() {
				that._toggleFPS();
			} );
		}
	};

	FpsDisplay.prototype._onGamePlayingStart = function( activity ) {
		if ( BUILD_TYPE === 'dev' && this._showFPS === false ) {
			this._toggleFPS();
		}
	};

	FpsDisplay.prototype._onNotifyTick = function( timeDelta ) {
		if ( this._showFPS ) {
			var elapsed;
			var now = Date.now();
			elapsed = now - this._lastTime;
			this._time = (this._time * 0.9) + (elapsed * 0.1);
			this._lastTime = now;

			this._FPS = 1000 / this._time;
			this._FPS = this._FPS.toFixed( 0 );

			if ( this._FPS !== this._lastFPS ) {
				this._lastFPS = this._FPS;
				if ( selectors.fpsDisplay.length > 0 ) {
					libs.client.utils.DOM.setText( selectors.fpsDisplay, '' + this._FPS, this._settings.text );
				}
			}
		}
	};

	FpsDisplay.prototype._onKeyPressed = function( keyCode, keyChar ) {
		if ( BUILD_TYPE === 'dev' ) {
			if ( keyChar === 'f' || keyChar === 'F' ) {
				this._toggleFPS();
			}
		}
	};

	FpsDisplay.prototype._toggleFPS = function() {
		this._showFPS = !this._showFPS;
		if ( this._showFPS ) {
			selectors.fpsDisplay.css( 'opacity', '1' );
		} else {
			selectors.fpsDisplay.css( 'opacity', '0' );
		}
	};

	// following functions are deprecated - will be removed when all games have had a merge from template
	FpsDisplay.prototype.draw = function( ctx ) {
	};

	FpsDisplay.prototype.drawIfDirty = function( ctx ) {
	};


	libs.client.presentation.FpsDisplay = FpsDisplay;

}());