goog.provide( 'libs.client.presentation.GameNameFooter' );

goog.require( 'libs.globalEventBus' );


(function() {
	'use strict';

	var defaultSettings = {
		text: {
			fontSize: 12
		}
	};

	var selectors = {
		container: $( '#footer' )
	};

	selectors.container.hide();

	var clientInfo = libs.client.core.clientInformation;

	var GameNameFooter = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._hasLoaded = false;
		this._orientation = null;

		if ( selectors.container.length ) {
			libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );
			libs.globalEventBus.connect( 'libs.event.notify.orientationChanged', function( orientation ) { that._onOrientationChanged( orientation ); } );

			libs.client.utils.DOM.setText( selectors.container, game.logic.GameData.friendlyGameName, this._settings.text );
		}
	};

	GameNameFooter.prototype._onGamePlayingStart = function( activity ) {
		this._hasLoaded = true;
		this._updateVisibility();
	};

	GameNameFooter.prototype._onOrientationChanged = function( orientation ) {
		this._orientation = orientation;
		this._updateVisibility();
	};

	GameNameFooter.prototype._updateVisibility = function() {
		if ( this._hasLoaded ) {
			if ( clientInfo.getDeviceType() === 'mobile' ) {
				if ( this._orientation === 'landscape' ) {
					selectors.container.show();
					return;
				}
			} else {
				selectors.container.show();
				return;
			}
		}

		selectors.container.hide();
	};


	libs.client.presentation.GameNameFooter = GameNameFooter;

})();