goog.provide( 'libs.client.presentation.HtmlButton' );

goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var HtmlButton = function( selector, popupSafe, callbackDown, callbackUp ) {
		var that = this;

		this._enabled = true;
		this._pressed = false;

		this._popupSafe = popupSafe;

		this._selector = selector;
		this._selector.downEvent( function() {that._downEvent();}, this._popupSafe );
		this._selector.upEvent( function() {that._upEvent();} );

		this._enabledClass = 'enabled';
		this._disabledClass = 'disabled';
		this._downClass = 'down';
		this._upClass = 'up';

		this._updateButtonClass();

		this._downEventCallback = callbackDown || null;
		this._upEventCallback = callbackUp || null;

		$( window ).on( 'mouseup touchend pointerup MSPointerUp', function() {
			that._upEvent();
			that._updateButtonClass();
		} );
	};

	HtmlButton.prototype._downEvent = function() {
		if ( this._enabled && !this._pressed ) {
			this._pressed = true;
			this._updateButtonClass();

			if ( this._downEventCallback ) {
				this._downEventCallback.call( this );
			}
		}
	};

	HtmlButton.prototype._upEvent = function() {
		if ( this._pressed ) {
			this._pressed = false;
			this._updateButtonClass();

			$( this ).focus().blur();

			if ( this._upEventCallback ) {
				this._upEventCallback.call( this );
			}
		}
	};

	HtmlButton.prototype._updateButtonClass = function() {
		if ( this._enabled ) {
			this._selector.removeClass( this._disabledClass );
			this._selector.addClass( this._enabledClass );
		} else {
			this._selector.removeClass( this._enabledClass );
			this._selector.addClass( this._disabledClass );
		}

		if ( this._pressed ) {
			this._selector.removeClass( this._upClass );
			this._selector.addClass( this._downClass );
		} else {
			this._selector.removeClass( this._downClass );
			this._selector.addClass( this._upClass );
		}
	};

	HtmlButton.prototype.enable = function() {
		this._enabled = true;
		this._updateButtonClass();
	};

	HtmlButton.prototype.disable = function() {
		this._enabled = false;
		this._updateButtonClass();
	};

	HtmlButton.prototype.show = function( duration ) {
		libs.client.utils.DOM.fadeIn( this._selector, duration );
	};

	HtmlButton.prototype.hide = function( duration ) {
		libs.client.utils.DOM.fadeOut( this._selector, duration );
	};

	HtmlButton.prototype.isEnabled = function() {
		return this._enabled;
	};


	libs.client.presentation.HtmlButton = HtmlButton;

}());