goog.require( 'libs.globalEventBus' );


(function( $, window, undefined ) {
	"use strict";

	// Create and add a hidden backdrop <div> element
	$( '#viewport' ).append( '<div id="theBackdrop">' );
	$( "#theBackdrop" ).css( {
		'backgroundColor': 'rgba(0,0,0,1)',
		'zIndex': '300',
		'width': '200%',
		'height': '200%',
		'display': 'none',
		'position': 'fixed',
		'opacity': 0,
		'top': 0,
		'left': 0
	} );

	/**
	 * Array of options
	 * [0] is the options applied to backdrop when first called
	 * [1] second, and so forth...
	 * Hiding will apply options for prevoius backdrop options in array, if only one then backdrop is completely hidden
	 */
	var optionsArray = [];
	var theBackdrop = $( "#theBackdrop" );


	libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.systemInit', function( activity ) {
		libs.globalEventBus.connect( 'libs.event.request.showBackdrop', function( el, options ) {

			// Default options for backdrop
			var defaults = {
				color: '#000000',
				zIndex: 0,
				backdropClass: 'backdrop',
				animationTime: 300,
				opacity: 0.8,
				closeOnClick: false,
				allowMultiple: false,
				customHTML: '',
				closeButton: null,
				confirmButton: null,
				closeOnConfirm: true,
				el: null,
				preInitFunction: function() {},
				postInitFunction: function() {},
				clickEvent: function() {},
				confirmActions: function() {},
				cancelActions: function() {}
			};


			var newOptions = $.extend( true, {}, defaults, options );
			newOptions.preInitFunction();

			theBackdrop.css( { zIndex: newOptions.zIndex, display: 'block' } );

			theBackdrop
				.velocity( 'stop', true )
				.velocity( { 'opacity': newOptions.opacity } );

			theBackdrop.backdropClass = newOptions.backdropClass;
			theBackdrop.el = el;


			// Store extended options if not already stored
			if ( findIndex( newOptions.backdropClass ) === -1 ) {
				var position = optionsArray.length;
				optionsArray[position] = newOptions;
			}

			// Define the confirm behaviour
			if ( newOptions.confirmButton !== null ) {
				$( newOptions.confirmButton ).downEvent( function() {
					var index = findIndex( theBackdrop.backdropClass );
					confirmation( index );

					// Incase confirmation function removes background, make sure index is still valid
					if ( index < optionsArray.length ) {
						if ( optionsArray[index].closeOnConfirm ) {
							clickEventCallback( index );
							closeBackdrop( index );
						}
					}
				} );
			}

			// Define the close behaviour
			if ( newOptions.closeButton !== null ) {
				$( newOptions.closeButton ).downEvent( function() {
					var index = findIndex( theBackdrop.backdropClass );
					cancel( index );

					// Incase cancel function removes background, make sure index is still valid
					if ( index < optionsArray.length ) {
						clickEventCallback( index );
						closeBackdrop( index );
					}
				} );
			}

			// Define the downEvent behaviour when backdrop <div> element is clicked/pressed
			theBackdrop.downEvent( function( e ) {
				var index = findIndex( theBackdrop.backdropClass );
				clickEventCallback( index );
				if ( optionsArray[index] ) {
					if ( optionsArray[index].closeOnClick ) {
						closeBackdrop( index );
					}
				}
			} );

			newOptions.postInitFunction();

		} ); // End of libs.globalEventBus.connect(...)


		libs.globalEventBus.connect( 'libs.event.request.hideBackdrop', function( el, bdClass ) {
			var index = findIndex( bdClass );
			if ( optionsArray[index] ) {
				closeBackdrop( index );
			}
		} );
	} );


	var findIndex = function( bdClass ) {
		var index = -1;
		for ( var i = 0; i < optionsArray.length; i++ ) {
			if ( optionsArray[i] ) {
				if ( optionsArray[i].backdropClass === bdClass ) {
					index = i;
					break;	// break out of for loop
				}
			}
		}
		return index;
	};


	var closeBackdrop = function( index ) {

		// Change backdrop to match the properties of the previously shown backdrop
		if ( index > 0 ) {
			theBackdrop.css( 'zIndex', optionsArray[index - 1].zIndex );

			theBackdrop
				.velocity( 'stop', true )
				.velocity(
				{ 'opacity': optionsArray[index - 1].opacity },
				{
					'duration': optionsArray[index].animationTime,
					'complete': function() {
						optionsArray.splice( index, 1 );
						theBackdrop.backdropClass = optionsArray[index - 1].backdropClass;
						theBackdrop.el = optionsArray[index - 1].el;
					}
				}
			);

			// Fade out backdrop if only one remains
		} else if ( optionsArray[index] ) {
			var tempOptions = optionsArray[index];
			if ( index !== -1 ) { optionsArray.splice( index, 1 ); }
			libs.client.utils.DOM.fadeOut( theBackdrop, tempOptions.animationTime );
		}
	};


	var confirmation = function( index ) {
		if ( optionsArray[index] ) {
			optionsArray[index].confirmActions.call( theBackdrop, theBackdrop.el, theBackdrop );
		}
	};


	var clickEventCallback = function( index ) {
		if ( optionsArray[index] ) {
			optionsArray[index].clickEvent.call( theBackdrop, theBackdrop.el, theBackdrop );
		}
	};


	var cancel = function( index ) {
		if ( optionsArray[index] ) {
			optionsArray[index].cancelActions.call( theBackdrop, theBackdrop.el, theBackdrop );
		}
	};

	var pluginName = 'insertBackdrop';
	$['insertBackdrop'] = {};
	$['insertBackdrop'].insertBackdrops = [];

	$['insertBackdrop'].unbind = function( $obj ) {
		var iBd = $obj.data( 'plugin_' + pluginName );
		if ( typeof iBd === 'undefined' ) {
			return;
		}
		$obj.removeData( 'plugin_' + pluginName );
		libs.log( 'insertBackdrop unbound' );
	};

}( jQuery, window ));