goog.provide( 'libs.client.presentation.IntroVideo' );
goog.provide( 'libs.mediaPlayer' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		text: {
			fontSize: 24
		}
	};

	var selectors = {
		videoContainer: $( '#intro-video-container' ),
		videoElement: $( '#intro-video-player' ),
		skipText: $( '#intro-video-skip' )
	};

	var IntroVideo = function(overrideSettings) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._language = null;
		this._handleDisplayIntroVideo = null;
		this._finishedWithIntroVideo = false;

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.systemInit', function( activity ) { that._onSystemInitStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.introVideo', function( activity ) { that._onDisplayIntroVideo( activity ); } );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
	};

	IntroVideo.prototype._onSystemInitStart = function( activity ) {
		var that = this;

		var options = {
			'startVolume': 1.0,
			'pluginPath': '/client/thirdParty/mediaelement/build/',
			'features': ['playpause', 'current', 'progress', 'duration', 'tracks', 'volume', 'fullscreen', 'sourcechooser'],
			'enableKeyboard': false,
			'keyActions': [],
			'success': function( media ) {
				var events = ['loadstart', 'play', 'pause', 'ended', 'canplay'];
				for ( var i = 0; i < events.length; ++i ) {
					var eventName = events[i];
					media.addEventListener( eventName, function( e ) {
						if ( e.type === 'ended' ) {
							that._removeIntroVideo();
						}
						if ( e.type === 'loadstart' ) {
						}
					} );
				}
			},
			'error': function() {
				that._removeIntroVideo();
			}
		};

		try {
			libs.mediaPlayer = new MediaElementPlayer( selectors.videoElement, options );
			libs.mediaPlayer.pause();
		} catch ( err ) {
			// no video, so end it
			this._removeIntroVideo();
			return;
		}

		selectors.videoContainer.downEvent( function() {
			that._removeIntroVideo();
		} );

		this._showIntroVideo( false );
	};

	IntroVideo.prototype._onDisplayIntroVideo = function( activity ) {
		if ( this._language === 'en_GB' ) {
			this._handleDisplayIntroVideo = activity.join();
			this._showIntroVideo( true );
		} else {
			this._removeIntroVideo();
		}
	};

	IntroVideo.prototype._showIntroVideo = function( show ) {
		if ( show === undefined || show === true ) {
			try {
				libs.mediaPlayer.play();
			} catch ( err ) {
				// no video, so end it
				this._removeIntroVideo();
				return;
			}

			libs.client.utils.DOM.fadeIn( selectors.videoContainer, 250 );
		} else {
			libs.client.utils.DOM.fadeOut( selectors.videoContainer, 125 );
		}
	};

	IntroVideo.prototype._removeIntroVideo = function() {
		if ( libs.mediaPlayer ) {
			try {
				libs.mediaPlayer.pause();
			} catch ( err ) {
			}
		}

		selectors.videoElement.remove();
		selectors.videoContainer.remove();

		if ( this._handleDisplayIntroVideo ) {
			this._handleDisplayIntroVideo.leave();
			this._handleDisplayIntroVideo = null;
		}

		this._finishedWithIntroVideo = true;
	};

	IntroVideo.prototype._onNotifyLanguageChanged = function( langCode ) {
		if ( !this._finishedWithIntroVideo ) {
			this._language = langCode;

			libs.client.utils.DOM.setText( selectors.skipText, libs.getTranslation( 'SKIP_INTRO' ), this._settings.text );

			if ( langCode !== 'en_GB' ) {
				this._removeIntroVideo();
			}
		}
	};


	libs.client.presentation.IntroVideo = IntroVideo;

}());