goog.provide( 'libs.client.presentation.LoadingScreen' );

goog.require( 'libs.globalEventBus' );


(function() {
	"use strict";

	// We R addition - handle cordova splashscreen using cordova's device ready event
	var isDeviceReady = false;
	var isAppReady = false;

	document.addEventListener('deviceready', function() {
		isDeviceReady = true;
		handleHideCordovaSplashscreen();
	}, false);

	function handleHideCordovaSplashscreen() {
		if (isDeviceReady && isAppReady) {
			navigator.splashscreen.hide();
		}
	}

	var defaultSettings = {
		gameWidth: undefined,
		maxBarWidth: 338
	};

	var selectors = {
		loadingBlackout: $( '#loading-blackout' ),
		containerbg: $( '#background-loading' ),
		containerfg: $( '#loading-container' ),
		progressBar: $( '#loading-progress-front' )
	};

	var LoadingScreen = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.systemInit', function( activity ) { that._onSystemInitStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gameLoading', function( activity ) { that._onGameLoadingStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.gameLoading', function() { that._onGameLoadingEnd(); } );

		libs.globalEventBus.connect( 'libs.event.notify.loadingProgress', function( percentage ) { that._onNotifyLoadingProgress( percentage ); } );
	};

	LoadingScreen.prototype._onSystemInitStart = function( activity ) {
		// make sure screen has re-orientated / re-sized itself before showing the loading screen
		libs.setTimeout( function() {
			$( '#loading-blackout' ).remove();
		}, 1000 );
	};

	LoadingScreen.prototype._onGameLoadingStart = function( activity ) {
		selectors.containerbg.show();
		selectors.containerfg.show();
		isAppReady = true;
		handleHideCordovaSplashscreen();
	};

	LoadingScreen.prototype._onGameLoadingEnd = function() {
		selectors.containerbg.remove();
		selectors.containerfg.remove();
	};

	LoadingScreen.prototype._onNotifyLoadingProgress = function( percentage ) {
		var widthInPixels = this._settings.maxBarWidth * percentage;
		var widthInPerc = widthInPixels / this._settings.gameWidth;

		selectors.progressBar.css( "width", widthInPerc + '%' );
	};


	libs.client.presentation.LoadingScreen = LoadingScreen;

}());