goog.provide( 'libs.client.presentation.MessageBar' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		fadeInTime: 400,
		fadeOutTime: 400,
		text: {
			message: {
				fontSize: 24,
				maxWidth: 540
			}
		}
	};

	var selectors = {
		container: $( '#message-bar-container' ),
		message: $( '#message-bar-text' )
	};

	selectors.container.hide();

	var MessageBar = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		this._isVisible = false;

		var that = this;

		this._timeoutId = null;

		libs.globalEventBus.connect( 'libs.event.request.showMessageBar', function( message, timeTilAutoClose ) { that._onRequestShowMessageBar( message, timeTilAutoClose ); } );
		libs.globalEventBus.connect( 'libs.event.request.hideMessageBar', function() { that._onRequestHideMessageBar(); } );
	};

	MessageBar.prototype._onRequestShowMessageBar = function( message, timeTilAutoClose ) {
		var that = this;

		libs.client.utils.DOM.setText( selectors.message, message, this._settings.text.message );
		if ( !this._isVisible ) {
			libs.globalEventBus.invoke( 'libs.event.notify.messageBarState', true );
			libs.client.utils.DOM.fadeIn( selectors.container, this._settings.fadeInTime, null, true );
			this._isVisible = true;

			libs.clearTimeoutTick( this._timeoutId );
			if ( timeTilAutoClose ) {
				this._timeoutId = libs.setTimeoutTick( function() {
					that._timeoutId = null;
					that._hideMessageBar();
				}, timeTilAutoClose );
			}
		}
	};

	MessageBar.prototype._onRequestHideMessageBar = function() {
		libs.clearTimeoutTick( this._timeoutId );
		this._hideMessageBar();
	};

	MessageBar.prototype._hideMessageBar = function() {
		if ( this._isVisible ) {
			libs.globalEventBus.invoke( 'libs.event.notify.messageBarState', false );
			libs.client.utils.DOM.fadeOut( selectors.container, this._settings.fadeOutTime, null, true );
			this._isVisible = false;
		}
	};


	libs.client.presentation.MessageBar = MessageBar;

}());