goog.provide( 'libs.client.presentation.BalanceMeter' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.presentation.MeterBaseClass' );


(function() {
	"use strict";

	var defaultSettings = {
		name: 'BalanceMeter',
		alwaysIncludeSubunit: true,
		canBeNegativeValue: false,
		fadeInTime: 0,
		fadeOutTime: 0,
		hideTextOnZeroValue: false,
		updateValueEvent: 'libs.event.notify.balance',
		model: libs.client.reels.meterModels.balance,
		translations: {
			title: 'BALANCE'
		},
		text: {
			title: {
				fontSize: 15,
				maxWidth: 140
			},
			value: {
				fontSize: 25,
				maxWidth: 125
			}
		}
	};

	var hillsVegasDesktopSettings = {
		text: {
			title: {
				fontSize: 17,
				maxWidth: 100
			},
			value: {
				fontSize: 19,
				maxWidth: 185
			}
		}
	};

	var selectors = {
		container: $( '.meter-balance-container' ),
		title: $( '.meter-balance-title' ),
		value: $( '.meter-balance-value' )
	};

	selectors.container.hide();

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	var isHillsVegas = PLATFORM_TYPE === 'hillsLocal' || PLATFORM_TYPE === 'hillsLive';
	if ( isDesktopDeviceType && isHillsVegas ){
		defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, hillsVegasDesktopSettings );
	}

	var BalanceMeter = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		libs.client.presentation.MeterBaseClass.call( this, this._settings, selectors );
	};

	BalanceMeter.prototype = Object.create( libs.client.presentation.MeterBaseClass.prototype );
	BalanceMeter.prototype.constructor = BalanceMeter;


	libs.client.presentation.BalanceMeter = BalanceMeter;

}());