goog.provide( 'libs.client.presentation.SoundSelection' );

goog.requireThirdParty( 'screenfull' );
goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.DOM' );
// WeR: Required to check if WeR amendments active
goog.require( 'game.settings' );


(function() {
	"use strict";

	var defaultSettings = {
		text: {
			title: {
				fontSize: 32,
				maxWidth: 320
			},
			description: {
				fontSize: 18,
				maxWidth: 320,
				maxHeight: 60
			},
			buttons: {
				fontSize: 18,
				maxWidth: 135
			}
		}
	};

	var selectors = {
		body: document.getElementById( 'body' ),
		canvasBase: $( '#canvas-base' ),
		backgroundSounds: $( '#background-sounds' ),
		selectionContainer: $( '#sound-selection-container' ),
		messageLine1: $( '#sound-selection-line1' ),
		messageLine2: $( '#sound-selection-line2' ),
		yesButton: $( '#sound-selection-button-yes' ),
		noButton: $( '#sound-selection-button-no' ),
		yesButtonTitle: $( '#sound-selection-button-yes-title' ),
		noButtonTitle: $( '#sound-selection-button-no-title' )
	};

	var clientInfo = libs.client.core.clientInformation;

	selectors.backgroundSounds.css( 'opacity', 0 );
	selectors.selectionContainer.hide();

	var SoundSelection = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._handleDisplaySoundSelection = null;
		this._finishedWithSoundSelection = false;

		libs.globalEventBus.connectActivityStart( 'libs.activity.display.soundSelection', function( activity ) {
			that._onDisplaySoundSelectionStart( activity );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) {
			that._onNotifyLanguageChanged( langCode );
		} );
	};

	SoundSelection.prototype._onDisplaySoundSelectionStart = function( activity ) {
        // WeR: Turn off sound selection
        if (! game.settings.WeR) {
            (function() {
                var that = this;
             
                this._handleDisplaySoundSelection = activity.join();
             
                selectors.yesButton.downEvent( function() {
                    that._optionSelected( true );
                }, false, true );
                selectors.noButton.downEvent( function() {
                    that._optionSelected( false );
                }, false, true );
             
                this._showSoundSelection( true );
                selectors.canvasBase.hide();
            })();
        }
	};

	SoundSelection.prototype._onNotifyLanguageChanged = function( langCode ) {
		if ( !this._finishedWithSoundSelection ) {
			libs.client.utils.DOM.setText( selectors.yesButtonTitle, libs.getTranslation( 'YES' ), this._settings.text.buttons );
			libs.client.utils.DOM.setText( selectors.noButtonTitle, libs.getTranslation( 'NO' ), this._settings.text.buttons );
			libs.client.utils.DOM.setText( selectors.messageLine1, libs.getTranslation( 'ENABLE_SOUNDS' ), this._settings.text.title );
			libs.client.utils.DOM.setText( selectors.messageLine2, libs.getTranslation( 'PLAYING_WITH_SOUND_MAY_AFFECT_YOUR_PERFORMANCE' ), this._settings.text.description );
		}
	};

	SoundSelection.prototype._optionSelected = function( enableSounds ) {
		if ( clientInfo.supportsFullscreenAPI() ) {
			screenfull.request( selectors.body );
		} else {
			libs.log( "Fullscreen API is not supported by this browser" );
		}

		if ( enableSounds ) {
			libs.globalEventBus.invoke( 'libs.event.request.enableSounds' );
		} else {
			libs.globalEventBus.invoke( 'libs.event.request.disableSounds' );
		}

		selectors.canvasBase.show();

		this._removeSoundSelection();
		this._handleDisplaySoundSelection.leave();
		this._handleDisplaySoundSelection = null;
	};

	SoundSelection.prototype._showSoundSelection = function( show ) {
		if ( show === undefined || show === true ) {
			libs.client.utils.DOM.showBackground( selectors.backgroundSounds, 0 );
			libs.client.utils.DOM.fadeIn( selectors.selectionContainer, 125 );
		} else {
			libs.client.utils.DOM.hideBackground( selectors.backgroundSounds, 125 );
			libs.client.utils.DOM.fadeOut( selectors.selectionContainer, 125 );
		}
	};

	SoundSelection.prototype._removeSoundSelection = function() {
		selectors.selectionContainer.remove();
		selectors.backgroundSounds.remove();
		this._finishedWithSoundSelection = true;
	};


	libs.client.presentation.SoundSelection = SoundSelection;

}());