goog.provide( 'libs.client.reels.BetStateController' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.reels.BetStateIdle' );
goog.require( 'libs.client.reels.BetStatePreBet' );
goog.require( 'libs.client.reels.BetStatePlayingBet' );
goog.require( 'libs.client.reels.BetStatePostBet' );


(function() {
	"use strict";

	var defaultSettings = {
		idle: {},
		preBet: {},
		playingBet: {},
		postBet: {}
	};

	var BetStateController = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._handleGamePlaying = null;

		this._betStateIdle = new libs.client.reels.BetStateIdle( this._settings.idle );
		this._betStatePreBet = new libs.client.reels.BetStatePreBet( this._settings.preBet );
		this._betStatePlayingBet = new libs.client.reels.BetStatePlayingBet( this._settings.playingBet );
		this._betStatePostBet = new libs.client.reels.BetStatePostBet( this._settings.postBet );

		this._betFailed = false;

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.idle', function( activity ) { that._onIdleStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.betState.idle', function() { that._onIdleEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.betState.preBet', function() { that._onPreBetEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.betState.playingBet', function() { that._onPlayingBetEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.betState.postBet', function() { that._onPostBetEnd(); } );

		libs.globalEventBus.connect( 'libs.event.notify.serverBetAborted', function() { that._onNotifyServerBetAborted(); } );
	};

	BetStateController.prototype._onGamePlayingStart = function( activity ) {
		this._handleGamePlaying = activity.join();

		// use settimeout rather than setimmediate to make 100% sure that gameplayingstart finishes before idle starts
		libs.setTimeout( function() {
			libs.globalEventBus.invokeActivity( 'libs.activity.betState.idle' );
		}, 1 );
	};

	BetStateController.prototype._onIdleStart = function( activity ) {
		this._betFailed = false;
	};

	BetStateController.prototype._onIdleEnd = function() {
		this._invokeActivity( 'libs.activity.betState.preBet' );
	};

	BetStateController.prototype._onPreBetEnd = function() {
		if ( this._betFailed ) {
			this._invokeActivity( 'libs.activity.betState.idle' );
		} else {
			this._invokeActivity( 'libs.activity.betState.playingBet' );
		}
	};

	BetStateController.prototype._onPlayingBetEnd = function() {
		this._invokeActivity( 'libs.activity.betState.postBet' );
	};

	BetStateController.prototype._onPostBetEnd = function() {
		this._invokeActivity( 'libs.activity.betState.idle' );
	};

	BetStateController.prototype._onNotifyServerBetAborted = function() {
		this._betFailed = true;
	};

	BetStateController.prototype._invokeActivity = function( activityName ) {
		// makes sure all calls on an activity end are processed before a new activity starts
		libs.setImmediate( function() {
			libs.globalEventBus.invokeActivity( activityName );
		} );
	};


	libs.client.reels.BetStateController = BetStateController;

}());