goog.provide( 'libs.client.reels.AutoplayButton' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.presentation.ButtonBaseClass' );


(function() {
	"use strict";

	var defaultSettings = {
		name: 'AutoplayButton',
		fadeInTime: 200,
		fadeOutTime: 200,
		keyMapping: {
			keyCode: null,
			keyChar: null
		},
		model: libs.client.reels.buttonModels.autoplay,
		popupSafe: false,
		text: {
			title: {
				fontSize: 19,
				maxWidth: 121,
				multiline: {
					numWordsToGroup: 0,
					reduceSpaceBetweenLinesBy: 10,
					marginTop: 10
				}
			}

		},
		translations: {
			title: 'AUTOPLAY',
			activeMultiple: 'SPINS',
			activeSingle: 'SPIN',
			stop: 'STOP'
		}
	};

	var defaultSettingsMobile = {
	};

	var defaultSettingsDesktop = {
		text: {
			title: {
				maxWidth: 110
			}
		}
	};

	var selectors = {
		container: $( '#button-autoplay-container' )
	};

	selectors.container.hide();

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );

	var AutoplayButton = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		libs.client.presentation.ButtonBaseClass.call( this, this._settings, selectors );

		var that = this;

		this._numberOfAutoplays = 0;

		libs.globalEventBus.connect( 'libs.event.notify.autoplaysRemaining', function( numberOfAutoplays ) { that._onNotifyAutoplaysRemaining( numberOfAutoplays ); } );

		this._updateButtonColour();
	};

	AutoplayButton.prototype = Object.create( libs.client.presentation.ButtonBaseClass.prototype );
	AutoplayButton.prototype.constructor = AutoplayButton;

	AutoplayButton.prototype._onNotifyLanguageChanged = function( langCode ) {
		this._updateButtonTitle();
	};

	AutoplayButton.prototype._onNotifyAutoplaysRemaining = function( numberOfAutoplays ) {
		this._numberOfAutoplays = numberOfAutoplays;
		this._updateButtonColour();
		this._updateButtonTitle();
	};

	AutoplayButton.prototype._updateButtonColour = function() {
		if ( this._numberOfAutoplays === 0 ) {
			selectors.container.removeClass( 'red' );
			selectors.container.addClass( 'blue' );
		} else {
			selectors.container.removeClass( 'blue' );
			selectors.container.addClass( 'red' );
		}
	};

	AutoplayButton.prototype._updateButtonTitle = function() {
		var textLine1 = '';
		var textLine2 = '';

		if ( this._numberOfAutoplays === 0 ) {
			textLine1 = libs.getTranslation( this._settings.translations.title );
		} else if ( this._numberOfAutoplays === -1 ) { // -1 means until bonus
			textLine1 = libs.getTranslation( this._settings.translations.stop );
		} else if ( this._numberOfAutoplays === 1 ) {
			textLine1 = String( this._numberOfAutoplays ) + " " + libs.getTranslation( this._settings.translations.activeSingle );
			textLine2 = libs.getTranslation( this._settings.translations.stop );
		} else {
			textLine1 = String( this._numberOfAutoplays ) + " " + libs.getTranslation( this._settings.translations.activeMultiple );
			textLine2 = libs.getTranslation( this._settings.translations.stop );
		}

		if ( textLine2 === '' ) {
			this._settings.text.title.multiline.numWordsToGroup = 0;
			libs.client.utils.DOM.setMultiLineText( selectors.container, textLine1, this._settings.text.title );
		} else {
			this._settings.text.title.multiline.numWordsToGroup = 1;
			libs.client.utils.DOM.setMultiLineText( selectors.container, textLine1 + ' ' + textLine2, this._settings.text.title );
		}
	};


	libs.client.reels.AutoplayButton = AutoplayButton;

}());

