goog.provide( 'libs.client.reels.buttonModels' );

goog.require( 'libs.globalEventBus' );


(function() {
	"use strict";

	// extra data for spin button
	var inQuickPlay = false;
	var skippedWin = false;
	var winThisSpin = false;
	libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.reelsLanding', function() { inQuickPlay = false; } );
	libs.globalEventBus.connect( 'libs.event.request.skipWinAnimations', function( enabled ) { skippedWin = true; } );
	libs.globalEventBus.connect( 'libs.event.notify.quickPlayState', function( enabled ) {
		inQuickPlay = enabled;
		skippedWin = false;
	} );
	libs.globalEventBus.connect( 'libs.event.notify.totalWin', function( totalWin ) {
		winThisSpin = false;
		if ( typeof totalWin === 'object' ) {
			if ( totalWin instanceof BigNumber ) {
				winThisSpin = !totalWin.isZero();
			}
		} else if ( typeof totalWin === 'number' ) {
			winThisSpin = totalWin > 0;
		}
	} );

	// extra date for stakeup / stakedown buttons
	var availableStakesPerLine = [];
	var currentStakePerLine = null;
	var nextStakePerLine = null;
	var prevStakePerLine = null;

	var updatePrevAndNextStakes = function() {
		if ( availableStakesPerLine.length > 0 && currentStakePerLine ) {
			nextStakePerLine = null;
			prevStakePerLine = null;

			var index = availableStakesPerLine.indexOf( currentStakePerLine );

			var prevIndex = index - 1;
			if ( prevIndex >= 0 ) {
				prevStakePerLine = availableStakesPerLine[prevIndex];
			}

			var nextIndex = index + 1;
			if ( nextIndex < availableStakesPerLine.length ) {
				nextStakePerLine = availableStakesPerLine[nextIndex];
			}
		}
	};

	libs.globalEventBus.connect( 'libs.event.notify.availableStakesPerLine', function( availableStakes ) {
		availableStakesPerLine = availableStakes;
		updatePrevAndNextStakes();
	} );
	libs.globalEventBus.connect( 'libs.event.notify.stakePerLine', function( stakePerLine ) {
		currentStakePerLine = stakePerLine;
		updatePrevAndNextStakes();
	} );

	libs.client.reels.buttonModels = {
		baseModel: {
			shouldBeEnabled: function() {
				if ( !libs.globalEventBus.isActivityRunning( 'libs.activity.systemState.gamePlaying' ) || !libs.globalEventBus.isActivityRunning( 'libs.activity.system.tickManagerActive' )
				) {
					return false;
				}

				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.display.error' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.display.popupMessage' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.display.history' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.display.menu' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.display.paytable' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.display.stakeSelector' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.display.autoplaySelector' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.display.bigWinner' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.display.xOfAKindSplash' )
				) {
					return false;
				}

				return true;
			},
			shouldBeVisible: function() {
				if ( !libs.globalEventBus.isActivityRunning( 'libs.activity.systemState.gamePlaying' ) ) {
					return false;
				}

				return true;
			},
			updateEvents: {
				activityStartAndEnd: [
					'libs.activity.systemState.gamePlaying',
					'libs.activity.system.tickManagerActive',
					'libs.activity.display.error',
					'libs.activity.display.popupMessage',
					'libs.activity.display.history',
					'libs.activity.display.menu',
					'libs.activity.display.paytable',
					'libs.activity.display.stakeSelector',
					'libs.activity.display.autoplaySelector',
					'libs.activity.display.bigWinner',
					'libs.activity.display.xOfAKindSplash'
				],
				activityStart: [],
				activityEnd: [],
				event: []
			}
		},

		autoplay: {
			updateEvents: {
				activityStartAndEnd: [
					'libs.activity.betState.idle',
					'libs.activity.display.freespinsUI',
					'libs.activity.gameMode.autoplay',
					'libs.activity.gameMode.betSequence'
				],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			shouldBeEnabled: function() {
				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.betSequence' ) && !libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' )
				) {
					return false;
				}

				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.betState.idle' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ) {
					return true;
				}

				return false;
			},
			shouldBeVisible: function() {
				return !libs.globalEventBus.isActivityRunning( 'libs.activity.display.freespinsUI' );
			},
			downEvent: function() {
				libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );

				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ) {
					libs.globalEventBus.invoke( 'libs.event.request.stopAutoplay' );
				} else {
					libs.globalEventBus.invoke( 'libs.event.request.disableZIndexHack' );
					libs.globalEventBus.invoke( 'libs.event.request.autoplaySelector' );
				}
			},
			upEvent: null
		},

		menu: {
			updateEvents: {
				activityStartAndEnd: [
					'libs.activity.betState.idle',
					'libs.activity.display.freespinsUI',
					'libs.activity.gameMode.autoplay',
					'libs.activity.gameMode.betSequence'
				],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			shouldBeEnabled: function() {
				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.betSequence' )
				) {
					return false;
				}

				return libs.globalEventBus.isActivityRunning( 'libs.activity.betState.idle' );
			},
			shouldBeVisible: function() {
				return !libs.globalEventBus.isActivityRunning( 'libs.activity.display.freespinsUI' );
			},
			downEvent: function() {
				libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
				libs.globalEventBus.invoke( 'libs.event.request.openMenu' );
			},
			upEvent: null
		},

		paytable: {
			updateEvents: {
				activityStartAndEnd: [
					'libs.activity.betState.idle',
					'libs.activity.display.freespinsUI',
					'libs.activity.gameMode.autoplay',
					'libs.activity.gameMode.betSequence'
				],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			shouldBeEnabled: function() {
				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.betSequence' )
				) {
					return false;
				}

				return libs.globalEventBus.isActivityRunning( 'libs.activity.betState.idle' );
			},
			shouldBeVisible: function() {
				return !libs.globalEventBus.isActivityRunning( 'libs.activity.display.freespinsUI' );
			},
			downEvent: function() {
				libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
				libs.globalEventBus.invoke( 'libs.event.request.openPaytable' );
			},
			upEvent: null
		},

		spin: {
			updateEvents: {
				activityStartAndEnd: [
					'libs.activity.gameMode.autoplay',
					'libs.activity.gameMode.betSequence'
				],
				activityStart: [
					'libs.activity.betState.idle',
					'libs.activity.betState.preBet',
					'libs.activity.betState.playingBet',
					'libs.activity.betState.postBet',
					'libs.activity.reelsState.preReelsLanding',
					'libs.activity.reelsState.reelsLanding',
					'libs.activity.reelsState.postReelsLanding',
					'libs.activity.reelsState.displayReelsWin',
					'libs.activity.reelsState.reelsBonusFeature',
					'libs.activity.reelsState.endOfReelsGameModals'
				],
				activityEnd: [],
				event: [
					'libs.event.notify.quickPlayState',
					'libs.event.request.skipWinAnimations'
				]
			},
			shouldBeEnabled: function() {
				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.betState.idle' ) && !libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) && !libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.betSequence' )
				) {
					return true;
				}

				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.betState.preBet' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.betState.postBet' )
				) {
					return false;
				}

				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.betState.playingBet' ) ) {
					if ( libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.preReelsLanding' ) ||
						libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.reelsLanding' ) ) {
						if ( inQuickPlay || skippedWin ) {
							return false;
						} else {
							return true;
						}
					}

					if ( libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.postReelsLanding' ) ||
						libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.reelsBonusFeature' ) ||
						libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.endOfReelsGameModals' )
					) {
						return false;
					}

					if ( libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.displayReelsWin' ) ) {
						return winThisSpin && !skippedWin;
					}
				}

				return false;
			},
			shouldBeVisible: function() {
				return true;
			},
			downEvent: function() {
				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.betState.idle' ) && !libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) && !libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.betSequence' )
				) {
					libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
					libs.globalEventBus.invoke( 'libs.event.request.disableZIndexHack' );
					libs.globalEventBus.invoke( 'libs.event.request.startBet' );
				} else if ( libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.preReelsLanding' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.reelsLanding' ) ) {
					libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
					libs.globalEventBus.invoke( 'libs.event.notify.quickPlayState', true );
				} else if ( libs.globalEventBus.isActivityRunning( 'libs.activity.reelsState.displayReelsWin' ) ) {
					libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
					libs.globalEventBus.invoke( 'libs.event.request.skipWinAnimations' );
				}
			},
			upEvent: null
		},

		stake: {
			updateEvents: {
				activityStartAndEnd: [
					'libs.activity.betState.idle',
					'libs.activity.display.freespinsUI',
					'libs.activity.gameMode.autoplay',
					'libs.activity.gameMode.betSequence'
				],
				activityStart: [],
				activityEnd: [],
				event: []
			},
			shouldBeEnabled: function() {
				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.betSequence' )
				) {
					return false;
				}

				return libs.globalEventBus.isActivityRunning( 'libs.activity.betState.idle' );
			},
			shouldBeVisible: function() {
				return !libs.globalEventBus.isActivityRunning( 'libs.activity.display.freespinsUI' );
			},
			downEvent: function() {
				libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
				libs.globalEventBus.invoke( 'libs.event.request.stakeSelector' );
				libs.globalEventBus.invoke( 'libs.event.request.disableZIndexHack' );
			},
			upEvent: null
		},

		stakeDown: {
			updateEvents: {
				activityStartAndEnd: [
					'libs.activity.betState.idle',
					'libs.activity.display.freespinsUI',
					'libs.activity.gameMode.autoplay',
					'libs.activity.gameMode.betSequence'
				],
				activityStart: [],
				activityEnd: [],
				event: [
					'libs.event.notify.availableStakesPerLine',
					'libs.event.notify.stakePerLine'
				]
			},
			shouldBeEnabled: function() {
				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.betSequence' )
				) {
					return false;
				}

				if ( !prevStakePerLine ) {
					return false;
				}

				return libs.globalEventBus.isActivityRunning( 'libs.activity.betState.idle' );
			},
			shouldBeVisible: function() {
				return !libs.globalEventBus.isActivityRunning( 'libs.activity.display.freespinsUI' );
			},
			downEvent: function() {
				if ( prevStakePerLine ) {
					libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
					libs.globalEventBus.invoke( 'libs.event.request.changeStakePerLine', prevStakePerLine );
				}
			},
			upEvent: null
		},

		stakeUp: {
			updateEvents: {
				activityStartAndEnd: [
					'libs.activity.betState.idle',
					'libs.activity.display.freespinsUI',
					'libs.activity.gameMode.autoplay',
					'libs.activity.gameMode.betSequence'
				],
				activityStart: [],
				activityEnd: [],
				event: [
					'libs.event.notify.availableStakesPerLine',
					'libs.event.notify.stakePerLine'
				]
			},
			shouldBeEnabled: function() {
				if ( libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ||
					libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.betSequence' )
				) {
					return false;
				}

				if ( !nextStakePerLine ) {
					return false;
				}

				return libs.globalEventBus.isActivityRunning( 'libs.activity.betState.idle' );
			},
			shouldBeVisible: function() {
				return !libs.globalEventBus.isActivityRunning( 'libs.activity.display.freespinsUI' );
			},
			downEvent: function() {
				if ( nextStakePerLine ) {
					libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 1 );
					libs.globalEventBus.invoke( 'libs.event.request.changeStakePerLine', nextStakePerLine );
				}
			},
			upEvent: null
		}
	};


}());