goog.provide( 'libs.client.reels.SpinButton' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.presentation.ButtonBaseClass' );


(function() {
	"use strict";

	var defaultSettings = {
		name: 'SpinButton',
		fadeInTime: 200,
		fadeOutTime: 200,
		keyMapping: {
			keyCode: null,
			keyChar: ' '
		},
		model: libs.client.reels.buttonModels.spin,
		popupSafe: false,
		translations: {
			title: 'SPIN'
		}
	};

	var defaultSettingsMobile = {
		text: {
			title: {
				fontSize: 42,
				maxWidth: 121,
				marginTop: 30
			}
		}
	};

	if ( IS_FORTUNE_SPINS_LAYOUT ) {
		defaultSettingsMobile = {
			text: {
				title: {
					fontSize: 38,
					maxWidth: 130,
					marginTop: 0
				}
			}
		};
	}

	var defaultSettingsDesktop = {
		text: {
			title: {
				fontSize: 30,
				maxWidth: 79,
				lineClass: 'multiline-margin-left'
			}
		}
	};


	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );

	var selectors = {
		container: $( '#button-spin-container' )
	};

	selectors.container.hide();

	var SpinButton = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		libs.client.presentation.ButtonBaseClass.call( this, this._settings, selectors );

		var that = this;

		this._reelDraggerEnabled = true;

		libs.globalEventBus.connect( 'libs.event.request.enableReelDragger', function() { that._onRequestEnableReelDragger(); } );
		libs.globalEventBus.connect( 'libs.event.request.disableReelDragger', function() { that._onRequestDisableReelDragger(); } );

		libs.globalEventBus.connect( 'libs.event.notify.inputGesture', function( gesture ) { that._onNotifyInputGesture( gesture ); } );
	};

	SpinButton.prototype = Object.create( libs.client.presentation.ButtonBaseClass.prototype );
	SpinButton.prototype.constructor = SpinButton;

	SpinButton.prototype._onRequestEnableReelDragger = function() {
		this._reelDraggerEnabled = true;
	};

	SpinButton.prototype._onRequestDisableReelDragger = function() {
		this._reelDraggerEnabled = false;
	};

	SpinButton.prototype._onNotifyInputGesture = function( gesture ) {
		if ( !isDesktopDeviceType && this._reelDraggerEnabled ) {
			if ( gesture === 'swipedown' || gesture === 'swipeup' ) {
				if ( this._button.isEnabled() && libs.globalEventBus.isActivityRunning( 'libs.activity.betState.idle' ) ) {
					libs.globalEventBus.invoke( 'libs.event.request.startBet', { noPullBack: true } ); //noPullBack, looks odd when swiping
				}
			}
		}
	};

	libs.client.reels.SpinButton = SpinButton;

}());