goog.provide( 'libs.client.reels.StakeButton' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.presentation.ButtonBaseClass' );


(function() {
	"use strict";

	var defaultSettings = {
		name: 'StakeButton',
		fadeInTime: 200,
		fadeOutTime: 200,
		keyMapping: {
			keyCode: null,
			keyChar: null
		},
		model: libs.client.reels.buttonModels.stake,
		popupSafe: false,
		isMultiLine: false,
		text: {
			title: {
				fontSize: 15,
				maxWidth: 95
			},
			value: {
				fontSize: 22,
				maxWidth: 87
			}
		},
		translations: {
			title: 'TOTAL_BET'
		}
	};

	var selectors = {
		container: $( '#button-stake-container' ),
		title: $( '#button-stake-title' ),
		value: $( '#button-stake-value' )
	};

	selectors.container.hide();

	var StakeButton = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
		libs.client.presentation.ButtonBaseClass.call( this, this._settings, selectors );

		var that = this;

		this._stakeTotal = new BigNumber( 0 );

		libs.globalEventBus.connect( 'libs.event.notify.stakeTotal', function( stakeTotal ) { that._onNotifyStakeTotal( stakeTotal ); } );
		libs.globalEventBus.connect( 'libs.event.notify.currencyChanged', function( currencyCode, formatForCurrency ) { that._onNotifyCurrencyChanged( currencyCode, formatForCurrency ); } );
	};

	StakeButton.prototype = Object.create( libs.client.presentation.ButtonBaseClass.prototype );
	StakeButton.prototype.constructor = StakeButton;

	StakeButton.prototype._onNotifyStakeTotal = function( stakeTotal ) {
		this._stakeTotal = stakeTotal;
		this._updateDOM();
	};

	StakeButton.prototype._onNotifyLanguageChanged = function( langCode ) {
		if ( this._settings.translations.title ) {
			var translation = libs.getTranslation( this._settings.translations.title );
			var textSettings = this._settings.text.title;

			if ( selectors.title ) {
				libs.client.utils.DOM.setText( selectors.title, translation, textSettings );
			}
		}
	};

	StakeButton.prototype._onNotifyCurrencyChanged = function( currencyCode, formatForCurrency ) {
		this._updateDOM();
	};

	StakeButton.prototype._updateDOM = function() {
		var stakeStr = libs.moneyFormat( this._stakeTotal, false );
		libs.client.utils.DOM.setText( selectors.value, stakeStr, this._settings.text.value );
	};


	libs.client.reels.StakeButton = StakeButton;

}());