goog.provide( 'libs.client.reels.MainSlotsBaseClass' );

goog.require( 'libs.moneyFormat' );
goog.require( 'libs.getTranslation' );

goog.require( 'libs.client.utils.misc' );

goog.require( 'libs.client.presentation.BigWinner' );
goog.require( 'libs.client.presentation.FpsDisplay' );
goog.require( 'libs.client.presentation.MessageBar' );
goog.require( 'libs.client.presentation.DigitalClock' );
goog.require( 'libs.client.presentation.GameNameFooter' );

goog.require( 'libs.client.presentation.BalanceMeter' );
goog.require( 'libs.client.presentation.TotalWinMeter' );

goog.require( 'libs.client.reels.ReelView' );
goog.require( 'libs.client.reels.ReelViewGroup' );
goog.require( 'libs.client.reels.SlotsGameModel' );
goog.require( 'libs.client.reels.WinlinesModel' );
goog.require( 'libs.client.reels.WinlinesView' );

goog.require( 'libs.client.reels.AutoplaySelector' );
goog.require( 'libs.client.reels.Paytable' );
goog.require( 'libs.client.reels.ReelsAnticipation' );
goog.require( 'libs.client.reels.XOfAKindSplash' );

goog.require( 'libs.client.reels.AutoplayButton' );
goog.require( 'libs.client.reels.SpinButton' );

goog.require( 'game.logic' );

// mobile
//goog.require('libs.client.reels.MenuButton');
//goog.require('libs.client.reels.StakeButton');
//goog.require('libs.client.hillsVegas.MobileMenu');
//goog.require('libs.client.reels.ScrollableStakeSelector');

// desktop
//goog.require('libs.client.reels.StakeDownButton');
//goog.require('libs.client.reels.StakeUpButton');
//goog.require('libs.client.reels.PaytableButton');
//goog.require('libs.client.presentation.TotalStakeMeter');
//goog.require('libs.client.hillsVegas.DesktopMenu');
//goog.require('libs.client.hillsVegas.HistoryDisplay');


(function() {
	"use strict";

	var defaultSettings = {
		symbolsData: undefined,
		imposeReelSymbolSets: null,
		anticipationSymbols: null,
		stopAutoplayOnSymbolsWin: null,
		stopAutoplayOnFreespins: true,
		specialLandingSounds: null,
		libs: {
			presentation: {
				showGameWinInMessageBox: true,
				allowMessageBoxInFreespins: false
			}
		},
		sounds: {
			reelSpin: 'SndReelSpin1',
			countUpWin: 'SndCountUpWin',
			reelStop: 'SndReelStop'
		},
		gameDelayFreespins: 1000,
		gameDelayAutoplayNormal: 100,
		gameDelayAutoplayAndroid: 350
	};


	var spinParameterDefaults = {

		maxSpeed: 1.6,
		bounceBackTime: 250,
		bounceDistance: 20,
		pullbackTime: 0,
		pullbackSpeed: 0.3,
		acceleration: 0.004,
		heavyDropSpeed: 0.4,
		reelSpinTimeGap: 0,
		reelBlurWaitTime: 1000,
		reelLandingGap: 150
	};

	// merging arrays manually, as having problems using jquery.extend to deep copy them - overrwrites arrays rather than merges
	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';

	var selectors = {
		backgroundNormal: $( '#background-normal' )
	};

	libs.client.utils.DOM.hideBackground( selectors.backgroundNormal, 1 );

	var clientInfo = libs.client.core.clientInformation;
	var isMobileDeviceType = clientInfo.getDeviceType() === 'mobile';
	var isNativeAndroid = clientInfo.isNativeAndroidBrowser();

	if ( PLATFORM_TYPE === 'virgo' || PLATFORM_TYPE === 'local' ) {
		$( ".platform-hillsVegas" ).remove();
	} else if ( PLATFORM_TYPE === 'hillsLocal' || PLATFORM_TYPE === 'hillsLive' ) {
		$( ".platform-virgo" ).remove();
	}

	var MainSlotsBaseClass = function( overrideSettings ) {
		if ( this.constructor === MainSlotsBaseClass ) {
			libs.logError( 'Cannot instantiate abstract MainSlotsBaseClass class' );
		}

		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );
	};

	MainSlotsBaseClass.prototype._initialise = function() {
		var that = this;

		this._defaultUpdateFunctions = {
			update: that._update,
			drawAllBase: function() {},
			drawDirtyBase: function() {},
			drawAllTop: function() {},
			drawDirtyTop: function() {},
			drawOverlay: function() {}
		};

		this._betRecoveryInfo = null;

		this._shouldActivateFreespinsAtEndOfCurrentBet = false;
		this._isFreespinsActive = false;
		this._isFreespinsUIActive = false;
		this._turboModeEnabled = false;
		this._isQuickPlay = false;
		this._isModalVisible = false;
		this._inXOfAKindSplash = false;

		this._reelSpinSoundCounter = 0;
		this._lastReelSpinSoundName = null;
		this._playingSymbolSound = null;
		this._playedCountUpWinSnd = false;
		this._currentSymbolCountUpWinnings = new BigNumber( 0 );

		this._specialLandingSoundsData = [];
		var i;
		if ( this._settings.specialLandingSounds && this._settings.specialLandingSounds.length > 0 ) {
			this._specialLandingSoundsData = this._settings.specialLandingSounds.slice( 0 );
			for ( i = 0; i < this._specialLandingSoundsData.length; ++i ) {
				this._specialLandingSoundsData[i].numberOfTimesLanded = 0;
			}
		}

		this._landingSoundPlayedOnReel = new Array( game.logic.GameData.numberOfReels );
		for ( i = 0; i < game.logic.GameData.numberOfReels; i++ ) {
			this._landingSoundPlayedOnReel[i] = false;
		}

		this._balance = new BigNumber( 0 );
		this._stakeTotal = new BigNumber( 0 );

		this._someoneJustWonAJackpot = false;
		this._playerJustWonAJackpot = false;

		this._createCoreReelsComponents();
		this._createReelsComponents();
		this._createPresentationComponents();
		this._createButtons();
		this._createMeters();

		if ( PLATFORM_TYPE === 'virgo' || PLATFORM_TYPE === 'local' ) {
			this._createVirgoComponents();
		} else if ( PLATFORM_TYPE === 'hillsLocal' || PLATFORM_TYPE === 'hillsLive' ) {
			this._createHillsVegasComponents();
		}

		libs.globalEventBus.connectActivityStart( 'libs.activity.display.modalVisible', function( activity ) {
			that._onModalVisibleStart( activity );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.modalVisible', function() {
			that._onModalVisibleEnd();
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.xOfAKindSplash', function( activity, numberOfWins ) {
			that._onXOfAKindSplashStart( activity, numberOfWins );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.xOfAKindSplash', function( numberOfWins ) {
			that._onXOfAKindSplashEnd( numberOfWins );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.quickPlayState', function( enabled ) {
			that._onNotifyQuickPlayState( enabled );
		} );

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) {
			that._onGamePlayingStart( activity );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.idle', function( activity ) {
			that._onIdleStart( activity );
		} );
		libs.globalEventBus.connect( 'libs.event.request.startBet', function( options ) {
			that._onRequestStartBet( options );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.reelsStarting', function( activity ) {
			that._onReelsStarting( activity );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.reelsLanding', function() {
			that._onReelsLandingEnd();
		} );
		libs.globalEventBus.connect( 'libs.event.notify.reelBounced', function( reelIndex, finalSymbols ) {
			that._onNotifyReelBounced( reelIndex, finalSymbols );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.reelStopped', function( reelIndex, finalSymbols ) {
			that._onNotifyReelStopped( reelIndex, finalSymbols );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.showingScatterWin', function( symbolId ) {
			that._onNotifyShowingScatterWin( symbolId );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.showingAllSymbolsOfWin', function( symbolId, win ) {
			that._onNotifyShowingAllSymbolsOfWin( symbolId, win );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.winlineShown', function( positionInRun, win ) {
			that._onNotifyWinlineShown( positionInRun, win );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.displayReelsWin', function( activity ) {
			that._onDisplayReelsWinEnd( activity );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.betState.postBet', function( activity ) {
			that._onPostBetEnd( activity );
		} );

		libs.globalEventBus.connectActivityStart( 'libs.activity.gameMode.betSequence', function( activity ) {
			that._onBetSequenceStart( activity );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.gameMode.betSequence', function( activity ) {
			that._onBetSequenceEnd();
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.gameMode.freespins', function( activity, inBetRecovery ) {
			that._onFreespinsStart( activity, inBetRecovery );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.gameMode.freespins', function() {
			that._onFreespinsEnd();
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.freespinsUI', function( activity, inBetRecovery ) {
			that._onFreespinsUIStart( activity, inBetRecovery );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.freespinsUI', function() {
			that._onFreespinsUIEnd();
		} );

		libs.globalEventBus.connect( 'libs.event.notify.betBonusInfo', function( bonusInfo ) {
			that._onNotifyBetBonusInfo( bonusInfo );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.betRecoveryInfo', function( betRecoveryInfo ) {
			that._onNotifyBetRecoveryInfo( betRecoveryInfo );
		} );

		libs.globalEventBus.connect( 'libs.event.notify.balance', function( balance ) {
			that._onNotifyBalance( balance );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.stakeTotal', function( stakeTotal ) {
			that._onNotifyStakeTotal( stakeTotal );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.jackpotValues', function( jackpotValues ) {
			that._onNotifyJackpotValues( jackpotValues );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.jackpotValueDecreased', function( jackpotIndex ) {
			that._onNotifyJackpotValueDecreased( jackpotIndex );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.jackpotWins', function( jackpotWins ) {
			that._onNotifyJackpotWins( jackpotWins );
		} );
		libs.globalEventBus.connect( 'libs.event.request.updateSpinParameters', function( sp ) {
			that._onRequestUpdateSpinParameters( sp );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.betState.preBet', function() { that._onActivityEndPreBet(); } );

		libs.globalEventBus.connect( 'libs.event.notify.turboModeState', function( enabled ) { that._onNotifyTurboModeState( enabled ); } );
	};

	MainSlotsBaseClass.prototype._onNotifyTurboModeState = function( enabled ) {
		this._turboModeEnabled = enabled;
	};

	MainSlotsBaseClass.prototype._createCoreReelsComponents = function() {
		var that = this;

		this._winAnimation = null;
		this._winlinesModel = null;
		this._winlinesView = null;
		this._reelsMachineModel = null;
		this._slotsGameModel = null;
		this._reelsViewGroup = null;

		if ( this._settings.libs.reels.winAnimation ) {
			this._winAnimation = new libs.client.reels.WinAnimation( this._settings.libs.reels.winAnimation );
		}
		if ( this._settings.libs.reels.winlinesModel ) {
			this._winlinesModel = new libs.client.reels.WinlinesModel( this._settings.libs.reels.winlinesModel );
		}
		if ( this._settings.libs.reels.winlinesView && this._winlinesModel ) {
			this._winlinesView = new libs.client.reels.WinlinesView( this._settings.libs.reels.winlinesView, this._winlinesModel );
		}
		if ( this._settings.libs.reels.reelsMachineModel ) {
			this._reelsMachineModel = new libs.client.reels.ReelsMachineModel( this._settings.libs.reels.reelsMachineModel );
		}
		if ( this._settings.libs.reels.slotsGameModel ) {
			this._slotsGameModel = new libs.client.reels.SlotsGameModel( this._settings.libs.reels.slotsGameModel );
			this._slotsGameModel.onServerBetModification( function( betResult ) {
				that._serverBetModification( betResult );
			} );
		}
		if ( that._settings.libs.reels.reelView && this._reelsMachineModel ) {
			// build the reels
			this._reelsViewGroup = new libs.client.reels.ReelViewGroup();
			var reelModels = this._reelsMachineModel.getReelModels();
			var buildReel = function( reel ) {
				var reelView = new libs.client.reels.ReelView( that._settings.libs.reels.reelView, reelModels[reel] );
				reelView.onBeforeSymbol( function( ctx, symInfo, landingInformation, drawingInformation ) {
					that._drawUnderSymbol( ctx, reel, symInfo, landingInformation, drawingInformation );
				} );
				reelView.onAfterSymbol( function( ctx, symInfo, landingInformation, drawingInformation ) {
					that._drawOverSymbol( ctx, reel, symInfo, landingInformation, drawingInformation );
				} );
				that._reelsViewGroup.reelViews.push( reelView );
			};

			for ( var i = 0; i < game.logic.GameData.numberOfReels; ++i ) {
				buildReel( i );
			}
		}
	};

	MainSlotsBaseClass.prototype._createButtons = function() {
		this._autoplayButton = null;
		this._spinButton = null;
		this._paytableButton = null;
		this._stakeDownButton = null;
		this._stakeUpButton = null;
		this._menuButton = null;
		this._stakeButton = null;

		if ( this._settings.libs.reels.autoplayButton ) {
			this._autoplayButton = new libs.client.reels.AutoplayButton( this._settings.libs.reels.autoplayButton );
		}
		if ( this._settings.libs.reels.spinButton ) {
			this._spinButton = new libs.client.reels.SpinButton( this._settings.libs.reels.spinButton );
		}
		if ( this._settings.libs.reels.paytableButton ) {
			this._paytableButton = new libs.client.reels.PaytableButton( this._settings.libs.reels.paytableButton );
		}
		if ( this._settings.libs.reels.stakeDownButton ) {
			this._stakeDownButton = new libs.client.reels.StakeDownButton( this._settings.libs.reels.stakeDownButton );
		}
		if ( this._settings.libs.reels.stakeUpButton ) {
			this._stakeUpButton = new libs.client.reels.StakeUpButton( this._settings.libs.reels.stakeUpButton );
		}
		if ( this._settings.libs.reels.stakeButton ) {
			this._stakeButton = new libs.client.reels.StakeButton( this._settings.libs.reels.stakeButton );
		}
		if ( this._settings.libs.reels.menuButton ) {
			this._menuButton = new libs.client.reels.MenuButton( this._settings.libs.reels.menuButton );
		}
	};

	MainSlotsBaseClass.prototype._createMeters = function() {
		this._balanceMeter = null;
		this._totalWinMeter = null;
		this._totalStakeMeter = null;

		if ( this._settings.libs.presentation.balanceMeter ) {
			this._balanceMeter = new libs.client.presentation.BalanceMeter( this._settings.libs.presentation.balanceMeter );
		}
		if ( this._settings.libs.presentation.totalWinMeter ) {
			this._totalWinMeter = new libs.client.presentation.TotalWinMeter( this._settings.libs.presentation.totalWinMeter );
		}
		if ( this._settings.libs.presentation.totalStakeMeter ) {
			this._totalStakeMeter = new libs.client.presentation.TotalStakeMeter( this._settings.libs.presentation.totalStakeMeter );
		}

	};

	MainSlotsBaseClass.prototype._createReelsComponents = function() {
		this._autoplaySelector = null;
		this._paytable = null;
		this._reelsAnticipation = null;
		this._xOfAKindSplash = null;
		this._scrollableStakeSelector = null;
		this._symbolTeaseAnimation = null;

		if ( this._settings.libs.reels.autoplaySelector ) {
			this._autoplaySelector = new libs.client.reels.AutoplaySelector( this._settings.libs.reels.autoplaySelector );
		}
		if ( this._settings.libs.reels.paytable ) {
			this._paytable = new libs.client.reels.Paytable( this._settings.libs.reels.paytable );
		}
		if ( this._settings.libs.reels.reelsAnticipation ) {
			this._reelsAnticipation = new libs.client.reels.ReelsAnticipation( this._settings.libs.reels.reelsAnticipation );
		}
		if ( this._settings.libs.reels.xOfAKindSplash ) {
			this._xOfAKindSplash = new libs.client.reels.XOfAKindSplash( this._settings.libs.reels.xOfAKindSplash );
		}
		if ( this._settings.libs.reels.scrollableStakeSelector ) {
			this._scrollableStakeSelector = new libs.client.reels.ScrollableStakeSelector( this._settings.libs.reels.scrollableStakeSelector );
		}
		if ( this._settings.libs.reels.symbolTeaseAnimation ) {
			this._symbolTeaseAnimation = new libs.client.reels.SymbolTeaseAnimation( this._settings.libs.reels.symbolTeaseAnimation );
		}
		if ( this._settings.libs.reels.bellAlarm ) {
			this._bellAlarm = new libs.client.reels.BellAlarm( this._settings.libs.reels.bellAlarm );
		}
	};

	MainSlotsBaseClass.prototype._createPresentationComponents = function() {
		this._bigWinner = null;
		this._fpsDisplay = null;
		this._messageBar = null;

		if ( this._settings.libs.presentation.bigWinner ) {
			this._bigWinner = new libs.client.presentation.BigWinner( this._settings.libs.presentation.bigWinner );
		}
		if ( this._settings.libs.presentation.fpsDisplay ) {
			this._fpsDisplay = new libs.client.presentation.FpsDisplay( this._settings.libs.presentation.fpsDisplay );
		}
		if ( this._settings.libs.presentation.messageBar ) {
			this._messageBar = new libs.client.presentation.MessageBar( this._settings.libs.presentation.messageBar );
		}
		if ( this._settings.libs.presentation.digitalClock ) {
			this._digitalClock = new libs.client.presentation.DigitalClock( this._settings.libs.presentation.digitalClock );
		}
		if ( this._settings.libs.presentation.gameNameFooter ) {
			this._gameNameFooter = new libs.client.presentation.GameNameFooter( this._settings.libs.presentation.gameNameFooter );
		}
	};

	MainSlotsBaseClass.prototype._createHillsVegasComponents = function() {
		this._menu = null;
		this._historyDisplay = null;

		if ( this._settings.libs.hillsVegas.mobileMenu ) {
			this._menu = new libs.client.hillsVegas.MobileMenu( this._settings.libs.hillsVegas.mobileMenu );
		}
		if ( this._settings.libs.hillsVegas.desktopMenu ) {
			this._menu = new libs.client.hillsVegas.DesktopMenu( this._settings.libs.hillsVegas.desktopMenu );
		}
		if ( this._settings.libs.hillsVegas.historyDisplay ) {
			this._historyDisplay = new libs.client.hillsVegas.HistoryDisplay( this._settings.libs.hillsVegas.historyDisplay );
		}
	};

	MainSlotsBaseClass.prototype._createVirgoComponents = function() {
		this._menu = null;
		this._historyDisplay = null;

		if ( this._settings.libs.virgo.menu ) {
			this._menu = new libs.client.virgo.Menu( this._settings.libs.virgo.menu );
		}
	};

	MainSlotsBaseClass.prototype._update = function( timeDelta ) {
		createjs.Tween.tick( timeDelta, false );

		if ( this._winAnimation ) {
			this._winAnimation.update( timeDelta );
		}
		if ( this._reelsMachineModel ) {
			this._reelsMachineModel.update( timeDelta );
		}
		if ( this._reelsViewGroup ) {
			this._reelsViewGroup.update( timeDelta );
		}
	};

	MainSlotsBaseClass.prototype._onActivityEndPreBet = function() {
		if ( this._betRecoveryInfo ) {
			this._gameSpecificBetRecovery();
		}
		this._betRecoveryInfo = null;
	};

	MainSlotsBaseClass.prototype._onModalVisibleStart = function( activity ) {
		this._isModalVisible = true;
	};

	MainSlotsBaseClass.prototype._onModalVisibleEnd = function() {
		this._isModalVisible = false;
	};

	MainSlotsBaseClass.prototype._onXOfAKindSplashStart = function( activity, numberOfWins ) {
		this._inXOfAKindSplash = true;
	};

	MainSlotsBaseClass.prototype._onXOfAKindSplashEnd = function( numberOfWins ) {
		this._inXOfAKindSplash = false;
	};

	MainSlotsBaseClass.prototype._onNotifyQuickPlayState = function( enabled ) {
		this._isQuickPlay = enabled;
	};

	MainSlotsBaseClass.prototype._onGamePlayingStart = function( activity ) {
		libs.globalEventBus.invoke( 'libs.event.notify.buttonBaseModel', libs.client.reels.buttonModels.baseModel );
		libs.globalEventBus.invoke( 'libs.event.notify.meterBaseModel', libs.client.reels.meterModels.baseModel );

		libs.globalEventBus.invoke( 'libs.event.notify.defaultUpdateFunctions', this._defaultUpdateFunctions );

		libs.client.utils.DOM.showBackground( selectors.backgroundNormal, 0 );
	};

	MainSlotsBaseClass.prototype._onIdleStart = function( activity ) {
		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );

		if ( this._betRecoveryInfo ) {
			this._startBetRecovery();
		} else {
			if ( this._isFreespinsActive ) {
				libs.setTimeoutTick( function() {
					libs.globalEventBus.invoke( 'libs.event.request.startBet' );
				}, this._settings.gameDelayFreespins );
			} else if ( libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ) {
				libs.setTimeoutTick( function() {
					if ( libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ) {
						libs.globalEventBus.invoke( 'libs.event.request.startBet' );
					} // extra time for native android browsers to sort out any redraws
				}, isNativeAndroid ? this._settings.gameDelayAutoplayAndroid : this._settings.gameDelayAutoplayNormal );
			} else {
				libs.globalEventBus.invoke( 'libs.event.request.enableReelDragger' );
				libs.globalEventBus.invoke( 'libs.event.request.enableMenu' );
			}
		}
	};

	MainSlotsBaseClass.prototype._onRequestStartBet = function( options ) {
		var that = this;

		// WE R - added request to keep device awake
		if (window.chrome && window.chrome.power && window.chrome.power.requestKeepAwake) {
			window.chrome.power.requestKeepAwake('display');
		}

		if ( this._slotsGameModel.isInBet() || this._announceAnotherPlayerJustWonAJackpot() ) {
			libs.globalEventBus.invoke( 'libs.event.request.stopAutoplay' );

			return false;
		}

		var isMoneySpin = !this._isFreespinsActive;
		if ( isMoneySpin && this._stakeTotal.greaterThan( this._balance ) ) {
			switch ( PLATFORM_TYPE ) {
				default:
				case 'local':
					// WE R - display message when stake is too big
					libs.globalEventBus.invoke( 'libs.event.request.showError', 'You have insufficient balance to place this bet. Please lower your stake to an affordable amount.', false );
					break;
				case 'virgo': break;
				case 'hillsLocal':
					libs.globalEventBus.invoke( 'libs.event.request.showError', libs.getTranslation( 'You_do_not_have_enough_points' ), false );
					break;
				case 'hillsLive':
					var negativeButton = {
						func: function() {
						},
						text: libs.getTranslation( 'BACK' )
					};

					var positiveButton = {
						func: function() {
							libs.client.utils.misc.openUrl( that._settings.hillsVegasLinks.deposit, true );
						},
						text: libs.getTranslation( 'DEPOSIT' )
					};

					libs.globalEventBus.invoke( 'libs.event.request.showCustomError', libs.getTranslation( 'You_have_insufficient_funds' ), negativeButton, positiveButton );
					break;
			}

			return false;
		}

		for ( var i = 0; i < this._specialLandingSoundsData.length; ++i ) {
			this._specialLandingSoundsData[i].numberOfTimesLanded = 0;
		}

		this._rebuildSpinParameters( options );
		libs.globalEventBus.invoke( 'libs.event.notify.betAllowed' );

		return true;
	};

	MainSlotsBaseClass.prototype._onRequestUpdateSpinParameters = function( spinParameters ) {
		this._spinParameters = jQuery.extend( true, {}, this._spinParameters, spinParameters );
		libs.globalEventBus.invoke( 'libs.event.notify.spinParameters', this._spinParameters );
	};

	MainSlotsBaseClass.prototype._rebuildSpinParameters = function( options ) {

		// merge with default spin parameter object
		this._spinParameters = jQuery.extend( true, {}, spinParameterDefaults, this._buildSpinParameters( options ) );
		libs.globalEventBus.invoke( 'libs.event.notify.spinParameters', this._spinParameters );
	};

	// Can be overriden by sub class
	MainSlotsBaseClass.prototype._buildSpinParameters = function( options ) {
		var spinParameters = jQuery.extend( true, {}, options );

		// Superimposesymbol patterns on the reel bands that pass the window. These will not be displayed on the final reel screen
		if ( this._settings.imposeReelSymbolSets ) {
			var randomIndex = libs.client.utils.misc.getRandomInt( 0, this._settings.imposeReelSymbolSets.length );
			spinParameters.imposeReelSymbols = this._settings.imposeReelSymbolSets[randomIndex];
		}
		return spinParameters;
	};

	MainSlotsBaseClass.prototype._onReelsStarting = function( activity ) {
		libs.globalEventBus.invoke( 'libs.event.request.disableReelDragger' );
		libs.globalEventBus.invoke( 'libs.event.request.disableMenu' );

		if ( !this._isFreespinsUIActive && !this._turboModeEnabled ) {
			libs.globalEventBus.invoke( 'libs.event.request.playSound', ['LQ', 'MQ', 'HQ'], this._getReelSpinSound() );
		}
		if ( this._landingSoundPlayedOnReel ) {
			for ( var i = 0; i < this._landingSoundPlayedOnReel.length; i++ ) {
				this._landingSoundPlayedOnReel[i] = false;
			}
		}
	};

	MainSlotsBaseClass.prototype._onNotifyReelBounced = function( reelIndex, finalSymbols ) {
		var moreThanOneSpecialLandSoundSymbol = this._getNumberOfSpecialLandingSoundsSymbols() > 0;
		this._checkIfLandingSoundShouldPlay( reelIndex, finalSymbols, moreThanOneSpecialLandSoundSymbol );
	};

	MainSlotsBaseClass.prototype._onNotifyReelStopped = function( reelIndex, finalSymbols ) {
		var playOnFirstReel = reelIndex === 0 && this._getNumberOfSpecialLandingSoundsSymbols() === 0;
		this._checkIfLandingSoundShouldPlay( reelIndex, finalSymbols, playOnFirstReel );
	};

	MainSlotsBaseClass.prototype._checkIfLandingSoundShouldPlay = function( reelIndex, finalSymbols, playOnQuickPlay ) {
		var playLandingSound = false;

		if ( this._isQuickPlay ) {
			//if quickplay and quickplay condition is true
			if ( playOnQuickPlay ) {
				playLandingSound = true;
			}
		} else {
			//only play if sound hasnt played for this reel
			if ( this._checkIfLandingSoundHasntPlayed( reelIndex ) ) {
				playLandingSound = true;
			}
		}

		if ( playLandingSound ) {
			this._playLandingSound( reelIndex, finalSymbols );
		}
	};

	MainSlotsBaseClass.prototype._getNumberOfSpecialLandingSoundsSymbols = function() {
		var numberOfSpecialLandingSoundsSymbols = 0;
		if ( this._specialLandingSoundsData && this._specialLandingSoundsData.length > 0 ) {
			for ( var i = 0; i < this._specialLandingSoundsData.length; ++i ) {
				numberOfSpecialLandingSoundsSymbols += this._specialLandingSoundsData[i].numberOfTimesLanded;
				break; // no need to carry on, only need to know if one exists or not
			}
		}

		return numberOfSpecialLandingSoundsSymbols;
	};

	MainSlotsBaseClass.prototype._checkIfLandingSoundHasntPlayed = function( reel ) {
		if ( this._landingSoundPlayedOnReel && !this._landingSoundPlayedOnReel[reel] ) {
			return true;
		}
		return false;
	};

	MainSlotsBaseClass.prototype._playLandingSound = function( reel, symbols ) {

		if ( this._spinParameters.disableReelSounds && this._spinParameters.disableReelSounds.indexOf( reel ) >= 0 ) {
			return; // dont play landing sound on this reel
		}

		var reelStopSound = null;

		var reelsRemaining = game.logic.GameData.numberOfReels - (reel + 1);

		for ( var i = 0; i < symbols.length; ++i ) {
			if ( !reelStopSound ) {
				for ( var j = 0; j < this._specialLandingSoundsData.length; ++j ) {
					var symbolSoundData = this._specialLandingSoundsData[j];
					if ( symbols[i].id === symbolSoundData.symbolId ) {
						++symbolSoundData.numberOfTimesLanded;

						// can specify in settings a valid reels array to tell the game which reels a bonus symbol can land on
						if ( symbolSoundData.validReels ) {
							var remainingCount = 0;
							for ( var r = reel; r < symbolSoundData.validReels.length; r++ ) {
								if ( symbolSoundData.validReels[r] ) {
									remainingCount++;
								}
							}
							reelsRemaining = remainingCount - 1;
						}

						if ( symbolSoundData.numberOfTimesLanded + reelsRemaining >= symbolSoundData.minFinalSymbolCount ) {
							reelStopSound = symbolSoundData.reelStopSound;
							break;
						}
					}
				}
			}
		}

		if ( reelStopSound ) { // must be a special one, so play haptics
			libs.globalEventBus.invoke( 'libs.event.request.playHaptic', 50 );
		} else { // no special one, so play standard reel stop sound
			reelStopSound = this._getReelStopSound();
		}

		if ( this._landingSoundPlayedOnReel ) {
			this._landingSoundPlayedOnReel[reel] = true;
		}

		var panLeft = -1;
		var panRight = 1;
		var panIncrement = (panRight - panLeft) / (game.logic.GameData.numberOfReels - 1);
		var landingPan = panLeft + (panIncrement * reel);
		libs.globalEventBus.invoke( 'libs.event.request.playSound', ['HQ'], reelStopSound, { pan: landingPan } );
	};

	MainSlotsBaseClass.prototype._onReelsLandingEnd = function() {
		if ( this._isQuickPlay && this._lastReelSpinSoundName ) {
			// Only stop the sound if the reels were skipped
			libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['MQ', 'HQ'], this._lastReelSpinSoundName );
			this._lastReelSpinSoundName = null;
		}
		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );
	};

	MainSlotsBaseClass.prototype._onNotifyShowingScatterWin = function( symbolId ) {
		this._playSymbolSound( symbolId );
		this._checkIfShouldStopAutoplay( symbolId );
	};

	MainSlotsBaseClass.prototype._onNotifyShowingAllSymbolsOfWin = function( symbolId, win ) {
		libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['HQ'], this._getCountUpWinSound() );

		this._playedCountUpWinSnd = false;
		this._currentSymbolCountUpWinnings = new BigNumber( 0 );

		this._playSymbolSound( symbolId );
		win = new BigNumber( win );
		if ( (!this._isFreespinsUIActive || this._settings.libs.presentation.allowMessageBoxInFreespins) && this._settings.libs.presentation.showGameWinInMessageBox ) {
			if ( win.greaterThan( 0 ) ) { // stops bar appearing for progressive wins
				var msg = libs.getTranslation( 'WIN' ) + ' ' + libs.moneyFormat( win, true );
				libs.globalEventBus.invoke( 'libs.event.request.showMessageBar', msg );
			}
		}

		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', false, true );

		this._checkIfShouldStopAutoplay( symbolId );
	};

	MainSlotsBaseClass.prototype._onNotifyWinlineShown = function( positionInRun, win ) {
		// if positionInRun === -1, then it is just a single winlines. we'll want to
		// skip functionality here as it will be handled in _allOfSymbolWin
		if ( positionInRun > -1 ) {
			if ( !this._playedCountUpWinSnd ) {
				libs.globalEventBus.invoke( 'libs.event.request.playSound', ['HQ'], this._getCountUpWinSound() );
				this._playedCountUpWinSnd = true;
			}

			if ( positionInRun === 0 ) {
				this._currentSymbolCountUpWinnings = new BigNumber( 0 );
			}

			if ( (!this._isFreespinsUIActive || this._settings.libs.presentation.allowMessageBoxInFreespins) && this._settings.libs.presentation.showGameWinInMessageBox ) {
				if ( win > 0 ) { // stops bar appearing for progressive wins
					this._currentSymbolCountUpWinnings = this._currentSymbolCountUpWinnings.plus( win );
					var msg = libs.getTranslation( 'WIN' ) + ' ' + libs.moneyFormat( this._currentSymbolCountUpWinnings, true );
					libs.globalEventBus.invoke( 'libs.event.request.showMessageBar', msg );
				}
			}
		}
	};

	MainSlotsBaseClass.prototype._onDisplayReelsWinEnd = function() {
		libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['HQ'], this._getCountUpWinSound() );
		this._playedCountUpWinSnd = false;
		libs.globalEventBus.invoke( 'libs.event.request.hideMessageBar' );
	};

	MainSlotsBaseClass.prototype._onPostBetEnd = function() {
		this._playingSymbolSound = null;
		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );

		if ( isNativeAndroid ) {
			$( '#viewport' ).redrawGentle();
		}
		
		// WE R - added request to keep device awake
		if (window.chrome && window.chrome.power && window.chrome.power.releaseKeepAwake) {
			window.chrome.power.releaseKeepAwake();
		}
	};

	MainSlotsBaseClass.prototype._onBetSequenceStart = function( activity ) {
		if ( !this._handleFreespins && this._shouldActivateFreespinsAtEndOfCurrentBet ) {
			libs.globalEventBus.invokeActivity( 'libs.activity.gameMode.freespins' );
		}

		this._shouldActivateFreespinsAtEndOfCurrentBet = false;
	};

	MainSlotsBaseClass.prototype._onBetSequenceEnd = function() {
		if ( this._handleFreespins ) {
			this._handleFreespins.leave();
			this._handleFreespins = null;
		}
	};

	MainSlotsBaseClass.prototype._onFreespinsStart = function( activity, inBetRecovery ) {
		this._handleFreespins = activity.join();
		this._isFreespinsActive = true;
		if ( this._settings.stopAutoplayOnFreespins && libs.globalEventBus.isActivityRunning( 'libs.activity.gameMode.autoplay' ) ) {
			libs.globalEventBus.invoke( 'libs.event.request.stopAutoplay' );
		}
	};

	MainSlotsBaseClass.prototype._onFreespinsEnd = function() {
		this._isFreespinsActive = false;
	};

	MainSlotsBaseClass.prototype._onFreespinsUIStart = function( activity, inBetRecovery ) {
		this._isFreespinsUIActive = true;
		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );
	};

	MainSlotsBaseClass.prototype._onFreespinsUIEnd = function() {
		this._isFreespinsUIActive = false;
		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );
	};

	MainSlotsBaseClass.prototype._onNotifyBetBonusInfo = function( bonusInfo ) {
		if ( bonusInfo['isFreeSpinWin'] ) {
			this._shouldActivateFreespinsAtEndOfCurrentBet = true;
		}
	};

	MainSlotsBaseClass.prototype._onNotifyBetRecoveryInfo = function( betRecoveryInfo ) {
		this._betRecoveryInfo = betRecoveryInfo;
	};

	MainSlotsBaseClass.prototype._startBetRecovery = function() {
		var startReels = true;
		if ( this._betRecoveryInfo && this._betRecoveryInfo['bonus'] ) {
			for ( var i = 0; i < this._betRecoveryInfo['bonus'].length; ++i ) {
				if ( this._betRecoveryInfo['bonus'][i]['noReelsSpin'] ) {
					startReels = false;
					break;
				}
			}
		}

		if ( startReels ) {
			this._rebuildSpinParameters();
		} else {
			libs.globalEventBus.invoke( 'libs.event.request.disableReelStates' );
		}

		libs.globalEventBus.invoke( 'libs.event.notify.betAllowed' );
	};

	MainSlotsBaseClass.prototype._gameSpecificBetRecovery = function() {
		libs.logError( 'Call to pure virtual method _gameSpecificBetRecovery' );
	};

	MainSlotsBaseClass.prototype._serverBetModification = function( serverBet ) {
		this._betModificationReelDelays( serverBet );
	};

	MainSlotsBaseClass.prototype._betModificationReelDelays = function( serverBet ) {
		var reelCount = game.logic.GameData.numberOfReels;
		var rowCount = game.logic.GameData.numberOfSymbolRows;

		var delaysArray = [];
		for ( var i = 0; i < reelCount; ++i ) {
			delaysArray.push( false );
		}
		serverBet['reelDelays'] = delaysArray;

		if ( this._settings.anticipationSymbols ) {
			if ( serverBet['reels'] ) {
				var screen = serverBet['reels']['screen'];

				if ( Array.isArray( screen ) ) {
					for ( var j = 0; j < this._settings.anticipationSymbols.length; ++j ) {
						var anticipation = this._settings.anticipationSymbols[j];
						var symbolsFoundCount = 0;
						for ( var reel = 0; reel < reelCount; ++reel ) {
							if ( serverBet['reelDelays'][reel] === false ) {

								if ( symbolsFoundCount >= anticipation.minSymbolCount && (!this._settings.anticipationReels || this._settings.anticipationReels[reel]) ) {
									serverBet['reelDelays'][reel] = true;
								}

								for ( var row = 0; row < rowCount; ++row ) {
									if ( screen[reel][row] === anticipation.symbolId ) {
										++symbolsFoundCount;
									}
								}

							}
						}
					}
				}
			}
		}
	};

	MainSlotsBaseClass.prototype._checkIfShouldStopAutoplay = function( winningSymbol ) {
		if ( this._settings.stopAutoplayOnSymbolsWin ) {
			if ( this._settings.stopAutoplayOnSymbolsWin.indexOf( winningSymbol ) !== -1 ) {
				libs.globalEventBus.invoke( 'libs.event.request.stopAutoplay' );
			}
		}
	};

	MainSlotsBaseClass.prototype._playSymbolSound = function( symbol ) {
		var winSoundName = this._settings.symbolsData[symbol].sound;

		if ( !this._isFreespinsUIActive ) {
			if ( this._playingSymbolSound ) {
				libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['MQ'], this._playingSymbolSound );
			}
			libs.globalEventBus.invoke( 'libs.event.request.playSound', ['MQ'], winSoundName );
		}

		if ( this._playingSymbolSound ) {
			libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['HQ'], this._playingSymbolSound );
		}
		libs.globalEventBus.invoke( 'libs.event.request.playSound', ['HQ'], winSoundName );

		this._playingSymbolSound = winSoundName;
	};

	MainSlotsBaseClass.prototype._shouldCycleReelSpinSound = function() {
		return !clientInfo.isIE() && clientInfo.getAudioQuality() !== 'LQ';
	};

	MainSlotsBaseClass.prototype._getReelSpinSound = function() {
		if ( this._lastReelSpinSoundName ) {
			libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['MQ', 'HQ'], this._lastReelSpinSoundName );
		}

		if ( typeof this._settings.sounds.reelSpin === 'string' ) {
			this._lastReelSpinSoundName = this._settings.sounds.reelSpin;
		} else if ( Array.isArray( this._settings.sounds.reelSpin ) ) {
			this._lastReelSpinSoundName = this._settings.sounds.reelSpin[this._reelSpinSoundCounter];
			if ( this._shouldCycleReelSpinSound() ) {
				this._reelSpinSoundCounter = ( this._reelSpinSoundCounter + 1 ) % this._settings.sounds.reelSpin.length;
			}
		} else {
			throw new Error( "Unexpected value for reelSpin setting" );
		}

		return this._lastReelSpinSoundName;
	};

	MainSlotsBaseClass.prototype._getReelStopSound = function() {
		return this._settings.sounds.reelStop;
	};

	MainSlotsBaseClass.prototype._getCountUpWinSound = function() {
		return this._settings.sounds.countUpWin;
	};

	MainSlotsBaseClass.prototype._isShowingSplashScreen = function() {
		return this._isModalVisible || this._inXOfAKindSplash;
	};

	MainSlotsBaseClass.prototype._drawUnderSymbol = function( ctx, reel, symInfo, landingInformation, drawingInformation ) {
	};

	MainSlotsBaseClass.prototype._drawOverSymbol = function( ctx, reel, symInfo, landingInformation, drawingInformation ) {
	};

	MainSlotsBaseClass.prototype._onNotifyBalance = function( balance ) {
		this._balance = balance;
	};

	MainSlotsBaseClass.prototype._onNotifyStakeTotal = function( stakeTotal ) {
		this._stakeTotal = stakeTotal;
	};

	MainSlotsBaseClass.prototype._onNotifyJackpotValues = function() {
		this._announceAnotherPlayerJustWonAJackpot();
	};

	MainSlotsBaseClass.prototype._onNotifyJackpotValueDecreased = function() {
		this._someoneJustWonAJackpot = true;
		this._announceAnotherPlayerJustWonAJackpot();
	};

	MainSlotsBaseClass.prototype._onNotifyJackpotWins = function( jackpotWins ) {
		if ( !this._playerJustWonAJackpot ) {
			// If flag has already been set, wait for the announce to come through to clear it
			this._playerJustWonAJackpot = jackpotWins.length > 0;
		}
	};

	MainSlotsBaseClass.prototype._announceAnotherPlayerJustWonAJackpot = function() {
		if ( this._someoneJustWonAJackpot ) {
			if ( this._playerJustWonAJackpot ) {
				this._playerJustWonAJackpot = false;
				this._someoneJustWonAJackpot = false;
			} else {
				if ( !this._slotsGameModel.isInBet() && !this._isFreespinsUIActive && !this._isShowingSplashScreen() ) {
					libs.globalEventBus.invoke( 'libs.event.request.showError', libs.getTranslation( 'Another_player_has_won_a_Jackpot' ), false );
					this._playerJustWonAJackpot = false;
					this._someoneJustWonAJackpot = false;
					return true;
				}
			}
		}

		return false;
	};


	libs.client.reels.MainSlotsBaseClass = MainSlotsBaseClass;

}());