goog.provide( 'libs.client.reels.XOfAKindSplash' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.StateMachine' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.rendering.SpriteGroup' );
goog.require( 'libs.client.utils.DOM' );


/*
 *
 * A few things left outstanding to complete the 5 of a kind
 *
 * The dimmed background couldbe done programmatically - it could possibly be drawn to the base canvas, but I'm not sure performance wise if that would be best.
 * Otherwise keep it in the DOm and style the CSS in the constructor to the dimensions of the reels. Gettinging the top left and bottom right positions of the reels.
 *
 * Once this is done you can remove the left, top, width, height params from the css for this ( however you'll need to still either leave the 'top' position for text or do that programmatically also)
 *
 * We are also waiting on a new sound... it will just be a single sound for both multiple and single 5 of a kinds...
 * this can be started in the same place (but without the delay which is being addeed on currently ) and stopped in the commented out section on line 266
 *
 *
 * For noe I'd say that's pretty much all I can think of... However... we may need 6 or 7... maybe 12... ( who knows!!!! ) of a kind in the future -
 *
 * which should be a simple switch to passing an extra param in the activity 'libs.activity.display.xOfAKindSplash'
 * for how many of a kind it is... and then just pick the translation up from there
 *
 *
 * I think this is all I have to say on this for now...
 *
 * C ya!
 *
 *
 *  XOXOXOXOXOXOXOXOXOXOXOXOX
 *
 * */

(function() {
	"use strict";

	var defaultSettings = {
		numberOfAKind: 5,
		soundSingle: 'SndXOfAKindSingle',
		soundMultiple: 'SndXOfAKindMultiple',
		explosionSettings: {
			imageName: 'Kind_Explosion',
			animSettings: {
				fps: 40,
				resetToFrameZeroOnEnd: false,
				timesToLoop: 0
			},
			drawSettings: {
				alpha: 0,
				offsetX: -100,	// half of the image width to find center point
				offsetY: -75	// half of the image height to find center point
			}
		},
		textSettings: {
			fontSize: 95,
			maxWidth: 600
		},
		preMessageDelay: 400,
		growTime: 600,
		postMessageDelay: 500,
		reelFadeDelay: 500,	//needed, to work with reel background fading
		gradientText: true
	};

	var selectors = {
		canvas: $( '#main' ),
		body: $('#body'),
		xOfAKindContainer: $( '#x-of-a-kind-container' ),
		xOfAKindMessage: $( '#x-of-a-kind-message' ),
		xOfAKindMessageLineOne: $( '#x-of-a-kind-message-line-one' ),
		xOfAKindMessageLineTwo: $( '#x-of-a-kind-message-line-two' ),
		xOfAKindMultiplier: $( '#x-of-a-kind-multiplier' )
	};

	selectors.xOfAKindContainer.hide();

	var XOfAKindSplash = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._stateMachine = new libs.StateMachine();

		this._numberOfWins = 0;
		this._explosionPositions = [];
		this._explosionSpriteGroup = new libs.client.rendering.SpriteGroup();
		this._explosionScale = 1;
		this._explosionCount = 0;

		this._handleXOfAKindSplash = null;
		this._xOfAKindText = [];
		this._domElements = [];

		this._shouldDoExplosions = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, this._settings.explosionSettings.imageName ).length > 0;
		this._createExplosionSprites();

		this._timeout = null;

		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) { that._onNotifyTick( timeDelta ); } );
		libs.globalEventBus.connect( 'libs.event.notify.keyPressed', function( keyCode, keyChar ) { that._onNotifyKeyPressed( keyCode, keyChar ); } );
		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerDown', function( point ) { that._onNotifyCanvasPointerDown( point ); } );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.xOfAKindSplash', function( activity, numberOfWins ) { that._onDisplayXOfAKindSplashStart( activity, numberOfWins ); } );
	};

	XOfAKindSplash.prototype._onNotifyTick = function( timeDelta ) {
		if ( this._handleXOfAKindSplash ) {
			this._stateMachine.update( timeDelta );
		}
	};

	XOfAKindSplash.prototype._onNotifyLanguageChanged = function( langCode ) {
		var translation = libs.getTranslation( 'X_OF_A_KIND', [this._settings.numberOfAKind] );
		var fontSize = libs.client.utils.DOM.calculateFontSize( translation, this._settings.textSettings );

		selectors.xOfAKindMessage.css( { fontSize: fontSize } );
		this._explosionScale = parseFloat( fontSize/10 );

		this._xOfAKindText.length = 0;
		this._xOfAKindText = translation.split( ' ' );
		this._createDomElements();
	};

	XOfAKindSplash.prototype._onNotifyKeyPressed = function( keyCode, keyChar ) {
		if ( keyChar === ' '  &&  this._handleXOfAKindSplash) {
			this._skipSplash();
		}
	};

	XOfAKindSplash.prototype._onNotifyCanvasPointerDown = function( point ) {
		this._skipSplash();
	};

	XOfAKindSplash.prototype._skipSplash = function() {
		if ( this._handleXOfAKindSplash ) {
			this._endSplash();
		}
	};

	XOfAKindSplash.prototype._onDisplayXOfAKindSplashStart = function( activity, numberOfWins ) {
		var that = this;

		this._handleXOfAKindSplash = activity.join();

		for ( var i = 0; i < game.logic.GameData.reels.nreels; i++ ) {
			libs.globalEventBus.invoke( 'libs.event.request.startForceReelFadedBackground', i, true );
		}
		// give the reels time to fade otherwise they won't fade completly
		
		this._timeout = libs.setTimeoutTick( function() {
			libs.globalEventBus.invoke( 'libs.event.request.enableCanvasOverlayMode' );
			that._numberOfWins = numberOfWins;
			that._stateMachine.update( 0 );
			that._stateMachine.stateGoto( function() {that._animateSplashWin();} );
		}, this._settings.reelFadeDelay );
	};

	XOfAKindSplash.prototype._createExplosionSprites = function() {
		if ( this._shouldDoExplosions ) {
			this._explosionSpriteGroup.sprites.length = 0;
			for ( var i = 0; i < 3; i++ ) {
				var explosionSettings = Object.create( this._settings.explosionSettings );
				var explosion = new libs.client.rendering.Sprite( explosionSettings );
				this._explosionSpriteGroup.push( explosion );
			}
		}
	};

	XOfAKindSplash.prototype._createDomElements = function() {
		this._domElements.length = 0;
		selectors.xOfAKindMessageLineTwo.empty();

		for ( var i = 0, len = this._xOfAKindText.length; i < len; i++ ) {
			var span = $( '<span />' );
			libs.client.utils.DOM.setText( span, this._xOfAKindText[i], {}, this._settings.gradientText );

			this._domElements.push( span );
			selectors.xOfAKindMessageLineTwo.append( span );
		}
		this._getExplosionPositions();
	};

	XOfAKindSplash.prototype._getExplosionPositions = function() {
		var that = this;

		this._explosionPositions.length = 0;

		// as we're doing a redraw to get the element positions we need to fade out the container again.
		libs.client.utils.DOM.fadeOut( selectors.xOfAKindContainer, 1 );
		selectors.xOfAKindContainer.redrawHarsh();

		var canvasPos = selectors.canvas[0].getBoundingClientRect();
		var scaleValue = selectors.body.css( 'font-size' );
		var scale = parseFloat( scaleValue ) / 16;

		function getPos( el, canvasPos, scale ) {
			var elPos = el.getBoundingClientRect();

			return {
				x: ((elPos.left - canvasPos.left) + (elPos.width / 2)) / scale,
				y: ((elPos.bottom - canvasPos.top) - (elPos.height / 2)) / scale
			};

		}

		selectors.xOfAKindMessage.find( 'span' ).each( function() {
			that._explosionPositions.push( getPos( this, canvasPos, scale ) );
		} );
	};

	XOfAKindSplash.prototype._animateSplashWin = function() {
		var that = this;

		if ( this._stateMachine.phaseOnce() ) {
			libs.client.utils.DOM.fadeIn( selectors.xOfAKindContainer, 200 );
			if ( this._numberOfWins > 1 ) {
				selectors.xOfAKindMessageLineOne.velocity( { 'opacity': 1, 'delay': 0 } );
			}
		}

		if ( this._stateMachine.phaseTimed( this._settings.preMessageDelay ) ) { }

		if ( this._stateMachine.phaseOnce() ) {
			this._createText();

			// the current sound only has 5 beats so needs to start after the multiplier has shown --- this does just that
			libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['HQ'], this._settings.soundMultiple );
			libs.setTimeoutTick( function() {
				libs.globalEventBus.invoke( 'libs.event.request.playSound', ['HQ'], that._settings.soundMultiple );
			}, this._numberOfWins > 1 ? 0 : this._settings.growTime );

		}

		if ( this._stateMachine.phaseOnce() ) {
			this._showMessage( selectors.xOfAKindMultiplier, 0 );
		}
		if ( this._stateMachine.phaseTimed( this._settings.growTime ) ) { }

		for ( var i = 0, len = this._domElements.length; i < len; i++ ) {
			if ( this._stateMachine.phaseOnce() ) {
				this._showMessage( this._domElements[i], i + 1 );
			}
			if ( this._stateMachine.phaseTimed( this._settings.growTime ) ) { }
		}

		if ( this._stateMachine.phaseOnce() ) {
			libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['HQ'], this._settings.soundMultiple );
		}

		if ( this._stateMachine.phaseTimed( this._settings.postMessageDelay ) ) { }

		if ( this._stateMachine.phaseOnce() ) {
			selectors.xOfAKindMultiplier.velocity( { 'opacity': 0 } );
			for ( i = 0, len = this._domElements.length; i < len; i++ ) {
				this._domElements[i].velocity( { 'opacity': 0 } );
			}
			libs.client.utils.DOM.fadeOut( selectors.xOfAKindContainer, 200 );
		}

		if ( this._stateMachine.phaseOnce() ) {
			this._endSplash();
		}
	};

	XOfAKindSplash.prototype._createText = function() {
		libs.client.utils.DOM.setText( selectors.xOfAKindMultiplier, this._numberOfWins + "X", null, this._settings.gradientText );
	};

	XOfAKindSplash.prototype._showMessage = function( text, cnt ) {
		if ( this._numberOfWins > 1 || cnt > 0 ) {
			text.velocity( { 'opacity': 1, 'delay': this._settings.growTime } );

			if ( this._shouldDoExplosions ) {
				this._explosionSpriteGroup.sprites[this._explosionCount].play();

				this._explosionSpriteGroup.sprites[this._explosionCount]
					.moveTo( this._explosionPositions[cnt] )
					.scaleTo( { x: this._explosionScale, y: this._explosionScale } )
					.fadeTo( { duration: 0, alpha: 1 } )
					.fadeTo( { duration: 500, alpha: 1 } )
					.fadeTo( { duration: 500, alpha: 0 } );

				this._explosionCount++;
				if ( this._explosionCount > 2 ) {
					this._explosionCount = 0;
				}
			}
		}
	};

	XOfAKindSplash.prototype._endSplash = function() {
		this._handleXOfAKindSplash.leave();
		this._handleXOfAKindSplash = null;

		this._stateMachine.stateGoto( null );

		this._numberOfWins = 0;

		libs.clearTimeoutTick(this._timeout);
		
		libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['HQ'], this._settings.soundMultiple );

		libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-xOfAKindSplash' );
		libs.globalEventBus.invoke( 'libs.event.request.disableCanvasOverlayMode' );

		libs.client.utils.DOM.forceDOMRedraw( libs.surfaceTop.jCanvas );

		selectors.xOfAKindMessageLineOne.velocity( { 'opacity': 0, 'delay': 0 } );
		for ( var i = 0, len = this._domElements.length; i < len; i++ ) {
			this._domElements[i].velocity( { 'opacity': 0, 'delay': 0 } );
		}
		libs.client.utils.DOM.fadeOut( selectors.xOfAKindContainer, 200 );
		for ( var i = 0; i < game.logic.GameData.reels.nreels; i++ ) {
			libs.globalEventBus.invoke( 'libs.event.request.stopForceReelFadedBackground', i );
		}
		selectors.xOfAKindContainer.fadeOut( 100 );
	};

	XOfAKindSplash.prototype.draw = function() {
		if ( this.isFeatureActive() ) {
			var ctx = libs.surfaceTop.ctx;
			this._explosionSpriteGroup.draw( ctx );
		}
	};

	XOfAKindSplash.prototype.isDirty = function() {
		return !!this._handleXOfAKindSplash;
	};

	XOfAKindSplash.prototype.isFeatureActive = function() {
		return !!this._handleXOfAKindSplash;
	};


	libs.client.reels.XOfAKindSplash = XOfAKindSplash;

}());