goog.provide( 'libs.client.reels.ReelsGameInit' );
goog.provide( 'libs.serverInstance' );

goog.require( 'libs.coreUtils' );
goog.require( 'libs.globalEventBus' );

goog.require( 'libs.client.core.DebugServerInterface' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.core.InputController' );
goog.require( 'libs.client.core.InputGestureController' );
goog.require( 'libs.client.core.Preloader' );
goog.require( 'libs.client.core.SystemStateController' );
goog.require( 'libs.client.core.UpdateManager' );

goog.require( 'libs.client.languages.LanguageController' );

goog.require( 'libs.client.displayHandler' );

goog.require( 'libs.client.rendering.SurfaceManager' );

goog.require( 'libs.client.presentation.DeviceCompatibilityDisplay' );
goog.require( 'libs.client.presentation.ErrorScreen' );
goog.require( 'libs.client.presentation.IntroVideo' );
goog.require( 'libs.client.presentation.LoadingScreen' );

goog.require( 'libs.client.reels.BetStateController' );
goog.require( 'libs.client.reels.ReelsStateController' );

goog.require( 'libs.client.sound.SoundPlayer' );
goog.require( 'libs.client.utils.HapticPlayer' );

// mobile only
// goog.require('libs.client.presentation.SoundSelection');
// goog.require('libs.client.rendering.NativeAndroidRenderingHacks');

// desktop only
// goog.require('libs.client.displayHandler.IFrameFocus');

// HillsVegas only
// goog.require('libs.client.hillsVegas.UrlChanger');


(function() {
	"use strict";

	var clientInfo = libs.client.core.clientInformation;
	var isMobileDeviceType = clientInfo.getDeviceType() === 'mobile';
	var isDesktopDeviceType = clientInfo.getDeviceType() === 'desktop';

	var ReelsGameInit = function( setupFlags, createServerFuncs, settings, manifestArray ) {
		var that = this;

		this._setupFlags = setupFlags;
		this._createServerFuncs = createServerFuncs;
		this._settings = settings;
		this._manifestArray = manifestArray;

		this._betRecoveryInfo = null;

		this._compatibilityPassed = true;
		this._gameInitFlowFinished = false;

		this._handleGameInit = null;
		this._handleServerInit = null;

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.systemInit', function( activity ) { that._onSystemInitStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.systemInit', function() { that._onSystemInitEnd(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.serverInit', function( activity ) { that._onServerInitStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gameInit', function( activity ) { that._onGameInitStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.compatibility', function() { that._onDisplayCompatibilityEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.soundSelection', function() { that._onDisplaySoundSelectionEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.introVideo', function() { that._onDisplayIntroVideoEnd(); } );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.gameInit', function() { that._onGameInitEnd(); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gamePlaying', function( activity ) { that._onGamePlayingStart( activity ); } );

		if ( this._setupFlags['buildType'] !== 'release' ) {
			this._debugServerInterface = new libs.client.core.DebugServerInterface( settings.libs.core.debugServerInterface, this._setupFlags );
		}

		if ( settings.libs.core.inputController ) {
			this._inputController = new libs.client.core.InputController( settings.libs.core.inputController );
		}
		if ( settings.libs.core.inputGestureController ) {
			this._inputGestureController = new libs.client.core.InputGestureController( settings.libs.core.inputGestureController );
		}
		if ( settings.libs.core.languageController ) {
			this._languageController = new libs.client.languages.LanguageController( settings.libs.core.languageController );
		}
		if ( settings.libs.core.preloader ) {
			this._preloader = new libs.client.core.Preloader( settings.libs.core.preloader );
		}
		if ( settings.libs.core.updateManager ) {
			this._updateManager = new libs.client.core.UpdateManager( settings.libs.core.updateManager );
		}

		if ( settings.libs.displayHandler ) {
			if ( isDesktopDeviceType ) {
				this._displayHandler = new libs.client.displayHandler.DesktopDisplayHandler( settings.libs.displayHandler );
			} else {
				this._displayHandler = new libs.client.displayHandler.MobileDisplayHandler( settings.libs.displayHandler );
			}
		}

		if ( isDesktopDeviceType ) {
			if ( settings.libs.iFrameFocus ) {
				this._iFrameFocus = new libs.client.displayHandler.IFrameFocus( settings.libs.iFrameFocus );
			}
		}

		if ( settings.libs.rendering.nativeAndroidRenderingHacks && isMobileDeviceType && clientInfo.isNativeAndroidBrowser() ) {
			this._nativeAndroidRenderingHacks = new libs.client.rendering.NativeAndroidRenderingHacks( settings.libs.rendering.nativeAndroidRenderingHacks );
		}

		if ( settings.libs.rendering.surfaceManager ) {
			this._surfaceManager = new libs.client.rendering.SurfaceManager( settings.libs.rendering.surfaceManager );
		}

		if ( settings.libs.presentation.deviceCompatibilityDisplay ) {
			this._deviceCompatibilityDisplay = new libs.client.presentation.DeviceCompatibilityDisplay( settings.libs.presentation.deviceCompatibilityDisplay );
		}

		if ( settings.libs.presentation.errorScreen ) {
			this._errorScreen = new libs.client.presentation.ErrorScreen( settings.libs.presentation.errorScreen );
		}

		if ( settings.libs.presentation.introVideo ) {
			this._introVideo = new libs.client.presentation.IntroVideo( settings.libs.presentation.introVideo );
		}

		if ( settings.libs.presentation.loadingScreen ) {
			this._loadingScreen = new libs.client.presentation.LoadingScreen( settings.libs.presentation.loadingScreen );
		}

		if ( settings.libs.presentation.soundSelection && isMobileDeviceType ) {
			this._soundSelection = new libs.client.presentation.SoundSelection( settings.libs.presentation.soundSelection );
		}

		if ( settings.libs.reels.betStateController ) {
			this._betStateController = new libs.client.reels.BetStateController( settings.libs.reels.betStateController );
		}

		if ( settings.libs.reels.reelsStateController ) {
			this._reelsStateController = new libs.client.reels.ReelsStateController( settings.libs.reels.reelsStateController );
		}

		if ( settings.libs.utils.hapticPlayer ) {
			this._hapticPlayer = new libs.client.utils.HapticPlayer( settings.libs.utils.hapticPlayer );
		}

		if ( settings.libs.sound.soundPlayer ) {
			this._soundPlayer = new libs.client.sound.SoundPlayer( settings.libs.sound.soundPlayer );
		}

		if ( this._setupFlags['platformType'] === 'hillsLive' || this._setupFlags['platformType'] === 'hillsLocal' ) {
			if ( isMobileDeviceType ) {
				if ( settings.libs.hillsVegas.urlChanger ) {
					this._urlChanger = new libs.client.hillsVegas.UrlChanger( settings.libs.hillsVegas.urlChanger );
				}
			}
		}

		// last as this kicks of the system state flow
		if ( settings.libs.core.systemStateController ) {
			this._systemStateController = new libs.client.core.SystemStateController( settings.libs.core.systemStateController );
		}
	};

	ReelsGameInit.prototype._onSystemInitStart = function( activity ) {
		if ( clientInfo.getDeviceType() === 'mobile' && clientInfo.requiresForceCSSHardwareAcceleration() ) {
			$( '#canvas-top' ).addClass( 'forceCSSHardwareAcceleration' );
			$( '#canvas-base' ).addClass( 'forceCSSHardwareAcceleration' );
			$( '#big-winner-container' ).addClass( 'forceCSSHardwareAcceleration' );
		}
	};

	ReelsGameInit.prototype._onSystemInitEnd = function() {
		$( window ).orientationchange();

		if ( BUILD_TYPE !== 'dev' ) {
			$( document ).bind( "contextmenu", function( e ) { e.preventDefault(); } );
		}

		$( '.preventTouchMove' ).on( 'touchmove', function( e ) { e.preventDefault(); } );
		$( 'body' ).on( 'touchmove', function( e ) { e.preventDefault(); } );

//		$(document).on('touchmove', function(e){
//			if (!$( e.target ).parents().hasClass('allowTouchMove')) {
//				e.preventDefault();
//			}
//		});

		$( 'img' ).on( 'dragstart', function( event ) { event.preventDefault(); } );

		libs.globalEventBus.invoke( 'libs.event.notify.manifestsToLoad', this._manifestArray );
		libs.globalEventBus.invokeActivity( 'libs.activity.display.compatibility' );

		// in meantime, sytemInit has ended, gameLoading will start and end, and when that has finished, move on serverInitStart
	};

	ReelsGameInit.prototype._onDisplayCompatibilityEnd = function() {
		this._compatibilityPassed = true;
		this._checkIfOKToLeaveGameInit();
	};

	ReelsGameInit.prototype._onServerInitStart = function( activity ) {
		var that = this;

		this._handleServerInit = activity.join();

		var serverCreateFunc = this._createServerFuncs[this._setupFlags['platformType']];
		if ( !serverCreateFunc || ( typeof serverCreateFunc !== 'function' ) ) {
			throw new Error( "Invalid platformType specified in game init setup flags! '" + this._setupFlags['platformType'] + "'" );
		}

		libs.serverInstance = serverCreateFunc();
		libs.serverInstance.onInit( function( betRecovery, playerPersistentState ) {that._onServerConnected( betRecovery, playerPersistentState );} );
		libs.serverInstance.connect();
	};

	ReelsGameInit.prototype._onServerConnected = function( betRecovery, playerPersistentState ) {
		this._betRecoveryInfo = betRecovery;

		libs.globalEventBus.invoke( 'libs.event.notify.isDebugMode', this._setupFlags['enableDebugMode'] );
		libs.globalEventBus.invoke( 'libs.event.notify.platformType', this._setupFlags['platformType'] );
		libs.globalEventBus.invoke( 'libs.event.notify.playerPersistentData', playerPersistentState );

		this._handleServerInit.leave();
	};

	ReelsGameInit.prototype._onGameInitStart = function( activity ) {
		this._handleGameInit = activity.join();
		libs.globalEventBus.invokeActivity( 'libs.activity.display.soundSelection' );
	};

	ReelsGameInit.prototype._onDisplaySoundSelectionEnd = function() {
		libs.setImmediate( function() {
			libs.globalEventBus.invokeActivity( 'libs.activity.display.introVideo' );
		} );
	};

	ReelsGameInit.prototype._onDisplayIntroVideoEnd = function() {
		this._gameInitFlowFinished = true;
		this._checkIfOKToLeaveGameInit();
	};

	ReelsGameInit.prototype._onGamePlayingStart = function( activity ) {
		$( window ).orientationchange();
	};

	ReelsGameInit.prototype._onGameInitEnd = function() {
		libs.globalEventBus.invoke( 'libs.event.notify.isDebugMode', this._setupFlags['enableDebugMode'] );
		libs.globalEventBus.invoke( 'libs.event.notify.platformType', this._setupFlags['platformType'] );

		libs.globalEventBus.invoke( 'libs.event.notify.betRecoveryInfo', this._betRecoveryInfo );

		this._deviceCompatibilityDisplay = null;
		this._loadingScreen = null;
	};

	ReelsGameInit.prototype._checkIfOKToLeaveGameInit = function( activity ) {
		var that = this;

		libs.setImmediate( function() {
			if ( that._compatibilityPassed && that._gameInitFlowFinished ) {
				that._handleGameInit.leave();
				that._handleGameInit = null;
			}
		} );
	};


	libs.client.reels.ReelsGameInit = ReelsGameInit;

}());