goog.provide( 'libs.client.reels.ReelsMachineAnimationBlurred' );

(function() {
	"use strict";

	var ReelsMachineAnimationBlurred = function( overrideSettings, reelModels ) {

		var that = this;
		this._settings = overrideSettings;
		this._spinParameters = {};
		this._reelModels = reelModels;
		this._isQuickPlay = false;
		this._betResult = null;
		this._animationStateMachine = new libs.StateMachine();
		this._blurredSpinTimeout = new libs.Timeout();

		libs.globalEventBus.connect( 'libs.event.notify.quickPlayState', function( enabled ) {
			that._isQuickPlay = enabled;
		} );
		libs.globalEventBus.connect( 'libs.event.notify.spinParameters', function( spinParameters ) {
			that._onNotifySpinParameters( spinParameters );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) { that._onNotifyBetResult( betResult ); } );
	};

	ReelsMachineAnimationBlurred.prototype._onNotifySpinParameters = function( spinParameters ) {
//		All of the options in spinParameters have fallback defaults so only specify the ones you wish to change. Here are all of them to use
//		Use the 'reelSpinSymbolsUntilLanding' and 'reelBlurStopTimeGap' to implement anticipation spins (long spins), for the 'real' spin and 'blurred' spin type respectively.
//		var spinParameters = { noPullBack: noPullBack, noBounce: false, maxSpeed: 2.8, acceleration: 0.004, bounceDistance: 30, bounceBackTime: 200, pullbackTime: 200, pullbackSpeed: 0.3,
//			reelBlurWaitTime: 1000, reelSpinSymbolsUntilLanding: [ 0, 8, 16, 24, 32 ], reelBlurStopTimeGap: [ 100, 100, 100, 100, 100 ], minimumSpinTime: 0, reverseReelSpin: false,
//      lockedReels: [0,1,2,3,4] };
		this._spinParameters = spinParameters;
	};

	ReelsMachineAnimationBlurred.prototype.startSpin = function() {
		var that = this;
		this._animationStateMachine.stateGoto( function( timeDelta ) {
			that._stateStartSpin( timeDelta );
		} );
	};

	ReelsMachineAnimationBlurred.prototype.abortAnim = function() {
		this._animationStateMachine.stateGoto( null );
	};

	ReelsMachineAnimationBlurred.prototype.update = function( timeDelta ) {
		this._animationStateMachine.update( timeDelta );
		this._blurredSpinTimeout.update( timeDelta );
	};

	ReelsMachineAnimationBlurred.prototype._stateStartSpin = function( timeDelta ) {
		var reelIndex;

		for ( reelIndex = 0; reelIndex < this._reelModels.length; ++reelIndex ) {
			if ( this._animationStateMachine.phaseOnce() ) {
				// stagger reel start times
				this._reelModels[reelIndex].startBlurredSpin();
			}

			if ( this._animationStateMachine.phaseTimed( this._spinParameters.reelSpinTimeGap ) ) {
			}
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			this._blurredSpinTimeout.reset( this._spinParameters.reelBlurWaitTime );
		}

		if ( this._animationStateMachine.phaseManual() ) {
			// wait for timeout to expire
			if ( this._blurredSpinTimeout.expired() ) {
				this._animationStateMachine.phaseStep();
				libs.globalEventBus.invoke( 'libs.event.notify.reelsAtSpeed', [] );
			}
		}
	};


	ReelsMachineAnimationBlurred.prototype._onNotifyBetResult = function( betResult ) {
		this._betResult = betResult;
		this._reelModels.forEach( function( reelModel ) { reelModel.switchReelSet( betResult['reels']['reelset'] ); } );
	};


	ReelsMachineAnimationBlurred.prototype.completeSpin = function() {
		var that = this;
		this._animationStateMachine.stateGoto( function( timeDelta ) {
			that._stateCompleteSpin( timeDelta );
		} );
	};


	ReelsMachineAnimationBlurred.prototype._stateCompleteSpin = function( timeDelta ) {

		for ( var reelIndex = 0; reelIndex < this._reelModels.length; ++reelIndex ) {
			// stop all the reels
			var stopWaitTime = 100;
			if ( !this._isQuickPlay ) {
				if ( isFinite( this._spinParameters.reelBlurStopTimeGap ) ) {
					stopWaitTime = this._spinParameters.reelBlurStopTimeGap;
				} else if ( this._spinParameters.reelBlurStopTimeGap && this._spinParameters.reelBlurStopTimeGap.length > reelIndex ) {
					stopWaitTime = this._spinParameters.reelBlurStopTimeGap[reelIndex];
				}
			} else {
				stopWaitTime = 0;
			}

			if ( this._animationStateMachine.phaseTimed( stopWaitTime ) ) {
			}

			if ( this._animationStateMachine.phaseOnce() ) {
				this._spinReelStop( reelIndex );
			}
		}

		if ( this._isQuickPlay ) {
			this._reelModels.forEach( function( reelModel ) { reelModel.jump(); } );
		}

		if ( this._animationStateMachine.phaseManual() ) {
			// wait for all reels to land
			var atLeastOneIsStillSpinning = this._reelModels.some( function( reelModel ) { return reelModel.isSpinning(); } );
			if ( !atLeastOneIsStillSpinning ) {
				this._animationStateMachine.phaseStep();
			}
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			libs.globalEventBus.invoke( 'libs.event.notify.allReelsStopped' );
		}
	};


	ReelsMachineAnimationBlurred.prototype._spinReelStop = function( reelID ) {
		var reelModel = this._reelModels[reelID];
		reelModel.blurredStop();
	};


	libs.client.reels.ReelsMachineAnimationBlurred = ReelsMachineAnimationBlurred;

}());