goog.provide( 'libs.client.reels.ReelsMachineAnimationNormal' );

(function() {
	"use strict";


	var ReelsMachineAnimationNormal = function( overrideSettings, reelModels ) {

		var that = this;
		this._settings = overrideSettings;
		this._reelModels = reelModels;
		this._spinParameters = {};
		this._isQuickPlay = false;
		this._betResult = null;
		this._animationStateMachine = new libs.StateMachine();
		libs.globalEventBus.connect( 'libs.event.notify.quickPlayState', function( enabled ) {
			that._isQuickPlay = enabled;
		} );
		libs.globalEventBus.connect( 'libs.event.notify.spinParameters', function( spinParameters ) {
			that._onNotifySpinParameters( spinParameters );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) { that._onNotifyBetResult( betResult ); } );
	};

	ReelsMachineAnimationNormal.prototype._onNotifySpinParameters = function( spinParameters ) {
//		All of the options in spinParameters have fallback defaults so only specify the ones you wish to change. Here are all of them to use
//		Use the 'reelSpinSymbolsUntilLanding' and 'reelBlurStopTimeGap' to implement anticipation spins (long spins), for the 'real' spin and 'blurred' spin type respectively.
//		var spinParameters = { noPullBack: noPullBack, noBounce: false, maxSpeed: 2.8, acceleration: 0.004, bounceDistance: 30, bounceBackTime: 200, pullbackTime: 200, pullbackSpeed: 0.3,
//			reelBlurWaitTime: 1000, reelSpinSymbolsUntilLanding: [ 0, 8, 16, 24, 32 ], reelBlurStopTimeGap: [ 100, 100, 100, 100, 100 ], minimumSpinTime: 0, reverseReelSpin: false,
//      lockedReels: [0,1,2,3,4] };
		this._spinParameters = spinParameters;
		if ( this._spinParameters.switchReelSet ) {
			// force immediate reel set switch
			this._switchReelSet();
		}
	};

	ReelsMachineAnimationNormal.prototype.startSpin = function() {
		var that = this;
		this._animationStateMachine.stateGoto( function( timeDelta ) {
			that._stateStartSpin( timeDelta );
		} );
	};

	ReelsMachineAnimationNormal.prototype.abortAnim = function() {
		this._animationStateMachine.stateGoto( null );
	};

	ReelsMachineAnimationNormal.prototype.update = function( timeDelta ) {
		this._animationStateMachine.update( timeDelta );
	};

	ReelsMachineAnimationNormal.prototype._shouldReelSkip = function( reelIndex ) {
		var skipForbidden = this._spinParameters.noSkip || ( this._spinParameters.noSkipReels && this._spinParameters.noSkipReels[reelIndex] );
		var shouldSkip = !skipForbidden && this._isQuickPlay;
		return shouldSkip;
	};

	ReelsMachineAnimationNormal.prototype._shouldReelHeavyDrop = function( reelIndex ) {
		return this._spinParameters.heavyDropAnimation && this._spinParameters.heavyDropAnimation.oldPositions && this._spinParameters.heavyDropAnimation.oldPositions[reelIndex] >= 0;
	};

	ReelsMachineAnimationNormal.prototype._isReelLocked = function( index ) {
		if ( this._spinParameters.lockedReels && this._spinParameters.lockedReels.indexOf( index ) > -1 ) {
			return true;
		}
		return false;
	};

	ReelsMachineAnimationNormal.prototype._stateStartSpin = function( timeDelta ) {

		var that = this;
		for ( var reelIndex = 0; reelIndex < this._reelModels.length; ++reelIndex ) {

			if ( this._animationStateMachine.phaseOnce() ) {
				// stagger reel start times
				if ( !this._shouldReelHeavyDrop( reelIndex ) ) {
					if ( !this._isReelLocked( reelIndex ) ) {
						this._reelModels[reelIndex].startReelSpin();
					}
				}
			}

			if ( this._spinParameters.reelSpinTimeGap > 0 ) {
				if ( this._animationStateMachine.phaseTimed( this._spinParameters.reelSpinTimeGap ) ) {
				}
			}
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			// start heavy drops all at the same time
			for ( var reelIndex2 = 0; reelIndex2 < this._reelModels.length; ++reelIndex2 ) {
				if ( this._shouldReelHeavyDrop( reelIndex2 ) ) {
					this._reelModels[reelIndex2].startHeavyDropSpin();
				}
			}
		}

		if ( this._animationStateMachine.phaseManual() ) {
			// make sure all reels are up to full speed, ready to stop
			var allFullSpeed = this._reelModels.every( function( reelModel, reelIndex ) {
				if ( !that._isReelLocked( reelIndex ) ) {
					return reelModel.isFullSpeed();
				} else {
					return true;
				}
			} );
			if ( allFullSpeed || this._isQuickPlay ) {
				libs.globalEventBus.invoke( 'libs.event.notify.reelsAtSpeed', [] );
				this._animationStateMachine.phaseStep();
			}
		}
	};


	ReelsMachineAnimationNormal.prototype._onNotifyBetResult = function( betResult ) {
		this._betResult = betResult;
		if ( !this._betResult['noReelSetSwitch'] ) {
			this._switchReelSet();
		}
	};


	ReelsMachineAnimationNormal.prototype._switchReelSet = function() {
		var that = this;
		if ( this._betResult['reels'] && isFinite( this._betResult['reels']['reelset'] ) ) {
			this._reelModels.forEach( function( reelModel ) {
				reelModel.switchReelSet( that._betResult['reels']['reelset'] );
			} );
		}
	};


	ReelsMachineAnimationNormal.prototype.completeSpin = function( betResult ) {
		var that = this;
		// tell all reels to stop
		for ( var reelIndex = 0; reelIndex < this._reelModels.length; ++reelIndex ) {
			if ( !this._isReelLocked( reelIndex ) ) {
				this._spinReelStop( reelIndex );
			}
		}
		this._animationStateMachine.stateGoto( function( timeDelta ) {
			that._stateCompleteSpin( timeDelta );
		} );
	};


	ReelsMachineAnimationNormal.prototype._stateCompleteSpin = function( timeDelta ) {

		var reelIndex;
		if ( this._animationStateMachine.phaseManual() ) {
			// wait for all reels to land
			var allFinished = true;
			for ( reelIndex = 0; reelIndex < this._reelModels.length; ++reelIndex ) {
				if ( !this._isReelLocked( reelIndex ) ) {
					var reelModel = this._reelModels[reelIndex];
					if ( reelModel.isSpinning() ) {
						allFinished = false;
					}
					// check if we should turbo the reel if quick spin has been pressed
					if ( this._shouldReelSkip( reelIndex ) ) {
						reelModel.jump();
					}
				}
			}
			if ( allFinished ) {
				this._animationStateMachine.phaseStep();
			}
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			return; // Wait a frame for the 'libs.activity.reelsState.reelsLanding' activity to start and end if in turbo mode
		}

		if ( this._animationStateMachine.phaseOnce() ) {
			libs.globalEventBus.invoke( 'libs.event.notify.allReelsStopped' );
		}
	};


	ReelsMachineAnimationNormal.prototype._spinReelStop = function( reelIndex ) {

		this._reelModels[reelIndex].spinToStop();
	};


	libs.client.reels.ReelsMachineAnimationNormal = ReelsMachineAnimationNormal;

}());


