goog.provide( 'libs.client.reels.ReelsStateEndOfReelsGameModals' );

goog.require( 'libs.globalEventBus' );


/** @module ReelsStateEndOfReelsGameModals */
(function() {
	"use strict";

	var defaultSettings = {
		modalEvents: [
			'libs.activity.display.bigWinner'
		]
	};

	/**
	 * A class that allows the game to flows through available game modals shown at the end of a game cycle
	 * @constructor
	 * @static
	 */
	var ReelsStateEndOfReelsGameModals = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._currentModal = 0;

		this._handleModalDisplay = null;
		this._handleModalVisible = null;

		for ( var i = 0; i < this._settings.modalEvents.length; ++i ) {
			libs.globalEventBus.connectActivityEnd( this._settings.modalEvents[i], function( activity ) { that._onSingleModalEnd( activity ); } );
		}

		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.endOfReelsGameModals', function( activity ) { that._onEndOfReelsGameModals( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.modalVisible', function( activity ) { that._onModalVisibleStart( activity ); } );
	};

	/**
	 * Joins the endOfReelsGameModals event if there are modals to display
	 * @private
	 * @function _onEndOfReelsGameModals
	 * @param {object} activity
	 */
	ReelsStateEndOfReelsGameModals.prototype._onEndOfReelsGameModals = function( activity ) {
		if ( this._settings.modalEvents.length > 0 ) {
			this._handleModalDisplay = activity.join();

			this._currentModal = 0;
			libs.globalEventBus.invokeActivity( this._settings.modalEvents[this._currentModal] );
		}
	};

	/**
	 * Joins the modalVisible event if a modal has accepted they want to display. This component keeps that handle;
	 * useful to drawing components can listen to this event to know when to draw or not rathar than endOfReelsGameModals
	 * as the later will ALWAYS be started and ended every game cycle, but modalVisible only if a modal was required
	 *
	 * @private
	 * @function _onModalVisibleStart
	 * @param {object} activity
	 */
	ReelsStateEndOfReelsGameModals.prototype._onModalVisibleStart = function( activity ) {
		this._handleModalVisible = activity.join();
	};

	/**
	 * Called when a modal activity from the settings list has ended. If it is the final modal, it will leave the handles
	 * to make the endOfReelsGameModals and modalVisible events end; otherwise it will start the next modal.
	 *
	 * @private
	 * @function _onSingleModalEnd
	 * @param {object} activity
	 */
	ReelsStateEndOfReelsGameModals.prototype._onSingleModalEnd = function( activity ) {
		var that = this;

		++this._currentModal;

		libs.setImmediate( function() {
			if ( that._currentModal < that._settings.modalEvents.length ) {
				libs.globalEventBus.invokeActivity( that._settings.modalEvents[that._currentModal] );
			} else {
				if ( that._handleModalVisible ) {
					that._handleModalVisible.leave();
					that._handleModalVisible = null;
				}
				that._handleModalDisplay.leave();
				that._handleModalDisplay = null;
			}
		} );
	};


	libs.client.reels.ReelsStateEndOfReelsGameModals = ReelsStateEndOfReelsGameModals;

}());