goog.provide( 'libs.client.reels.ReelsStateController' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.reels.ReelsStateEndOfReelsGameModals' );


(function() {
	"use strict";

	var defaultSettings = {
		endOfReelsGameModals: []
	};

	var ReelsStateController = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;
		this._reelStatesEnabled = true;

		this._reelsStateEndOfReelsGameModals = new libs.client.reels.ReelsStateEndOfReelsGameModals( this._settings.endOfReelsGameModals );

		this._handlePlayingBet = null;
		this._handleReelsStarting = null;

		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.preBet', function( activity ) {
			that._onPreBetStart( activity );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.playingBet', function( activity ) {
			that._onPlayingBetStart( activity );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.reelsStarting', function( activity ) {
			that._onReelsStartingStart( activity );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.preReelsLanding', function() {
			that._onPreReelsLandingEnd();
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.reelsLanding', function() {
			that._onReelsLandingEnd();
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.postReelsLanding', function() {
			that._onPostReelsLandingEnd();
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.preDisplayReelsWin', function() {
			that._onPreDisplayReelsWinEnd();
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.displayReelsWin', function() {
			that._onDisplayReelsWinEnd();
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.postDisplayReelsWin', function() {
			that._onPostDisplayReelsWinEnd();
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.reelsBonusFeature', function() {
			that._onReelsBonusFeatureEnd();
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.reelsState.endOfReelsGameModals', function() {
			that._onEndOfReelsGameModals();
		} );

		libs.globalEventBus.connect( 'libs.event.request.disableReelStates', function() {
			that._onRequestDisableReelStates();
		} );
		libs.globalEventBus.connect( 'libs.event.request.enableReelStates', function() {
			that._onRequestEnableReelStates();
		} );
	};

	ReelsStateController.prototype._onPreBetStart = function( activity ) {
		if ( this._reelStatesEnabled ) {
			this._invokeActivity( 'libs.activity.reelsState.reelsStarting' );
		}
	};

	ReelsStateController.prototype._onReelsStartingStart = function( activity ) {
		this._handleReelsStarting = activity.join();
	};

	ReelsStateController.prototype._onPlayingBetStart = function( activity ) {
		if ( this._reelStatesEnabled ) {
			this._handleReelsStarting.leave();
			this._handleReelsStarting = null;
			this._invokeActivity( 'libs.activity.reelsState.preReelsLanding' );
			this._handlePlayingBet = activity.join();
		}
	};

	ReelsStateController.prototype._onPreReelsLandingEnd = function() {
		this._invokeActivity( 'libs.activity.reelsState.reelsLanding' );
	};

	ReelsStateController.prototype._onReelsLandingEnd = function() {
		this._invokeActivity( 'libs.activity.reelsState.postReelsLanding' );
	};

	ReelsStateController.prototype._onPostReelsLandingEnd = function() {
		this._invokeActivity( 'libs.activity.reelsState.preDisplayReelsWin' );
	};

	ReelsStateController.prototype._onPreDisplayReelsWinEnd = function() {
		this._invokeActivity( 'libs.activity.reelsState.displayReelsWin' );
	};

	ReelsStateController.prototype._onDisplayReelsWinEnd = function() {
		this._invokeActivity( 'libs.activity.reelsState.postDisplayReelsWin' );
	};

	ReelsStateController.prototype._onPostDisplayReelsWinEnd = function() {
		this._invokeActivity( 'libs.activity.reelsState.reelsBonusFeature' );
	};

	ReelsStateController.prototype._onReelsBonusFeatureEnd = function() {
		this._invokeActivity( 'libs.activity.reelsState.endOfReelsGameModals' );
	};

	ReelsStateController.prototype._onEndOfReelsGameModals = function() {
		if ( this._handlePlayingBet ) {
			this._handlePlayingBet.leave();
		}
		this._handlePlayingBet = null;
	};

	ReelsStateController.prototype._invokeActivity = function( activityName ) {
		// makes sure all calls on an activity end are processed before a new activity starts
		libs.setImmediate( function() {
			libs.globalEventBus.invokeActivity( activityName );
		} );
	};

	ReelsStateController.prototype._onRequestDisableReelStates = function() {
		this._reelStatesEnabled = false;
	};

	ReelsStateController.prototype._onRequestEnableReelStates = function() {
		this._reelStatesEnabled = true;
	};

	libs.client.reels.ReelsStateController = ReelsStateController;

}());