goog.provide( 'libs.client.reels.WinlinesModel' );

goog.require( 'libs.globalEventBus' );

goog.require( 'game.logic' );


(function() {
	"use strict";

	var defaultSettings = {};

	var WinlinesModel = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this.lineMapping = game.logic.GameData.reels.winLineDefinitionArray;
		this.numLines = this.lineMapping.length;
		this.lines = [];

		for ( var i = 0; i < this.numLines; ++i ) {
			var line = { visible: false, dirty: false, run: 0 };
			this.lines.push( line );
		}

		libs.globalEventBus.connect( 'libs.event.request.showWinlines', function( linesInfo, positionInRun ) { that._onRequestShowWinlines( linesInfo, positionInRun ); } );
	};

	WinlinesModel.prototype._onRequestShowWinlines = function( linesInfo, positionInRun ) {
		var that = this;
		var win = new BigNumber( 0 );

		var anyLineVisible = function() {
			return that.lines.some( function( line ) {return line.visible;} );
		};

		if ( anyLineVisible() ) {
			libs.globalEventBus.invoke( 'libs.event.notify.winlineHidden' );
		}

		this._hideAllLines();
		for ( var i = 0; i < linesInfo.length; ++i ) {
			this._showLine( linesInfo[i] );
			win = win.plus( linesInfo[i]['win'] );
		}

		if ( anyLineVisible() ) {
			libs.globalEventBus.invoke( 'libs.event.notify.winlineShown', positionInRun, win );
		}
	};

	WinlinesModel.prototype._showLine = function( lineInfo ) {
		this.lines[lineInfo['line']].visible = true;
		this.lines[lineInfo['line']].dirty = true;
		this.lines[lineInfo['line']].run = lineInfo['run'];
	};

	WinlinesModel.prototype._hideAllLines = function() {
		this.lines.forEach( function( line ) {
			line['visible'] = false;
			line['run'] = 0;
		} );
	};


	libs.client.reels.WinlinesModel = WinlinesModel;

}());