goog.provide( 'libs.client.rendering.CanvasText' );


(function() {
	"use strict";

	// be aware - some android devices (like HTC One) do not respect the max boundaries
	var CanvasText = function( text, textSettings ) {
		this._text = text;
		this._boundaries = textSettings !== undefined && textSettings.boundaries !== undefined ? textSettings.boundaries : 0;
		this._colours = textSettings !== undefined && textSettings.colours !== undefined ? textSettings.colours : ["#FFFFFF"];
		this._font = textSettings !== undefined && textSettings.font !== undefined ? textSettings.font : '40px Arial';

		this._isDirty = true;
	};

	CanvasText.prototype.setText = function( newText ) {
		if ( newText !== undefined ) {
			this._text = newText;
			this._isDirty = true;
		}
	};

	CanvasText.prototype.setColours = function( newColours ) {
		if ( newColours !== undefined ) {
			this._colours = newColours;
			this._isDirty = true;
		}
	};

	CanvasText.prototype.setFont = function( newFont ) {
		if ( newFont !== undefined ) {
			this._font = newFont;
			this._isDirty = true;
		}
	};

	CanvasText.prototype.moveTo = function( newPosition ) {
		if ( newPosition !== undefined ) {

			this._boundaries = {
				x: newPosition.x || this._boundaries.x,
				y: newPosition.y || this._boundaries.y,
				width: this._boundaries.width,
				height: this._boundaries.height
			};

			this._isDirty = true;
		}
	};

	CanvasText.prototype.moveBy = function( newPosition ) {
		if ( newPosition !== undefined ) {
			this._boundaries.x += newPosition.x || 0;
			this._boundaries.y += newPosition.y || 0;
			this._isDirty = true;
		}
	};

	CanvasText.prototype.draw = function( ctx ) {

		var centerX = this._boundaries.x + (this._boundaries.width / 2);
		var centerY = this._boundaries.y + (this._boundaries.height / 2);

		ctx.save();

		var numOfColours = this._colours.length;
		if ( numOfColours > 1 ) {
			var grd = ctx.createLinearGradient( centerX, this._boundaries.y - (this._boundaries.height / 2), centerX, centerY );

			var highestIndex = numOfColours - 1;
			for ( var i = 0; i < numOfColours; ++i ) {
				var distance = i / highestIndex;
				grd.addColorStop( distance, this._colours[i] );
			}
			ctx.fillStyle = grd;
		} else {
			ctx.fillStyle = this._colours[0];
		}

		ctx.textBaseline = "center";
		ctx.textAlign = "center";
		ctx.font = this._font;

		ctx.fillText( this._text, centerX, centerY, this._boundaries.width );
		ctx.restore();

		this._isDirty = false;
	};

	CanvasText.prototype.isDirty = function() {
		return this._isDirty;
	};

	CanvasText.prototype.drawIfDirty = function( ctx ) {
		if ( this._isDirty ) {
			this.draw( ctx );
		}
	};


	libs.client.rendering.CanvasText = CanvasText;

}());