goog.provide( 'libs.client.rendering.imageManipulations' );


(function() {
	'use strict';

	var imageManipulation = {

		/**
		 * Change image color to passed RBG value
		 * @param image
		 * @param data - must contain: color: { r: 0, g: 0, b: 0 }
		 * @returns {HTMLElement|*}
		 */
		changeColor: function( image, data ) {
			var canvas = document.createElement( "canvas" );
			var ctx = canvas.getContext( '2d' );

			canvas.width = data.width;
			canvas.height = data.height;

			ctx.drawImage( image, 0, 0 );

			var imgData = ctx.getImageData( 0, 0, data.width, data.height );

			var imgDataLen = imgData.data.length;
			for ( var i = 0; i < imgDataLen; i += 4 ) {
				imgData.data[i] = data.color.r;
				imgData.data[i + 1] = data.color.g;
				imgData.data[i + 2] = data.color.b;
			}

			ctx.putImageData( imgData, 0, 0 );

			return canvas;
		},


		/**
		 * Flips image in given direction.
		 * @param image
		 * @param data - must contain: flipDirection: a string, either 'horizontal', 'vertical' or 'both'
		 * @returns {HTMLElement|*}
		 */
		flip: function( image, data, allImageData ) {

			if ( data.untrimmedSize === undefined ) {
				throw new Error( "libs.client.rendering.imageManipulations.flip requires untrimmedSize parameter - update your JSON manifest to include this!" );
			}

			var canvas = document.createElement( "canvas" );
			var ctx = canvas.getContext( '2d' );

			canvas.width = data.width;
			canvas.height = data.height;

			var flipDirection = data.flipDirection || allImageData.flipDirection;

			ctx.save();
			if ( flipDirection === 'horizontal' ) {
				data.trimmedOffsetX = data.untrimmedSize.width - ( data.trimmedOffsetX + data.width );
				ctx.translate( data.width, 0 );
				ctx.scale( -1, 1 );
			} else if ( flipDirection === 'vertical' ) {
				data.trimmedOffsetY = data.untrimmedSize.height - ( data.trimmedOffsetY + data.height );
				ctx.translate( 0, data.height );
				ctx.scale( 1, -1 );
			} else if ( flipDirection === 'both' ) {
				data.trimmedOffsetX = data.untrimmedSize.width - ( data.trimmedOffsetX + data.width );
				data.trimmedOffsetY = data.untrimmedSize.height - ( data.trimmedOffsetY + data.height );
				ctx.translate( data.width, 0 );
				ctx.translate( 0, data.height );
				ctx.scale( -1, -1 );
			} else {
				throw new Error( "Invalid argument passed to libs.client.rendering.imageManipulations.flip: '" + flipDirection + "'" );
			}
			ctx.drawImage( image, data.x, data.y, data.width, data.height, 0, 0, data.width, data.height );
			ctx.restore();

			return canvas;
		}
	};

	libs.client.rendering.imageManipulations = imageManipulation;

}());