goog.provide( 'libs.client.rendering.LegacySprite' );

goog.require( 'libs.StateMachine' );


(function() {
	"use strict";

	// single image on a canvas
	var LegacySprite = function( src, drawSettings ) {
		this._src = src;

		if ( src.type === "metagu_sub_image" ) {
			this.image = src.image;
			this._isSub = true;
		} else {
			this.image = src;
			this._isSub = false;
		}

		this.width = src.width;
		this.height = src.height;

		this.x = drawSettings !== undefined && drawSettings.x !== undefined ? drawSettings.x : 0;
		this.y = drawSettings !== undefined && drawSettings.y !== undefined ? drawSettings.y : 0;

		this.scalex = drawSettings !== undefined && drawSettings.scalex !== undefined ? drawSettings.scalex : 1.0;
		this.scaley = drawSettings !== undefined && drawSettings.scaley !== undefined ? drawSettings.scaley : 1.0;

		this.alpha = drawSettings !== undefined && drawSettings.alpha !== undefined ? drawSettings.alpha : 1.0;

		this._fadeTime = 0;

		this._fadeStateMachine = new libs.StateMachine();

		this.dirtyFlag = true;
	};

	LegacySprite.prototype.update = function( timeDelta ) {
		this._fadeStateMachine.update( timeDelta );
	};

	LegacySprite.prototype._stateFadeIn = function() {
		if ( this._fadeStateMachine.phaseTimed( this._fadeTime ) ) {
			this.dirtyFlag = true;
			this.alpha = this._fadeStateMachine.phaseTimedGetProgress();
		}
	};

	LegacySprite.prototype._stateFadeOut = function() {
		if ( this._fadeStateMachine.phaseTimed( this._fadeTime ) ) {
			this.dirtyFlag = true;
			this.alpha = this._fadeStateMachine.phaseTimedGetRevProgress();
		}
	};

	LegacySprite.prototype.draw = function( ctx ) {
		this.dirtyFlag = false;

		if ( this.alpha > 0 ) {
			if ( this.alpha < 1 ) {
				ctx.save();
				ctx.globalAlpha = this.alpha;
			}

			if ( this._isSub ) {
				ctx.drawImage( this.image,
					this._src.x, this._src.y, this.width, this.height,
					this.x, this.y, this.width * this.scalex, this.height * this.scaley );

			} else {
				ctx.drawImage( this.image, 0, 0, this.width, this.height, this.x, this.y, this.width * this.scalex, this.height * this.scaley );
			}

			if ( this.alpha < 1 ) {
				ctx.restore();
			}
		}
	};

	LegacySprite.prototype.drawImage = function( ctx, srcX, srcY, srcWidth, srcHeight, x, y, width, height ) {
		this.dirtyFlag = false;

		if ( this.alpha > 0 ) {
			if ( this.alpha < 1 ) {
				ctx.save();
				ctx.globalAlpha = this.alpha;
			}

			x = x !== undefined ? x : srcX;
			y = y !== undefined ? y : srcY;
			width = width !== undefined ? width : srcWidth;
			height = height !== undefined ? height : srcHeight;

			ctx.drawImage( this.image,
				srcX, srcY, srcWidth, srcHeight,
				x, y, width, height );

			if ( this.alpha < 1 ) {
				ctx.restore();
			}
		}
	};

	LegacySprite.prototype.drawIfDirty = function( ctx ) {
		if ( this.dirtyFlag ) {
			this.draw( ctx );
		}
	};

	LegacySprite.prototype.move = function( x, y ) {
		this.clear = {
			x: this.x,
			y: this.y,
			width: this.width,
			height: this.height
		};
		this.x += x;
		this.y += y;
		this.dirtyFlag = true;
	};

	LegacySprite.prototype.fadeIn = function( time ) {
		var that = this;
		this._fadeTime = time || 1000;
		this._fadeStateMachine.stateGoto( function() { that._stateFadeIn();} );
	};

	LegacySprite.prototype.fadeOut = function( time ) {
		var that = this;
		this._fadeTime = time || 1000;
		this._fadeStateMachine.stateGoto( function() { that._stateFadeOut();} );
	};

	LegacySprite.prototype.clone = function() {
		var returned;

		returned = new LegacySprite( this._src, undefined );
		returned.x = this.x;
		returned.y = this.y;

		return returned;
	};

	LegacySprite.prototype.isDirty = function() {
		return this.dirtyFlag;
	};


	libs.client.rendering.LegacySprite = LegacySprite;

}());