goog.provide( 'libs.client.rendering.NativeAndroidRenderingHacks' );

goog.require( 'libs.client.core.clientInformation' );


(function() {
	"use strict";

	var defaultSettings = {};

	var clientInfo = libs.client.core.clientInformation;
	var isMobileDeviceType = clientInfo.getDeviceType() === 'mobile';
	var isNativeAndroid = clientInfo.isNativeAndroidBrowser();

	var NativeAndroidRenderingHacks = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.error', function() { that._onDisplayErrorEnd(); } );
		libs.globalEventBus.connect( 'libs.event.notify.orientationChanged', function( orientation ) { that._onNotifyOrientationChanged( orientation ); } );
		libs.globalEventBus.connect( 'libs.event.request.startBet', function( options ) { that._onRequestStartBet( options ); } );
		libs.globalEventBus.connect( 'libs.event.notify.reelStopped', function( reelIndex, finalSymbols ) { that._onNotifyReelStopped( reelIndex, finalSymbols ); } );
		libs.globalEventBus.connect( 'libs.event.request.enableCanvasOverlayMode', function() { that._onRequestEnableCanvasOverlayMode(); } );
	};

	NativeAndroidRenderingHacks.prototype._onDisplayErrorEnd = function() {
		if ( libs.globalEventBus.isActivityRunning( 'libs.activity.systemState.gamePlaying' ) ) {
			libs.globalEventBus.invoke( 'libs.event.request.enableZIndexHack' );
		}
	};

	NativeAndroidRenderingHacks.prototype._onNotifyOrientationChanged = function( orientation ) {
		if ( libs.globalEventBus.isActivityRunning( 'libs.activity.systemState.gamePlaying' ) ) {
			if ( isMobileDeviceType ) {
				if ( orientation === 'landscape' ) {
					libs.globalEventBus.invoke( 'libs.event.request.enableZIndexHack' );
				}
			}
		}
	};

	NativeAndroidRenderingHacks.prototype._onRequestStartBet = function( options ) {
		libs.globalEventBus.invoke( 'libs.event.request.disableZIndexHack' );
	};

	NativeAndroidRenderingHacks.prototype._onNotifyReelStopped = function( reelIndex, finalSymbols ) {
		var lastReel = game.logic.GameData.numberOfReels - 1;
		if ( reelIndex === lastReel && isNativeAndroid ) {
			libs.globalEventBus.invoke( 'libs.event.request.enableZIndexHack' );
		}
	};

	NativeAndroidRenderingHacks.prototype._onRequestEnableCanvasOverlayMode = function() {
		libs.globalEventBus.invoke( 'libs.event.request.enableZIndexHack' );
	};


	libs.client.rendering.NativeAndroidRenderingHacks = NativeAndroidRenderingHacks;

}());