goog.provide( 'libs.client.rendering.SpriteGroup' );


(function() {
	"use strict";

	// Helper to handle a collection of sprites.
	var SpriteGroup = function( sprites ) {
		if ( sprites ) {
			this.sprites = sprites;
		} else {
			this.sprites = [];
		}
	};

	SpriteGroup.prototype.each = function( toCall ) {
		for ( var i = 0; i < this.sprites.length; i++ ) {
			if ( this.sprites[i] ) {
				toCall( this.sprites[i], i );
			}
		}
	};

	SpriteGroup.prototype.getIndex = function( index ) {
		return this.sprites[index];
	};

	SpriteGroup.prototype.push = function( sprite ) {
		this.sprites.push( sprite );
	};

	SpriteGroup.prototype.pop = function( sprite ) {
		return this.sprites.pop();
	};

	SpriteGroup.prototype.length = function() {
		//Deprecated function use getLength
		return this.sprites.length;
	};

	SpriteGroup.prototype.getLength = function() {
		return this.sprites.length;
	};

	SpriteGroup.prototype.clear = function() {
		this.sprites.length = 0;
	};

	SpriteGroup.prototype.play = function() {
		this.each( function( sprite ) {sprite.play();} );
	};

	SpriteGroup.prototype.stop = function() {
		this.each( function( sprite ) {sprite.stop();} );
	};

	SpriteGroup.prototype.draw = function( ctxPrimary, ctxSecondary ) {
		this.each( function( sprite ) {sprite.draw( ctxPrimary, ctxSecondary );} );
	};

	SpriteGroup.prototype.update = function( timeDelta ) {
		this.each( function( sprite ) {sprite.update( timeDelta );} );
	};

	SpriteGroup.prototype.drawIfDirty = function( ctxPrimary, ctxSecondary ) {
		this.each( function( sprite ) {sprite.drawIfDirty( ctxPrimary, ctxSecondary );} );
	};

	SpriteGroup.prototype.isDirty = function() {
		return this.sprites.some( function( sprite ) {return sprite.isDirty();} );
	};

	SpriteGroup.prototype.moveBy = function( options ) {
		this.each( function( sprite ) {sprite.moveBy( options );} );
		return this;
	};

	SpriteGroup.prototype.moveTo = function( options ) {
		this.each( function( sprite ) { sprite.moveTo( options ); } );
		return this;
	};

	SpriteGroup.prototype.scaleBy = function( options ) {
		this.each( function( sprite ) {sprite.scaleBy( options );} );
		return this;
	};

	SpriteGroup.prototype.scaleTo = function( options ) {
		this.each( function( sprite ) {sprite.scaleTo( options );} );
		return this;
	};

	SpriteGroup.prototype.rotateBy = function( options ) {
		this.each( function( sprite ) {sprite.rotateBy( options );} );
		return this;
	};

	SpriteGroup.prototype.rotateTo = function( options ) {
		this.each( function( sprite ) {sprite.rotateTo( options );} );
		return this;
	};

	SpriteGroup.prototype.fadeBy = function( options ) {
		this.each( function( sprite ) {sprite.fadeBy( options );} );
		return this;
	};

	SpriteGroup.prototype.fadeTo = function( options ) {
		this.each( function( sprite ) {sprite.fadeTo( options );} );
		return this;
	};

	libs.client.rendering.SpriteGroup = SpriteGroup;

}());