goog.provide( 'libs.client.rendering.Surface' );

goog.require( 'libs.client.core.clientInformation' );


(function() {
	"use strict";

	var clientInfo = libs.client.core.clientInformation;
	var isHTCOne = clientInfo.isHTCOne();
	var isNativeSonyXperiaZ = clientInfo.isSonyXperiaZ() && !clientInfo.isHardwareAcceleratedChrome();

	var Surface = function( canvas, drawWidth, drawHeight ) {
		this.canvas = canvas.get( 0 );
		this.ctx = this.canvas.getContext( "2d" );
		this.jCanvas = $( this.canvas );

		this.canvas.width = drawWidth;
		this.canvas.height = drawHeight;

		this._blankCanvasData = this.ctx.getImageData( 0, 0, drawWidth, drawHeight );
	};

	Surface.prototype._resize = function( width, height ) {
		this.canvas.style.width = width + "px";
		this.canvas.style.height = height + "px";

		this.styleWidth = width;
		this.styleHeight = height;
	};

	Surface.prototype.setBlankCanvasData = function( data ) {
		this._blankCanvasData = data;
	};

	Surface.prototype.clearRect = function( x, y, width, height, forcePutImageData ) {
		if ( forcePutImageData || isHTCOne || isNativeSonyXperiaZ ) { // hack because clearrect doesn't always work on certain devices
			this.ctx.putImageData( this._blankCanvasData, 0, 0, x, y, width, height );
		} else {
			this.ctx.clearRect( x, y, width, height );
		}
	};

	Surface.prototype.show = function() {
	};

	Surface.prototype.hide = function() {
		this.clearRect( 0, 0, this.canvas.width, this.canvas.height );
	};

	Surface.prototype.resizeWithAspect = function( width, height ) {
		var scale = getScale( this.canvas, width, height );
		this._resize( scale.styleWidth, scale.styleHeight );
	};

	//Max size of the  canvas while keeping aspect ratio.
	var getScale = function( canvas, maxWidth, maxHeight ) {
		var xStretch, yStretch, stretch;
		var styleWidth, styleHeight;
		var top, left;

		xStretch = maxWidth / canvas.width;
		yStretch = maxHeight / canvas.height;
		stretch = Math.min( xStretch, yStretch );

		styleWidth = canvas.width * stretch;
		styleHeight = canvas.height * stretch;

		top = Math.floor( (maxHeight - styleHeight) / 2 );
		left = Math.floor( (maxWidth - styleWidth) / 2 );

		return {
			styleWidth: styleWidth,
			styleHeight: styleHeight,
			top: top,
			left: left
		};
	};


	libs.client.rendering.Surface = Surface;

}());