goog.provide( 'libs.client.utils.HapticPlayer' );

goog.require( 'libs.globalEventBus' );


(function() {
	"use strict";

	var defaultSettings = {};

	var HapticPlayer = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._hapticAvailable = window.navigator.vibrate !== null && window.navigator.vibrate !== undefined;
		this._enabled = this._hapticAvailable;

		libs.globalEventBus.connectActivityEnd( 'libs.activity.systemState.gameInit', function( activity ) { that._onGameInitEnd(); } );

		libs.globalEventBus.connect( 'libs.event.request.enableHaptics', function() { that._onRequestEnableHaptics(); } );
		libs.globalEventBus.connect( 'libs.event.request.disableHaptics', function() { that._onRequestDisableHaptics(); } );
		libs.globalEventBus.connect( 'libs.event.request.playHaptic', function( pattern ) { that._onRequestPlayHaptic( pattern ); } );
		libs.globalEventBus.connect( 'libs.event.request.stopHaptic', function() { that._onRequestStopHaptic(); } );
	};

	HapticPlayer.prototype._onGameInitEnd = function() {
		libs.globalEventBus.invoke( 'libs.event.notify.hapticsState', this._hapticAvailable && this._enabled );
	};

	HapticPlayer.prototype._onRequestEnableHaptics = function() {
		this._enabled = true;

		libs.globalEventBus.invoke( 'libs.event.notify.hapticsState', this._hapticAvailable && this._enabled );
	};

	HapticPlayer.prototype._onRequestDisableHaptics = function() {
		this._enabled = false;
		if ( this._hapticAvailable ) {
			window.navigator.vibrate( 0 ); // if disabled, stop any current haptics
		}

		libs.globalEventBus.invoke( 'libs.event.notify.hapticsState', this._hapticAvailable && this._enabled );
	};

	HapticPlayer.prototype._onRequestPlayHaptic = function( pattern ) {
		if ( this._hapticAvailable && this._enabled ) {
			window.navigator.vibrate( pattern );
		}
	};

	HapticPlayer.prototype._onRequestStopHaptic = function() {
		if ( this._hapticAvailable ) {
			window.navigator.vibrate( 0 );
		}
	};


	libs.client.utils.HapticPlayer = HapticPlayer;

}());