goog.provide( 'libs.client.utils.tweens' );


(function() {
	"use strict";

	libs.client.utils.tweens = {
		linear: function( k ) {
			return k;
		},
		quadratic: {
			in: function( k ) {
				return k * k;
			},
			out: function( k ) {
				return k * (2 - k);
			},
			inout: function( k ) {
				if ( (k *= 2) < 1 ) {
					return 0.5 * k * k;
				}
				return -0.5 * (--k * (k - 2) - 1);
			}
		},
		cubic: {
			in: function( k ) {
				return k * k * k;
			},
			out: function( k ) {
				return --k * k * k + 1;
			},
			inout: function( k ) {
				if ( (k *= 2) < 1 ) {
					return 0.5 * k * k * k;
				}
				return 0.5 * ((k -= 2) * k * k + 2);
			}
		},
		quartic: {
			in: function( k ) {
				return k * k * k * k;
			},
			out: function( k ) {
				return 1 - (--k * k * k * k);
			},
			inout: function( k ) {
				if ( (k *= 2) < 1 ) {
					return 0.5 * k * k * k * k;
				}
				return -0.5 * ((k -= 2) * k * k * k - 2);
			}
		},
		quintic: {
			in: function( k ) {
				return k * k * k * k * k;
			},
			out: function( k ) {
				return --k * k * k * k * k + 1;
			},
			inout: function( k ) {
				if ( (k *= 2) < 1 ) {
					return 0.5 * k * k * k * k * k;
				}
				return 0.5 * ((k -= 2) * k * k * k * k + 2);
			}
		},
		sinusoidal: {
			in: function( k ) {
				return 1 - Math.cos( k * Math.PI / 2 );
			},
			out: function( k ) {
				return Math.sin( k * Math.PI / 2 );
			},
			inout: function( k ) {
				return 0.5 * (1 - Math.cos( Math.PI * k ));
			}
		},
		exponential: {
			in: function( k ) {
				return k === 0 ? 0 : Math.pow( 1024, k - 1 );
			},
			out: function( k ) {
				return k === 1 ? 1 : 1 - Math.pow( 2, -10 * k );
			},
			inout: function( k ) {
				if ( k === 0 ) {
					return 0;
				}
				if ( k === 1 ) {
					return 1;
				}
				if ( (k *= 2) < 1 ) {
					return 0.5 * Math.pow( 1024, k - 1 );
				}
				return 0.5 * (-Math.pow( 2, -10 * (k - 1) ) + 2);
			}
		},
		circular: {
			in: function( k ) {
				return 1 - Math.sqrt( 1 - k * k );
			},
			out: function( k ) {
				return Math.sqrt( 1 - (--k * k) );
			},
			inout: function( k ) {
				if ( (k *= 2) < 1 ) {
					return -0.5 * (Math.sqrt( 1 - k * k ) - 1);
				}
				return 0.5 * (Math.sqrt( 1 - (k -= 2) * k ) + 1);
			}
		},
		elastic: {
			in: function( k ) {
				var s, a = 0.1,
					p = 0.4;
				if ( k === 0 ) {
					return 0;
				}
				if ( k === 1 ) {
					return 1;
				}
				if ( !a || a < 1 ) {
					a = 1;
					s = p / 4;
				} else {
					s = p * Math.asin( 1 / a ) / (2 * Math.PI);
				}
				return -(a * Math.pow( 2, 10 * (k -= 1) ) * Math.sin( (k - s) * (2 * Math.PI) / p ));
			},
			out: function( k ) {
				var s, a = 0.1,
					p = 0.4;
				if ( k === 0 ) {
					return 0;
				}
				if ( k === 1 ) {
					return 1;
				}
				if ( !a || a < 1 ) {
					a = 1;
					s = p / 4;
				} else {
					s = p * Math.asin( 1 / a ) / (2 * Math.PI);
				}
				return (a * Math.pow( 2, -10 * k ) * Math.sin( (k - s) * (2 * Math.PI) / p ) + 1);
			},
			inout: function( k ) {
				var s, a = 0.1,
					p = 0.4;
				if ( k === 0 ) {
					return 0;
				}
				if ( k === 1 ) {
					return 1;
				}
				if ( !a || a < 1 ) {
					a = 1;
					s = p / 4;
				} else { s = p * Math.asin( 1 / a ) / (2 * Math.PI); }
				if ( (k *= 2) < 1 ) {
					return -0.5 * (a * Math.pow( 2, 10 * (k -= 1) ) * Math.sin( (k - s) * (2 * Math.PI) / p ));
				}
				return a * Math.pow( 2, -10 * (k -= 1) ) * Math.sin( (k - s) * (2 * Math.PI) / p ) * 0.5 + 1;
			}
		},
		back: {
			in: function( k ) {
				var s = 1.70158;
				return k * k * ((s + 1) * k - s);
			},
			out: function( k ) {
				var s = 1.70158;
				return --k * k * ((s + 1) * k + s) + 1;
			},
			inout: function( k ) {
				var s = 1.70158 * 1.525;
				if ( (k *= 2) < 1 ) {
					return 0.5 * (k * k * ((s + 1) * k - s));
				}
				return 0.5 * ((k -= 2) * k * ((s + 1) * k + s) + 2);
			}
		},
		bounce: {
			in: function( k ) {
				return 1 - libs.client.utils.tweens.bounce.out( 1 - k );
			},
			out: function( k ) {
				if ( k < (1 / 2.75) ) {
					return 7.5625 * k * k;
				} else if ( k < (2 / 2.75) ) {
					return 7.5625 * (k -= (1.5 / 2.75)) * k + 0.75;
				} else if ( k < (2.5 / 2.75) ) {
					return 7.5625 * (k -= (2.25 / 2.75)) * k + 0.9375;
				} else {
					return 7.5625 * (k -= (2.625 / 2.75)) * k + 0.984375;
				}
			},
			inout: function( k ) {
				if ( k < 0.5 ) {
					return libs.client.utils.tweens.bounce.in( k * 2 ) * 0.5;
				}
				return libs.client.utils.tweens.bounce.out( k * 2 - 1 ) * 0.5 + 0.5;
			}
		}
	};

}());