goog.provide( 'libs.client.utils.misc' );

goog.require( 'libs.client.core.clientInformation' );


(function() {
	"use strict";

	var isIphoneLike = libs.client.core.clientInformation.isIPhoneLike();

	var jWin = $( window );
	var currentLanguage = 'en_GB';

	libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) {
		currentLanguage = langCode;
	} );

	libs.client.utils.misc = function() {};

	libs.client.utils.misc.screenSize = function() {
		// This should be easy, except iphone always reports the size as if the URL bar is on screen.
		// Unless it's in full screen when it reports correctly.
		var urlHeight = 60;

		var width = jWin.width();
		var height = jWin.height();

		if ( isIphoneLike && height !== 320 ) {
			height += urlHeight;
		}

		return {
			width: width,
			height: height
		};
	};

	libs.client.utils.misc.roundedRect = function( ctx, x, y, w, h, radius, fill, stroke ) {
		ctx.save();
		ctx.beginPath();
		ctx.moveTo( x + radius, y );
		ctx.arcTo( x + w, y, x + w, y + radius, radius );
		ctx.arcTo( x + w, y + h, x + w - radius, y + h, radius );
		ctx.arcTo( x, y + h, x, y + h - radius, radius );
		ctx.arcTo( x, y, x + radius, y, radius );
		if ( fill ) {
			ctx.fill();
		}
		if ( stroke ) {
			ctx.stroke();
		}
		ctx.restore();
	};

	libs.client.utils.misc.shuffle = function( array ) {
		//From http://stackoverflow.com/questions/2450954/how-to-randomize-a-javascript-array
		for ( var i = array.length - 1; i > 0; i-- ) {
			var j = libs.client.utils.misc.getRandomInt( 0, i + 1 );
			var temp = array[i];
			array[i] = array[j];
			array[j] = temp;
		}
		return array;
	};

	libs.client.utils.misc.openUrl = function( url, newWindow ) {
		if ( url && url.length && url.length > 1 ) {
			if ( newWindow ) {
				window.open( libs.client.assetUrlPrefix + 'redirect.html?ingglink=' + url );
			} else {
				window.location = libs.client.assetUrlPrefix + 'redirect.html?ingglink=' + url;
			}
		} else {
			libs.logError( 'openUrl: invalid url of ' + url );
		}

	};

//	Normally not used by the game directly, included in xml as the menu items handler.
//	Called when a menu option is tapped. Delegates back to any game registered functions if no URL is specified or opens URL otherwise.
	libs.client.utils.misc.listeners = {};
	libs.client.utils.misc.menuOptionTapped = function( menuOption, url, newWindow ) {
		if ( url && url !== "" ) {
			libs.client.utils.misc.openUrl( url, newWindow );
		} else {
			if ( libs.client.utils.misc.listeners[menuOption] ) {
				try {
					libs.client.utils.misc.listeners[menuOption]();
				} catch ( e ) {
					if ( console ) {
						console.error( e.message );
					}
				}
			}
		}
	};

//	Function to be used by the game to add call-backs to display menu options that are provided by the game itself and not in an external page.
	libs.client.utils.misc.onMenuTappedAddCallback = function( menuOption, onMenuTappedCallback ) {
		libs.client.utils.misc.listeners[menuOption] = onMenuTappedCallback;
	};

	//Is the point inside the rect
	libs.client.utils.misc.collision = function( rect, point ) {
		return rect.x <= point.x && (rect.x + rect.width) > point.x &&
			rect.y <= point.y && (rect.y + rect.height) > point.y;
	};

	//Do either of the two rects overlap
	libs.client.utils.misc.overlap = function( rect1, rect2 ) {
		var left1, left2;
		var right1, right2;
		var top1, top2;
		var bottom1, bottom2;

		left1 = rect1.x;
		left2 = rect2.x;
		right1 = rect1.x + rect1.width;
		right2 = rect2.x + rect2.width;
		top1 = rect1.y;
		top2 = rect2.y;
		bottom1 = rect1.y + rect1.height;
		bottom2 = rect2.y + rect2.height;

		return !(bottom1 < top2 || bottom2 < top1 ||
		right1 < left2 || right2 < left1);
	};

	libs.client.utils.misc.loadAnimationFrames = function( images, base_name ) {
		var padToFive = function( number ) {
			if ( number <= 99999 ) {
				number = ("0000" + number).slice( -5 );
			}
			return number;
		};

		var i, image_list = [];

		for ( i = 0; i < 1500; i++ ) {
			var frameName = base_name + '_' + padToFive( i );
			if ( images[frameName] ) {
				image_list.push( images[frameName] );
			} else if ( i > 1 ) {
				break;
			}
		}

		return image_list;
	};

	libs.client.utils.misc.toFixed = function( number, precision ) {
		var multiplier = Math.pow( 10, precision + 1 ),
			wholeNumber = Math.floor( number * multiplier );
		return Math.round( wholeNumber / 10 ) * 10 / multiplier;
	};

	libs.client.utils.misc.toBigNumber = function( number ) {
		return new BigNumber( number || 0 );
	};

	libs.client.utils.misc.logSettings = function( settings ) {
		if ( BUILD_TYPE === 'dev' ) {
			libs.log( 'settings: ' + JSON.stringify( settings, null, '\t' ) );
		}
	};

	// http://davidwalsh.name/javascript-clone
	// use this for cloning objects over jquery.extend for occasions where you need
	// null and undefined properties to be copied across too
	libs.client.utils.misc.clone = function( src ) {
		function mixin( dest, source, copyFunc ) {
			var name, s, i, empty = {};
			for ( name in source ) {
				// the (!(name in empty) || empty[name] !== s) condition avoids copying properties in "source"
				// inherited from Object.prototype.	 For example, if dest has a custom toString() method,
				// don't overwrite it with the toString() method that source inherited from Object.prototype
				s = source[name];
				if ( !(name in dest) || (dest[name] !== s && (!(name in empty) || empty[name] !== s)) ) {
					dest[name] = copyFunc ? copyFunc( s ) : s;
				}
			}
			return dest;
		}

		if ( !src || typeof src != "object" || Object.prototype.toString.call( src ) === "[object Function]" ) {
			// null, undefined, any non-object, or function
			return src;	// anything
		}
		if ( src.nodeType && "cloneNode" in src ) {
			// DOM Node
			return src.cloneNode( true ); // Node
		}
		if ( src instanceof Date ) {
			// Date
			return new Date( src.getTime() );	// Date
		}
		if ( src instanceof RegExp ) {
			// RegExp
			return new RegExp( src );   // RegExp
		}
		var r, i, l;
		if ( src instanceof Array ) {
			// array
			r = [];
			for ( i = 0, l = src.length; i < l; ++i ) {
				if ( i in src ) {
					r.push( libs.client.utils.misc.clone( src[i] ) );
				}
			}
			// we don't clone functions for performance reasons
			//		}else if(d.isFunction(src)){
			//			// function
			//			r = function(){ return src.apply(this, arguments); };
		} else {
			// generic objects
			r = src.constructor ? new src.constructor() : {};
		}
		return mixin( r, src, libs.client.utils.misc.clone );
	};

	var checkNoUndefinedProperty = function( overrideSettings, parent, isArrayChild ) {
		parent = parent || '';
		for ( var key in overrideSettings ) {
			var property = overrideSettings[key];
			var isArray = property ? typeof property.length !== 'undefined' : false;
			var fullname = parent !== '' ? parent + '.' + key : key;
			if ( isArrayChild ) {
				fullname = parent !== '' ? parent + '[' + key + ']' : key;
			}
			if ( typeof property === 'object' ) {
				checkNoUndefinedProperty( property, fullname, isArray );
			} else if ( typeof property === 'undefined' ) {
				libs.logError( 'mergeSettings: ' + fullname + ' is undefined' );
			}
		}
	};

	libs.client.utils.misc.mergeSettings = function( defaultSettings, overrideSettings, skipCheckForUndefinedProperties ) {
		var mergedSettings = libs.client.utils.misc.clone( defaultSettings );
		jQuery.extend( true, mergedSettings, overrideSettings );
		if ( !skipCheckForUndefinedProperties ) {
			checkNoUndefinedProperty( mergedSettings );
		}
		return mergedSettings;
	};

	libs.client.utils.misc.replaceAssetPathsInText = function( text ) {
		return text.replace( /\{\{assetPath\}\}/g, libs.client.assetUrlPrefix );
	};

	libs.client.utils.misc.replaceLanguageCodeInText = function( text ) {
		return text.replace( /\{\{languageCode\}\}/g, currentLanguage );
	};

	// random int - inclusive of min, exclusive of max
	libs.client.utils.misc.getRandomInt = function( min, max ) {
		return Math.floor( Math.random() * (max - min) ) + min;
	};

	libs.client.utils.misc.wordSplitter = function( sentence, maxLines ) {
		var lines = [];
		var wordArray = sentence.split( ' ' );
		var maxPerLine = Math.ceil( wordArray.length / (maxLines || 1) );
		var currentLine = 0;
		var cycle = 0;

		for ( var i = 0, len = wordArray.length; i < len; i++ ) {
			cycle++;
			lines[currentLine] = lines[currentLine] || [];
			lines[currentLine].push( wordArray[i] );
			if ( cycle >= maxPerLine ) {
				currentLine++;
				cycle = 0;
			}
		}

		return lines.map( function( line ) {
			return line.join( ' ' );
		} );
	};

	/**
	 * Get a value in a percent form
	 * @method getValueAsPercent
	 * @param {Number} value - amount to convert
	 * @param {Numebr} total - the total value
	 * @param {Boolean} appendPercent - should a % symbol be appended
	 */

	libs.client.utils.misc.getValueAsPercent = function( value, total, appendPercent ) {
		var percent = 0;

		if ( value && total ) {
			percent = (value / total) * 100;
		}

		if ( appendPercent ) {
			percent += '%';
		}

		return percent;
	};

	/**
	 * Get a value in a percent form
	 * @method getValueAsPercent
	 * @param {Number} percent - percent of total
	 * @param {Numebr} total - the total value
	 */

	libs.client.utils.misc.getPercentOfTotal = function( percent, total ) {
		var value = 0;

		if ( percent && total ) {
			value = (total / 100) * percent;
		}

		return value;
	};

	// This function is accessed from menu.xml so export it using string literals
	window['GLOBAL_menuOptionTapped'] = libs.client.utils.misc.menuOptionTapped;

}());