goog.provide( 'libs.getDefaultFormatForCurrencyCode' );
goog.provide( 'libs.moneyFormat' );

goog.require( 'libs.globalEventBus' );


(function() {
	"use strict";


	var defaultFormatForCurrency = {
		// symbol unicodes from https://gist.github.com/bzerangue/5484121
		// formatting via Windows Region and Language
		'ARS': {
			code: "ARS",
			description: "Argentinian Pesos",
			symbol: String.fromCharCode( 36 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "{0}-{1}",
			seperatorDecimal: ",",
			separatorThousands: ".",
			numberOfDigitsAfterDecimal: 2
		},
		'AUD': {
			code: "AUD",
			description: "Australian Dollar",
			symbol: String.fromCharCode( 36 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'BGN': {
			code: "BGN",
			description: "Bulgarian Lev",
			symbol: String.fromCharCode( 1083 ) + String.fromCharCode( 1074 ),
			positiveFormat: "{1} {0}",
			negativeFormat: "-{1} {0}",
			seperatorDecimal: ",",
			separatorThousands: " ",
			numberOfDigitsAfterDecimal: 2
		},
		'BRL': {
			code: "BRL",
			description: "Brazilian Real",
			symbol: String.fromCharCode( 82 ) + String.fromCharCode( 36 ),
			positiveFormat: "{0} {1}",
			negativeFormat: "-{0} {1}",
			seperatorDecimal: ",",
			separatorThousands: ".",
			numberOfDigitsAfterDecimal: 2
		},
		'CAD': {
			code: "CAD",
			description: "Canadian Dollar",
			symbol: String.fromCharCode( 36 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'CHF': {
			code: "CHF",
			description: "Swiss Franc",
			symbol: String.fromCharCode( 67 ) + String.fromCharCode( 72 ) + String.fromCharCode( 70 ),
			positiveFormat: "{0} {1}",
			negativeFormat: "-{0} {1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'CZK': {
			code: "CZK",
			description: "Czech Republic Koruna",
			symbol: String.fromCharCode( 75 ) + String.fromCharCode( 269 ),
			positiveFormat: "{1} {0}",
			negativeFormat: "-{1} {0}",
			seperatorDecimal: ",",
			separatorThousands: " ",
			numberOfDigitsAfterDecimal: 2
		},
		'DKK': {
			code: "DKK",
			description: "Danish Krone",
			symbol: String.fromCharCode( 107 ) + String.fromCharCode( 114 ) + String.fromCharCode( 46 ),
			positiveFormat: "{0} {1}",
			negativeFormat: "{0} -{1}",
			seperatorDecimal: ",",
			separatorThousands: ".",
			numberOfDigitsAfterDecimal: 2
		},
		'EUR': {
			code: "EUR",
			description: "European Union Euro",
			symbol: String.fromCharCode( 8364 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ",",
			separatorThousands: ".",
			numberOfDigitsAfterDecimal: 2
		},
		'GBP': {
			code: "GBP",
			description: "British Pound",
			symbol: String.fromCharCode( 163 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'HKD': {
			code: "HKD",
			description: "Hong Kong Dollar",
			symbol: String.fromCharCode( 36 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "({0}{1})",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'HUF': {
			code: "HUF",
			description: "Hungarian Forint",
			symbol: String.fromCharCode( 70 ) + String.fromCharCode( 116 ),
			positiveFormat: "{1} {0}",
			negativeFormat: "-{1} {0}",
			seperatorDecimal: ",",
			separatorThousands: " ",
			numberOfDigitsAfterDecimal: 2
		},
		'INR': {
			code: "INR",
			description: "Indian Rupee",
			symbol: String.fromCharCode( 8377 ),
			positiveFormat: "{0} {1}",
			negativeFormat: "{0} -{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'ISK': {
			code: "ISK",
			description: "Icelandic Krona",
			symbol: String.fromCharCode( 107 ) + String.fromCharCode( 114 ) + String.fromCharCode( 46 ),
			positiveFormat: "{1} {0}",
			negativeFormat: "-{1} {0}",
			seperatorDecimal: ",",
			separatorThousands: ".",
			numberOfDigitsAfterDecimal: 0
		},
		'JPY': {
			code: "JPY",
			description: "Japanese Yen",
			symbol: String.fromCharCode( 165 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 0
		},
		'MXN': {
			code: "MXN",
			description: "Mexican Peso",
			symbol: String.fromCharCode( 36 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'MYR': {
			code: "MYR",
			description: "Malaysian Ringgit",
			symbol: String.fromCharCode( 82 ) + String.fromCharCode( 77 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "({0}{1})",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 0
		},
		'NOK': {
			code: "NOK",
			description: "Norwegian Krone",
			symbol: String.fromCharCode( 107 ) + String.fromCharCode( 114 ),
			positiveFormat: "{0} {1}",
			negativeFormat: "{0} -{1}",
			seperatorDecimal: ",",
			separatorThousands: " ",
			numberOfDigitsAfterDecimal: 2
		},
		'NZD': {
			code: "NZD",
			description: "New Zealand Dollar",
			symbol: String.fromCharCode( 36 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'PLN': {
			code: "PLN",
			description: "Polish Zloty",
			symbol: String.fromCharCode( 122 ) + String.fromCharCode( 322 ),
			positiveFormat: "{1} {0}",
			negativeFormat: "-{1} {0}",
			seperatorDecimal: ",",
			separatorThousands: " ",
			numberOfDigitsAfterDecimal: 2
		},
		'CNY': {
			code: "CNY",
			description: "Chinese Renminbi",
			symbol: String.fromCharCode( 165 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "{0}-{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'RON': {
			code: "RON",
			description: "Romanian New Leu",
			symbol: String.fromCharCode( 108 ) + String.fromCharCode( 101 ) + String.fromCharCode( 105 ),
			positiveFormat: "{1} {0}",
			negativeFormat: "-{1} {0}",
			seperatorDecimal: ",",
			separatorThousands: ".",
			numberOfDigitsAfterDecimal: 2
		},
		'SEK': {
			code: "SEK",
			description: "Swedish Krona",
			symbol: String.fromCharCode( 107 ) + String.fromCharCode( 114 ),
			positiveFormat: "{1} {0}",
			negativeFormat: "-{1} {0}",
			seperatorDecimal: ",",
			separatorThousands: ".",
			numberOfDigitsAfterDecimal: 2
		},
		'SGD': {
			code: "SGD",
			description: "Singapor Dollar",
			symbol: String.fromCharCode( 36 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "({0}{1})",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'THB': {
			code: "THB",
			description: "Thai Baht",
			symbol: String.fromCharCode( 3647 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'TWD': {
			code: "TWD",
			description: "Taiwan New Dollar",
			symbol: String.fromCharCode( 78 ) + String.fromCharCode( 84 ) + String.fromCharCode( 36 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'USD': {
			code: "USD",
			description: "United States Dollar",
			symbol: String.fromCharCode( 36 ),
			positiveFormat: "{0}{1}",
			negativeFormat: "({0}{1})",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		},
		'ZAR': {
			code: "ZAR",
			description: "South African Rand",
			symbol: String.fromCharCode( 82 ),
			positiveFormat: "{0} {1}",
			negativeFormat: "{0}-{1}",
			seperatorDecimal: ",",
			separatorThousands: " ",
			numberOfDigitsAfterDecimal: 2
		},
		'XXX': {
			code: "XXX",
			description: "Demonstration Mode Points",
			symbol: "",
			positiveFormat: "{0}{1}",
			negativeFormat: "-{0}{1}",
			seperatorDecimal: ".",
			separatorThousands: ",",
			numberOfDigitsAfterDecimal: 2
		}
	};

	var currentCurrencyFormat = jQuery.extend( true, {}, defaultFormatForCurrency['GBP'] );

	var getDefaultFormatForCurrencyCode = function( currencyCode ) {
		return defaultFormatForCurrency[currencyCode];
	};

	libs.globalEventBus.connect( 'libs.event.request.changeCurrency', function( currencyCode, formatForCurrency ) {
		currentCurrencyFormat = jQuery.extend( true, {}, defaultFormatForCurrency[currencyCode], formatForCurrency );
		libs.globalEventBus.invoke( 'libs.event.notify.currencyChanged', currencyCode, formatForCurrency );
	} );

	//Take in a Big Number and format it for display
	var moneyFormat = function( money, alwaysIncludeSubunit ) {
		if ( !(money instanceof BigNumber ) ) {
			var msg = "Invalid 'money' passed to moneyFormat() method - must be a BigNumber instance";
			libs.logError( msg );
			throw new Error( msg );
		}

		alwaysIncludeSubunit = alwaysIncludeSubunit !== undefined ? alwaysIncludeSubunit : true;

		//Pence to pounds
		money = money.div( 100 );
		//Convert it to a higher number
		money = money.absoluteValue();

		var moneyString = money.toString().split( '.' );
		if ( alwaysIncludeSubunit === true || (typeof moneyString[1] !== "undefined" && moneyString[1] > "0") ) {
			if ( moneyString.length === 1 ) {
				if ( currentCurrencyFormat.numberOfDigitsAfterDecimal > 0 ) {
					moneyString[1] = "00";
				}
			} else if ( moneyString[1].length === 1 ) {
				moneyString[1] = moneyString[1] + "0";
			}
		}

		moneyString[0] = moneyString[0].replace( /\B(?=(\d{3})+(?!\d))/g, currentCurrencyFormat.separatorThousands );

		return libs.formatString( money.isNegative() ? currentCurrencyFormat.negativeFormat : currentCurrencyFormat.positiveFormat, [currentCurrencyFormat.symbol, moneyString.join( currentCurrencyFormat.seperatorDecimal )] );
	};


	libs.moneyFormat = moneyFormat;
	libs.getDefaultFormatForCurrencyCode = getDefaultFormatForCurrencyCode;

}());