// JScript requires all top level namespaces to be declared like this
var libs = libs || {}; // jshint ignore:line
var game = game || {}; // jshint ignore:line
var goog = goog || {}; // jshint ignore:line
var BigNumber = BigNumber || {}; // jshint ignore:line
var globalThis = this;

/** @define {boolean} */
var CLOSURE_COMPILE = false; // jshint ignore:line


var __getContainer = function( container ) {
	"use strict";
	try {
		return ( container || window ) || globalThis;
	}
	catch ( err ) { // JScript throws an exception if the 'window' object is attempted to be referenced. Catch this and return the global this pointer instead
	}
	return globalThis;
};


(function() {
	"use strict";

	var error = function( error ) {
		libs.logError( error );
	};

	var provide = function( name, container ) {
		var ns = name.split( '.' ),
			o = __getContainer( container ),
			i,
			len;
		for ( i = 0, len = ns.length; i < len; i++ ) {
			o = o[ns[i]] = o[ns[i]] || {};
		}
		return o;
	};

	var require = function( name, container ) {
		var ns = name.split( '.' ),
			o = __getContainer( container ),
			i,
			len;
		for ( i = 0, len = ns.length; i < len; i++ ) {
			o = o[ns[i]] = o[ns[i]] || error( 'goog.require() failed: ' + name );
		}
		return o;
	};

	var requireThirdParty = function( name, container ) {
		// Does exactly what goog.require() does, except isn't parsed by the closure compiler.
		// This is used to check the existence of objects that do not have an equivalent goog.provide().
		// Logic is changed slightly so global objects that are booleans that are set to false (e.g. screenfull object) do not fail.
		var ns = name.split( '.' ),
			o = __getContainer( container ),
			i,
			len;
		for ( i = 0, len = ns.length; i < len; i++ ) {
			o = o[ns[i]];
			if ( o === undefined || o === null ) {
				error( 'goog.requireThirdParty() failed: ' + name );
			}
		}
		return o;
	};

	goog.provide = provide;
	goog.require = require;
	goog.requireThirdParty = requireThirdParty;
}());


goog.provide( 'libs' );
goog.provide( 'game' );


(function() {
	"use strict";

	var getExternalSetting = function( name, defaultValue, container ) {

		var ns = ( 'externalSettings.' + name ).split( '.' ),
			o = __getContainer( container ),
			i,
			len;
		for ( i = 0, len = ns.length; i < len; i++ ) {
			if ( o[ns[i]] ) {
				o = o[ns[i]];
			} else {
				return defaultValue;
			}
		}
		if ( typeof o === 'string' || typeof o === 'number' ) {
			return o;
		} else {
			return defaultValue;
		}
	};

	var getExternalSettingArray = function( name, defaultValue, container ) {
		var ns = ( 'externalSettings.' + name ).split( '.' ),
			o = __getContainer( container ),
			i,
			len;
		for ( i = 0, len = ns.length; i < len; i++ ) {
			if ( o[ns[i]] ) {
				o = o[ns[i]];
			} else {
				return defaultValue;
			}
		}
		if ( o instanceof Array ) {
			return o;
		} else {
			return defaultValue;
		}
	};


	libs.log = function() {};
	libs.getExternalSetting = getExternalSetting;
	libs.getExternalArray = getExternalSettingArray;
}());


