goog.provide( 'libs.UpdateTimer' );


(function() {
	"use strict";

	// A timer that works off updates instead of real time.
	// This means things like stopping and pausing the game will work correctly.
	var UpdateTimer = function() {
		this.running = []; //This is a sparse array be careful.
		this.toDelete = [];
	};

	UpdateTimer.prototype.update = function( timeDelta ) {
		var that = this;
		this.toDelete.length = 0;

		this.running.forEach( function( action, index ) {
			action.fire -= timeDelta;
			if ( action.fire <= 0 ) {
				action.callback();
				if ( action.repeat ) {
					action.fire = action.repeat;
				} else {
					that.toDelete.push( index );
				}
			}

		} );

		this.toDelete.forEach( function( index ) {
			that.stopTimer( index + 1 );
		} );
	};

	UpdateTimer.prototype.addTimeout = function( timeSpan, callback ) {

		return this._addTimer( {
			start: timeSpan,
			fire: timeSpan,
			repeat: 0,
			callback: callback
		} );
	};

	UpdateTimer.prototype.addInterval = function( timeSpan, callback ) {
		return this._addTimer( {
			start: timeSpan,
			fire: timeSpan,
			repeat: timeSpan,
			callback: callback
		} );
	};

	UpdateTimer.prototype.stopTimer = function( id ) {
		var index = id - 1;
		if ( index >= 0 && index < this.running.length ) {
			this.running.splice( index, 1 );
		}
	};

	UpdateTimer.prototype.stopAllTimers = function() {
		this.running.length = 0;
	};

	UpdateTimer.prototype._addTimer = function( timer ) {
		var index = this.running.length;
		this.running[index] = timer;

		return index + 1;
	};

	UpdateTimer.prototype.getProgress = function( id ) {
		var timer = this.running[id - 1];
		var elapsed = timer.start - timer.fire;
		var progress = elapsed / timer.start;
		return progress || 0;
	};


	libs.UpdateTimer = UpdateTimer;

}());