/*jslint node: true */
// Game cycle execution code - this is executed within a separate Node.js VM context

"use strict";


var platformInterface = null;
var expectedJackpotValues = [];


var update_platform_values = function() {
	expectedJackpotValues = [];
	for ( var i = 0; i < platformInterface.getJackpotCount(); ++i ) {
		expectedJackpotValues.push( platformInterface.getJackpotValue( i ) );
	}
};


var do_init = function( initSettings ) {

	platformInterface = new libs.client.hillsVegas.LocalServerPlatformInterface( initSettings );
	var initResponse = game.logic.BuildInitResponse( platformInterface.getCurrencyCode() );
	update_platform_values();
	return initResponse;
};


var do_entire_game_cycle = function( stakePerLine, lineCount, forcedRNG, staticDataObject, postData ) {

	var totalStake = stakePerLine * lineCount;
	var currentSequence = 0;
	var inBetSequence = true;
	var gameState = { 'game': {}, 'player': staticDataObject };
	var betResults = [];

	if ( forcedRNG && forcedRNG.length > 0 ) {
		libs.client.hillsVegas.LocalServerPlatformInterface.forceRng( forcedRNG );
	}

	update_platform_values();

	var betSelection = {
		'stake': new BigNumber( stakePerLine ),
		'total_stake': new BigNumber( totalStake ),
		'lines': lineCount,
		'checkpointId': currentSequence,
		'jackpot': expectedJackpotValues
	};

	platformInterface.startFirstBetOperation();

	while ( inBetSequence ) {

		betSelection['checkpointId'] = currentSequence;
		var serverBetResults = game.logic.BuildBetResults( platformInterface, {
			'betSelection': betSelection,
			'serverState': gameState,
			'postData': postData
		} );

		inBetSequence = serverBetResults['expectCheckpoint'];

		platformInterface.takeStake( currentSequence, totalStake, serverBetResults['win'], true );
		betResults.push( serverBetResults );

		// update sequence number and server state
		currentSequence++;
	}

	platformInterface.closeBet();

	return betResults;
};


var singleCheckpointState = null;


var do_checkpoint_game_cycle = function( stakePerLine, lineCount, forcedRNG, staticDataObject, postData ) {

	var totalStake = stakePerLine * lineCount;
	var currentSequence = 0;
	var isNewGame = false;
	update_platform_values();

	if ( forcedRNG && forcedRNG.length > 0 ) {
		libs.client.hillsVegas.LocalServerPlatformInterface.forceRng( forcedRNG );
	}

	if ( !singleCheckpointState ) {
		singleCheckpointState = {
			'serverState': {
				'game': {}
			},
			'betSelection': {
				'stake': new BigNumber( stakePerLine ),
				'total_stake': new BigNumber( totalStake ),
				'lines': lineCount,
				'checkpointId': currentSequence,
				'jackpot': expectedJackpotValues
			}
		};
		platformInterface.startFirstBetOperation();
		isNewGame = true;
	} else {
		currentSequence = singleCheckpointState['betSelection']['checkpointId'];
		currentSequence++;
		singleCheckpointState['betSelection']['checkpointId'] = currentSequence;
	}

	if ( staticDataObject ) {
		singleCheckpointState['serverState']['player'] = staticDataObject;
	}
	if ( postData ) {
		singleCheckpointState['postData'] = postData;
	}

	var betResult = game.logic.BuildBetResults( platformInterface, singleCheckpointState );

	if ( isNewGame ) {
		platformInterface.takeStake( currentSequence, totalStake, betResult['win'], true );
	}

	if ( !betResult['inBetSequence'] ) {
		platformInterface.closeBet();
		singleCheckpointState = null;
	}

	return betResult;
};



