goog.provide( 'libs.server.platform.debug.acquireForcedRandomNumbers' );
goog.provide( 'libs.server.platform.debug.logRandomNumberStringPlatform' );
goog.provide( 'libs.server.platform.debug.logRandomNumberStringLocal' );
goog.provide( 'libs.server.platform.debug.forceRNGUsingApplicationContents' );
goog.provide( 'libs.server.platform.debug.saveRNGValueUsingApplicationContents' );
goog.provide( 'libs.server.platform.debug.clearSavedRNGValueUsingApplicationContents' );
goog.provide( 'libs.server.platform.debug.getSavedRNGValueUsingApplicationContents' );


(function() {
	"use strict";

	libs.server.platform.debug.acquireForcedRandomNumbers = function( platformInterface, accountId, gameCode, type, lowestInclusive, highestInclusive, numberCount ) {

		var numbers = [];
		var key = "IV:" + accountId + ":" + gameCode;
		var setRngsCSV = libs.server.getApplicationContents( key );
		if ( setRngsCSV ) {
			var setRngsArr = setRngsCSV.split( "," );
			var numbersToGet = Math.min( setRngsArr.length, numberCount );
			for ( var i = 0; i < numbersToGet; ++i ) {
				var chance = Number( setRngsArr.shift() );
				if ( isFinite( chance ) ) {
					if ( (chance < lowestInclusive) || (chance > highestInclusive) ) {
						chance = lowestInclusive + ( chance % ( highestInclusive - lowestInclusive ) ); // in the forced version, just modulus the number so it comes within range.
					}
					numbers.push( chance );
				}
			}
			libs.server.setApplicationContents( key, setRngsArr.join( "," ) );
			if ( numbers.length > 0 ) {
				platformInterface.logSuspicious( "acquireForcedRandomNumbers FORCED VALUE(S) type='" + type + "' numbers='" + numbers.join( "," ) + "'" );
			}
		}

		return numbers;
	};


	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	libs.server.platform.debug.forceRNGUsingApplicationContents = function( accountIdList, gameCode, rngString ) {
		if ( accountIdList ) {
			var accountIds = accountIdList.split( ',' );

			accountIds.forEach( function( accountId ) {
				var key = "IV:" + accountId + ":" + gameCode;
				if ( rngString === "" ) {
					libs.server.removeApplicationContents( key );
				}
				else {
					if ( rngString.charAt( 0 ) === "+" ) {
						rngString = rngString.slice( 1 );
						var currentRng = libs.server.getApplicationContents( key );
						if ( currentRng ) {
							rngString = currentRng + rngString;
						}
					}

					var setRngsArr = rngString.split( "," );
					for ( var i = 0; i < setRngsArr.length; ++i ) {
						if ( setRngsArr[i] === '*' ) {
							setRngsArr[i] = Math.floor( Math.random() * 1000000 );
						}
					}
					rngString = setRngsArr.join( "," );

					libs.server.setApplicationContents( key, rngString );
				}
			} );
		}
	};


	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	libs.server.platform.debug.saveRNGValueUsingApplicationContents = function( accountId, gameCode, rngArray ) {

		var key = "RNGLOG:" + accountId + ":" + gameCode;
		var existingString = String( libs.server.getApplicationContents( key ) );
		var str;
		if ( existingString.length > 0 ) {
			str = existingString + "," + rngArray.join( "," );
		} else {
			str = rngArray.join( "," );
		}
		libs.server.setApplicationContents( key, str );
	};


	libs.server.platform.debug.clearSavedRNGValueUsingApplicationContents = function( accountId, gameCode ) {
		var key = "RNGLOG:" + accountId + ":" + gameCode;
		libs.server.setApplicationContents( key, '' );
	};


	libs.server.platform.debug.getSavedRNGValueUsingApplicationContents = function( accountId, gameCode ) {
		var key = "RNGLOG:" + accountId + ":" + gameCode;
		return libs.server.getApplicationContents( key );
	};

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	libs.server.platform.debug.saveStaticDataUsingApplicationContents = function( accountId, gameCode, staticDataString ) {

		var key = "STATICLOG:" + accountId + ":" + gameCode;
		libs.server.setApplicationContents( key, staticDataString );
	};


	libs.server.platform.debug.clearStaticDataUsingApplicationContents = function( accountId, gameCode ) {
		var key = "STATICLOG:" + accountId + ":" + gameCode;
		libs.server.setApplicationContents( key, '' );
	};


	libs.server.platform.debug.getStaticDataUsingApplicationContents = function( accountId, gameCode ) {
		var key = "STATICLOG:" + accountId + ":" + gameCode;
		return libs.server.getApplicationContents( key );
	};


	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	libs.server.platform.debug.sendErrorsToClient = function() {
		return true;
	};

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	libs.server.platform.debug.logRandomNumberStringPlatform = function( platformInterface, accountId, gameCode ) {
		var previousGameRng = libs.server.platform.debug.getSavedRNGValueUsingApplicationContents( accountId, gameCode );
		// platformInterface.logMessage( "Previous Game RNG: " + previousGameRng );
	};

	libs.server.platform.debug.logRandomNumberStringLocal = function( rngStr ) {
		// libs.log( "Previous Game RNG: " + rngStr );
	};

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


}());

