/*global Application,Request,Server*/

goog.require( 'libs.xmlUtils' );

// Utility functions only used by the ASP server side (this should not be included on the client side)

goog.provide( 'libs.server.loadStateData' );
goog.provide( 'libs.server.saveStateData' );
goog.provide( 'libs.server.loadBetSelectionData' );
goog.provide( 'libs.server.saveBetSelectionData' );
goog.provide( 'libs.server.saveBetResults' );
goog.provide( 'libs.server.saveResponseHeader' );
goog.provide( 'libs.server.saveInitResponse' );
goog.provide( 'libs.server.getApplicationContents' );
goog.provide( 'libs.server.setApplicationContents' );
goog.provide( 'libs.server.removeApplicationContents' );
goog.provide( 'libs.server.getRequestCookie' );


(function() {
	"use strict";

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	// Equivalent of Server.HTMLDecode
	libs.server.htmlDecode = function( str ) {
		str = str.replace( /&quot;/g, "\"" );
		str = str.replace( /&lt;/g, "<" );
		str = str.replace( /&gt;/g, ">" );
		str = str.replace( /&amp;/g, "&" );
		str = str.replace( /&nbsp;/g, " " );

		for ( var i = 1; i <= 255; ++i ) {
			str = str.replace( new RegExp( "&#" + i + ";", "g" ), String.fromCharCode( i ) );
		}

		return str;
	};


	libs.server.getApplicationContents = function( keyName ) {
		var o = Application.Contents( keyName );
		if ( o !== undefined ) {
			return libs.server.htmlDecode( String( o ) );
		}
		return '';
	};


	libs.server.setApplicationContents = function( keyName, value ) {
		Application.Contents( keyName ) = Server.HTMLEncode( value ); // jshint ignore:line
	};


	libs.server.removeApplicationContents = function( keyName ) {
		Application.Contents.Remove( keyName );
	};


	libs.server.getRequestCookie = function( keyName, subKeyName, doHTMLEncode ) {
		var val = Request.Cookies( keyName )( subKeyName );
		if ( doHTMLEncode ) {
			return Server.HTMLEncode( val );
		} else {
			return val;
		}
	};


	libs.server.logError = function( platformInterface, moduleName, err ) {
		if ( platformInterface ) {
			var prefix = "";
			if ( moduleName ) {
				prefix = ": ";
			}
			if ( err.suspicious ) {
				platformInterface.logSuspicious( prefix + "suspicious: " + err.message );
			} else {
				platformInterface.logError( prefix + "error: " + err.message );
			}
		}

		var friendlyErrorMessage = err.friendlyMessage || "An error has occurred. Please reload the game.";
		var errorType = err.errorType || 0;//"fatal";

		if ( libs.server.platform.debug.sendErrorsToClient() ) {
			// debug mode: send full error
			return libs.xmlUtils.wrapJsonInXml( "jsonResponse", {
				'error': {
					'id': errorType,
					'text': Server.HTMLEncode( err.message || friendlyErrorMessage )
				}
			} );
		} else {
			// send generic friendly error message
			return libs.xmlUtils.wrapJsonInXml( "jsonResponse", {
				'error': {
					'id': errorType,
					'text': Server.HTMLEncode( friendlyErrorMessage )
				}
			} );
		}
	};
}());

