// History object builds the string to be sent to the game history log. ASP server only
goog.provide( 'libs.server.williamHill.historyRecordListToObject' );
goog.provide( 'libs.server.williamHill.betResultsToHistoryString' );

goog.require( 'libs.getDateString' );

goog.require( 'game.logic' );


(function() {
	"use strict";


	var historyRecordListToObject = function( recordList ) {

		var rawRegex = 'time\\d+=(.*?)&ref\\d+=(.*?)&action\\d+=(.*?)&amount\\d+=(.*?)&won\\d+=(.*?)&picked\\d+=(.*?)&selection\\d+=(.*?)&result\\d+=(.*?)&detail\\d+=(.*?)&';
		var matches = recordList.match( new RegExp( rawRegex, 'g' ) );
		var entries = [];

		if ( matches && matches.length > 0 ) {
			for ( var matchIndex = 0; matchIndex < matches.length; ++matchIndex ) {

				var subMatches = matches[matchIndex].match( new RegExp( rawRegex ) );
				var time = subMatches[1];
				var ref = subMatches[2];
				var action = subMatches[3];
				var amount = subMatches[4];
				var won = subMatches[5];
				var picked = subMatches[6];
				var selection = subMatches[7];
				if ( selection === "NormalSpin" ) {
					selection = "Normal Spin";
				} else if ( selection === "FreeSpin" ) {
					selection = "Free Spin";
				}
				var result = subMatches[8];

				var detailsString = subMatches[9];
				// unescape json string stored in history
				detailsString = detailsString.replace( /%26/g, '&' );
				detailsString = detailsString.replace( /%%/g, '%' );
				var detail = JSON.parse( detailsString );
				entries.push( {
					'time': time, 'ref': ref, 'action': action, 'amount': amount, 'won': won, 'picked': picked,
					'selection': selection, 'result': result, 'detail': detail
				} );
			}
		}

		return entries;
	};


	var betResultsToHistoryString = function( platformInterface, config, betResults ) {

		function _to2decimals( money ) {
			var result = (money / 100).toFixed( 2 );
			return result;
		}

		function formatScreenNames( screen ) {
			// replace symbol IDs with string names.
			// reverse screen column order (WH require array of rows - so 5x3 game would be 3 subarrays of 5 symbols each
			var newScreen = [];
			for ( var row = 0; row < screen[0].length; ++row ) {
				var newReel = [];
				for ( var reel = 0; reel < screen.length; ++reel ) {
					newReel.push( game.logic.GameData.reels.symbols.shortNameToLongName( screen[reel][row] ) );
				}
				newScreen.push( newReel );
			}
			return newScreen;
		}

		function formatWinlinesArray( betResultWinlines ) {

			return betResultWinlines.map( function( winline ) {
				var longSymbolName = game.logic.GameData.reels.symbols.shortNameToLongName( winline['symbol'] );
				return {
					'lineId': winline['line'],
					'award': _to2decimals( winline['win'].toN() ),
					'symbol': longSymbolName,
					'length': winline['run'],
					'multiplier': 1
				};
			} );
		}

		var firstOperation = config['betSelection']['checkpointId'] === 0;

		var history_game_code = platformInterface.getGameCode();
		var history_ref = platformInterface.getBetId();
		var history_date_string = libs.getDateString();

		var history_bet_value = _to2decimals( firstOperation ? config['betSelection']['total_stake'].toN() : 0 );
		var stakePerLine = _to2decimals( firstOperation ? config['betSelection']['stake'].toN() : 0 );
		var history_winnings = _to2decimals( betResults['win'].toN() );

		var history_selections = betResults['checkpointName'];

		function appendToResultsString( results, append ) {
			if ( results.length > 0 ) {
				results = results + '+';
			}
			return results + append;
		}

		var results = "";
		if ( betResults['win'].toN() > 0 ) {
			results = appendToResultsString( results, "Win" );
		}
		if ( betResults['bonus'] && betResults['bonus'].length > 0 ) {
			results = appendToResultsString( results, "Bonus" );
		}
		if ( betResults['jackpotWins'] && betResults['jackpotWins'].length > 0 ) {
			results = appendToResultsString( results, "Jackpot" );
		}
		if ( !results ) {
			results = "NoWin";
		}

		var legacyHeader = ''; // TODO: replace '99' with game version string, major.minor.patch
		legacyHeader += "History:1;1;" + history_game_code + ";" + history_ref + ";" + history_date_string + ";" + history_bet_value + ";" + history_winnings + ";99;" + stakePerLine;
		legacyHeader += ";" + history_selections + ";" + results + ";";

		var translateJackpotObj = function( jpArray ) {
			if ( jpArray ) {
				return jpArray.map( function( jp ) {
					return {
						'tier': (jp['id'] + 1),
						'value': jp['win'].div( 100 ).toFixed( 2 )
					};
				} );
			} else {
				return [];
			}
		};

		var jsonObj = {
			'cost': history_bet_value,
			'won': history_winnings,
			'value': _to2decimals( config['betSelection']['total_stake'].toN() ),
			'extras': {
				// Put freespins stuff / any extra features in here
				'winCount': betResults['winCount'],
				'stakePerLine': stakePerLine,
				'bonus': betResults['bonus'],
				'jackpotWins': translateJackpotObj( betResults['jackpotWins'] )
			}
		};

		if ( betResults['reels'] ) {
			if ( betResults['reels']['reelset'] ) {
				jsonObj['reelSet'] = betResults['reels']['reelset'];
			}
			if ( betResults['reels']['stops'] ) {
				jsonObj['reelPos'] = betResults['reels']['stops'];
			}
			if ( betResults['reels']['screen'] ) {
				jsonObj['visibleSymbols'] = formatScreenNames( betResults['reels']['screen'] );
			}
			if ( betResults['reels']['winline'] ) {
				jsonObj['winLines'] = formatWinlinesArray( betResults['reels']['winline'] );
			}
		}

		function replaceBigNumber( key, value ) {
			if ( value instanceof BigNumber ) {
				return value.toString();
			}
			return value;
		}

		var jsonString = JSON.stringify( jsonObj, replaceBigNumber );
		// escape all ampersands as this is what WH history logging splits fields by - use percent encoding
		jsonString = jsonString.replace( /%/g, '%%' ); // replace single percents with doubles.
		jsonString = jsonString.replace( /&/g, '%26' ); // replace ampersand with percent 26h
		return legacyHeader + jsonString;
	};


	libs.server.williamHill.betResultsToHistoryString = betResultsToHistoryString;
	libs.server.williamHill.historyRecordListToObject = historyRecordListToObject;

}());
