/*global ActiveXObject,Response*/
goog.provide( 'libs.server.platform.AspServerWilliamHills' );


(function() {
	"use strict";

	var AspServerWilliamHills = function( specificSessionId, specificGameCode ) {

		this._balance = 0;
		this._sessionId = '';
		this._gameCode = '';
		this._configGameCode = ''; // GetConfig / SaveConfig methods require a seperate code to the gamecode, which is the same for both mobile/desktop.
		this._jackpotValues = [];
		this._gameSupportsJackpots = game.logic.GameData.jackpots && Array.isArray( game.logic.GameData.jackpots.list ) && game.logic.GameData.jackpots.list.length > 0;
		this.connect( specificSessionId, specificGameCode );

	};

	AspServerWilliamHills.prototype._setupSDKJackpots = function() {

		if ( this._gameSupportsJackpots ) {
			var jpInfo = game.logic.GameData.jackpots.list;

			// Calculate total stake contribution
			var totalStakeContrib = 0.0;
			var jackpotIndex;
			for ( jackpotIndex = 0; jackpotIndex < jpInfo.length; ++jackpotIndex ) {
				totalStakeContrib += jpInfo[jackpotIndex].stakeContribution;
			}

			this._objGameSvr.ClearJackpot( this._gameCode ); // <- Clear any jackpot settings for the Jackpot for game code PW
			this._objGameSvr.SetJackpotType( this._gameCode, "NORMAL" ); // <- Set the Jackpot Type to Normal Jackpot
			this._objGameSvr.SetJackpotAllocationPercentage( this._gameCode, totalStakeContrib ); // <- Set the % of stake allocated to jackpot

			for ( jackpotIndex = 0; jackpotIndex < jpInfo.length; ++jackpotIndex ) {
				var jp = jpInfo[jackpotIndex];
				var jpContrib = Math.round( ( jp.stakeContribution / totalStakeContrib ) * 1000 ) / 1000;

				this._objGameSvr.SetJackpotTierAmount( this._gameCode, jackpotIndex + 1, jp.initialValue / 100 ); // <- Create a jackpot tier with an initial amount
				this._objGameSvr.SetJackpotTierReseedAmount( this._gameCode, jackpotIndex + 1, jp.reseedValue / 100 ); // <- Set the reseed amount for the tier
				this._objGameSvr.SetJackpotTierAllocationPercentage( this._gameCode, jackpotIndex + 1, jpContrib ); // <- Set the % of the jackpot allocation to go to
			}
		}
	};


	///////////////////////// Setup


	AspServerWilliamHills.prototype.connect = function( specificSessionId, specificGameCode ) {

		// Get session info from the cookie (as set by game_load.asp)
		if ( specificSessionId && specificSessionId.length > 0 ) {
			this._sessionId = specificSessionId;
		} else {
			this._sessionId = libs.server.getRequestCookie( "GameRoom", "ID", true ); // encode the value just in case it has been tampered with
		}

		if ( !this._sessionId || this._sessionId.length === 0 ) {
			throw( { message: "Failed to retrieve session ID - do you have cookies enabled?" } );
		}

		this._objGameCon = new ActiveXObject( "GameServer.GameCon" );

		if ( !this._objGameCon ) {
			throw( { message: "Failed to get GameServer.GameCon object" } );
		}

		this._objGameSvr = this._objGameCon.Gamesvr;
		if ( !this._objGameSvr ) {
			throw( { message: "Failed to get GameServer object" } );
		}

		// See if GameServer online
		var online = this._objGameSvr.GetOnline();
		if ( !online ) {
			throw( { message: "GameServer is not online" } );
		}

		// Get the Game code
		if ( specificGameCode && specificGameCode.length > 0 ) {
			this._gameCode = specificGameCode;
		} else {
			var maxGameCodeLen = 8; // Have seen 2 and four codes so 8 should sane.
			this._gameCode = this._objGameSvr.GetGameCode( this._sessionId );
			if ( !((this._gameCode) && (this._gameCode.length <= maxGameCodeLen)) ) { //This may be an error message
				throw( { message: "Session " + this._sessionId + " has invalid gameCode: " + this._gameCode + " (possibly session not created)" } );
			}
		}

		this._configGameCode = game.logic.GameData.williamHill.configGameCode;

		// Get balance and jackpot info.
		this._jackpotValues = [];
		if ( this._gameSupportsJackpots ) {
			var jackpotDetails;
			try {
				jackpotDetails = this._objGameSvr.GetJackpotDetails( this._sessionId ); //changed from GetJackpotDetailsForGame(this._sessionId, this._gameCode);
			}
			catch ( err ) {
				// Probably due to jackpots not being set in SDK. Set them up and try again
				try {
					this._setupSDKJackpots();
					jackpotDetails = this._objGameSvr.GetJackpotDetails( this._sessionId );
				}
				catch ( err2 ) {
					throw( { friendlyMessage: err.message + " Failed to get jackpot details for gamecode '" + this._gameCode + "' (possibly not set)" } );
				}
			}

			// Multiply out each jackpot detail to become currency units
			if ( jackpotDetails ) {
				var jackpotDetailsArr = jackpotDetails.split( "|" );
				for ( var i = 0; i < jackpotDetailsArr.length; i++ ) {
					this._jackpotValues.push( {
						'id': i,
						'money': new BigNumber( jackpotDetailsArr[i] ).times( 100 )
					} );
				}
			}
		}
		this._balance = Math.round( 100 * this._objGameSvr.GetAccountBalance( this._sessionId ) );
	};


	AspServerWilliamHills.prototype.getBalance = function() {
		return this._balance;
	};


	AspServerWilliamHills.prototype.getPlayerSessionId = function() {
		return this._sessionId;
	};


	AspServerWilliamHills.prototype.getGameCode = function() {
		return this._gameCode;
	};


	AspServerWilliamHills.prototype.getJackpotValue = function( jackpotId ) {
		if ( this._gameSupportsJackpots ) {
			if ( jackpotId < this._jackpotValues.length ) {
				return this._jackpotValues[jackpotId].money;
			} else {
				throw( { message: "Logic error: platformInterface.getJackpotValue() called with invalid value: jackpotId=" + jackpotId } );
			}
		} else {
			throw( { message: "Logic error: platformInterface.getJackpotValue() called even though no jackpots are defined in gameData.js" } );
		}
	};


	AspServerWilliamHills.prototype.getJackpotCount = function() {
		if ( this._gameSupportsJackpots ) {
			return this._jackpotValues.length;
		} else {
			return 0;
		}
	};


	AspServerWilliamHills.prototype.takeStake = function( currentSequence, stakeValue, win, eligibleForJackpot ) {

		var recordStake = stakeValue;
		if ( currentSequence === 0 ) {
			this._balance -= stakeValue;

			if ( eligibleForJackpot ) {
				this._objGameSvr.RecordProgressiveStake( this._sessionId, stakeValue / 100 );
				recordStake = 0;
			}
		} else {
			recordStake = 0;
		}

		// Tell server what happened. ??Do we do this if have no win or if won a jackpot?
		var balance = Math.round( 100 * this._objGameSvr.RecordStakeAndReturn( this._sessionId, recordStake / 100, win / 100 ) );
		this.logMessage( "Bet: sequence=" + currentSequence + " recorded - win=" + win + " balance=" + balance );
	};


	AspServerWilliamHills.prototype.createXmlResponse = function( responseObj ) {

		var ts = ( new Date() ).valueOf();
		return libs.xmlUtils.wrapJsonInXml( "jsonResponse", {
			'data': responseObj,
			'timestamp': ts,
			'money': this._balance,
			'jackpots': this._jackpotValues
		} );
	};


	///////////////////////// Logging


	AspServerWilliamHills.prototype._formatLogMessage = function( msg ) {

		return this._gameCode + ': ' + msg;
	};


	AspServerWilliamHills.prototype.logMessage = function( msg ) {

		this._objGameSvr.LogMess( this._sessionId, this._formatLogMessage( msg ) );
	};


	AspServerWilliamHills.prototype.logSuspicious = function( msg ) {

		this._objGameSvr.LogSuspicious( this._sessionId, this._formatLogMessage( msg ) );
	};


	AspServerWilliamHills.prototype.logError = function( msg ) {

		this._objGameSvr.LogError( this._sessionId, this._formatLogMessage( msg ) );
	};


	///////////////////////// Game state


	AspServerWilliamHills.prototype.persistentStateLoad = function() {

		var override = libs.server.platform.debug.getStaticDataUsingApplicationContents( this._sessionId, this._gameCode );

		if ( override && override.length > 0 ) {
			try {
				var obj = libs.xmlUtils.jsonParse( override );
				libs.server.platform.debug.clearStaticDataUsingApplicationContents( this._sessionId, this._gameCode );
				return obj;
			}
			catch ( err ) {}
		}

		var rawString;
		try {
			rawString = this._objGameSvr.GetConfig( this._sessionId, this._configGameCode );
			//this.logMessage( "persistentStateLoad: '" + rawString + "'" );
		}
		catch ( err ) {
			throw new Error( "Error executing gameServer.GetConfig() - Are you running the latest WH platform?" );
		}
		if ( rawString.length > 0 ) {
			return libs.xmlUtils.unwrapJsonFromXml( 'playerState', rawString );
		}

		return null;
	};


	AspServerWilliamHills.prototype.persistentStateSave = function( stateObject ) {

		var rawString = libs.xmlUtils.wrapJsonInXml( 'playerState', stateObject );
		//this.logMessage( "persistentStateSave: '" + rawString + "'" );
		this._objGameSvr.SaveConfig( this._sessionId, this._configGameCode, rawString );
	};


	AspServerWilliamHills.prototype.persistentStateClear = function() {

		this._objGameSvr.ClearConfig( this._sessionId, this._configGameCode );
	};


	AspServerWilliamHills.prototype.stateLoad = function() {

		try {
			var rawString = this._objGameSvr.stateLoad( this._sessionId, this._gameCode );
			if ( rawString.length > 0 ) {
				return libs.xmlUtils.unwrapJsonFromXml( 'state', rawString );
			}
		}
		catch ( err ) {
		}
		return null;
	};


	AspServerWilliamHills.prototype.stateSave = function( stateObject, totalStake ) {

		// Last argument is the total stake however bubble agent always passed zero to it
		// Update 24/2/2014 - Feedback from WH say pass the stake to it (Rohit IG-46)
		var rawString = libs.xmlUtils.wrapJsonInXml( 'state', stateObject );
		this._objGameSvr.stateSave( this._sessionId, this._gameCode, rawString, Math.floor( totalStake / 100 ) );
	};


	AspServerWilliamHills.prototype.stateClear = function() {

		this._objGameSvr.stateClear( this._sessionId, this._gameCode );
	};


	///////////////////////// Jackpots


	AspServerWilliamHills.prototype.claimJackpot = function( jackpotId ) {

		if ( this._gameSupportsJackpots && this._jackpotValues.length > jackpotId ) {
			var jackpotTier = jackpotId + 1;
			var jackpotExpectedValue = this.getJackpotValue( jackpotId );
			var actualValueWon = new BigNumber( this._objGameSvr.RecordJackpotWin( this._sessionId, jackpotTier, 0, jackpotExpectedValue.dividedBy( 100 ).toN() ) ).times( 100 );

			// Check that jackpotWon >= jackpotExpected
			if ( actualValueWon.lessThan( jackpotExpectedValue ) ) {
				throw( { message: "Server cannot allow bet at this time - try again", errorType: "1" } );
			}

			return actualValueWon;
		} else {
			throw( { message: "Logic error: platformInterface.claimJackpot() called with invalid jackpotId (" + jackpotId + ")" } );
		}
	};


	///////////////////////// Cashing out


	AspServerWilliamHills.prototype.getCurrencyCode = function() {

		// TODO: cache these get functions
		return this._objGameSvr.GetCurrCode( this._sessionId );
	};


	// Note: Is this an sdk-only function?
	AspServerWilliamHills.prototype.cashOutReset = function() {

		var currencyCode = this.getCurrencyCode();
		var accountId = this._objGameSvr.GetAccountId( this._sessionId );

		// In the docs it says amount should be zero but BubbleAgent passes the current balance
		this._objGameSvr.CashOut( this._sessionId, currencyCode, this._balance / 100, '' );

		this._sessionId = this._objGameSvr.CreateSession( accountId, currencyCode, this._gameCode, /*payPerPlay*/'Y' );
		Response.AddHeader( "Set-Cookie", "GameRoom=ID=" + this._sessionId + "; path=/; HttpOnly" );

		var balance = Math.round( 100 * this._objGameSvr.GetAccountBalance( this._sessionId ) );
		// Re-establish the balance
		this._objGameSvr.RecordStakeAndReturn( this._sessionId, (balance - this._balance) / 100, 0 );
	};


	AspServerWilliamHills.prototype.setBalance = function( newBalance ) {

		var amountToTake = this._balance - newBalance;
		if ( amountToTake > 0 ) {
			this._objGameSvr.RecordStakeAndReturn( this._sessionId, amountToTake / 100, 0 );
		} else if ( amountToTake < 0 ) {
			this._objGameSvr.RecordStakeAndReturn( this._sessionId, 0, -amountToTake / 100 );
		}
	};


	///////////////////////// History


	AspServerWilliamHills.prototype.getBetId = function() {

		// TODO: cache these get functions
		return this._objGameSvr.GetBetId();
	};


	AspServerWilliamHills.prototype.historyGetSession = function() {

		return this._objGameSvr.GetSessionHistory( this._sessionId, this._gameCode );
	};


	AspServerWilliamHills.prototype.historyUpdate = function( historyStr ) {

		return this._objGameSvr.UpdateHistory( this._sessionId, historyStr );
	};


	///////////////////////// Random number retrieval


	AspServerWilliamHills.prototype._getNumbers = function( logString, lowestInclusive, highestInclusive, numberCount ) {

		// WH required logging of rng calls, accurate 21/2/2014 (IG-15)
		this.logMessage( "RNG: Requesting random numbers sportsCode=" + this._gameCode + " event=" + logString + " unique=false upper=" + highestInclusive + " lower=" + lowestInclusive + " size=" + numberCount );

		var range = highestInclusive - lowestInclusive;
		var numbersCSV = this._objGameSvr.GetNumbersCSV( this._sessionId, this._gameCode, logString, false, range, 0, numberCount );

		this.logMessage( "RNG: Received response " + numbersCSV );

		var numbersStringArray = numbersCSV.split( "," );
		var numbers = [];
		for ( var i = 0; i < numbersStringArray.length; i++ ) {
			var num = Number( numbersStringArray[i] );
			// Verify number falls within requested range (IG-163)
			if ( num < 0 || num > range ) {
				throw {
					message: 'Unexpected result from GetNumbersCSV: Random number received not in requested range',
					friendlyMessage: 'Invalid response received from server'
				};
			}
			numbers.push( lowestInclusive + num );
		}

		if ( numbers.length !== numberCount ) {
			throw {
				message: 'Unexpected result from GetNumbersCSV: Number count received does not match requested count',
				friendlyMessage: 'Invalid response received from server'
			};
		}

		return numbers;
	};


	AspServerWilliamHills.prototype.getRandomNumber = function( type, lowestInclusive, highestInclusive ) {
		return this.getRandomNumbers( type, lowestInclusive, highestInclusive, 1 )[0];
	};

	AspServerWilliamHills.prototype.getRandomNumbers = function( type, lowestInclusive, highestInclusive, numberCount ) {
		var MaxGetNumbersHighest = 99999;

		numberCount = numberCount || 1;

		var accountId = this._objGameSvr.GetAccountId( this._sessionId );
		var numbers = libs.server.platform.debug.acquireForcedRandomNumbers( this, accountId, this._gameCode, type, lowestInclusive, highestInclusive, numberCount );

		var numbersLeft = numberCount - numbers.length;
		if ( numbersLeft > 0 ) {
			var range = highestInclusive - lowestInclusive;
			if ( range < MaxGetNumbersHighest ) {
				numbers = numbers.concat( this._getNumbers( type, lowestInclusive, highestInclusive, numbersLeft ) );
			}
			else {
				var ExtensionFactor = 1000;

				if ( ( ( range + 1 ) / ExtensionFactor ) < MaxGetNumbersHighest && ( ( range + 1 ) % ExtensionFactor ) === 0 ) { //range divisible
					var topRange = ( range + 1 ) / ExtensionFactor;

					var firstNumber = this._getNumbers( type, 0, topRange - 1, numbersLeft );
					var secondNumber = this._getNumbers( type, 0, ExtensionFactor - 1, numbersLeft );

					for ( var i = 0; i < firstNumber.length; i++ ) {
						var chance = firstNumber[i] * ExtensionFactor + secondNumber[i] + lowestInclusive;
						numbers.push( chance );
					}
				}
				else {
					throw {
						message: "Range " + (range + 1) + " not divisible by extension factor " + ExtensionFactor,
						friendlyMessage: 'Invalid response received from server'
					};
				}
			}
		}

		libs.server.platform.debug.saveRNGValueUsingApplicationContents( accountId, this._gameCode, numbers );

		return numbers;
	};


	AspServerWilliamHills.prototype.closeBet = function() {
		var accountId = this._objGameSvr.GetAccountId( this._sessionId );
		this.logMessage( 'closeBet: accountId=' + accountId + ' this._sessionId=' + this._sessionId );
		libs.server.platform.debug.logRandomNumberStringPlatform( this, accountId, this._gameCode );
	};


	AspServerWilliamHills.prototype.startFirstBetOperation = function() {
		var accountId = this._objGameSvr.GetAccountId( this._sessionId );
		libs.server.platform.debug.clearSavedRNGValueUsingApplicationContents( accountId, this._gameCode );
	};


	AspServerWilliamHills.prototype.getPreviousRngString = function() {
		var accountId = this._objGameSvr.GetAccountId( this._sessionId );
		return libs.server.platform.debug.getSavedRNGValueUsingApplicationContents( accountId, this._gameCode );
	};


	///////////////////////// SDK only static methods


	AspServerWilliamHills.forceCreateSession = function( accountId, gameCode, currCode ) {

		currCode = currCode || 'GBP';
		var objGameCon = new ActiveXObject( "GameServer.GameCon" );
		if ( objGameCon ) {
			var objGameSvr = objGameCon.Gamesvr;
			if ( objGameSvr ) {
				var online = objGameSvr.GetOnline();
				if ( online ) {
					var sessionId = objGameSvr.CreateSession( accountId, currCode, gameCode, /*payPerPlay*/'Y' );
					Response.AddHeader( "Set-Cookie", "GameRoom=ID=" + sessionId + "; path=/; HttpOnly" );
				}
			}
		}
	};


	AspServerWilliamHills.forceRng = function( rngString, accountId, gameCode ) {
		return libs.server.platform.debug.forceRNGUsingApplicationContents( accountId, gameCode, rngString );
	};


	AspServerWilliamHills.forceStaticData = function( staticDataString, accountId, gameCode ) {
		libs.server.platform.debug.saveStaticDataUsingApplicationContents( accountId, gameCode, staticDataString );
	};


	libs.server.platform.AspServerWilliamHills = AspServerWilliamHills;

}());

