<%
' *--------------------------------------------------------------------------------
' Bet String functions for all new Arcade games
' *--------------------------------------------------------------------------------
' * Author: Daniel triggs (AshGaming)
'This file requires the errorHandlingFunctions.asp file to be included for the errors to get raised (and the strGameCode and ID etc set) 
'It also requires the numberFunctions.asp to be included

'These global variables is used to store the bet details (placebet and other files that include this one should not use this variable name)
Dim a_glb_bets
a_glb_bets= Array() ' set is as a blank array to start with (so always set and is the correct type)

'This function takes the betstring and then splits it up to get each of the different bets and the parts within each bet
'Note: this function wont work with current games (such as blackjack and joker poker) due to the way the betstrings are done in different ways, but for future games they should be done in a standard way so this can be used.
'Bet strings should be structured like this:
'Bet type:stake(s):details|Bet type:stake(s):details
'Where each bet is split using "|" symbols (if only 1 bet then wont contain that at all), the different parts of the bet (type of bet, stake and any other details) are split using ":" and if there are several stakes or details then those are split using ",". 
'If there is no stake or details for that request then those parts will be blank/empty.
'For example: 
'B1JBET:1.00,1.00,0.00,1.00,0.00:
'B1JSTD:: 
'QBFRES:10.00:01|QBFHME:3.00:03|QBFSPE:0.50:04
Sub parseBetString(s_betString)
	's_betString should first have been validated to make sure it only contains the characters allowed in a bet string and isnt too long/blank
	on error resume next
	'Split the string into the individual bets:
	dim a_bets
	a_bets = Split(s_betString,"|")
	if err.Number<>0 then 'error occurred trying to split array
		returnError "Error with bets", "Error occurred while checking Bet string " & Err.Description & " Bet string=" & s_betString,true
	End if

	a_glb_bets = a_bets ' set the a_glb_bets variable to make it an array that is the same length as the number of bets

	'Now loop through each of the bets and split the individual parts up into arrays and check they are valid
	dim i,j,a_tempBetDetails,s_stake,a_stakes

	for i=0 To UBound(a_bets)
		'split this bet into the 3 individual parts
		a_tempBetDetails = Split(a_bets(i),":")
		if UBound(a_tempBetDetails)<>2 OR Err.Number<>0 then ' this bet didnt contain two ":"'s to split it into 3 parts so invalid bet
			returnError "Error with bets"& Err.Description, "Error occurred while checking Bet in position " & i &". "& Err.Description & " Full Bet string=" & s_betString,true
			exit for
		End if

		'check the stake value to see if it contains multiple stakes
		s_stake = a_tempBetDetails(1)
		if len(s_stake)>0 then ' if not >0 then skip this (could be Stand or Hit request in blackjack etc so no stake required)
			if instr(",",s_stake) >=0 then ' contains a comma so multiple stakes
				a_stakes = Split(s_stake,",")
				'check each of the stakes is a valid double (if not raise error)
				for j=0 to UBound(a_stakes)
					a_stakes(j) = stringToDouble(a_stakes(j),2,false) ' convert the string value to a double and check it is a valid number with no more than 2 decimal places and not negative 
					if Err.Number<>0 OR isNull(a_stakes(j)) then ' stringTodouble return null so value is invalid
						returnError "Error with bets", "Error with the stakes for bet in position " & i &". "& Err.Description & " Full Bet string=" & s_betString,true
					End if
				Next
			Else
				s_stake = stringToDouble(s_stake)
				if Err.Number<>0 OR isNull(s_stake) then ' stringTodouble return null so value is invalid
					returnError "Error with bets", "Error with the stake for bet in position " & i &". "& Err.Description & " Full Bet string=" & s_betString,true
				End if
			End if
		End if
		a_glb_bets(i) = a_tempBetDetails
'		Response.write("<br>" & i & ")" & Join(a_glb_bets(i),":"))
	Next
End Sub

'This function simply returns the total number of bets that were placed
Function getTotalBetStrings()
	getTotalBetStrings = Ubound(a_glb_bets)+1
End Function

'This function returns the type of bet for the bet in the given index
'returns null if the index is out of range
Function getBetCode(i_index)
	if i_index < 0 OR i_index > Ubound(a_glb_bets) then ' out of range
		getBetCode=Null
	Else 
		getBetCode = a_glb_bets(i_index)(0)
	End if
End Function

'This function returns the stake (or stakes) for the bet in the given index
'returns null if the index is out of range or there was no stake for that bet
'returns a double if there is just one stake or returns all the stakes as a comma separated string if there are several stakes
Function getBetStake(i_index)
	if i_index < 0 OR i_index > Ubound(a_glb_bets) then ' out of range
		getBetStake=Null
	Else 
		Dim s_stake
		s_stake = a_glb_bets(i_index)(1)
		if instr(",",CStr(s_stake)) >=0 then
			getBetStake = s_stake ' return all the stakes as they are
		Else
			getBetStake = CDbl(s_stake)
		End if
	End if
End Function

'This function returns the details for the bet in the given index. The details are returned as they are (in a string) as some games will have characters here and others numbers
'returns null if the index is out of range
Function getBetDetails(i_index)
	if i_index < 0 OR i_index > Ubound(a_glb_bets) then ' out of range
		getBetDetails=Null
	Else 
		getBetDetails = a_glb_bets(i_index)(2)
	End if
End Function

%>
