<%
' *--------------------------------------------------------------------------------
' Convenience Functions for all new Arcade games
' *--------------------------------------------------------------------------------
' * Author: Daniel triggs (AshGaming)
' * 25/Apr/08 - updated the f_getPosFromWeightings function so it still works if a weighting is 0

'Used to sort an array of integer numbers (ASP doesn't have a sort function)
Function sortIntegerArray(ByVal a_array,b_descending)
'note - this will only sort integers (due to the Cint part) - must be within range for int numbers
'The second variable should be a boolean - if it is true then the values are sorted in decending order, otherwise they are sorted in ascending order
	dim i,j,temp
	for i = UBound(a_array) - 1 To 0 Step -1 ' go from second to last pos backwards
		for j= 0 to i
			if b_descending then
				if CInt(a_array(j))<CInt(a_array(j+1)) then
					temp=a_array(j+1)
					a_array(j+1)=a_array(j)
					a_array(j)=temp
				end if
			else 
				if CInt(a_array(j))>CInt(a_array(j+1)) then
					temp=a_array(j+1)
					a_array(j+1)=a_array(j)
					a_array(j)=temp
				end if
			End if
		next
	next
	sortIntegerArray = a_array ' return the sorted array
End Function

'Used to sort an array of double numbers
Function sortDoubleArray(ByVal a_array,b_descending)
'The second variable should be a boolean - if it is true then the values are sorted in decending order, otherwise they are sorted in ascending order
	dim i,j,temp
	for i = UBound(a_array) - 1 To 0 Step -1 ' go from second to last pos backwards
		for j= 0 to i
			if b_descending then
				if CDbl(a_array(j))<CDbl(a_array(j+1)) then
					temp=a_array(j+1)
					a_array(j+1)=a_array(j)
					a_array(j)=temp
				end if
			else 
				if CDbl(a_array(j))>CDbl(a_array(j+1)) then
					temp=a_array(j+1)
					a_array(j+1)=a_array(j)
					a_array(j)=temp
				end if
			End if
		next
	next
	sortDoubleArray = a_array ' return the sorted array
End Function

'sorts strings into alphabetical order
Function sortStringArray(ByVal a_array,b_descending)
'The second variable should be a boolean - if it is true then the values are sorted in decending order, otherwise they are sorted in ascending order
	dim i,j,temp
	for i = UBound(a_array) - 1 To 0 Step -1 ' go from second to last pos backwards
		for j= 0 to i
			if b_descending then
				if CStr(a_array(j))<CStr(a_array(j+1)) then
					temp=a_array(j+1)
					a_array(j+1)=a_array(j)
					a_array(j)=temp
				end if
			else 
				if CStr(a_array(j))>CStr(a_array(j+1)) then
					temp=a_array(j+1)
					a_array(j+1)=a_array(j)
					a_array(j)=temp
				end if
			End if
		next
	next
	sortStringArray = a_array ' return the sorted array
End Function

Function StringToIntArray (s_string,s_separator)
	'Note - all the values must be in the valid range for integers (-32768 OR i_num>32767)
	Dim a_array,i
	a_array = Split(s_string,s_separator)
	For i=0 To UBound(a_array)
		a_array(i) = CInt(a_array(i))
	Next
	StringToIntArray = a_array
End Function

Function StringToDoubleArray (s_string,s_separator)
	Dim a_array,i
	a_array = Split(s_string,s_separator)
	For i=0 To UBound(a_array)
		a_array(i) = CDbl(a_array(i))
	Next
	StringToDoubleArray = a_array
End Function

Function IIf(i,j,k) ' immediate if statement
'The IIf() function is used in place of If..Then..Else statements, where a single expression is being tested and specific values are returned if the expression evaluates as true or false
	If i Then IIf = j Else IIf = k
End Function

'This function takes and index number and an array as variables and deletes the element in the given index from the given array.
'The array is passed by reference and updated so no output is returned
'a_array should contain the array that needs the element deleted from (this should be an array containing at least one item)
'i_index should be an integer number between 0 and the size of the array -1
Sub deleteElement(ByRef a_array, ByVal i_index) 'previously called f_deleteFromArray
'Deletes the element at position i_index from the given array (by moving all the elements after it down one and resizing the array). 
'The array is passed by reference - so changes to it made here will change the actual array.
	Dim i,ub
	ub = UBound(a_array) - 1
	i_index = CInt(i_index)
	if i_index>=0 AND i_index<=UBound(a_array) then 
		For i = i_index To ub
			a_array(i) = a_array(i + 1)
		Next
		ReDim Preserve a_array(ub)
	Else 
		Err.Raise vbObjectError +100, "deleteElement","deleteElement Invalid input: Index to delete is not between 0 and UBound for the given array. "& Err.description
	End if
End Sub

'This function is similar to "deleteElement" but it deletes several elements from the given array instead of just one element.
'It takes two arrays - the one with the data in and one with the list of indexes to remove 
'Deletes all the elements in the index positions listed, from the given array.
'Note: can't use the f_deleteFromArray function and loop round as once the first one is removed the indexes for the remaining ones could be affected (eg if need to remove positions 0,2 and 4 then as soon as remove 0 the value that was in 2 will now be in 1 - so deleting index 2 would actually be deleting the original index 3)
Sub deleteElements(ByRef a_array, ByVal a_indexes) 'previously called f_deleteSeveralFromArray
	Dim i,j,new_Array,i_newArraySize,i_counter,b_valueUsed
	if UBound(a_array)>=UBound(a_indexes) AND UBound(a_indexes)>=0 then ' lengths ok so continue
		i_newArraySize = (UBound(a_array) - UBound(a_indexes))-1 ' calulate new size for array. -1 at end because Array(5) actually makes it 6 long in asp
		new_Array = Array(i_newArraySize) ' declare is as an array
		ReDim new_Array(i_newArraySize) ' set size of the array to the new size
		i_counter = 0
		For i = 0 To UBound(a_array)
			b_valueUsed = false
			if UBound(a_indexes)>=0 then
				For j = 0 To UBound(a_indexes)
					if i= CInt(a_indexes(j)) then
						b_valueUsed = true
						deleteElement a_indexes,j
						exit For
					end if
				Next
			end if
			if b_valueUsed=false then
				new_Array(i_counter) = a_array(i)
				i_counter = i_counter +1
			end if
		Next
		a_array = Array()
		ReDim a_array(i_newArraySize) 'resize the a_array to the new size
		a_array = new_Array ' update the a_array with the new values
	Else
		Err.Raise vbObjectError +100, "deleteElements","deleteElements Invalid input: The number of elements to delete must be greater than 0 but not greater than the number of elements in the given array. "& Err.description
	End if
End Sub

'f_sumUpArray: sum up the all the values in an array (used to find the total value for weightings then can get rng number between 1 and that number and use f_getPosFromWeightings function to find which position it was in)
Function f_sumUpArray(byVal a_weightings)
	dim i,tempTotal
	tempTotal = 0
	for i = 0 to UBound(a_weightings)
		tempTotal = tempTotal + CLng(a_weightings(i))
	next
	f_sumUpArray = CLng(tempTotal)
End Function

'works out the result from the weightings array and random number given. (Random number should be between 1 and sum of weightings values)
'For example if weightings were 5,10,5,10 and the random number was 6 then it would return 1 as it would be the 2nd result (response is 0 based)
Function f_getPosFromWeightings(a_weightings,randomNumber)
	dim i,currentNumber, previousNumber, whichPosition
	randomNumber = CLng(randomNumber) ' long used instead of Int so can use numbers over 32k if ever need to
	whichPosition = -1
	currentNumber =0
	for i = 0 to UBound(a_weightings)
		if a_weightings(i)>0 then ' if the weighting is 0 then skip it - so can have weighting such as 0,4,0,5,1 and will never result in 1st and 3rd results
			currentNumber = currentNumber + a_weightings(i)
			if i = 0 then
				if randomNumber <= currentNumber AND randomNumber > 0 then
					whichPosition = i
					exit for
				end if
			else
				previousNumber = a_weightings(i-1)
				if randomNumber <= currentNumber AND randomNumber > previousNumber then
					whichPosition = i
					exit for
				end if
			end if
		End if
	next
	f_getPosFromWeightings = whichPosition
End Function
%>