<%
Function MFCurrencyAmount(strAmount, cur)
    'Verifies str is a valid positive cash amount for the currency, code cur.
    '
    'returns -1 for unrecognised/unsupported currency code
    'returns -2 for zero length supplied amount as string
    'returns -3 for too many decimal places after the decimal point for given currency
    'returns -4 for decimal point present in currency which has no decimal places
    'returns -5 for additional decimal point present in amount
    'returns -6 for invalid character present in string, includes negative sign
    'else returns the amount as a CDbl
    '
    'Note - only continues until first error condition met, only returns first condition found, may be more
    

    'Example -    MFCurrencyAmount("20.05", "GBP") returns 20.05
    'Example -    MFCurrencyAmount("20.005", "GBP") returns -3
    'Example -    MFCurrencyAmount("21", "INR") returns -1 - if indian rupees supported, needs to be included below with it's dp's, plus info for that currency added to other functions

    Dim strLen
    Dim iLooper
    Dim bIsNumeric
    Dim bIsDecimalPoint
    Dim iDecimalPlaces
    Dim iDecimalFoundAt
    Dim strChar
    Dim strTemp 
     
    MFCurrencyAmount = 0
    strLen = len(strAmount)
    iDecimalFoundAt = -1
    iDecimalPlaces = MFCurrencyDP(cur)
    
		if strLen = 0 then MFCurrencyAmount = -2 : exit function
		if iDecimalPlaces = -1 then MFCurrencyAmount = -1 : exit function
    
		for iLooper = 1 to strLen
			strChar = mid(strAmount,iLooper,1)
			bIsNumeric = (instr("0123456789", strChar) > 0)
			bIsDecimalPoint = (instr(".", strChar) > 0)
			if bIsNumeric then
				strTemp = strTemp & strChar
			elseif bIsDecimalPoint then 
				if iDecimalFoundAt >= 0 then ' already found a decimal point
					MFCurrencyAmount = -5 : exit function
				end if
				if iDecimalPlaces = 0 then ' currency shouldn't include decimal
					MFCurrencyAmount = -4 : exit function
				end if
				if strLen > iLooper + iDecimalPlaces then ' string to be checked has too many numbers after the decimal point
					MFCurrencyAmount = -3 : exit function
				else
					iDecimalFoundAt = iLooper
				end if
				strTemp = strTemp & strChar
			else
				MFCurrencyAmount = -6 : exit function ' string to be checked included an invalid character
			end if
		next
    
    if strTemp = "" then MFCurrencyAmount = -2 : exit function
    
    MFCurrencyAmount = CDbl(strTemp)
End Function
%>

<%
Function MFCurrencyDP(cur)
    'Returns decimal places for supported currency, code cur.
    'e.g. GBP - 2 dp
    '     JPY - 0 dp etc. - note current system uses 2dp, incorrectly, so for consitency also use 2d
    'returns -1 for unrecognised/unsupported currency code
    
    MFCurrencyDP = -1
    Select Case cur
			Case "GBP"
				MFCurrencyDP = 2
			Case "AUD"
				MFCurrencyDP = 2
			Case "CAD"
				MFCurrencyDP = 2
			Case "CHF"
				MFCurrencyDP = 2
			Case "SGD"
				MFCurrencyDP = 2
			Case "DKK"
				MFCurrencyDP = 2
			Case "HKD"
				MFCurrencyDP = 2
			Case "EUR"
				MFCurrencyDP = 2
			Case "USD"
				MFCurrencyDP = 2
			Case "SEK"
				MFCurrencyDP = 2
			Case "JPY"
				MFCurrencyDP = 2
		End Select
End Function
%>

<%
Function MFCurrencyMaxPayout(cur, gameCode)
    'Returns max payout for supported currency, code cur.
    'e.g. GBP - 100000
    '			JPY - 50000000 etc.
    'returns -1 for unrecognised/unsupported currency code
    
    MFCurrencyMaxPayout = -1
    Select Case cur
			Case "GBP"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 100000
					Case Else
						MFCurrencyMaxPayout = 100000
				End Select
			Case "AUD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 100000
					Case Else
						MFCurrencyMaxPayout = 100000
				End Select
			Case "CAD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 100000
					Case Else
						MFCurrencyMaxPayout = 100000
				End Select
			Case "CHF"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 100000
					Case Else
						MFCurrencyMaxPayout = 100000
				End Select
			Case "SGD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 100000
					Case Else
						MFCurrencyMaxPayout = 100000
				End Select
			Case "DKK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 1000000
					Case Else
						MFCurrencyMaxPayout = 1000000
				End Select
			Case "HKD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 1000000
					Case Else
						MFCurrencyMaxPayout = 1000000
				End Select
			Case "EUR"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 100000
					Case Else
						MFCurrencyMaxPayout = 100000
				End Select
			Case "USD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 100000
					Case Else
						MFCurrencyMaxPayout = 100000
				End Select
			Case "SEK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 1000000
					Case Else
						MFCurrencyMaxPayout = 1000000
				End Select
			Case "JPY"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxPayout = 10000000
					Case Else
						MFCurrencyMaxPayout = 10000000
				End Select
		End Select
End Function
%>

<%
Function MFCurrencyMaxStake(cur, gameCode)
    'Returns max stake for supported currency, code cur.
    'e.g. GBP - 10000
    '			JPY - 5000000 etc.
    'returns -1 for unrecognised/unsupported currency code

    'amended CMH 11/01/06 increased default max stakes (Heat ref 36860)
    
    MFCurrencyMaxStake = -1
    Select Case cur
			Case "GBP"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 1000
					Case Else
						MFCurrencyMaxStake = 10000
				End Select
			Case "AUD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 1000
					Case Else
						MFCurrencyMaxStake = 24400
				End Select
			Case "CAD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 1000
					Case Else
						MFCurrencyMaxStake = 23000
				End Select
			Case "CHF"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 1000
					Case Else
						MFCurrencyMaxStake = 22300
				End Select
			Case "SGD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 1000
					Case Else
						MFCurrencyMaxStake = 30600
				End Select
			Case "DKK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 10000
					Case Else
						MFCurrencyMaxStake = 107000
				End Select
			Case "HKD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 10000
					Case Else
						MFCurrencyMaxStake = 145500
				End Select
			Case "EUR"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 1000
					Case Else
						MFCurrencyMaxStake = 14500
				End Select
			Case "USD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 1000
					Case Else
						MFCurrencyMaxStake = 19000
				End Select
			Case "SEK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 10000
					Case Else
						MFCurrencyMaxStake = 131000
				End Select
			Case "JPY"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMaxStake = 200000
					Case Else
						MFCurrencyMaxStake = 2000000
				End Select
		End Select
End Function
%>

<%
Function MFCurrencyMinStake(cur, gameCode)
    'Returns max stake for supported currency, code cur.
    'e.g. GBP - 10000
    '			JPY - 5000000 etc.
    'returns -1 for unrecognised/unsupported currency code
    
    MFCurrencyMinStake = 99999999999999
    Select Case cur
			Case "GBP"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 0.25
					Case Else
						MFCurrencyMinStake = 0.25
				End Select
			Case "AUD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 0.25
					Case Else
						MFCurrencyMinStake = 0.25
				End Select
			Case "CAD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 0.25
					Case Else
						MFCurrencyMinStake = 0.25
				End Select
			Case "CHF"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 0.25
					Case Else
						MFCurrencyMinStake = 0.25
				End Select
			Case "SGD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 0.25
					Case Else
						MFCurrencyMinStake = 0.25
				End Select
			Case "DKK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 1
					Case Else
						MFCurrencyMinStake = 1
				End Select
			Case "HKD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 1
					Case Else
						MFCurrencyMinStake = 1
				End Select
			Case "EUR"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 0.25
					Case Else
						MFCurrencyMinStake = 0.25
				End Select
			Case "USD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 0.25
					Case Else
						MFCurrencyMinStake = 0.25
				End Select
			Case "SEK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 1
					Case Else
						MFCurrencyMinStake = 1
				End Select
			Case "JPY"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyMinStake = 100
					Case Else
						MFCurrencyMinStake = 100
				End Select
		End Select
End Function
%>

<%
Function MFCurrencyMinChipStake(cur, gameCode)
    'Returns min stake for supported currency, code cur.
    'e.g. GBP - 0.1
    '			JPY - 1 etc.
    'returns 99999999999999 for unrecognised/unsupported currency code
    
    MFCurrencyMinChipStake = 99999999999999
    Select Case cur
			Case "GBP"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 0.1
					Case Else
						MFCurrencyMinChipStake = 0.1
				End Select
			Case "AUD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 0.1
					Case Else
						MFCurrencyMinChipStake = 0.1
				End Select
			Case "CAD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 0.1
					Case Else
						MFCurrencyMinChipStake = 0.1
				End Select
			Case "CHF"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 0.1
					Case Else
						MFCurrencyMinChipStake = 0.1
				End Select
			Case "SGD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 0.1
					Case Else
						MFCurrencyMinChipStake = 0.1
				End Select
			Case "DKK"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 1
					Case Else
						MFCurrencyMinChipStake = 1
				End Select
			Case "HKD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 1
					Case Else
						MFCurrencyMinChipStake = 1
				End Select
			Case "EUR"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 0.1
					Case Else
						MFCurrencyMinChipStake = 0.1
				End Select
			Case "USD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 0.1
					Case Else
						MFCurrencyMinChipStake = 0.1
				End Select
			Case "SEK"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 1
					Case Else
						MFCurrencyMinChipStake = 1
				End Select
			Case "JPY"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyMinChipStake = 10
					Case Else
						MFCurrencyMinChipStake = 10
				End Select
		End Select
End Function
%>

<%
Function MFCurrencyDefaultStake(cur, gameCode)
    'Returns default stake for supported currency, code cur.
    'e.g. GBP - 10
    '			JPY - 5000 etc.
    'returns -1 for unrecognised/unsupported currency code
    
    MFCurrencyDefaultStake = -1
    Select Case cur
			Case "GBP"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 2.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 1.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 1.00
					Case Else
						MFCurrencyDefaultStake = 2.00
				End Select
			Case "AUD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 2.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 1.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 1.00
					Case Else
						MFCurrencyDefaultStake = 2.00
				End Select
			Case "CAD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 2.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 1.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 1.00
					Case Else
						MFCurrencyDefaultStake = 2.00
				End Select
			Case "CHF"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 2.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 1.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 1.00
					Case Else
						MFCurrencyDefaultStake = 2.00
				End Select
			Case "SGD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 2.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 1.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 1.00
					Case Else
						MFCurrencyDefaultStake = 2.00
				End Select
			Case "DKK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 10.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 10.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 10.00
					Case Else
						MFCurrencyDefaultStake = 10.00
				End Select
			Case "HKD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 10.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 10.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 10.00
					Case Else
						MFCurrencyDefaultStake = 10.00
				End Select
			Case "EUR"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 2.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 1.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 1.00
					Case Else
						MFCurrencyDefaultStake = 2.00
				End Select
			Case "USD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 2.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 1.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 1.00
					Case Else
						MFCurrencyDefaultStake = 2.00
				End Select
			Case "SEK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 10.00
					Case "QBHEX"
						MFCurrencyDefaultStake = 10.00
					Case "QBPKR"
						MFCurrencyDefaultStake = 10.00
					Case Else
						MFCurrencyDefaultStake = 10.00
				End Select
			Case "JPY"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyDefaultStake = 1000
					Case "QBHEX"
						MFCurrencyDefaultStake = 100
					Case "QBPKR"
						MFCurrencyDefaultStake = 100
					Case Else
						MFCurrencyDefaultStake = 1000
				End Select
		End Select
End Function
%>

<%
Function MFCurrencyStakeList(cur, gameCode)
    'Returns stake list for supported currency, code cur.
    'e.g. GBP - 0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00
    '     JPY - 0.00|100|200|500|1000|2000|5000|10000|20000|50000|100000|200000|500000|1000000
    'returns -1 for unrecognised/unsupported currency code
    
    MFCurrencyStakeList = ""
    Select Case cur
			Case "GBP"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
					Case Else
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
				End Select
			Case "AUD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
					Case Else
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
				End Select
			Case "CAD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
					Case Else
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
				End Select
			Case "CHF"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
					Case Else
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
				End Select
			Case "SGD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
					Case Else
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
				End Select
			Case "DKK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00|2000.00|5000.00|10000.00"
					Case Else
						MFCurrencyStakeList = "0.00|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00|2000.00|5000.00|10000.00"
				End Select
			Case "HKD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00|2000.00|5000.00|10000.00"
					Case Else
						MFCurrencyStakeList = "0.00|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00|2000.00|5000.00|10000.00"
				End Select
			Case "EUR"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
					Case Else
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
				End Select
			Case "USD"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
					Case Else
						MFCurrencyStakeList = "0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00"
				End Select
			Case "SEK"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00|2000.00|5000.00|10000.00"
					Case Else
						MFCurrencyStakeList = "0.00|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00|2000.00|5000.00|10000.00"
				End Select
			Case "JPY"
				Select Case gameCode
					Case "QBTHL"
						MFCurrencyStakeList = "0.00|100|200|500|1000|2000|5000|10000|20000|50000|100000|200000|500000|1000000"
					Case Else
						MFCurrencyStakeList = "0.00|100|200|500|1000|2000|5000|10000|20000|50000|100000|200000|500000|1000000"
				End Select
		End Select
End Function
%>

<%
Function MFCurrencyChipList(cur, gameCode)
    'Returns chip value list for supported currency, code cur.
    'e.g. GBP - 0.00|0.25|0.50|1.00|2.00|3.00|4.00|5.00|6.00|8.00|10.00|15.00|25.00|50.00|100.00|150.00|250.00|500.00|1000.00
    '			JPY - 0.00|100|200|500|1000|2000|5000|10000|20000|50000|100000|200000|500000|1000000
    'returns -1 for unrecognised/unsupported currency code
    
    MFCurrencyChipList = ""
    Select Case cur
			Case "GBP"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
					Case Else
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
				End Select
			Case "AUD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
					Case Else
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
				End Select
			Case "CAD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
					Case Else
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
				End Select
			Case "CHF"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
					Case Else
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
				End Select
			Case "SGD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
					Case Else
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
				End Select
			Case "DKK"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "1|5|10|25|100|500|1000"
					Case Else
						MFCurrencyChipList = "1|5|10|25|100|500|1000"
				End Select
			Case "HKD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "1|5|10|25|100|500|1000"
					Case Else
						MFCurrencyChipList = "1|5|10|25|100|500|1000"
				End Select
			Case "EUR"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
					Case Else
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
				End Select
			Case "USD"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
					Case Else
						MFCurrencyChipList = "0.10|0.25|1|5|10|25|100"
				End Select
			Case "SEK"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "1|5|10|25|100|500|1000"
					Case Else
						MFCurrencyChipList = "1|5|10|25|100|500|1000"
				End Select
			Case "JPY"
				Select Case gameCode
					Case "QBHEX"
						MFCurrencyChipList = "10|25|100|500|1000|2500|10000"
					Case Else
						MFCurrencyChipList = "10|25|100|500|1000|2500|10000"
				End Select
		End Select
End Function
%>