<%
' *--------------------------------------------------------------------------------
' Number functions for all new Arcade games
' *--------------------------------------------------------------------------------
' * Author: Daniel triggs (AshGaming)
' * 7 Oct 08 - (DTriggs). Small change to the doubleCeil, doubleRound and doubleFloor functions to round the value after doing i_num * (10^i_places) as this line was introducing some problems with floating number problems in some rare instances (eg 0.14 to 2dp was rounded up to 0.15)
'****************************************************

'Rounds the value up to the specified number of decimal places
'This works for negative numbers as well, -1.234 rounded to 0 decimal places returns -1
'Note: only the first 10 decimal places are used (rounded after that point) and due to limitations with doubles only 15 significant figures are used (so 1,000,000 and over can not be rounded to 9 decimal places) 
Function doubleCeil(ByVal iInput_number, ByVal iNumPlaces)
	On Error Resume Next
	dim i_num,i_places
	i_num = CDbl(iInput_number)
	i_places = CInt(iNumPlaces)
	if Err.number <> 0 OR i_places<0 OR i_places>9  then ' return Null for the response and raise a custom error in asp
		Err.Raise vbObjectError +100, "doubleCeil","Number to be ceiled must be valid number and number of decimal places must be a valid integer (0-9)"& Err.description
		doubleCeil = Null
		Exit Function
	End if
	i_num = Round(i_num * (10^i_places),10) ' multiply by 10 to the power of number of decimal places. rounded to 10dp to remove the floating number problem this line creates
	i_num = Int(i_num) + Sgn(i_num - Int(i_num)) ' then call int() on the function - if the number is different to the original then add 1
	doubleCeil = Round(i_num/(10^i_places),i_places) ' divide the number again to get back to correct number of decimals.
End Function




'Rounds the value down to the specified number of decimal places
'This works for negative numbers as well, -1.234 rounded to 0 decimal places returns -2
'Note: only the first 10 decimal places are used (rounded after that point) and due to limitations with doubles only 15 significant figures are used (so 1,000,000 and over can not be rounded to 9 decimal places) 
Function doubleFloor(ByVal iInput_number, ByVal iNumPlaces)
	On Error Resume Next
	dim i_num,i_places
	i_num = CDbl(iInput_number)
	i_places = CInt(iNumPlaces)
	if Err.number <> 0 OR i_places<0 OR i_places>9 then 'if error or number of places not an integer between 0 and 9 then raise error and return Null as the response
		Err.clear
		Err.Raise vbObjectError +100, "doubleFloor","Number to be floored must be valid number and number of decimal places must be a valid integer (0-9)"& Err.description
		doubleFloor = Null
		Exit Function
	End if
	i_num = Round((10^i_places) *i_num,10) 'rounded to 10dp to remove the floating number problem this line creates
	i_num =Int(i_num)/10^i_places 
	doubleFloor =Round(i_num,i_places) ' round done at end to check no floating number problems after division above (small chance these could still occur if several decimal places were used though).
End function 

'Rounds the value up or down to the specified number of decimal places.
'This works for negative numbers as well, -1.234 rounded to 0 decimal places returns -2
'Note: the built in asp round function uses "Bankers rounding" so .5 is sometimes rounded up and other times rounded down (rounded to nearest even number). But this doubleRound function always rounds 0.5 up if positive, and down if negative numbers (so always rounds away from 0)
'only the first 10 decimal places are used (rounded after that point) and due to limitations with doubles only 15 significant figures are used (so 1,000,000 and over can not be rounded to 9 decimal places) 
Function doubleRound(ByVal iInput_number, ByVal iNumPlaces)
	On Error Resume Next
	dim i_num,i_places
	i_num = CDbl(iInput_number)
	i_places = CInt(iNumPlaces)
	if Err.number <> 0 OR i_places<0 OR i_places>9 then 'if error or number of places not an integer between 0 and 9 then raise error and return Null as the response
		Err.Raise vbObjectError +100, "doubleRound","Number to be rounded must be valid number and number of decimal places must be a valid integer (0-9)"& Err.description
		doubleRound = Null
		Exit Function
	End if
	i_num = Round(i_num * (10^i_places),10) ' multiply by 10 to the power of number of decimal places. 'rounded to 10dp to remove the floating number problem this line creates
	i_num = Fix(i_num + 0.5 * Sgn(i_num)) / (10^i_places)
	doubleRound =Round(i_num,i_places) ' round done at end to check no floating number problems after division above (small chance these could still occur if several decimal places were used though).
End function 

' Checks that the value given is a valid double number (and doesn't contain any exponential notation) and has no more than the specified number of decimal places
'Note: accepts positive and negative numbers (as sometimes negative are needed - and negative can be used in stringToDouble as well)
Function isValidDouble(ByVal sInput_number, iNumPlaces)
	dim i_character_pos,s_Char,s_string,i_chars_in_num,b_result,i_decimalPos,i_places
	s_string = CStr(sInput_number)
	b_result = True ' set to true to start with, if any of the characters are non numeric though then it sets it to false
	if NOT(isValidWholeNumber(iNumPlaces)) then
		b_result = False 
		' raise a custom error - as the problem is with the code that called this function as not called correctly
		Err.Raise vbObjectError +100, "isValidDouble","Number of decimal places for double was not set to valid whole number"
	Else 
		i_places = CInt(iNumPlaces)
		if NOT(IsNumeric(s_string)) then 'if its not numeric then return false
			b_result = False
		else 
			'now check that the value given does not contain any exponential notation (only contains digits or a decimal point)
			i_chars_in_num = len(s_string)
			i_decimalPos = -1
			for i_character_pos = 1 to i_chars_in_num ' loop through all the characters in the bet val one at time and check they are numbers or a decimal point
				s_Char = mid(s_string,i_character_pos,1)
				if s_Char="." then
					i_decimalPos = i_character_pos
				Else 
					if(instr("0123456789.-", s_Char) < 1) then 'if the current character does not contain any of these characters then it is invalid
						'Note:the game only uses decimal points whatever currency is used - so a comma would not be valid
						b_result = False
						exit for
					end if
				End if
			next
			'if there is a decimal point then check the number of characters after that are not greater than the number of places specified. 
			'Also check that the decimal point is not at the start of the value (so .5 is not allowed but 0.5 is)
			if (i_decimalPos=1 OR (i_decimalPos>1 AND (i_chars_in_num-i_decimalPos)>i_places)) then 
				b_result = false
			End if 
		end if
	End if
	isValidDouble=b_result
End Function

'Converts a string to a double, also checks the double is valid for the given number of decimal places
Function stringToDouble(ByVal sInput_number, iNumPlaces, bAllowNegative) ' returns null if error occurs
'Note: added iNumPlaces to this (as that is needed for the call to isValidDouble)
	if NOT(isValidDouble(sInput_number,iNumPlaces)) then
		stringToDouble = Null
	Else 
		Dim i_num
		i_num = CDbl(sInput_number)
		if (i_num<0 AND bAllowNegative=false) then
			stringToDouble = Null
		Else
			stringToDouble = i_num
		End if
	End if
End Function

Function isValidInteger(ByVal sInput_number)
' Checks that the value given is a valid integer number (not a decimal and doesn't contain any exponential notation)
	On Error Resume Next
	dim i_character_pos,s_Char,s_string,i_chars_in_num,b_result 
	s_string = CStr(sInput_number)
	b_result = True ' set to true to start with, if any of the characters are non numeric though then it sets it to false
	if NOT(IsNumeric(s_string)) then 'if its not numeric then return false
		b_result = False
	else 
		'now check that the value given does not contain any exponential notation (only contains digits or a decimal point)
		i_chars_in_num = len(s_string)
		for i_character_pos = 1 to i_chars_in_num ' loop through all the characters in the bet val one at time and check they are numbers or a decimal point
			s_Char = mid(s_string,i_character_pos,1)
			if(instr("0123456789-", s_Char) < 1) then 'if the character does not contain any of these characters then it is invalid
				'Note:the game only uses decimal points whatever currency is used - so a comma would not be valid
				b_result = False
				exit for
			end if
		next
	end if
	If Err.Number<>0 then ' casting to a string could raise an error, catch it (and any other errors) here and return false
		b_result = False
	End if
	isValidInteger=b_result
End Function

Function stringToInteger(ByVal sInput_number)
'Converts string to integer (if the value is not in the range for ints (-32768 to 32767) then it converts to a long instead (as CInt throws an error for values outside this range)
	if NOT(isValidInteger(sInput_number)) then
		stringToInteger = Null
	Else 
		dim i_num
		i_num = CLng(sInput_number)
		if (i_num< -32768 OR i_num>32767) then
			stringToInteger = i_num ' leave as long value
		Else
			stringToInteger = CInt(sInput_number) ' convert to int
		End if
	End if
End Function

Function isValidWholeNumber(ByVal sInput_number)
' Checks that the value given is a valid integer number (not a decimal and doesn't contain any exponential notation) and that it is a positive number (or 0)
	if isValidInteger(sInput_number) then 'first test it is a valid integer
		if CLng(sInput_number)>=0 then ' if it is an integer then can test if it is positive
			isValidWholeNumber = true
		End if
	Else 	
		isValidWholeNumber= false
	End if
End Function

Function stringToWholeNumber(ByVal sInput_number) ' same as stringToInteger but this doesn't accept negative values
'Converts string to positive integer (if the value is not in the range for ints (0 to 32767) then it converts to a long instead (as CInt throws an error for values outside this range)
	if NOT(isValidWholeNumber(sInput_number)) then
		stringToWholeNumber = Null
	Else 
		dim i_num
		i_num = CLng(sInput_number)
		if (i_num>32767) then
			stringToWholeNumber = i_num ' leave as long value
		Else
			stringToWholeNumber = CInt(sInput_number) ' convert to int
		End if
	End if
End Function
%>
