<%
' *--------------------------------------------------------------------------------
' General Utilities file for all new Arcade games
' *--------------------------------------------------------------------------------
' * Author: Daniel triggs (AshGaming)
' * Initial version - 26th June 2007
' * 2/July/07 (DTriggs) - added f_checkRNGNums function and HTMLEncode in the f_return_error function
'----------------------------------------------------------------------------------
'This file is used to provide functions that are used by several of the games 
' so the code can just be checked once and used by all the games - rather than repeated
' in the placebet file of each game
'---------------------------------------------------------------------------------

'This subroutine sends an error to the game and writes the given error message to the log, it also ends the response so code after this call is not run
'NOTE: this function requires the following variables to be set before calling it:
'objGameSvr,objGameCon,ID, strGameCode
'apart from strGameCode (which is set in gamecommon.inc) these variables all get set by just including the set_server.inc file
Sub f_return_error(client_message, log_message,b_suspicious) ' b_suspicious must be included - should be either True (for suspicious errors) or False - for other errors
	Response.Write("&errorStr="&client_message)
	If b_suspicious=true then
		objGameSvr.LogSuspicious ID,strGameCode & " " & Server.HTMLEncode(log_message) 'make sure any data written to logs is encoded
	Else 
		objGameSvr.LogError ID, strGameCode & " " & Server.HTMLEncode(log_message)
	End if
	objGameSvr.DelClient ID 'delete the client from session (cash out)
	Session.Abandon 'quit the users session
	set objGameSvr = nothing 'clear the server variables
	set objGameCon = nothing
	Response.End 'quit running the rest of the code
End Sub

'This function checks the RNG string contains the correct amount of numbers and they are all in the correct range
'It also converts the string into an array of integrer numbers and then returns it so the numbers are ready to use
Function f_checkRNGNums(s_numString,i_max,i_min,i_count) ' s_numString is the string from RNG, i_count is expected number of values, i_min and i_max are the min and max values expected in range
	on error resume next
	i_max = CInt(i_max)
	i_min = CInt(i_min)
	i_count = CInt(i_count)
	if len(s_numString)<1 then
		f_return_error "RNG Error", "Error with numbers back from RNG  - No numbers were returned",false
	End if
	dim a_Results,i
	a_Results =Split(s_numString, ",")
	if Err.number <> 0 then
		f_return_error "RNG Error", "Error with numbers back from RNG (" & s_numString & ") : " & Err.Description,false
	End if	

	if Ubound(a_Results)<>(i_count-1) then
		f_return_error "RNG Error", "Error with numbers back from RNG. Expected "& i_count & " numbers but there were " & (Ubound(a_Results)+1),false
	End if

	For i=0 To UBound(a_Results) 'loop through array and convert each value to an integer and then check it is in correct range
		a_Results(i) = CInt(a_Results(i))
		if a_Results(i) < i_min OR a_Results(i)> i_max OR Err.number <> 0 then ' too low, too high or error occurred when converting it to an integer
			f_return_error "RNG Error", "Error with numbers back from RNG. Value in position " & i & " was not a number in the expected range of " & i_min & " to " & i_max & ". Numbers:" & s_numString & " " & Err.Description,false
			Exit For
		End if
	Next
	f_checkRNGNums = a_Results ' return the array of numbers (note - always returned as an array even if only one value - so need to save the response in a variable and then use (0) to get the first (and only) value.
End Function

'Simple function written by dan to check if number is an integer (returns true if is, otherwise returns false)
Function isInteger(ByVal input_number)
	Dim b_result
	if(isValidNumber(input_number)) then
		if(InStr(CStr(input_number),".")=0) then
			isInteger=True
		else
			isInteger=False
		end if
	else
		isInteger=False
	end if
'	isInteger = b_result
End Function

Function isValidNumber(ByVal input_number)
' Checks that the value given is a valid float number (numeric and doesn't use any exponential notation
	dim i_character_pos,s_Char,i_chars_in_num,b_result 
	b_result = True ' set to true to start with, if any of the characters are non numeric though then it sets it to false
	if NOT(IsNumeric(input_number)) then 'if its not numeric then return false
		b_result = False
	else 
		'now check that the value given does not contain any exponential notation (only contains digits or a decimal point)
		i_chars_in_num = len(input_number)
		for i_character_pos = 1 to i_chars_in_num ' loop through all the characters in the bet val one at time and check they are numbers or a decimal point
			s_Char = mid(input_number,i_character_pos,1)
			if(instr("0123456789.", s_Char) < 1) then 'if the character does not contain any of these characters then it is invalid
				'Note:the game only uses decimal points whatever currency is used - so a comma would not be valid
				b_result = False
				exit for
			end if
		next
	end if
	isValidNumber=b_result
End Function

'Rounds the value up to nearest penny (2dp) - (ASP doesn't have a ceil function)
'ie if num = 1.561, returns 1.57,  if num = 1.56 returns 1.56
Function Ceil(ByVal input_number)
	dim i_num
	i_num = CDbl(input_number) * 100 'get to pence
	if isInteger(i_num) then ' if its an integer already then can just return it (rounds to 2dp again just incase the /100 causes number problem)
		Ceil = Round(i_num/100,2)
	else
		Ceil = Int(Fix(i_num)+1)/100
	end if 
End Function

'Floors a number to 2 decimal places (asp doesn't have a floor function)
Function Floor(ByVal input_number) ' number to input should contain 2 digits after decimal place
	Dim i_num
	'note: the number should be checked before hand to make sure it is numeric
	i_num = Round(10000 *CDbl(input_number))/100 'x winnings by 10000 then round them to get rid of floating point number problems. 
	'Then divide by 100 to get to pennies.
	i_num =Round(Fix(i_num)/100,2)	' Then (fix) that number which removes everything after the "." 
	'then divide by 100 to get back to pounds. Finally round to 2 dp (just incase funny number problem doing/100)
	Floor = i_num
End function 

'Used to sort an array of numbers (ASP doesn't have a sort function)
Function f_sort(ByVal a_array,b_descending)
'note - this will only sort integers (due to the Cint part)
'The second variable should be a boolean - if it is true then the values are sorted in decending order, otherwise they are sorted in ascending order
	dim i,j,temp
	for i = UBound(a_array) - 1 To 0 Step -1 ' go from second to last pos backwards
		for j= 0 to i
			if b_descending then
				if CInt(a_array(j))<CInt(a_array(j+1)) then
					temp=a_array(j+1)
					a_array(j+1)=a_array(j)
					a_array(j)=temp
				end if
			else 
				if CInt(a_array(j))>CInt(a_array(j+1)) then
					temp=a_array(j+1)
					a_array(j+1)=a_array(j)
					a_array(j)=temp
				end if
			End if
		next
	next
	f_sort = a_array ' return the sorted array
End Function

%>