<%
' *--------------------------------------------------------------------------------
' Validation Functions for all new Arcade games
' *--------------------------------------------------------------------------------
' * Author: Daniel triggs (AshGaming)
' * 24 Oct 2007 - (DTriggs) - changed validateRNGString so it uses Int rather than CInt - this is so that large values (like those needed for jackpots) can still be verified as the range for CInt is only -32,768 to 32,767

'NOTE: errorHandlingFunctions.as MUST also be included for some of these functions to work (namely the validateRNGString function!!

'This function checks the RNG string contains the correct amount of numbers and they are all in the correct range
'It also converts the string into an array of integrer numbers and then returns it so the numbers are ready to use
Function validateRNGString(s_numString,byVal i_max, byVal i_min,byVal i_count) ' s_numString is the string from RNG, i_count is expected number of values, i_min and i_max are the min and max values expected in range
	on error resume next
	i_max = Int(i_max) ' note - as use Int rather than Cint now can go up to 2,147,483,647
	i_min = Int(i_min)
	i_count = CInt(i_count)
	if len(s_numString)<1 then
		returnError "RNG Error", "Error with numbers back from RNG  - No numbers were returned",false
	End if
	dim a_Results,i
	a_Results =Split(s_numString, ",")
	if Err.number <> 0 then
		returnError "RNG Error", "Error with numbers back from RNG (" & s_numString & ") : " & Err.Description,false
	End if

	if Ubound(a_Results)<>(i_count-1) then
		returnError "RNG Error", "Error with numbers back from RNG. Expected "& i_count & " numbers but there were " & (Ubound(a_Results)+1),false
	End if

	For i=0 To UBound(a_Results) 'loop through array and convert each value to an integer (or a Long if too big) and then check it is in correct range
		a_Results(i) = Int(a_Results(i))
		if a_Results(i) < i_min OR a_Results(i)> i_max OR Err.number <> 0 then ' too low, too high or error occurred when converting it to an integer
			returnError "RNG Error", "Error with numbers back from RNG. Value in position " & i & " was not a number in the expected range of " & i_min & " to " & i_max & ". Numbers:" & s_numString & " " & Err.Description,false
			Exit For
		End if
	Next
	validateRNGString = a_Results ' return the array of numbers (note - always returned as an array even if only one value - so need to save the response in a variable and then use (0) to get the first (and only) value.
End Function

Function matchesWhiteList(s_string,s_whitelist)
	dim i_chars_in_num,i_character_pos,s_char,b_matches
	i_chars_in_num = len(s_string)
	if (i_chars_in_num>0 AND len(s_whitelist)>0) then
		b_matches = true
		for i_character_pos = 1 to i_chars_in_num ' loop through all the characters in the bet val one at time and check they only contain the values in the whitelist
			s_char = mid(s_string,i_character_pos,1)
			if(instr(s_whitelist, s_char) < 1) then 'if the character does not contain any of these characters then it is invalid
				b_matches = False
				exit for
			end if
		next
	Else 'error with input
		Err.Raise vbObjectError +100, "matchesWhiteList","matchesWhiteList Invalid input: Both the string inputs need to contain 1 or more characters"& Err.description
		b_matches = False
	End if
	matchesWhiteList = b_matches
End Function

Function validateRange(i_number, i_min, i_max)
	if i_number>=i_min AND i_number<=i_max then
		validateRange = true
	Else
		validateRange = false
	End if
End Function

Function isValidFormatAndLength(s_string, s_whitelist, byVal i_maxLength)
	on error resume next
	i_maxLength = CInt(i_maxLength)
	If Err.number <> 0 OR i_maxLength<1 then
		Dim s_description
		s_description = Err.description
		Err.clear
		Err.Raise vbObjectError +100, "isValidFormatAndLength","isValidFormatAndLength Invalid input: max length must be a valid positive integer: "& s_description
		isValidFormatAndLength = false
	Else 'ok so now check string length
		if len(s_string)>i_maxLength then 'string too long so no point checking white list
			isValidFormatAndLength = false
		Else
			isValidFormatAndLength = matchesWhiteList(s_string,s_whitelist) ' if whitelist is ok then format and length are valid
		End if
	End if
End Function

Function URLDecode(sConvert)
    Dim aSplit
    Dim sOutput
    Dim I
    If IsNull(sConvert) Then
       URLDecode = ""
       Exit Function
    End If

    ' convert all spaces to pluses
    sOutput = REPLACE(sConvert, " ", "+")

    ' next convert %hexdigits to the character
    aSplit = Split(sOutput, "%")

    If IsArray(aSplit) Then
      sOutput = aSplit(0)
      For I = 0 to UBound(aSplit) - 1
        sOutput = sOutput & _
          Chr("&H" & Left(aSplit(i + 1), 2)) &_
          Right(aSplit(i + 1), Len(aSplit(i + 1)) - 2)
      Next
    End If

    URLDecode = sOutput
End Function

%>