/*jslint node: true */

"use strict";

var sync = require( 'synchronize' );
var http = require( 'http' );
var url = require( 'url' );
var UUID = require( 'node-uuid' );


var VirgoPlatformInterface = function( requestParams, currencyCode ) {

	this._requestParams = requestParams;
	this._onExternalHttpRequest = null;
	this._onExternalHttpResponse = null;
	this._jackpotValues = [];
	this._jackpotClaimTokens = {};
	this._previousRngString = '';
	this._currencyCode = currencyCode;// || 'GBP';
};


VirgoPlatformInterface.prototype.onExternalHttpRequest = function( cb ) {
	this._onExternalHttpRequest = cb;
};


VirgoPlatformInterface.prototype.onExternalHttpResponse = function( cb ) {
	this._onExternalHttpResponse = cb;
};


///////////////////////// Jackpots


VirgoPlatformInterface.prototype.claimJackpot = function( jackpotId, totalStake ) {

	if ( !( typeof totalStake === 'object' && totalStake instanceof BigNumber ) ) {
		throw new Error( "VirgoPlatformInterface.prototype.claimJackpot: totalStake must be a BigNumber instance" );
	}
	var amountActuallyWon = 0;
	var progressiveToken = this._requestParams['DataRetrievalContext']['TokenProgressive'];
	if ( this._jackpotValues && this._jackpotValues.length > jackpotId ) {
		if ( progressiveToken['AccessLevel'] === "LOCAL_DETERMINATION" && progressiveToken['Token'] && progressiveToken['Token']['Token'] && progressiveToken['Token']['Token'] !== "INVALID" ) {

			var serviceURI = progressiveToken['ServiceUri'];
			var token = progressiveToken['Token']['Token'];
			var jackpotData = this._jackpotValues[jackpotId];
			// Progressive claim
			var requestObj = {
				'ClaimedJackpot': jackpotData,
				'Stake': { 'Amount': totalStake.toN(), 'CurrencyCode': this._currencyCode },
				'RequestToken': { 'Token': token }
			};
			var data = this._sendHttpRequestSync( serviceURI, requestObj, 'claimJackpot' );
			// Record win: {"jsonrpc":"2.0","id":"c6531710-f24b-11e3-90e1-333f466f68d6","result
			// ":{"Win":{"Jackpot":{"Id":3,"Value":{"Amount":1100,"CurrencyCode":"GBP"},"LevelN
			// ame":"Silver","LevelIndex":2,"Type":"GAME"},"ClaimToken":{"Token":"e09641b9-5028
			// -49d3-bd8e-7a44af2e99e2"}}}}
			//console.log( JSON.stringify( data ) );
			// TODO: Double check returned ID matches the requested jackpot ID
			if ( data['result'] && data['result']['Win'] && data['result']['Win']['Jackpot'] && data['result']['Win']['Jackpot']['Value'] ) {
				amountActuallyWon = data['result']['Win']['Jackpot']['Value']['Amount'];

				// store claim token for later
				this._jackpotClaimTokens[jackpotId] = data['result']['Win']['ClaimToken'];

				console.log( "Jackpot claimed: " + jackpotId + " : " + JSON.stringify( this._jackpotClaimTokens[jackpotId] ) );
			}
		} else {
			throw new Error( "Attempted to claim an invalid Jackpot!" );
		}
	} else {
		throw new Error( "Attempted to claim an invalid Jackpot!" );
	}
	return new BigNumber( amountActuallyWon );
};


VirgoPlatformInterface.prototype.getJackpotClaimToken = function( jackpotId ) {

	return this._jackpotClaimTokens[jackpotId];
};


// TODO: Do we need this? Looks like it informs the server we haven't won a jackpot.
// VirgoPlatformInterface.prototype.jackpotRecordLose = function( jackpotId ) {

// var progressiveToken = this._requestParams[ 'DataRetrievalContext' ][ 'TokenProgressive' ];
// if( this._jackpotValues && this._jackpotValues.length > jackpotId ) {
// if( this._requestParams[ 'DataRetrievalContext' ][ 'TokenProgressive' ][ 'AccessLevel' ] === "LOCAL_DETERMINATION" ) {
// var serviceURI = progressiveToken[ 'ServiceUri' ];
// var token = progressiveToken[ 'Token' ][ 'Token' ];
// var jackpotData = this._jackpotValues[ jackpotId ];
// // Record failed
// var data = this._sendHttpRequestSync( serviceURI, { 'InvolvedJackpots': jackpotData, 'Stake': this._stake, 'RequestToken': { 'Token': token } }, 'recordFailedJackpot' );
// }
// }
// return 0;
// };


// VirgoPlatformInterface.prototype.jackpotMarkEligible = function( jackpotId, stake ) {

// // enter draw
// var progressiveToken = this._requestParams[ 'DataRetrievalContext' ][ 'TokenProgressive' ];
// if( this._jackpotValues && this._jackpotValues.length > jackpotId ) {
// if( this._requestParams[ 'DataRetrievalContext' ][ 'TokenProgressive' ][ 'AccessLevel' ] === "CENTRAL_DETERMINATION" ) {
// var serviceURI = progressiveToken[ 'ServiceUri' ];
// var token = progressiveToken[ 'Token' ][ 'Token' ];
// var jackpotData = this._jackpotValues[ jackpotId ];
// var data = this._sendHttpRequestSync( serviceURI, { 'InvolvedJackpots': jackpotData, 'Stake': stake.toN(), 'RequestToken': { 'Token': token } }, 'enterProgressiveDraw' );
// }
// }
// };


VirgoPlatformInterface.prototype.updateJackpotValues = function() {

	// Progressive status
	if ( this._requestParams['DataRetrievalContext'] && this._requestParams['DataRetrievalContext']['TokenProgressive'] ) {
		var progressiveToken = this._requestParams['DataRetrievalContext']['TokenProgressive'];
		var accessLevel = progressiveToken['AccessLevel'];

		if ( accessLevel !== 'NONE' ) {
			var serviceURI = progressiveToken['ServiceUri'];
			var token = progressiveToken['Token']['Token'];
			var data = this._sendHttpRequestSync( serviceURI, {
				/*'Currency': this.getCurrencyCode(),*/
				'RequestToken': { 'Token': token }
			}, 'getJackpotStatus' );

			if ( !data.error && data.result.JackpotList ) {

				this._jackpotValues = data.result.JackpotList.slice( 0 );

				// "JackpotList": [
				// {
				// "Id": 1,
				// "Value": {
				// "Amount": 124,
				// "CurrencyCode": "GBP"
				// },
				// "LevelName": "Gold",
				// "LevelIndex": 1,
				// "Type": "GAME"
				// },
				// {
				// "Id": 2,
				// "Value": {
				// "Amount": 22,
				// "CurrencyCode": "GBP"
				// },
				// "LevelName": "Silver",
				// "LevelIndex": 2,
				// "Type": "GAME"
				// }
				// ]
			}
		}
	}
};


VirgoPlatformInterface.prototype.getBalance = function() {
	return 0; // Virgo does not expose the balance to the logic server
};


VirgoPlatformInterface.prototype.getJackpotValue = function( jackpotId ) {

	if ( this._jackpotValues.length > jackpotId && jackpotId >= 0 ) {
		return this._jackpotValues[jackpotId]['Value']['Amount'];
	} else {
		return 0;
	}
};


VirgoPlatformInterface.prototype.getJackpotCount = function() {

	return this._jackpotValues.length;
};


///////////////////////// Logging


VirgoPlatformInterface.prototype.logMessage = function( msg ) {

};


VirgoPlatformInterface.prototype.logSuspicious = function( msg ) {

};


VirgoPlatformInterface.prototype.logError = function( msg ) {

};


///////////////////////// Cashing out


VirgoPlatformInterface.prototype.getCurrencyCode = function() {

	return this._currencyCode;
};


VirgoPlatformInterface.prototype._sendHttpRequestSync = function( urlString, gameLogicRequestJSON, interfaceMethod ) {

	// Use synchronize library to convert asynchronous code to synchronous.
	var urlObj = url.parse( urlString );
	var requestObj = {
		'params': gameLogicRequestJSON,
		'jsonrpc': '2.0',
		'id': UUID.v1(),
		'method': interfaceMethod
	};
	var requestDataJSON = JSON.stringify( requestObj );

	var options = {
		host: urlObj.hostName,
		port: urlObj.port,
		path: urlObj.path,
		method: 'POST',
		headers: {
			'Content-Type': 'application/json',
			'Content-Length': Buffer.byteLength( requestDataJSON ),
			'Accept-Encoding': 'gzip, deflate'
		}
	};

	var responseBody = '';

	var sendHttpRequest = function( options, callback ) {
		return http.request( options, function( resp ) {
			callback( null, resp );
		} );
	};

	if ( this._onExternalHttpRequest ) {
		// used for logging
		this._onExternalHttpRequest( urlString, requestObj );
	}
	var req = sendHttpRequest( options, sync.defer() );
	req.write( requestDataJSON );
	req.end();

	// wait for response, read it in
	var response = sync.await();
	response.setEncoding( 'utf8' );
	response.on( 'data', function( chunk ) {
		responseBody += chunk;
	} );
	response.on( 'end', sync.defer() );
	sync.await();

	var responseObject = JSON.parse( responseBody );

	if ( this._onExternalHttpResponse ) {
		this._onExternalHttpResponse( urlString, responseObject );
	}

	if ( responseObject['error'] ) {
		throw new Error( JSON.stringify( responseObject['error'], null, '\t' ) );
	}

	return responseObject;
};


///////////////////////// Random number retrieval


VirgoPlatformInterface.prototype.getRandomNumber = function( type, lowestInclusive, highestInclusive ) {

	return this.getRandomNumbers( type, lowestInclusive, highestInclusive, 1 )[0];
};


VirgoPlatformInterface.prototype.getRandomNumbers = function( type, lowestInclusive, highestInclusive, numberCount ) {

	var forcedNumbers = VirgoPlatformInterface.forcedRngString.split( "," ).map( function( num ) { return parseInt( num, 10 ); } );

	var range = highestInclusive - lowestInclusive;
	var numbers = [];
	var forcedCount = Math.min( forcedNumbers.length, numberCount );
	for ( var i = 0; i < forcedCount; ++i ) {
		var num = forcedNumbers.shift();
		if ( isFinite( num ) ) {
			while ( num > highestInclusive ) {
				num -= range;
			}
			numbers.push( num );
		}
	}

	VirgoPlatformInterface.forcedRngString = forcedNumbers.join( "," );

	var numbersLeft = numberCount - numbers.length;

//	console.log( JSON.stringify( this._requestParams, null, '\t' ) );

	if ( numbersLeft > 0 ) {
		var randomToken = this._requestParams['DataRetrievalContext']['TokenRandom']['Token']['Token'];
		var randomURI = this._requestParams['DataRetrievalContext']['TokenRandom']['ServiceUri'];
		var randomDataSource = this._requestParams['DataRetrievalContext']['TokenRandom']['DataSource'];

		if ( randomDataSource === 'NONE' ) {
			throw { message: 'Invalid random number source received in Virgo token' };
		}

		// Send HTTP request to virgo to get random numbers
		var data = this._sendHttpRequestSync( randomURI, {
			'Count': numbersLeft,
			'UpperBound': ( range + 1 ),
			'RequestToken': { 'Token': randomToken }
		}, 'getRandomNumber' );

		var receivedNumbers = data['result']['Data'];

		if ( !Array.isArray( receivedNumbers ) || receivedNumbers.length !== numbersLeft ) {
			throw { message: "Invalid random number response received" };
		}

		for ( var j = 0; j < numbersLeft; ++j ) {
			var newNumber = ( receivedNumbers[j] % ( range + 1 ) ) + lowestInclusive;
			numbers.push( newNumber );
		}
	}

	if ( this._previousRngString.length > 0 ) {
		this._previousRngString += ',';
	}
	this._previousRngString += numbers.join( ',' );

	return numbers;
};


VirgoPlatformInterface.prototype.closeBet = function() {

};


VirgoPlatformInterface.prototype.startFirstBetOperation = function() {

	this._previousRngString = '';
};


VirgoPlatformInterface.prototype.getPreviousRngString = function() {

	return this._previousRngString;
};


VirgoPlatformInterface.forcedRngString = '';

VirgoPlatformInterface.forceRng = function( rngString, accountId, gameCode ) {

	VirgoPlatformInterface.forcedRngString = rngString;
};


exports.VirgoPlatformInterface = VirgoPlatformInterface;



