goog.provide( 'game.FreespinsCoins' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.rendering.LegacyAnimatedSprite' );
goog.require( 'libs.client.rendering.SpriteGroup' );
goog.require( 'libs.client.utils.DOM' );
goog.require( 'libs.client.utils.misc' );


( function() {
	"use strict";

	var defaultSettings = {
		coinflipAnim: {
			autoplay: false,
			timesToLoop: 0,
			fps: 30,
			normalOffsetX: 0,
			extraSpinsOffsetX: 0
		},
		retriggerAnim: {
			autoplay: false,
			timesToLoop: 0,
			fps: 15,
			resetToFrameZeroOnEnd: false
		},
		postRevealDelay: 100
	};

	var coinPosYMobile = 472;
	var defaultSettingsMobile = {
		coinPositions: [{
			x: 82,
			y: coinPosYMobile
		}, {
			x: 231,
			y: coinPosYMobile
		}, {
			x: 377,
			y: coinPosYMobile
		}, {
			x: 526,
			y: coinPosYMobile
		}, {
			x: 676,
			y: coinPosYMobile
		}],
		text: {
			fontSize:20, 
			maxWidth:380
		}
	};

	var coinPosYDesktop = 484;
	var defaultSettingsDesktop = {
		coinPositions: [{
			x: 76,
			y: coinPosYDesktop
		}, {
			x: 225,
			y: coinPosYDesktop
		}, {
			x: 371,
			y: coinPosYDesktop
		}, {
			x: 520,
			y: coinPosYDesktop
		}, {
			x: 670,
			y: coinPosYDesktop
		}],
		text: {
			fontSize:20, 
			maxWidth:400
		}
	};

	var selectors = {
		coinPanel: $( '#freespins-coin-panel' ),
		infoPanel: $( '#freespins-info-panel' ),
		infoText: $( '#freespins-info-text' ),
		infoTextExtra_spins_one: $( '#freespins-info-text-extra-spins-red' ),
		infoTextIn_a_row_awards: $( '#freespins-info-text-in-a-row' ),
		infoTextEight: $( '#freespins-info-text-eight' ),
		infoTextExtra_spins_two: $( '#freespins-info-text-extra-spins-yellow' )
	};

	selectors.coinPanel.hide();
	selectors.infoPanel.hide();

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';
	defaultSettings = libs.client.utils.misc.mergeSettings( defaultSettings, isDesktopDeviceType ? defaultSettingsDesktop : defaultSettingsMobile, true );

	var FreespinsCoins = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._isFreespinsUIActive = false;
		this._bonusInfo = null;

		this._numberOfCoins = 5;

		this._numberOfExtraSpinCoins = 0;
		this._numberOfCoinsStartedFlipping = 0;
		this._numberOfCoinsFinishedFlipping = 0;
		this._flippingCoinsForward = true;

		this._coinRedrawArea = {
			x: this._settings.coinPositions[0].x - 25,
			y: this._settings.coinPositions[0].y - 20,
			width: this._settings.coinPositions[4].x - this._settings.coinPositions[0].x + 25,
			height: 90

		};

		this._coinAnimsNormal = new libs.client.rendering.SpriteGroup( [] );
		this._coinAnimsExtraSpins = new libs.client.rendering.SpriteGroup( [] );
		this._coinRetriggerAnims = null;

		var coinAnimLoopedFunc = function() {
			that._coinAnimFinished();
		};
		var coinFrames = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, 'RS_CoinSpin' );
		var coinFramesExtraSpins = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, 'RS_CoinSpin_ExtraSpins' );
		var coinRetriggerFrames = libs.client.utils.misc.loadAnimationFrames( libs.loadedImages, 'RS_CoinRetrigger' );

		if ( coinRetriggerFrames.length > 0 ) {
			this._coinRetriggerAnims = new libs.client.rendering.SpriteGroup( [] );
		}

		var coinAnimSettings = this._settings.coinflipAnim;
		var coinPosSettings = this._settings.coinPositions;
		for ( var i = 0; i < this._numberOfCoins; i++ ) {
			var coinAnim = new libs.client.rendering.LegacyAnimatedSprite( coinFrames, coinAnimSettings, {
				x: coinPosSettings[i].x + coinAnimSettings.normalOffsetX,
				y: coinPosSettings[i].y
			} );
			coinAnim.onLooped( coinAnimLoopedFunc );
			this._coinAnimsNormal.sprites.push( coinAnim );

			var coinAnim2 = new libs.client.rendering.LegacyAnimatedSprite( coinFramesExtraSpins, coinAnimSettings, {
				x: coinPosSettings[i].x + coinAnimSettings.extraSpinsOffsetX,
				y: coinPosSettings[i].y
			} );
			coinAnim2.onLooped( coinAnimLoopedFunc );
			this._coinAnimsExtraSpins.sprites.push( coinAnim2 );

			if ( this._coinRetriggerAnims ) {
				var retriggerAnims = new libs.client.rendering.LegacyAnimatedSprite( coinRetriggerFrames, this._settings.retriggerAnim, {
					x: coinPosSettings[i].x + 15,
					y: coinPosSettings[i].y - 13
				} );
				this._coinRetriggerAnims.sprites.push( retriggerAnims );
			}
		}

		libs.globalEventBus.connect( 'libs.event.notify.tick', function( timeDelta ) {
			that._onNotifyTick( timeDelta );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.betBonusInfo', function( bonusInfo ) {
			that._onNotifyBetBonusInfo( bonusInfo );
		} );
		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) {
			that._onNotifyLanguageChanged( langCode );
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.freespinsUI', function( activity, inBetRecovery ) {
			that._onFreespinsUIStart( activity, inBetRecovery );
		} );
		libs.globalEventBus.connectActivityEnd( 'libs.activity.display.freespinsUI', function() {
			that._onFreespinsUIEnd();
		} );
		libs.globalEventBus.connectActivityStart( 'libs.activity.reelsState.reelsBonusFeature', function( activity ) {
			that._onReelsBonusFeatureStart( activity );
		} );
	};

	FreespinsCoins.prototype._onNotifyTick = function( timeDelta ) {
		this._coinAnimsNormal.update( timeDelta );
		this._coinAnimsExtraSpins.update( timeDelta );
		if ( this._coinRetriggerAnims ) {
			this._coinRetriggerAnims.update( timeDelta );
		}
	};

	FreespinsCoins.prototype._onNotifyBetBonusInfo = function( bonusInfo ) {
		if ( bonusInfo['name'] === "freeSpinWin" || bonusInfo['name'] === "freeSpinCollect" ) {
			this._bonusInfo = bonusInfo;
		}
	};

	FreespinsCoins.prototype._onNotifyLanguageChanged = function( langCode ) {
		var translation = libs.getTranslation( 'X_EXTRA_SPINS_IN A_ROW_AWARDS_X_EXTRA_SPINS' , [5, 8] );
		var newString = "<span class=\"red\">'EXTRA SPINS'</span>"
		var newTranslation = translation.replace("'EXTRA SPINS'", newString);
		selectors.infoText.html(newTranslation);
		var fontSize = libs.client.utils.DOM.calculateFontSize(translation, this._settings.text)
		selectors.infoText.css('fontSize',fontSize);
	};

	FreespinsCoins.prototype._onFreespinsUIStart = function( activity, inBetRecovery ) {
		this._isFreespinsUIActive = true;
		libs.client.utils.DOM.fadeIn( selectors.coinPanel, 500 );
		libs.client.utils.DOM.fadeIn( selectors.infoPanel, 500 );

	};

	FreespinsCoins.prototype._onFreespinsUIEnd = function() {
		this._isFreespinsUIActive = false;
		libs.client.utils.DOM.fadeOut( selectors.coinPanel, 500 );
		libs.client.utils.DOM.fadeOut( selectors.infoPanel, 500 );
	};

	FreespinsCoins.prototype._onReelsBonusFeatureStart = function( activity ) {
		if ( this._bonusInfo && this._bonusInfo['coinsState'] ) {
			this._handleReelsBonusFeature = activity.join();
			libs.globalEventBus.invoke( 'libs.event.request.enableZIndexHack' );
			this._startCoinFlip( this._bonusInfo['coinsState'] );
		}
	};

	FreespinsCoins.prototype._startCoinFlip = function( coinsState ) {
		var coinStateArray = coinsState;
		this._numberOfExtraSpinCoins = 0;
		this._setFlipCoinsForward( true );
		for ( var i = 0; i < this._numberOfCoins; ++i ) {
			if ( coinStateArray[i] === 1 ) {
				++this._numberOfExtraSpinCoins;
			}
		}
		this._flipNextCoinForward();
	};

	FreespinsCoins.prototype._flipNextCoinForward = function() {
		var that = this;
		if ( this._numberOfExtraSpinCoins > this._numberOfCoinsFinishedFlipping ) {
			this._coinAnimsExtraSpins.sprites[this._numberOfCoinsFinishedFlipping].play();
			if ( this._coinRetriggerAnims ) {
				this._coinRetriggerAnims.sprites[this._numberOfCoinsFinishedFlipping].play();
			}

			var coinSoundString = "ExtraSpins" + this._numberOfCoinsFinishedFlipping;
			libs.globalEventBus.invoke( 'libs.event.request.playSound', ['MQ', 'HQ'], coinSoundString );
		} else {
			this._coinAnimsNormal.sprites[this._numberOfCoinsFinishedFlipping].play();
		}

		this._numberOfCoinsStartedFlipping++;

		if ( this._numberOfCoinsFinishedFlipping === 4 ) {
			libs.setTimeoutTick( function() {
				that._flipAllCoinsBackward();
			}, 1000 );
		}
	};

	FreespinsCoins.prototype._flipAllCoinsBackward = function() {
		this._setFlipCoinsForward( false );
		this._numberOfExtraSpinCoins = 0;
		this._numberOfCoinsStartedFlipping = 0;
		this._numberOfCoinsFinishedFlipping = 0;
		for ( var i = 0; i < this._numberOfCoins; i++ ) {
			this._coinAnimsNormal.sprites[i].play();
			this._coinAnimsExtraSpins.sprites[i].play();
			++this._numberOfCoinsStartedFlipping;
		}
	};

	FreespinsCoins.prototype._endCoinFlip = function() {
		var that = this;

		this._bonusInfo = null;
		this._numberOfExtraSpinCoins = 0;
		this._numberOfCoinsStartedFlipping = 0;
		this._numberOfCoinsFinishedFlipping = 0;

		for ( var i = 0; i < this._numberOfCoins; ++i ) {
			this._coinAnimsNormal.sprites[i].stop();
			this._coinAnimsExtraSpins.sprites[i].stop();
			if ( this._coinRetriggerAnims && this._coinRetriggerAnims.sprites && this._coinRetriggerAnims.sprites[i] ) {
				this._coinRetriggerAnims.sprites[i].stop();
			}
		}

		libs.setTimeoutTick( function() {
			that._handleReelsBonusFeature.leave();
			that._handleReelsBonusFeature = null;
		}, this._settings.postRevealDelay );
	};

	FreespinsCoins.prototype._setFlipCoinsForward = function( flipForward ) {
		this._flippingCoinsForward = flipForward;
		for ( var i = 0; i < this._numberOfCoins; i++ ) {
			this._coinAnimsNormal.sprites[i].updateAnimationSettings( {
				playForward: flipForward
			} );
			this._coinAnimsExtraSpins.sprites[i].updateAnimationSettings( {
				playForward: flipForward
			} );
		}
	};

	FreespinsCoins.prototype._coinAnimFinished = function() {
		++this._numberOfCoinsFinishedFlipping;
		if ( this._flippingCoinsForward ) {
			if ( this._numberOfCoinsFinishedFlipping < this._numberOfCoins ) {
				this._flipNextCoinForward();
			}
		} else {
			if ( this._numberOfCoinsFinishedFlipping === this._numberOfCoins ) {
				this._endCoinFlip();
			}
		}
	};

	FreespinsCoins.prototype.getCoinRedrawArea = function() {
		return this._coinRedrawArea;
	};

	FreespinsCoins.prototype.draw = function( ctx ) {
		for ( var j = 0; j < this._numberOfCoins; j++ ) {
			if ( j < this._numberOfExtraSpinCoins && j < this._numberOfCoinsStartedFlipping ) {
				this._coinAnimsExtraSpins.sprites[j].draw( ctx );
				if ( this._coinRetriggerAnims && this._numberOfCoinsFinishedFlipping >= j ) {
					this._coinRetriggerAnims.sprites[j].draw( ctx );
				}
			} else {
				this._coinAnimsNormal.sprites[j].draw( ctx );
			}
		}
	};

	FreespinsCoins.prototype.isDirty = function() {
		return this._isFreespinsUIActive;
	};

	FreespinsCoins.prototype.drawIfDirty = function( ctx ) {
		if ( this.isDirty() ) {
			this.draw( ctx );
		}
	};


	game.FreespinsCoins = FreespinsCoins;

}() );