goog.provide( 'game.FreespinsModalExtra' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		modalTimeout: 3000,
		delayBeforeShowingModal: 3000,
		characterFadeInTime: 50,
		characterFadeOutTime: 1500,
		text : {
			largeText : {
				fontSize:36,
				maxWidth:411
			},
			smallText: {
				fontSize:24, 
				maxWidth:411
			},
			numberText :{
				fontSize: 66, 
				maxWidth:411
			}
		}
	};

	var selectors = {
		container: $( '#freespins-extra-container' ),
		messageLine1: $( '#freespins-extra-line1' ),
		messageLine2: $( '#freespins-extra-line2' ),
	};

	selectors.container.hide();

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';

	var FreespinsModalExtra = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._timeoutId = null;

		this._bonusInfo = null;
		this._handleDisplayFreespinsExtra = null;
		this._handlePostBet = null;

		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connect( 'libs.event.notify.betBonusInfo', function( bonusInfo ) { that._onNotifyBetBonusInfo( bonusInfo ); } );
		libs.globalEventBus.connect( 'libs.event.notify.keyPressed', function( keyCode, keyChar ) { that._onNotifyKeyPressed( keyCode, keyChar ); } );
		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerDown', function( point ) { that._onNotifyCanvasPointerDown( point ); } );
		libs.globalEventBus.connectActivityStart( 'game.activity.freespins.modalExtra', function( activity ) { that._onFreespinsModalExtra( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.postBet', function( activity ) { that._onPostBetStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'game.activity.character.animating', function() { that._onCharacterAnimatingEnd(); } );
	};

	FreespinsModalExtra.prototype._onNotifyLanguageChanged = function( langCode ) {
		libs.client.utils.DOM.setText( selectors.messageLine1, libs.getTranslation( 'FREESPINS_RETRIGGER' ), this._settings.text.largeText );
		libs.client.utils.DOM.setText( selectors.messageLine2, libs.getTranslation( 'AWARDED_X_EXTRA_FREESPINS', 8 ), this._settings.text.smallText );
	};

	FreespinsModalExtra.prototype._onNotifyBetBonusInfo = function( bonusInfo ) {
		if ( bonusInfo['name'] === "freeSpinCollect" && bonusInfo['retrigger'] > 0 ) {
			this._bonusInfo = bonusInfo;
		}
	};

	FreespinsModalExtra.prototype._onNotifyKeyPressed = function( keyCode, keyChar ) {
		if ( keyChar === ' ' ) {
			this._skipExtra();
		}
	};

	FreespinsModalExtra.prototype._onNotifyCanvasPointerDown = function( point ) {
		this._skipExtra();
	};

	FreespinsModalExtra.prototype._skipExtra = function() {
		if ( this._handleDisplayFreespinsExtra ) {
			libs.clearTimeoutTick( this._timeoutId );
			this._timeoutId = null;
			this._hideFreespinsExtra();
		}
	};

	FreespinsModalExtra.prototype._onFreespinsModalExtra = function( activity ) {
		if ( this._bonusInfo ) {
			this._handleDisplayFreespinsExtra = activity.join();
			this._showFreespinsExtra();
		}
	};

	FreespinsModalExtra.prototype._showFreespinsExtra = function() {
		var that = this;

		if ( !libs.globalEventBus.isActivityRunning( 'libs.activity.display.modalVisible' ) ) {
			libs.globalEventBus.invokeActivity( 'libs.activity.display.modalVisible' );
		}

		// libs.globalEventBus.invoke( 'libs.event.request.changeTopCanvasZIndex', libs.surfaceSettings.top.zIndexMovedToFront );
		libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
			backdropClass: 'backdrop-freespinsExtra',
			zIndex: libs.surfaceSettings.top.zIndexMovedToFront - 2,
			animationTime: 200
		} );

	
		libs.client.utils.DOM.fadeIn( selectors.container, 750 );

		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );

		this._timeoutId = libs.setTimeoutTick( function() {
			that._hideFreespinsExtra();
		}, this._settings.modalTimeout );
	};

	FreespinsModalExtra.prototype._hideFreespinsExtra = function() {
		var that = this;

		this._handleDisplayFreespinsExtra.leave();
		this._handleDisplayFreespinsExtra = null;
		if(!isDesktopDeviceType){
			this._bonusInfo = null;
		}

		libs.client.utils.DOM.fadeOut( selectors.container, 500 );

		libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-freespinsExtra' );

		libs.setTimeoutTick( function() {
			libs.globalEventBus.invoke( 'game.event.request.playCharacterAnimation' );

			if ( isDesktopDeviceType ) {
				libs.globalEventBus.invoke( 'game.event.request.fadeInCharacterAnimation', that._settings.characterFadeInTime );
			}
		}, 1500 );
	};

	FreespinsModalExtra.prototype._onPostBetStart = function( activity ) {
		if ( this._bonusInfo ) {
			this._handlePostBet = activity.join();
		}
	};

	FreespinsModalExtra.prototype._onCharacterAnimatingEnd = function() {
		var that = this;

		if ( this._bonusInfo ) {
			if ( isDesktopDeviceType ) {
				libs.globalEventBus.invoke( 'game.event.request.fadeOutCharacterAnimation', this._settings.characterFadeOutTime );
				this._bonusInfo = null;
			}
			libs.setTimeoutTick( function() {
				if ( that._handlePostBet ) {
					that._handlePostBet.leave();
					that._handlePostBet = null;
				}

				libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, false );
			}, isDesktopDeviceType ? this._settings.characterFadeOutTime + 1 : 0 );
		}

	};


	game.FreespinsModalExtra = FreespinsModalExtra;

}());