goog.provide( 'game.FreespinsModalIntro' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		modalTimeout: 10000,
		characterFadeInTime: 50,
		characterFadeOutTime: 1500,
		text: {
			largeText: {
				maxWidth: 411, 
				fontSize: 29
			},
			smallText: {
				maxWidth: 411, 
				maxHeight: 70,
				fontSize:16
			}
		}
	};

	var selectors = {
		container: $( '#freespins-intro-container' ),
		messageLine1: $( '#freespins-intro-line1' ),
		messageLine2: $( '#freespins-intro-line2' ),
		messageLine3: $( '#freespins-intro-line3' ),
		messageLine4: $( '#freespins-intro-line4' ),
		bottom: $( '#freespins-intro-bottom' )
	};

	selectors.container.hide();

	var isDesktopDeviceType = libs.client.core.clientInformation.getDeviceType() === 'desktop';

	var FreespinsModalIntro = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._timeoutId = null;

		this._bonusInfo = null;
		this._handleDisplayFreespinsIntro = null;
		this._handlePostBet = null;

		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connect( 'libs.event.notify.betBonusInfo', function( bonusInfo ) { that._onNotifyBetBonusInfo( bonusInfo ); } );
		libs.globalEventBus.connect( 'libs.event.notify.keyPressed', function( keyCode, keyChar ) { that._onNotifyKeyPressed( keyCode, keyChar ); } );
		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerDown', function( point ) { that._onNotifyCanvasPointerDown( point ); } );
		libs.globalEventBus.connectActivityStart( 'game.activity.freespins.modalIntro', function( activity ) { that._onFreespinsModalIntroStart( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.betState.postBet', function( activity ) { that._onPostBetStart( activity ); } );
		libs.globalEventBus.connectActivityEnd( 'game.activity.character.animating', function() { that._onCharacterAnimatingEnd(); } );
	};

	FreespinsModalIntro.prototype._onNotifyLanguageChanged = function( langCode ) {

		libs.client.utils.DOM.setText( selectors.messageLine1, libs.getTranslation( 'CONGRATULATIONS' ), this._settings.text.largeText );
		libs.client.utils.DOM.setText( selectors.messageLine2, libs.getTranslation( 'YOU_HAVE_BEEN_AWARDED' ), this._settings.text.smallText );
		libs.client.utils.DOM.setText( selectors.messageLine3, "8 " + libs.getTranslation( 'FREESPINS' ), this._settings.text.largeText );
		libs.client.utils.DOM.setText( selectors.messageLine4, libs.getTranslation( 'DURING_FREESPINS_ALL_WILDS_WILL_STEP_DOWN' ), this._settings.text.smallText );


		if ( isDesktopDeviceType ) {
			libs.client.utils.DOM.setText( selectors.bottom, libs.getTranslation( 'CLICK_SCREEN_TO_START' ), this._settings.text.smallText );
		} else {
			libs.client.utils.DOM.setText( selectors.bottom, libs.getTranslation( 'TOUCH_SCREEN_TO_START' ), this._settings.text.smallText );
		}
	};

	FreespinsModalIntro.prototype._onNotifyBetBonusInfo = function( bonusInfo ) {
		if ( bonusInfo['name'] === "freeSpinWin" ) {
			this._bonusInfo = bonusInfo;
		}
	};

	FreespinsModalIntro.prototype._onNotifyKeyPressed = function( keyCode, keyChar ) {
		if ( keyChar === ' ' ) {
			this._skipIntro();
		}
	};

	FreespinsModalIntro.prototype._onNotifyCanvasPointerDown = function( point ) {
		this._skipIntro();
	};

	FreespinsModalIntro.prototype._skipIntro = function() {
		if ( this._handleDisplayFreespinsIntro ) {
			libs.clearTimeoutTick( this._timeoutId );
			this._timeoutId = null;
			this._hideFreespinsIntro();
		}
	};

	FreespinsModalIntro.prototype._onFreespinsModalIntroStart = function( activity ) {
		if ( this._bonusInfo ) {
			this._handleDisplayFreespinsIntro = activity.join();
			this._showFreespinsIntro();
		}
	};

	FreespinsModalIntro.prototype._showFreespinsIntro = function() {
		var that = this;

		if ( !libs.globalEventBus.isActivityRunning( 'libs.activity.display.modalVisible' ) ) {
			libs.globalEventBus.invokeActivity( 'libs.activity.display.modalVisible' );
		}

		libs.globalEventBus.invoke( 'game.event.request.playFreespinsMusic' );
		libs.globalEventBus.invoke( 'libs.event.request.changeTopCanvasZIndex', libs.surfaceSettings.top.zIndexMovedToFront );
		libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
			backdropClass: 'backdrop-freespinsIntro',
			zIndex: libs.surfaceSettings.top.zIndexMovedToFront - 2,
			animationTime: 200
		} );

		libs.client.utils.DOM.setText( selectors.messageLine3, this._bonusInfo['awarded'] + " " + libs.getTranslation( 'FREESPINS' ) );

		selectors.container.css( 'z-index', libs.surfaceSettings.top.zIndexMovedToFront - 1 );
		libs.client.utils.DOM.fadeIn( selectors.container, 750 );

		libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );

		this._timeoutId = libs.setTimeoutTick( function() {
			that._hideFreespinsIntro();
		}, this._settings.modalTimeout );
	};

	FreespinsModalIntro.prototype._hideFreespinsIntro = function() {
		var that = this;

		this._handleDisplayFreespinsIntro.leave();
		this._handleDisplayFreespinsIntro = null;

		libs.client.utils.DOM.fadeOut( selectors.container, 500 );

		libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-freespinsIntro' );
		libs.globalEventBus.invoke( 'libs.event.request.changeTopCanvasZIndex', libs.surfaceSettings.top.zIndexNormal );

		libs.globalEventBus.invokeActivity( 'libs.activity.display.freespinsUI', false );

		libs.setTimeoutTick( function() {
			libs.globalEventBus.invoke( 'game.event.request.fadeInCharacterAnimation', that._settings.characterFadeInTime );
			libs.globalEventBus.invoke( 'game.event.request.playCharacterAnimation' );
		}, 1500 );
	};

	FreespinsModalIntro.prototype._onPostBetStart = function( activity ) {
		if ( this._bonusInfo ) {
			this._handlePostBet = activity.join();
		}
	};

	FreespinsModalIntro.prototype._onCharacterAnimatingEnd = function() {
		var that = this;

		if ( this._bonusInfo ) {
			if ( isDesktopDeviceType ) {
				libs.globalEventBus.invoke( 'game.event.request.fadeOutCharacterAnimation', this._settings.characterFadeOutTime );
			}

			libs.setTimeoutTick( function() {
				if ( that._handlePostBet ) {
					that._handlePostBet.leave();
					that._handlePostBet = null;
				}

				that._bonusInfo = null;

				libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, false );
			}, isDesktopDeviceType ? this._settings.characterFadeOutTime + 1 : 0 );
		}
	};


	game.FreespinsModalIntro = FreespinsModalIntro;

}());