goog.provide( 'game.FreespinsModalOutro' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.getTranslation' );
goog.require( 'libs.client.utils.DOM' );


(function() {
	"use strict";

	var defaultSettings = {
		delayBeforeShowingModal: 2000,
		modalTimeout: 5000,
		text: {
			value: {
				fontSize: 36,
				maxWidth:200
			},
			largeText: {
				fontSize:26, 
				maxWidth:411
			}, 
			smallText: {
				fontSize:20, 
				maxWidth:411
			}
		}
	};

	var selectors = {
		container: $( '#freespins-outro-container' ),
		messageLine1: $( '#freespins-outro-line1' ),
		messageLine2: $( '#freespins-outro-line2' ),
		messageLine3: $( '#freespins-outro-line3' ),
		value: $( '#freespins-outro-line4' )
	};

	selectors.container.hide();

	var FreespinsModalOutro = function( overrideSettings ) {
		this._settings = libs.client.utils.misc.mergeSettings( defaultSettings, overrideSettings );

		var that = this;

		this._timeoutId = null;
		this._displayingOutro = false;

		this._bonusInfo = null;
		this._betResult = null;
		this._handleDisplayFreespinsOutro = null;
		this._handleFreespinsUI = null;

		libs.globalEventBus.connect( 'libs.event.notify.languageChanged', function( langCode ) { that._onNotifyLanguageChanged( langCode ); } );
		libs.globalEventBus.connect( 'libs.event.notify.betBonusInfo', function( bonusInfo ) { that._onNotifyBetBonusInfo( bonusInfo ); } );
		libs.globalEventBus.connect( 'libs.event.notify.betResult', function( betResult ) { that._onNotifyBetResult( betResult ); } );
		libs.globalEventBus.connect( 'libs.event.notify.keyPressed', function( keyCode, keyChar ) { that._onNotifyKeyPressed( keyCode, keyChar ); } );
		libs.globalEventBus.connect( 'libs.event.notify.canvasPointerDown', function( point ) { that._onNotifyCanvasPointerDown( point ); } );
		libs.globalEventBus.connectActivityStart( 'game.activity.freespins.modalOutro', function( activity ) { that._onFreespinsModalOutro( activity ); } );
		libs.globalEventBus.connectActivityStart( 'libs.activity.display.freespinsUI', function( activity, inBetRecovery ) { that._onFreespinsUIStart( activity, inBetRecovery ); } );
	};

	FreespinsModalOutro.prototype._onNotifyLanguageChanged = function( langCode ) {
		libs.client.utils.DOM.setText( selectors.messageLine1, libs.getTranslation( 'FREESPINS_OVER' ), this._settings.text.smallText );
		libs.client.utils.DOM.setText( selectors.messageLine2, libs.getTranslation( 'CONGRATULATIONS' ), this._settings.text.largeText );
		libs.client.utils.DOM.setText( selectors.messageLine3, libs.getTranslation( 'YOU_HAVE_WON' ), this._settings.text.largeText );
	};

	FreespinsModalOutro.prototype._onNotifyBetBonusInfo = function( bonusInfo ) {
		if ( bonusInfo['name'] === "freeSpinCollect" ) {
			this._bonusInfo = bonusInfo;
		}
	};

	FreespinsModalOutro.prototype._onNotifyBetResult = function( betResult ) {
		if ( this._bonusInfo && betResult['expectCheckpoint'] === false ) {
			this._betResult = betResult;
		}
	};

	FreespinsModalOutro.prototype._onNotifyKeyPressed = function( keyCode, keyChar ) {
		if ( keyChar === ' ' ) {
			this._skipOutro();
		}
	};

	FreespinsModalOutro.prototype._onNotifyCanvasPointerDown = function( point ) {
		this._skipOutro();
	};

	FreespinsModalOutro.prototype._skipOutro = function() {
		if ( this._displayingOutro ) {
			libs.clearTimeoutTick( this._timeoutId );
			this._timeoutId = null;
			this._hideFreespinsOutro();
		}
	};

	FreespinsModalOutro.prototype._onFreespinsModalOutro = function( activity ) {
		if ( this._bonusInfo && this._betResult ) {
			this._handleDisplayFreespinsOutro = activity.join();
			this._showFreespinsOutro();
		} else {
			this._bonusInfo = null;
			this._betResult = null;
		}
	};

	FreespinsModalOutro.prototype._onFreespinsUIStart = function( activity, inBetRecovery ) {
		this._handleFreespinsUI = activity.join();
	};

	FreespinsModalOutro.prototype._showFreespinsOutro = function() {
		var that = this;

		if ( !libs.globalEventBus.isActivityRunning( 'libs.activity.display.modalVisible' ) ) {
			libs.globalEventBus.invokeActivity( 'libs.activity.display.modalVisible' );
		}

		libs.globalEventBus.invoke( 'game.event.request.stopFreespinsMusic' );

		libs.setTimeoutTick( function() {
			that._displayingOutro = true;

			libs.globalEventBus.invoke( 'libs.event.request.changeTopCanvasZIndex', libs.surfaceSettings.top.zIndexMovedToFront );
			libs.globalEventBus.invoke( 'libs.event.request.showBackdrop', 'body', {
				backdropClass: 'backdrop-freespinsOutro',
				zIndex: libs.surfaceSettings.top.zIndexMovedToFront - 2,
				animationTime: 200
			} );

			// total win is the accumulative win so far.... plus any win on the very last spin
			var text = libs.moneyFormat( that._bonusInfo['accumulativeWin'].plus( that._bonusInfo['win'] ), true );

			libs.client.utils.DOM.setText( selectors.value, text, that._settings.text.value );

			selectors.container.css( 'z-index', libs.surfaceSettings.top.zIndexMovedToFront - 1 );
			libs.client.utils.DOM.fadeIn( selectors.container, 750 );

			libs.globalEventBus.invoke( 'libs.event.request.canvasFullDraw', true, true );

			that._handleFreespinsUI.leave();
			that._handleFreespinsUI = null;

			that._timeoutId = libs.setTimeoutTick( function() {
				that._hideFreespinsOutro();
			}, that._settings.modalTimeout );
		}, that._settings.delayBeforeShowingModal );
	};

	FreespinsModalOutro.prototype._hideFreespinsOutro = function() {
		var that = this;
		this._displayingOutro = false;

		libs.client.utils.DOM.fadeOut( selectors.container, 500, function() {
			that._handleDisplayFreespinsOutro.leave();
			that._handleDisplayFreespinsOutro = null;
		} );

		libs.globalEventBus.invoke( 'libs.event.request.hideBackdrop', 'body', 'backdrop-freespinsOutro' );
		libs.globalEventBus.invoke( 'libs.event.request.changeTopCanvasZIndex', libs.surfaceSettings.top.zIndexNormal );

		this._bonusInfo = null;
		this._betResult = null;
	};


	game.FreespinsModalOutro = FreespinsModalOutro;

}());