goog.provide( 'game.FreespinsMusic' );

goog.require( 'libs.globalEventBus' );


(function() {
	"use strict";


	var FreespinsMusic = function() {
		var that = this;

		this._currentFreeSpinsLoopToPlay = 1;

		libs.globalEventBus.connect( 'game.event.request.playFreespinsMusic', function( forcePlay ) { that._onRequestPlayFreespinsMusic( forcePlay ); } );
		libs.globalEventBus.connect( 'game.event.request.stopFreespinsMusic', function() { that._onRequestStopFreespinsMusic(); } );
	};

	FreespinsMusic.prototype._onRequestPlayFreespinsMusic = function( forcePlay ) {
		this._currentFreeSpinsLoopToPlay = this._getFreeSpinsMusicIterationToPlay();
		var loopCallbackFunc = libs.callback( function() {
			var newSpinLoopToPlay = this._getFreeSpinsMusicIterationToPlay();
			if ( this._currentFreeSpinsLoopToPlay !== newSpinLoopToPlay || forcePlay ) {
				var oldLoopName = this._formatFreeSpinsSoundName( this._currentFreeSpinsLoopToPlay );
				var newLoopName = this._formatFreeSpinsSoundName( newSpinLoopToPlay );

				libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['MQ', 'HQ'], oldLoopName );
				libs.globalEventBus.invoke( 'libs.event.request.playSound', ['MQ', 'HQ'], newLoopName, {
					loopCount: -1,
					loopCallback: loopCallbackFunc
				} );

				this._currentFreeSpinsLoopToPlay = newSpinLoopToPlay;
			}
		}, this );

		libs.globalEventBus.invoke( 'libs.event.request.playSound', ['MQ', 'HQ'], this._formatFreeSpinsSoundName( this._currentFreeSpinsLoopToPlay ), {
			loopCount: -1,
			loopCallback: loopCallbackFunc
		} );
	};

	FreespinsMusic.prototype._onRequestStopFreespinsMusic = function() {
		libs.globalEventBus.invoke( 'libs.event.request.stopSound', ['MQ', 'HQ'], this._formatFreeSpinsSoundName( this._currentFreeSpinsLoopToPlay ) );
		libs.globalEventBus.invoke( 'libs.event.request.playSound', ['MQ', 'HQ'], 'SndFreeSpinsLoopEnd' );
	};

	FreespinsMusic.prototype._formatFreeSpinsSoundName = function( iteration ) {
		return "SndFreeSpinsLoop" + iteration;
	};

	FreespinsMusic.prototype._getFreeSpinsMusicIterationToPlay = function() {
		return 1;
	};

	game.FreespinsMusic = FreespinsMusic;

}());