goog.provide( 'game.GameInit' );

goog.require( 'libs.client.core.buildVariables' );
goog.require( 'libs.client.core.clientInformation' );
goog.require( 'libs.client.reels.ReelsGameInit' );
goog.require( 'libs.client.hillsVegas.createLocalServer' );
goog.require( 'libs.client.hillsVegas.createLiveServer' );
goog.require( 'libs.client.virgo.createLiveServer' );
goog.require( 'game.manifestAudio' );
goog.require( 'game.manifestSlot' );
goog.require( 'game.manifestSymbols' );
goog.require( 'game.settings' );


(function() {
	"use strict";

	var enableDebugMode = false;
//	enableDebugMode = true;

	var clientInfo = libs.client.core.clientInformation;

	libs.log( '\n------ Compile Information ------' +
	'\n\tCLOSURE_COMPILE: ' + CLOSURE_COMPILE +
	'\n\tPLATFORM_TYPE: ' + PLATFORM_TYPE +
	'\n\tCUSTOMER_NAME: ' + CUSTOMER_NAME +
	'\n\tIS_FORTUNE_SPINS_LAYOUT: ' + IS_FORTUNE_SPINS_LAYOUT +
	'\n\tBUILD_TYPE: ' + BUILD_TYPE +
	'\n\tBUILD_VERSION: ' + BUILD_VERSION +
	'\n\tDevice Type: ' + game.deviceCompatibility.deviceType +
	'\n---------------------------------' );

	var GameInit = function() {
		var that = this;

		libs.globalEventBus.connectActivityStart( 'libs.activity.systemState.gameInit', function( activity ) { that._onGameInitStart( activity ); } );

		var manifestArray = [game.manifestSymbols, game.manifestSlot];
		if ( clientInfo.getAudioQuality() !== "LQ" ) {
			manifestArray.push( game.manifestAudio );
		}

		var setupFlags = {
			'enableDebugMode': enableDebugMode,
			'platformType': PLATFORM_TYPE,
			'customerName': CUSTOMER_NAME,
			'isFortuneSpinsLayout': IS_FORTUNE_SPINS_LAYOUT,
			'buildType': BUILD_TYPE
		};

		var createServerFuncs = {
			'local': function() { return libs.client.hillsVegas.createLocalServer(); },
			'fake': function() { throw new Error( "Fake server not supported" ); },
			'hillsLive': function() { return libs.client.hillsVegas.createLiveServer(); },
			'hillsLocal': function() { return libs.client.hillsVegas.createLocalServer(); },
			'virgo': function() { return libs.client.virgo.createLiveServer(); }
		};

		this._reelsGameInit = new libs.client.reels.ReelsGameInit( setupFlags, createServerFuncs, game.settings, manifestArray );
	};

	GameInit.prototype._onGameInitStart = function( activity ) {
        // WeR: Switch to enable WeR changes for Marketing build
        game.settings.WeR = true;
		game.settings.customerName = CUSTOMER_NAME;
		game.settings.platformType = PLATFORM_TYPE;
		game.settings.isFortuneSpinsLayout = IS_FORTUNE_SPINS_LAYOUT;
		game.mainSlots = new game.MainSlots( game.settings );
	};

	$( document ).ready( libs.callback( function() {
		game.gameInit = new GameInit();
	} ) );

}());