goog.provide( 'game.MainSlots' );

goog.require( 'libs.globalEventBus' );
goog.require( 'libs.client.core.clientInformation' );

goog.require( 'libs.client.rendering.Sprite' );

goog.require( 'libs.client.reels.MainSlotsBaseClass' );

goog.require( 'libs.client.presentation.JackpotTierOneMeter' );
goog.require( 'libs.client.presentation.JackpotTierTwoMeter' );
goog.require( 'libs.client.presentation.FreespinsJackpotTierOneMeter' );
goog.require( 'libs.client.presentation.FreespinsWinningsMeter' );

goog.require( 'game.Freespins' );
goog.require( 'game.FreespinsCharacter' );
goog.require( 'game.FreespinsCoins' );
goog.require( 'game.FreespinsModalIntro' );
goog.require( 'game.FreespinsModalExtra' );
goog.require( 'game.FreespinsModalOutro' );
goog.require( 'game.FreespinsCoins' );
goog.require( 'game.FreespinsMusic' );


(function() {
	"use strict";

	var selectors = {
		backgroundNormal: $( '#background-normal' ),
		backgroundFreespins: $( '#background-freespins' ),
		freespinsPlusLogo: $( '#freespinsplus-logo' ),
		inspiredFooterLogo: $('#wer-marketing-inspired-logo')
	};

	selectors.freespinsPlusLogo.hide();
	selectors.inspiredFooterLogo.hide();
	libs.client.utils.DOM.hideBackground( selectors.backgroundFreespins, 1 );

	var clientInfo = libs.client.core.clientInformation;
	var isMobileDeviceType = clientInfo.getDeviceType() === 'mobile';
	var isDesktopDeviceType = clientInfo.getDeviceType() === 'desktop';

	var MainSlots = function( settings ) {
		libs.client.reels.MainSlotsBaseClass.call( this, settings );
		this._base = libs.client.reels.MainSlotsBaseClass.prototype;
		this._base._initialise.call( this );
		this._defaultUpdateFunctions = this._setUpdateFunctions();

		var that = this;

		// game specific variables created here


		// game specific images created here
		this._reelFrame = new libs.client.rendering.Sprite( this._settings.reelsFrame );
		this._backgroundStrips = new libs.client.rendering.Sprite( this._settings.backgroundStrips );
		this._backgroundStripsFreespins = new libs.client.rendering.Sprite( jQuery.extend( true, {}, this._settings.backgroundStrips, { imageName: 'FS_Background_Strips' } ) );

		// game specific components created below
		this._jackpotTierOneMeter = new libs.client.presentation.JackpotTierOneMeter( this._settings.libs.presentation.jackpotTierOneMeter );
		this._jackpotTierTwoMeter = new libs.client.presentation.JackpotTierTwoMeter( this._settings.libs.presentation.jackpotTierTwoMeter );

		this._freespinsJackpotTierOneMeter = new libs.client.presentation.FreespinsJackpotTierOneMeter( this._settings.libs.presentation.freespinsJackpotTierOneMeter );
		this._freespinsWinningsMeter = new libs.client.presentation.FreespinsWinningsMeter( this._settings.libs.presentation.freespinsWinningsMeter );

		this._hiddenWilds = new game.HiddenWilds( this._settings.hiddenWilds );
		this._freespins = new game.Freespins( this._settings.freespins );
		this._freespinsCharacter = new game.FreespinsCharacter( this._settings.freespinsCharacter );
		this._freespinsCoins = new game.FreespinsCoins( this._settings.freespinsCoins );
		this._freespinsModalIntro = new game.FreespinsModalIntro( this._settings.freespinsModalIntro );
		this._freespinsModalExtra = new game.FreespinsModalExtra( this._settings.freespinsModalExtra );
		this._freespinsModalOutro = new game.FreespinsModalOutro( this._settings.freespinsModalOutro );
		this._freespinsMusic = new game.FreespinsMusic( this._settings.freespinsMusic );

		if ( isDesktopDeviceType && (this._settings.platformType === 'hillsLocal' || this._settings.platformType === 'hillsLive' ) ) {
			this._logoAnimation = new libs.client.reels.LogoAnimation( this._settings.libs.reels.logoAnimation );
		}

		// game specific activities joined here

	};

	MainSlots.prototype = Object.create( libs.client.reels.MainSlotsBaseClass.prototype );
	MainSlots.prototype.constructor = MainSlots;

	MainSlots.prototype._onGamePlayingStart = function( activity ) {
		this._base._onGamePlayingStart.call( this, activity );

		if ( isMobileDeviceType ) {
			libs.client.utils.DOM.fadeIn( selectors.freespinsPlusLogo, 500 );
			libs.client.utils.DOM.fadeIn( selectors.inspiredFooterLogo, 500 );
		}
	};

	MainSlots.prototype._onFreespinsUIStart = function( activity, inBetRecovery ) {
		this._base._onFreespinsUIStart.call( this, activity, inBetRecovery );

		libs.client.utils.DOM.hideBackground( selectors.backgroundNormal, 1000 );
		libs.client.utils.DOM.showBackground( selectors.backgroundFreespins, 1000 );

		if ( isMobileDeviceType ) {
			libs.client.utils.DOM.fadeOut( selectors.freespinsPlusLogo, 500 );
			libs.client.utils.DOM.fadeOut( selectors.inspiredFooterLogo, 500 );
		}
	};

	MainSlots.prototype._onFreespinsUIEnd = function() {
		this._base._onFreespinsUIEnd.call( this );

		libs.client.utils.DOM.hideBackground( selectors.backgroundFreespins, 1000 );
		libs.client.utils.DOM.showBackground( selectors.backgroundNormal, 1000 );

		if ( isMobileDeviceType ) {
			libs.client.utils.DOM.fadeIn( selectors.freespinsPlusLogo, 500 );
			libs.client.utils.DOM.fadeIn( selectors.inspiredFooterLogo, 500 );
		}
	};

	MainSlots.prototype._buildSpinParameters = function( options ) {
		var spinParameters = jQuery.extend( true, {}, options );

		if ( this._isFreespinsActive ) {
			// if reels were locked from the previous spin, pass the heavy animation spin parameter to the reels to move it 1 position down.
			var reelsLocked = this._freespins.reelsLocked();
			if ( reelsLocked ) {
				spinParameters.heavyDropSpeed = 0.1;
				spinParameters.heavyDropAnimation = {
					oldPositions: reelsLocked.reels,
					distances: reelsLocked.reels.map( function( pos ) {
						return 1;
					} ),
					reelsetIndex: reelsLocked.set
				};
			}

			// use a slower spin speed than normal in free spins mode so the player can see the fireball
			spinParameters.maxSpeed = this._settings.freespins.fireball.freeSpinsSpinSpeed;
		}
		// Superimpose mystery symbol patterns on the reel bands that pass the window.
		// These will not be displayed on the final reel screen
		if ( this._settings.imposeReelSymbolSets ) {
			var randomIndex = Math.floor( Math.random() * this._settings.imposeReelSymbolSets.length );
			spinParameters.imposeReelSymbols = this._settings.imposeReelSymbolSets[randomIndex];
		}
		return spinParameters;
	};

	MainSlots.prototype._gameSpecificBetRecovery = function() {
		if ( this._betRecoveryInfo['bonus'] ) {
			for ( var i = 0; i < this._betRecoveryInfo['bonus'].length; ++i ) {
				var bonusInfo = this._betRecoveryInfo['bonus'][i];
				if ( bonusInfo['name'] === "freeSpinCollect" ) {
					libs.globalEventBus.invoke( 'game.event.request.playFreespinsMusic' );

					libs.setTimeoutTick( function() {
						if ( isMobileDeviceType ) {
							libs.globalEventBus.invoke( 'game.event.request.fadeInCharacterAnimation', 50 );
							libs.globalEventBus.invoke( 'game.event.request.playCharacterAnimation' );
						}
					}, 250 );
					break;
				}
			}
		}
	};

	MainSlots.prototype._setUpdateFunctions = function() {
		var that = this;

		var update = function( timeDelta ) {
			that._base._update.call( that, timeDelta );
		};

		var drawAllBase = function() {
			var surface = libs.surfaceBase;
			var ctx = libs.surfaceBase.ctx;

			surface.clearRect( 0, 0, that._settings.gameWidth, that._settings.gameHeight );

			var shouldDrawLogo = that._logoAnimation && !that._isFreespinsUIActive;

			if ( that._isFreespinsUIActive ) {
				that._backgroundStripsFreespins.draw( ctx );
				that._freespins.draw( ctx );
			} else {
				that._backgroundStrips.draw( ctx );
			}

			that._reelsViewGroup.drawReelband( ctx );

			if ( shouldDrawLogo ) {
				that._logoAnimation.draw( ctx );
			}

			that._reelsViewGroup.drawFadedBackgroundIfDirty( ctx );
			that._reelFrame.draw( ctx );
		};

		var drawDirtyBase = function() {
			var surface = libs.surfaceBase;
			var ctx = libs.surfaceBase.ctx;

			var shouldDrawLogo = that._logoAnimation && !that._isFreespinsUIActive;

			if ( that._reelsViewGroup.isDirtyReelband() || that._reelsViewGroup.isDirtyFadedBackground() || shouldDrawLogo ) {
				surface.clearRect( that._reelFrame.getDrawnPosition().x, that._reelFrame.getDrawnPosition().y, that._reelFrame.getDrawnDimensions().width, that._reelFrame.getDrawnDimensions().height );
				if ( shouldDrawLogo ) {
					var logoDrawSettings = that._logoAnimation.getDrawSettings();
					surface.clearRect( logoDrawSettings.x, logoDrawSettings.y, logoDrawSettings.width, logoDrawSettings.height );
				}

				if ( that._isFreespinsUIActive ) {
					that._backgroundStripsFreespins.draw( ctx );
				} else {
					that._backgroundStrips.draw( ctx );
				}

				that._reelsViewGroup.drawReelband( ctx );

				if ( shouldDrawLogo ) {
					that._logoAnimation.draw( ctx );
				}

				that._reelsViewGroup.drawFadedBackgroundIfDirty( ctx );
				that._reelFrame.draw( ctx );
			}
		};

		var drawAllTop = function() {
			var surface = libs.surfaceTop;
			var ctx = libs.surfaceTop.ctx;

			surface.clearRect( 0, 0, that._settings.gameWidth, that._settings.gameHeight );

			if ( that._isFreespinsUIActive ) {
				that._freespins.draw( ctx );
			}

			that._winlinesView.draw( ctx );
			that._reelsViewGroup.drawAnimation( ctx );

			that._reelsAnticipation.draw( ctx );

			if ( that._isFreespinsUIActive ) {
				that._freespinsCharacter.draw( ctx );
				that._freespinsCoins.draw( ctx );
			}

			if ( that._hiddenWilds.featureActive ) {
				that._hiddenWilds.draw( ctx );
			}
		};

		var drawDirtyTop = function() {
			var surface = libs.surfaceTop;
			var ctx = libs.surfaceTop.ctx;

			var clearedAll = false;

			var reelsDirty = that._reelsViewGroup.isDirtyAnimation() ||
				that._winlinesView.isDirty() || that._hiddenWilds.isDirty() ||
				that._freespins.isDirty() || that._reelsAnticipation.isDirty();

			if ( reelsDirty ) {
				if ( that._reelsAnticipation.isDirty() || that._hiddenWilds.isDirty() ) {
					surface.clearRect( 0, 0, that._settings.gameWidth, that._settings.gameHeight );
					clearedAll = true;
				} else if ( that._freespins.isDirty() || (that._reelsViewGroup.isDirtyAnimation() && !that._isFreespinsUIActive) ) {
					surface.clearRect( that._reelFrame.getDrawnPosition().x, 0, that._reelFrame.getDrawnDimensions().width, that._settings.gameHeight );
				} else {
					surface.clearRect( that._reelFrame.getDrawnPosition().x, that._reelFrame.getDrawnPosition().y, that._reelFrame.getDrawnDimensions().width, that._reelFrame.getDrawnDimensions().height );
				}
			}

			if ( that._isFreespinsUIActive && that._freespinsCharacter.isDirty() ) {
				if ( !clearedAll ) { // no need to clear if whole screen has been anyway
					var drawSettings = that._freespinsCharacter.getDrawSettings();
					surface.clearRect( drawSettings.x - 20, drawSettings.y - 20, drawSettings.width + drawSettings.trimmedOffsetX + 40, drawSettings.height + 40 );
				}
			}

			if ( that._isFreespinsUIActive ) {
				surface.clearRect( that._freespinsCoins.getCoinRedrawArea().x,
					that._freespinsCoins.getCoinRedrawArea().y,
					that._freespinsCoins.getCoinRedrawArea().width,
					that._freespinsCoins.getCoinRedrawArea().height
				);
			}

			if ( reelsDirty ) {
				that._winlinesView.draw( ctx );
				that._reelsViewGroup.drawAnimationIfDirty( ctx );
			}

			if ( that._reelsAnticipation.isDirty() ) {
				that._reelsAnticipation.draw( ctx );
			}

			if ( that._isFreespinsUIActive ) {
				if ( that._freespinsCharacter.isDirty() || clearedAll ) {
					that._freespinsCharacter.draw( ctx );
				} else {
					that._freespins.drawIfDirty( ctx );
				}
			}

			if ( that._isFreespinsUIActive ) {
				that._freespinsCoins.drawIfDirty( ctx );
			}

			if ( reelsDirty ) {
				if ( that._hiddenWilds.featureActive ) {
					that._hiddenWilds.draw( ctx );
				}
			}
		};

		var drawOverlay = function() {
			var surface = libs.surfaceTop;
			var ctx = libs.surfaceTop.ctx;

			surface.clearRect( 0, 0, that._settings.gameWidth, that._settings.gameHeight );

			if ( that._xOfAKindSplash.isFeatureActive() ) {
				that._xOfAKindSplash.draw( ctx );
			}

			if ( that._bigWinner.isFeatureActive() ) {
				that._bigWinner.draw( ctx );
			}

			if ( that._historyDisplay && that._historyDisplay.isFeatureActive() ) {
				that._historyDisplay.draw( ctx );
			}
		};

		var updateFunctions = {
			update: update,
			drawAllBase: drawAllBase,
			drawDirtyBase: drawDirtyBase,
			drawAllTop: drawAllTop,
			drawDirtyTop: drawDirtyTop,
			drawOverlay: drawOverlay
		};

		return updateFunctions;
	};


	game.MainSlots = MainSlots;

}());