goog.provide('game.manifestAudio');

goog.require('libs.client.core.clientInformation');


(function(){
	"use strict";

	var clientInfo = libs.client.core.clientInformation;
	var manifest_audio = [];

	var isIE = clientInfo.isIE();

	// manifest here is for MQ and HQ sounds only (ie. uses soundjs. LQ uses a direct Audio Tag
	function createSrcString( srcString ) {

		function replaceAll( str, find, replace) {
			return str.replace(new RegExp(find, 'g'), replace);
		}

		// Note: Firefox 25 web audio has a bug loading MP3 on Windows 7, always prefer to load OGG first
		var resultString = "";
		if ( clientInfo.getAudioQuality() === "HQ" ){
			resultString += replaceAll( srcString, "%1%", "stereo" );
		} else{
			resultString += replaceAll( srcString, "%1%", "mono" );
		}
		return resultString;
	}

	// data: x signifies how many of that kind of sound can be played at once. Usually can just put 1, but occasionally, if the same sound
	// may need to be played multiple times at one, it needs increasing or else the previous sound gets cut off. But be aware that IE
	// can only support a certain number of tags (20-25 is the target - the reality is there is no defined limit, it's based on hardware/memory/IE version 
	// at runtime) which is the sum of all of the numbers in data. For IE, you may want to accept the cutting off of a sound rather than losing the ability
	// to load additional sounds
	var common = [
		{id: "SndSilence", src: createSrcString( "assets/sounds/%1%/General/Silence.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndFreeSpinsLoop1", src: createSrcString( "assets/sounds/%1%/Freespins/FreeSpinsLoop1.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndFreeSpinsLoopEnd", src: createSrcString( "assets/sounds/%1%/Freespins/FreeSpinsLoopEnd.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndReelSpin1", src: createSrcString( "assets/sounds/%1%/Reels/ReelSpin1.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbol0", src: createSrcString( "assets/sounds/%1%/Reels/Symbol0.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbol1", src: createSrcString( "assets/sounds/%1%/Reels/Symbol1.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbol2", src: createSrcString( "assets/sounds/%1%/Reels/Symbol2.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbol3", src: createSrcString( "assets/sounds/%1%/Reels/Symbol3.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbol4", src: createSrcString( "assets/sounds/%1%/Reels/Symbol4.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbol5", src: createSrcString( "assets/sounds/%1%/Reels/Symbol5.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbol6", src: createSrcString( "assets/sounds/%1%/Reels/Symbol6.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbol7", src: createSrcString( "assets/sounds/%1%/Reels/Symbol7.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbolFreespins", src: createSrcString( "assets/sounds/%1%/Reels/SymbolFreespins.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndSymbolJackpot", src: createSrcString( "assets/sounds/%1%/Reels/Symbol8.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },

		{id: "MysterySymbol", src: createSrcString( "assets/sounds/%1%/Reels/MysterySymbol.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
        		{id: "ExtraSpins0", src: createSrcString( "assets/sounds/%1%/Freespins/ExtraSpins1.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "ExtraSpins1", src: createSrcString( "assets/sounds/%1%/Freespins/ExtraSpins2.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "ExtraSpins2", src: createSrcString( "assets/sounds/%1%/Freespins/ExtraSpins3.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "ExtraSpins3", src: createSrcString( "assets/sounds/%1%/Freespins/ExtraSpins4.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "ExtraSpins4", src: createSrcString( "assets/sounds/%1%/Freespins/ExtraSpins5.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndRockSlide", src: createSrcString( "assets/sounds/%1%/Freespins/RockSlide.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndFireballWhoosh", src: createSrcString( "assets/sounds/%1%/Freespins/FireBallWhoosh0.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndBigWinner", src: createSrcString( "assets/sounds/%1%/General/BigWinner.ogg" ), data:1 , type: createjs.PreloadJS.SOUND }

	];

	var commonMQ = [];

	var commonHQ_All = [
		{id: "SndReelStop", src: createSrcString( "assets/sounds/%1%/Reels/ReelStop.ogg" ), data:2 , type: createjs.PreloadJS.SOUND },
		{id: "SndCountUpWin", src : createSrcString ( "assets/sounds/%1%/Reels/CountUpWin.ogg" ), data:1, type: createjs.PreloadJS.SOUND },
		{id: "SndFeatureSymbolLand", src: createSrcString( "assets/sounds/%1%/Reels/FeatureSymbolLand.ogg" ), data:isIE?1:2 , type: createjs.PreloadJS.SOUND },
		{id: "SndXOfAKindMultiple", src: createSrcString( "assets/sounds/%1%/Reels/XOfAKindMultiple.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndFireballWhoosh", src: createSrcString( "assets/sounds/%1%/Freespins/FireBallWhoosh0.ogg" ), data:1 , type: createjs.PreloadJS.SOUND },
		{id: "SndLongSpinLoop", src: createSrcString( "assets/sounds/%1%/Reels/LongSpinLoop.ogg" ), data:1 , type: createjs.PreloadJS.SOUND }
	];
	
	var commonHQ_NonIE = [
	];

	if ( clientInfo.getAudioQuality() !== "LQ" ){
		manifest_audio.push.apply( manifest_audio, common );
		if ( clientInfo.getAudioQuality() === "MQ"){
			manifest_audio.push.apply( manifest_audio, commonMQ );
		} else if ( clientInfo.getAudioQuality() === "HQ" ){
			manifest_audio.push.apply( manifest_audio, commonHQ_All );
			if ( !isIE ){
				manifest_audio.push.apply( manifest_audio, commonHQ_NonIE );
			}
		}
	}

	game.manifestAudio = manifest_audio;

}());