goog.provide( 'game.settings' );
goog.require( 'game.logic' );


(function() {
	"use strict";

	var verticalSymbolSpacing = game.logic.GameData.numberOfSymbolRows === 4 ? 92 : 123;

	var symbolsData = {
		'S': {
			name: 'RS_Symbol0', sound: 'SndSymbol0', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: 0, y: 1 }
		}, // Saphire
		'E': {
			name: 'RS_Symbol1', sound: 'SndSymbol1', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: 0, y: 2 }
		}, // Emerald
		'R': {
			name: 'RS_Symbol2', sound: 'SndSymbol2', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: 0, y: 1 }
		}, // Ruby
		'SS': {
			name: 'RS_Symbol3', sound: 'SndSymbol3', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: 0, y: 1 }
		}, // Sword and Shield
		'T': {
			name: 'RS_Symbol4', sound: 'SndSymbol4', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: 0, y: 1 }
		}, // Toucan
		'I': {
			name: 'RS_Symbol5', sound: 'SndSymbol5', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: 0, y: 0 }
		}, // Iguana
		'J': {
			name: 'RS_Symbol6', sound: 'SndSymbol6', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: -1, y: 0 }
		}, // Jaguar
		'CG': {
			name: 'RS_Symbol7', sound: 'SndSymbol7', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: 0, y: 0 }
		}, // City of Gold
		'GA': {
			name: 'RS_Jackpot', sound: 'SndSymbolJackpot', zOrder: 0,
			animSettings: { autoplay: false, fps: 15, timesToLoop: 1, reverseLoop: true },
			animOffset: { x: 0, y: -1 }
		}, // Goddess Of The Amazon
		'WI': {
			name: 'RS_Wild', sound: 'SndSilence', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: -2, y: -2 }
		}, // Wild
		'FW': {
			name: 'FS_Wild', sound: 'SndSilence', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 3, reverseLoop: true },
			animOffset: { x: -2, y: -2 }
		}, // Freespins Wild
		'MS': {
			name: 'RS_Mystery', sound: 'SndSilence', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 1, reverseLoop: true },
			animOffset: { x: 0, y: 0 }
		}, // Mystery symbol
		'FS': {
			name: 'RS_FreeSpins', sound: 'SndSymbolFreespins', zOrder: 1,
			animSettings: { autoplay: false, fps: 10, timesToLoop: 0, reverseLoop: true },
			animOffset: { x: 0, y: 0 }
		}, // Freespins
		'GP': {
			name: 'RS_Prog1', sound: 'SndSymbolJackpot', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 9, reverseLoop: true },
			animOffset: { x: -1, y: -1 }
		}, // Gold Jackpot
		'SP': {
			name: 'RS_Prog2', sound: 'SndSymbolJackpot', zOrder: 0,
			animSettings: { autoplay: false, fps: 20, timesToLoop: 9, reverseLoop: true },
			animOffset: { x: -1, y: -1 }
		}  // Silver Jackpot
	};

	var hideDuringFreespinsVisibilityFunc = function() {
		return !libs.globalEventBus.isActivityRunning( 'libs.activity.display.freespinsUI' );
	};

	var extraActivities = [
		'libs.activity.betState.idle',
		'libs.activity.systemState.gamePlaying',
		'libs.activity.display.freespinsUI'
	];

	libs.client.reels.buttonModels.baseModel.updateEvents.activityStartAndEnd = libs.client.reels.buttonModels.baseModel.updateEvents.activityStartAndEnd.concat( extraActivities );
	libs.client.reels.meterModels.baseModel.updateEvents.activityStartAndEnd = libs.client.reels.meterModels.baseModel.updateEvents.activityStartAndEnd.concat( extraActivities );

	var settings = {
		customerName: 'inspired',
		symbolsData: symbolsData,
		imposeReelSymbolSets: [
			{
				randomiseStartIndex: true,
				data: [
					[-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', -1, -1, -1, -1, -1],
					[-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1],
					[-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1],
					[-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1],
					[-1, -1, -1, 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'MS', 'MS', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1]
				]
			}
		],
		anticipationSymbols: [
			{
				symbolId: 'FS',
				minSymbolCount: 2
			},
			{
				symbolId: 'SP',
				minSymbolCount: 4
			},
			{
				symbolId: 'GP',
				minSymbolCount: 4
			}
		],
		specialLandingSounds: [
			{
				symbolId: 'FS',
				minFinalSymbolCount: 3,
				reelStopSound: 'SndFeatureSymbolLand'
			}
		],
		stopAutoplayOnSymbolsWin: [
			'FS',
			'SP',
			'GP'
		],
		sounds: {
			reelSpin: 'SndReelSpin1',
			countUpWin: 'SndCountUpWin',
			reelStop: 'SndReelStop'
		},
		backgroundStrips: {
			imageName: 'RS_Background_Strips'
		},
		reelsFrame: {
			imageName: 'RS_ReelFrame'
		},
		freespins: {
			fireball: {
				freeSpinsSpinSpeed: 1.7,
				verticalSymbolSpacing: verticalSymbolSpacing
			}
		},
		freespinsCharacter: {},
		freespinsCoins: {},
		freespinsModalIntro: {},
		freespinsModalExtra: {},
		freespinsModalOutro: {},
		freespinsMusic: {},
		hiddenWilds: {
			verticalSymbolSpacing: verticalSymbolSpacing
		},
		models: {
			hideDuringFreespinsVisibilityFunc: hideDuringFreespinsVisibilityFunc
		},
		libs: {
			core: {
				inputController: {},
				inputGestureController: {},
				languageController: {},
				preloader: {},
				systemStateController: {},
				updateManager: {},
				debugServerInterface: {}
			},
			displayHandler: {
				resizeForFooter: true
			},
			presentation: {
				bigWinner: {
					awardLevels: [
						{ animName: 'RS_BigWinCoin' },
						{ animName: 'RS_BigWinStar' },
						{ animName: 'RS_BigWinGem' }
					]
				},
				deviceCompatibilityDisplay: {},
				errorScreen: {},
				fpsDisplay: false,
				introVideo: {},
				loadingScreen: {},
				soundSelection: {},
				balanceMeter: {},
				freespinsJackpotTierOneMeter: {},
				freespinsWinningsMeter: {},
				jackpotTierOneMeter: {},
				jackpotTierTwoMeter: {
					model: {
						shouldBeVisible: hideDuringFreespinsVisibilityFunc
					}
				},
				totalWinMeter: {
					model: {
						shouldBeVisible: hideDuringFreespinsVisibilityFunc
					}
				},
				messageBar: {},
				gameNameFooter: {}
			},
			reels: {
				autoplayButton: {
					model: {
						shouldBeVisible: hideDuringFreespinsVisibilityFunc
					}
				},
				spinButton: {},
				autoplaySelector: {},
				betStateController: {},
				reelsStateController: {
					endOfReelsGameModals: {
						modalEvents: [
							'game.activity.freespins.modalOutro',
							'libs.activity.display.bigWinner',
							'game.activity.freespins.modalIntro',
							'game.activity.freespins.modalExtra'
						]
					}
				},
				paytable: {
					version: BUILD_VERSION
				},
				reelsAnticipation: {},
				reelView: {
					symbolsData: symbolsData,
					verticalSymbolSpacing: verticalSymbolSpacing
				},
				slotsGameModel: {},
				reelsMachineModel: {
					verticalSymbolSpacing: verticalSymbolSpacing
				},
				winAnimation: {},
				winlinesModel: {},
				winlinesView: {},
				xOfAKindSplash: {
					growTime: 400
				}
			},
			rendering: {
				surfaceManager: {}
			},
			sound: {
				soundPlayer: {}
			},
			utils: {
				hapticPlayer: {}
			}

		}
	};


	game.settings = settings;

}());